# This -*- perl -*- script checks for required and recommended Perl
# modules 

require 5.004;
use strict;
use Config;

$| = 1;
my $done=0;
my $separator= '-'x70 . "\n";
my $sect_separator= "\n" . '*'x70 . "\n";
my $ans="y";
my (@errors,$docDir,$scriptDir);
$::configChanges="";

print $separator;
print "     HEASARC BROWSE Check Perl Modules SCRIPT:\n";
print "     -----------------------------------\n";
print "\nThis script will ";
print "try to locate necessary Perl modules.\n";
print "\nAfter installing any missing modules rerun this script to check\n" .
	"if the Perl module installation is complete.\n";

print "\n** Continue? (y/n) [y] ";
chomp($ans=<STDIN>);  
$ans="y" unless $ans;
exit unless $ans =~ /y/i;


print $separator;

#**********************************************************************
#--- Check for required Perl libraries
#
print "\nRequired Perl modules will be searched for in the following " .
"directories:\n@INC\n";

print "\n** Do you want to add any directories to this list? (y/n) [n] ";
chomp($ans=<STDIN>);  
$ans="n" unless $ans;
my $extraDirs;
my $extraDirsConfig=$_;
if ($ans =~ /y/i) {
   print "\n** Enter additional directories separated by a comma ";
   chomp($extraDirs=<STDIN>);  
   if ($extraDirs) {
      $_=$extraDirs;
      s/,/","/;
      s/\s//;
      $extraDirsConfig=$_;
   
      print "Adding $extraDirs\n";
      @::browselibs=split(/,/,$extraDirs);
   
      unshift (@INC,@::browselibs);

      print "\nPerl path is now: @INC\n\n";
      $::configChanges .= "\n" .'@::browselibs' . 
   	"= (\"$extraDirsConfig\",\"./lib\")\n"  .
	"unshift(". '@INC' . "," . '@::browselibs' .")\n";


        print "\n\nconfig=$::configChanges";
   } else {
      print "\n>>No directories added to current Perl library path\n";
   }
} 

print $separator;

required_mods();
#---------------------------------------------------------------------

print $sect_separator;



my $perllib=$ENV{PERL5LIB};
print "Finished with Browse Perl Module check!\n";
if ($perllib ||$::configChanges) {
print "\n- If there are no errors below you should now edit \n".
	"- $scriptDir/lib/Browse/Config.pm\n";
print "- to set variables specific to your installation. \n";

print "\n- The code that defines the " . '@INC '. 
	"- variables should be changed as follows:\n" .
	"$::configChanges" if $::configChanges;

if ($perllib) {
   if ($::configChanges =~ /browselibs/) {
      print "\n- You may need to add the following directories (currently in" .
      "\n- the PERL5LIB environment variable) to the " . '@::browselibs list' .
      "\n- or separately as in the following example code:\n"
   } else {
      print "\n- You may need to include the following directories \n" .
      "- (currently in the PERL5LIB environment variable) to Config.pm." .
      "\n- The following example code should work;\n";
   }
      my @libs=split(/:/,$perllib);
      my $liblist="";
      for (my $i=0; $i <=$#libs; ++$i ) {
         $liblist .= "\"$libs[$i]\"";
         if ($i <$#libs) {
            $liblist .= ",";
         }
      }
   
      print "\n" .'@perllibs' . 
   	"= ($liblist)\n"  .
	"unshift(". '@INC' . "," . '@perllibs' .")\n";
}
}
$done=1;
# Make sure we print a list of all errors.
END {
   my $err;
   exit unless scalar(@errors);
   print "\n"."="x75 ."\nSummary of all errors:\n\n";
   foreach $err (@errors) {
      print "* " . $err;
   }
   print "\nThe README file may have information necessary to fix these" .
   	"\nerrors.  Rerun this script after missing mudules are " .
	"installed.\n";
   print "runme.pl terminated abnormally! $!\n" unless $done;
   print "\n"."="x75 ."\n";
}

#-----------------------------------------------------------------------------
sub required_mods {
# Check for non-standard modules that are used by Browse

  my %check = (
     "LWP.pm,REQUIRED" => "The LWP module is needed for querying VizieR tables, and for startfv.pl, which slurps in remote FITS files for viewing.",
     "DBI.pm,REQUIRED" => "The DBI module is REQUIRED for querying the databases. ",
     "GDBM_File.pm,REQUIRED" => "The GMDB_File module is required for resolving object names.",
     "CGI.pm,REQUIRED" => "The CGI module is required for Error messages.",
     "XML/Parser.pm,REQUIRED" => "The XML::Parser module is required for querying VizieR tables.",
     "RPC/PlClient.pm,recommended for testing database connection code by  connecting to Heasarc database" => "The RPC module is recommended for testing database connections by connecting to the HEASARC database via a DBI proxy.",
     "Filesys/DiskSpace.pm,optional but requires code alteration to leave out" => "The Filesys::DiskSpace module is used in Products.pm for determining how much disk space is available before creating a tar file.  You can replace this code with a call to 'du' (du is not installed in our chrooted web tree, for security.).",
  );

  my $failed=0;
  foreach my $key (keys %check) {
     my ($mod, $status) = split(',',$key);
     print "Checking for $mod...........";
     eval { require $mod; };
     if ($@) {
        if ($status =~ /REQUIRED/) {$failed=1;}
        print " failed\n";
        Error("$mod module is $status!");
        print "$@\n$check{$key}\n\n";
        sleep(2);
        $@='';
     } else {
        print " ok\n";
        print "   Available DBI drivers: ",
               join(', ',DBI->available_drivers),"\n" if $mod =~ /DBI/;
     }
  }
  if ($failed) {
print $separator;
   my $err;
   print "\nSummary of Perl Module Test\n\n";  
   foreach $err (@errors) {
      print "* " . $err;
   }
   print $separator;
   print "\nRequired Perl modules are missing. ";  
   print "Perl Module installation help\n" .
      "can be found in the README files.\n";
   
  }
}

#-----------------------------------------------------------------------------
sub Error {
# print out each error as it comes in, and append it to a list of all errors.
   print "\n"."-"x75 . "\n@_\n" ."-"x75 ."\n";
   push @errors,"@_\n";
}



#-----------------------------------------------------------------------------
sub quit {
   print "\nOkay, but installation is not complete. Rerun \"$^X $0\" to try again.\n\n";
   exit;
}
