package heasarc.w3browse.plot;

/**
 *           W3Browse Plotting Applet                           **
 *                                                              **
 * SOURCE: PlotSpecs.java                                       **
 *                                                              **
 * @author Saima Zobair (RITSS)   saima@lheamail.gsfc.nasa.gov  **
 *                                                              **
 *****************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import heasarc.w3browse.*;

/** This class contains all data that defines the type of plot */
public class PlotSpecs {
    
  /** Plot type (such as scatter, histogram, cumulative, aitoff */
  public String plottype;
    
  /** Name of right ascension variable in database */
  public String ra_name;
    
  /** Name of declination variable in database */
  public String dec_name;
    
  /** histogram  number of bins */
  public int bin;
    
  /** parameter name for xaxis name */ 
  public String xaxis;
    
  /** parameter name for yaxis name */ 
  public String yaxis;
    
  /** parameter name for optional yaxis */ 
  public String yaxis2;
    
  /** parameter name for optional yaxis */ 
  public String yaxis3;
    
  /** indicates if 2nd additional dataset is requested */
  public boolean isYOption2; 
    
  /** indicates if 3rd additional dataset is requested */
  public boolean isYOption3; 
    
  /** indicates if 3rd additional dataset is requested */
  public boolean showErrorBars; 
    
  /** indicates if the x axis is a log axis */
  public boolean isxlog; 
    
  /** indicates if the y axis is a log axis */
  public boolean isylog;
    
  /** sets xminimum range */
  public double xmin = Double.NaN;
    
  /** sets xmaximum range */
  public double xmax = Double.NaN;
    
  /** sets yminimum range */
  public double ymin = Double.NaN;
    
  /** sets ymaximum range */
  public double ymax = Double.NaN;
    
  /** sets yminimum range */
  public double y2min = Double.NaN;
    
  /** sets ymaximum range */
  public double y2max = Double.NaN;
    
  /** sets yminimum range */
  public double y3min = Double.NaN;
    
  /** sets ymaximum range */
  public double y3max = Double.NaN;
    
  /** is sampling on ? */
  public boolean isSample;
    
  /** criteria to base sample parameter on */
  public String criteria;
    
  /** parameter number for sample parameter */
  public String sample_param;
    
  /** Process number for creating giffile name */
  public String procnum = null;
    
  /** URL to query to generate data. */
  String url;
    
  /** Information to be posted to the URL */
  String postData;
    
  public HeasarcQuery hquery;

  /** Constructor for PlotSpecs */
  public PlotSpecs(String [] args) throws InputException{
      
    String urlbase = null;
      
    for(int i=0;i<args.length;i++) {

      if (args[i].startsWith("plot-type=")){
         plottype = args[i].substring(10);

      } else if(args[i].startsWith("bin=")){
         bin = getBin(args[i].substring(4));

      } else if(args[i].startsWith("xaxis=")){
         xaxis = args[i].substring(6);

      } else if(args[i].startsWith("yaxis=")){
         yaxis = args[i].substring(6);

      } else if(args[i].startsWith("xlog=")){
         isxlog = (Boolean.valueOf(args[i].substring(5))).booleanValue();

      } else if(args[i].startsWith("ylog=")){
         isylog = (Boolean.valueOf(args[i].substring(5))).booleanValue();

      } else if(args[i].startsWith("xmin=")){
         xmin = getValue(args[i].substring(5));

      } else if(args[i].startsWith("xmax=")){
         xmax = getValue(args[i].substring(5));

      } else if(args[i].startsWith("ymax=")){
         ymax = getValue(args[i].substring(5));

      } else if(args[i].startsWith("ymin=")){
         ymin = getValue(args[i].substring(5));

      } else if(args[i].startsWith("ra=")){
         ra_name = args[i].substring(3);

      } else if(args[i].startsWith("dec=")){
         dec_name = args[i].substring(4);

      } else if(args[i].startsWith("sample_param=")){
         sample_param = args[i].substring(13);

      } else if(args[i].startsWith("criteria=")){
         criteria = args[i].substring(9);

      } else if(args[i].startsWith("isSample=")){
         isSample = (Boolean.valueOf(args[i].substring(9))).booleanValue();

      } else if(args[i].startsWith("procnum=")){
         procnum  = args[i].substring(8);

      } else if(args[i].startsWith("urlbase=")){
         urlbase  = args[i].substring(8);

      } else if(args[i].startsWith("request=")){
         postData = "dispmode=StreamDisplay&request="+
	                         args[i].substring(8);
      } else if (args[i].startsWith("url=")) {
	 url      = args[i].substring(4);  

      } else {
         System.out.println ("Error: Unknown parameter " + args[i]);
      }
    }
      
    if (urlbase != null) {
	 url = urlbase+url;
    }
	
    //displayall();

  }
    
  /** Constructor which takes Options panel to determine plotspecs */
  public PlotSpecs (PlotOptions pOptions, String ra, String dec, String url)
    throws InputException{
	
      plottype = pOptions.getPlotType();
      hquery   = pOptions.getQuery();

      xaxis = pOptions.getXAxis();
      yaxis = pOptions.getYAxis();

      isYOption2 = pOptions.isYOption2();
      isYOption3 = pOptions.isYOption3();
      if (isYOption2) {yaxis2 = pOptions.getY2Axis();}
      if (isYOption3) {yaxis3 = pOptions.getY3Axis();}

      showErrorBars = pOptions.showErrorBars();
      String bins = pOptions.getBin();
      bin = getBin(bins);

      isxlog = pOptions.isXLog();
      isylog = pOptions.isYLog();

      String xmins = pOptions.getXmin();
      xmin = getValue(xmins);
      String xmaxs = pOptions.getXmax();
      xmax = getValue(xmaxs);

      String ymins = pOptions.getYmin();
      ymin = getValue(ymins);
      String ymaxs = pOptions.getYmax();
      ymax = getValue(ymaxs);

      //--- min and max values for optional y axis
      if (isYOption2) {
         String y2mins = pOptions.getY2min();
         y2min = getValue(y2mins);
	 String y2maxs = pOptions.getY2max();
         y2max = getValue(y2maxs);
      }

      //--- min and max values for optional y axis
      if (isYOption3) {
         String y3mins = pOptions.getY3min();
         y3min = getValue(y3mins);
	 String y3maxs = pOptions.getY3max();
         y3max = getValue(y3maxs);
      }

      ra_name  = ra;
      dec_name = dec;

      isSample = pOptions.getIsSample(); 
      criteria = pOptions.getSampleText();
      sample_param  = pOptions.getParam();
      this.url = url;

      //displayall();
  }
    
  /** for debugging purposes displays all parsed data */
  public void displayall() {
      System.out.println("PlotType:" + plottype);
      System.out.println("XAXIS:" + xaxis + " YAXIS:" + yaxis);
      if (isYOption2) { System.out.println("YAXIS2:" + yaxis2 );}
      if (isYOption3) { System.out.println("YAXIS3:" + yaxis3 );}
      System.out.println("Errorbars:" + showErrorBars);
      System.out.println("IsXLog:" + isxlog + " IsYLog:" + isylog);
      System.out.println("XMIN:" + xmin + " XMAX:" + xmax);
      System.out.println("YMIN:" + ymin + " YMAX:" + ymax);
      System.out.println("Bin:" + bin);
      System.out.println("isSample:" + isSample + " SampleParm: " + sample_param);
      System.out.println("Criteria:" + criteria);
      System.out.println("URL:" + url);
  }
    
  /** parses an integer */
  private int tryparseInt(String s) throws InputException {
    int num = 0;
    try {
      num = Integer.parseInt(s);
    }catch(NumberFormatException e) {
      throw new InputException("Incorrect number format "+s);
    }
    return(num);
  }
    
  /** gets Bin int value for PlotSpecs */
  private int getBin(String bins) throws InputException {
    int bin = 0;
    if(bins.equals("")) {
      return (-1);
    }
    bin = tryparseInt(bins);
    if(bin <1){
        throw new InputException("Bin is not a positive integer.");
    }
    return(bin);
  }

  /** returns the double equivalant of entered value number */
  protected double getValue(String value) throws InputException {
    double val=0.0;
    if (value.equals("")) {
      return(Double.NaN);
    }
    try {
       Double dval=Double.valueOf(value);
       val = dval.doubleValue();
    } catch (NumberFormatException e) {
       throw new InputException("Incorrect number format in setting min/max ranges.");
    }
    return(val);
  }
    
  /** Get the URL queried for this table. */
  public String getURL() {
      return url;
  }
  /** Get the POST informaiton for the query. */
  public String getPostData() {
      return postData;
  }

}

