#!/usr1/local/bin/perl5.8.6
#=============================================================================#
# Program Name:   getvotable.pl
# Module Author:  Edward J. Sabol <sabol@lheamail.gsfc.nasa.gov>
# Creation Date:  2006/03/17
# Project:        HEASARC
#=============================================================================#
# Program Description:
#   Takes a CGI argument specifying the name of a table in the HEASARC DBMS
#   and then dumps it in VOTable 1.1 XML format.
#=============================================================================#
#
# $Id: getvotable.pl,v 1.2 2008/11/15 00:29:07 sabol Exp $
#
#=============================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but, unless you change it
# significantly, please leave the above author names and address intact.
#=============================================================================#

BEGIN {
    $ENV{'HOME'} = '/tmp';
    $ENV{'PATH'} = '/bin';
}

use strict;
use FileHandle;
use CGI qw(-oldstyle_urls -unique_headers);
use lib '/www/htdocs/cgi-bin/lib/modperl';
use Error qw(:try);
use lib '/www/htdocs/cgi-bin/lib/heasarc';
use Util qw(DumpHash);
use HEASARC::TDAT;

our $debug = 0;
# $debug = $HEASARC::TDAT::debug = 1;

our $max_concurrent_downloads_total = 10;
our $max_concurrent_downloads_per_ip = 3;

our $form = new CGI;

try {
    if ($form->param() && (my $name = $form->param('name'))) {
	$name = lc($name);
	$name =~ s/^heasarc_//;
	my $file = "/FTP/heasarc/dbase/tdat_files/heasarc_${name}.tdat.gz";
	my $tdat = new HEASARC::TDAT('file' => $file,
				     'defer_data' => 1);
	&check_concurrent_downloads($name);
	print &content_header($name);
	$tdat->dump_as_VOTable;
    } else {
	throw HEASARC::Exception(-text => "No HEASARC table name specified.");
    }
} catch Error with {
    my $exception = shift;
    &handle_exception($exception);
};

#------------------------------------------------------------------------------
sub content_header {
    my($name) = @_;
    $name ||= 'error';
    return($form->header(-type => 'application/x-votable+xml',
			 -charset => 'utf-8',
			 -attachment => "${name}.xml"));

} #eosub--content_header
#------------------------------------------------------------------------------
sub handle_exception {
    my($exception) = @_;

    my $error = &$HEASARC::TDAT::xml_header_encoder(ref($exception)
						    ? $exception->text
						    : defined($exception)
						      ? "$exception" : '');
    $error = "Unknown error encountered." unless ($error ne '');
    print &content_header('error'),
          qq{<?xml version="1.0"?>
<VOTABLE version="1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="http://www.ivoa.net/xml/VOTable/VOTable/v1.1">
  <RESOURCE>
    <INFO>$error</INFO>
  </RESOURCE>
</VOTABLE>
};

} #eosub--handle_exception
#------------------------------------------------------------------------------
sub check_concurrent_downloads {

    my($name) = @_;

    my $uid = $>;
    my $procname = quotemeta($ENV{SCRIPT_NAME});
    my $procregex = qr/$procname/;
    $0 .= " name=$name REMOTE_ADDR=$ENV{'REMOTE_ADDR'}";
    my @psoutput = `ps -u $uid ww`;
    my @pids = ();
    my %remote_addrs = ();
    foreach my $psline (@psoutput) {
	next unless ($psline =~ /$procregex/);
	my $pid = int(substr($psline,0,6));
	push(@pids,$pid) if $pid;
	if ($psline =~ /REMOTE_ADDR=([\d+\.]+)/i) {
	    $remote_addrs{$1}++;
	}
    }
    if (scalar(@pids) > $max_concurrent_downloads_total) {
	throw HEASARC::Exception(-text => 'The maximum number of concurrent HEAsoft downloads has been reached. Please wait a couple of minutes and try again. We apologize for the inconvenience. If the problem persists, please contact us at the Browse Help Desk: http://heasarc.gsfc.nasa.gov/cgi-bin/Feedback');
    } elsif ($remote_addrs{$ENV{'REMOTE_ADDR'}}
	     > $max_concurrent_downloads_per_ip) {
	throw HEASARC::Exception(-text => 'There are a limited number VOTable downloads allowed per IP address. Please wait until your other VOTable downloads have completed or cancel them before submitting another VOTable download request or just try again in a few minutes. We apologize for the inconvenience. If you continue to have difficulty downloading HEASARC database tables in VOTable format, please contact us at the Browse Help Desk: http://heasarc.gsfc.nasa.gov/cgi-bin/Feedback');
    }
    return;

} #eosub--check_concurrent_downloads
#------------------------------------------------------------------------------
#########
#  EOF  #
#########
