# $Id: CGI.pm,v 1.13 2008/12/30 20:51:08 lmm Exp $
# $Log: CGI.pm,v $
# Revision 1.13  2008/12/30 20:51:08  lmm
# Made some corrections to code that deals with coordinate file upload.
#
# Revision 1.12  2005/02/24 14:38:14  lmm
# Added semicolon between multiple objects entered by user (for master
#   table query)
#
# Revision 1.11  2004/01/15 18:43:17  lmm
# Added some debug statements
#
# Revision 1.10  2003/12/05 21:08:35  lmm
# Removed call to non-existent error handling method
#
# Revision 1.9  2003/12/02 19:37:06  lmm
# Added functionality to read and parse query and target file uploads
#
# Revision 1.8  2002/03/15 23:36:14  cwh
# removed writeheader from Header().
#
# Revision 1.7  2001/11/07 23:38:40  cwh
# Moved Header from old HTML.pm in here.
#
# Revision 1.6  2001/04/25 16:45:29  cwh
# Added Deform and Enform.
#
# Revision 1.5  2000/03/08 20:06:54  saima
# W3Browse to Browse name change.
#
# Revision 1.4  1999/09/15 14:15:07  saima
# Removed reference to CGI::Action(), and CGI::SimpleSubmit() deprecated
#   routines.
#
# Revision 1.3  1999/08/26 19:39:17  saima
# Modified ReadParse() so that it returns the input line on request.
#   This feature is used for the creation of the HEASARC-control
#   form used by HERA.
#
# Revision 1.2  1999/05/27 15:50:05  tam
# Modified to use mod_perl in Apache.  All references to &smart removed
#
# Revision 1.1  1998/12/22 20:05:52  saima
# Previosuly known as CGI.pl. Other minor tweaks.
#
# Revision 1.8  1998/06/12 19:47:51  saima
# Modified routines to include JavaScript code for validating user input.
#
# Revision 1.7  1997/11/25 23:07:27  saima
# Added -T switch to perl version.
#
# Revision 1.6  1997/04/17 22:03:24  sabol
# HTML table headers in SimpleSubmit are now designated as such by <th> tags
#   instead of previously incorrect <td> tags.
#
# Revision 1.5  1997/03/21 23:29:02  sabol
# Added SimpleSubmit() and PrintVariables() functions. The latter is based on
#   a function from Steven Brenner's cgi-lib.pl.
#
# Revision 1.4  1996/12/19 22:25:04  sabol
# Changed $max_columns to 4 (previously 3). Change HTML output slightly for
#   better style.
#
# Revision 1.3  1996/11/22 20:38:43  sabol
# Added ReadParse, Encode, and Decode routines.
#
# Revision 1.2  1996/09/20 22:44:58  sabol
# If there are only one or two actions and the browser doesn't support tables,
#   then it doesn't use a select menu for displaying the actions.
#
# Revision 1.1  1996/09/20 21:01:46  sabol
# Added CGI.pl.
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=======================================================================#
# Module Name:    CGI.pl
# Module Authors: Tom McGlynn (USRA), Edward J. Sabol (Hughes STX)
# Date:           09/96
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
# Constructs buttons and actions for use in Browse.
#
#=======================================================================#

use strict;

use Browse::CGILIB;
use Browse::Parse;   # parse input file with targets

package Browse::CGI;

#------------------------------------------------------------------------
sub Header {
    return("Content-type: text/html\n\n");
} #eosub--Header
#--------------------------------------------------------------------------
sub ReadParse {

    my($getInputString) = 0;
    if (scalar(@_)) {
      $getInputString = @_;
    }

    my(%in);
    my($in) = {};
    my($input); 

   if ($ENV{'CONTENT_TYPE'} =~ m#^multipart/form-data#) {

      #-- If CGI method is POST with multipart/form-data content type
      #-- call the CGILIB package routine which can handle data read
      #-- from user file.  

      # --------------- variables for upload file ---------------------
      my (
          %cgi_cfn,   # The uploaded file(s) client-provided name(s)
          %cgi_ct,    # The uploaded file(s) content-type(s).  These are
                      #   set by the user's browser and may be unreliable
          %cgi_sfn,   # The uploaded file name on the server (this machine)
          $ret,       # Return value of the ReadParse call.       
          $buf        # Buffer for data read from disk.
         );

      # --------------- start reading upload file ---------------------
      # Start off by reading and parsing the data.  Save the return value.
      # Pass references to retreive the data, the filenames, and the 
      # content-type

      $ret = Browse::CGILIB::LibReadParse(\%{$in},\%cgi_cfn,
		\%cgi_ct,\%cgi_sfn);
      if ($::debug) {print "CGI: method=" ." multipart<br>";}
      if ($::debug) {
         my($k);
         foreach $k (keys  %{$in}) {
            print "from CGI::" .$k . " ... " .$in->{$k} ."<br>\n";
         }
      }

      #-- Check for existence or problems with upload file before reading
      print "CGI: upload file ? " . $cgi_sfn{'upfile'} . ".<br>" if $::debug;

      if (defined  $cgi_sfn{'upfile'} && -s $cgi_sfn{'upfile'}) {
         open (UPFILE, $cgi_sfn{'upfile'}) or 
          Browse::Utils::Error("Error: Unable to open file $cgi_sfn{'upfile'}");
     
         $in->{"userTargetFile"}=$cgi_cfn{'upfile'};

         $buf = "";    
         my $dataFromFile  = "";    
         while (read (UPFILE, $buf, 8192)) {
            print "from CGI::buf=" . $buf . "<br>\n" if $::debug;

            # Munge the uploaded text so that it doesn't contain HTML elements
            # This munging isn't complete -- lots of illegal characters are left
            # However, it takes care of the most common culprits.  
            $buf =~ s/</&lt;/g;
            $buf =~ s/>/&gt;/g;
            $dataFromFile .= $buf;
         }
         close (UPFILE);
      
         # Parse the file content -- may be list of targets, coords
         # delimited by semicolons and/or newlines or "Pure Text" output
         print "from CGI::calling parse<br>\n" if $::debug;
      
         my ($entry,$errorMsg)=Browse::Parse::parseInputFile($dataFromFile,
	   $in->{Coordinates});

         if ($errorMsg ne "") {
            print "CGI: error " . $errorMsg . "<br>" if $::debug;
            Browse::Utils::Error("$errorMsg" );
         }

         #-- If targets have been entered elsewhere on the form 
         #-- append new targets from file
         $in->{"Entry"} .=  ";" unless $in->{"Entry"} eq "";
         $in->{"Entry"} .=  $entry;

         unlink ($cgi_sfn{'upfile'}) if -e $cgi_sfn{'upfile'};
         #unlink ($cgi_sfn{'upfile'}) or
             #&CgiError("Error: Unable to delete file",
             #"Error: Unable to delete file $cgi_sfn{'upfile'}: $!\n");
      }  #--- End upload file processing
  
  } else {

    # Based on Steven E. Brenner's cgi-lib Perl routines. This version has
    # been jazzed up to be more Perl 5-savvy. It should also be faster.
    #
    # Reads in GET or POST data, converts it to unescaped text, and returns a
    # reference to a key/value hash of the data, using '\0' to separate multiple
    # values with the same key.
    #
    # Function  input: (optional) reference to a hash to be used by this routine
    # Function output: reference to a key/value hash of the GET or POST CGI data
    #
    # For data read from file a routine is called from CGILIB (more of 
    # Brenner's work) which deals with multipart/form-data (POST only)



    #my($entry,$input,$key,$val);
    my($entry,$key,$val);

    # Read in text.
    print "::before parse:" .$ENV{'CONTENT_TYPE'} . " " .$ENV{'REQUEST_METHOD'}.
	"<br>" if $::debug;
    if ($ENV{'REQUEST_METHOD'} eq "GET") {
	$input = $ENV{'QUERY_STRING'};
        print "GET:: $input<br>" if $::debug;
    } elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
	read(STDIN,$input,$ENV{'CONTENT_LENGTH'});
        print "$input<br>" if $::debug;
        print "POST:: " .Decode($input) . "<br>" if $::debug;
    } 
    
    foreach $entry (split(/&/,$input)) {

	# Convert %XX from hex numbers to alphanumeric and '+' to ' '.
	$entry = Decode($entry);

	# Split into key and value.
	($key,$val) = split(/=/,$entry,2);
        #print "CGI:: " . $key . "=" . $val ."<br>\n" if $::debug;

	# Associate key and value.
	$in->{$key} .= "\0" if (defined($in->{$key})); # \0 is the separator
	$in->{$key} .= $val;

    }
   }
      # separate multiple entries .Added for master table   
      #$in->{"Entry"} =~  s/\0/;/g unless ($in->{"Entry"} eq "" || !defined($in->{"Entry"}));
      if ($in->{"Entry"}) {
         $in->{"Entry"} =~  s/\0/;/g;
       }
      #$in->{"Entry"} =~  s/\W/"&#".ord($&).";"/ge;
 

    if ($getInputString){ # if script needs the $input info (see w3query.pl)
      return($in,$input);
    } else {
      return($in);
    }

} #eosub--ReadParse
#--------------------------------------------------------------------------
#sub ReadFile {
#my$tmpfile='/tmp';
#} #eosub--ReadFile
#--------------------------------------------------------------------------
sub Decode {

    my($string) = shift(@_);
    $string =~ tr/\+/ /;
    $string =~ s/%([0-9a-fA-F][0-9a-fA-F])/chr(hex($1))/ge;

    return($string);

} #eosub--Decode
#--------------------------------------------------------------------------
sub Encode {

    my($string) = shift(@_);

    $string =~ s/([^ 0-9A-Za-z])/sprintf("%%%02X",ord($1))/ge;
    $string =~ tr/ /+/;

    return($string);

} #eosub--Encode
#--------------------------------------------------------------------------
sub PrintVariables {

    my($in) = shift(@_);

    my($key,$value,$output);
    $output = "<dl compact>";
    foreach $key (sort keys(%{ $in })) {
	foreach $value (split("\0",$in->{$key})) {
	    $value =~ s/\n/<br>/mg;
	    $output .= "<dt><b>$key</b><dd><i>$value</i><br>";
	}
    }
    $output .= "</dl>";

    return($output);

} #eosub--PrintVariables
#--------------------------------------------------------------------------
sub Deform{

    my ($str) = @_;

    my (%decoding);
    my ($key);
    my ($val);

    my @fields = split('&', $str);


    foreach (@fields) {
        ($key, $val) = split('=', $_);

        ($key) = Decode ($key);
        ($val) = Decode ($val);

        if (defined($decoding{$key})) {
            $decoding{$key} .= "\0$val";
        } else {
            # First time
            $decoding{$key} = $val;
        }
    }

return \%decoding;
}



sub Enform {

    my ($form) = @_;

    my $str;

    my $key;
    my $val;

    my $amp;

    while (($key,$val) = each(%$form) ) {

        my (@vals) = split("\0", $val);
        foreach $val (@vals) {
            $str .= $amp . Encode($key) . '=' . Encode($val);
            $amp = '&';
        }
    }
    return $str;
}

1;
#########
#  EOF  #
#########
