# $Id: DPQuery.pm,v 1.2 2005/03/07 18:37:26 lmm Exp $
# $Log: DPQuery.pm,v $
# Revision 1.2  2005/03/07 18:37:26  lmm
# Made an error message more meaningful.
#
# Revision 1.1  2002/12/16 21:17:42  cwh
# Parent to HDPQuery.pm.
#
# Revision 1.11  2002/03/22 22:48:35  cwh
# Correctly passing along data set descriptive names.
#
# Revision 1.10  2001/12/05 23:30:45  cwh
# In findKeys, use _unique_id as unique_key if it exists.  Fall back to
#   query_key and unique_key.  Set unique key to 'recno' for Vizier.
# getDataProducts() now uses the keys that are passed in to it, instead of
#   using getKey to get the keys.
# Allow unique keys to be specified as keyname=keyvalue in getDataProducts.
#
# Revision 1.9  2001/11/07 23:34:17  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.8  2001/05/17 21:06:41  tam
# Changed separator in keys to '|' to agree with usage elsewhere.
#
# Revision 1.7  2001/04/25 16:56:05  cwh
# got rid of changes introduced in last commit.
# Tom: Added support for VizieR tables.
#
# Revision 1.6  2001/04/06 20:04:59  cwh
# Changes Ed made:
#   changes findKeys() to first check for the _unique_id parameter and before
#     falling back to the query_key and unique_key virtual parameters.
#
# Revision 1.5  2001/02/15 21:19:13  cwh
# Added more debug info.
#
# Revision 1.4  2001/01/18 22:00:14  cwh
# Changes Tom made: First uses query_key as the unique key, if it exists.  If
#   query_key doesn't exist, then uses unique_key.
#
# Revision 1.3  2000/10/04 20:00:04  tam
# Assume _unique_id columns are ints.  They
# generally do not have information in the database.
#
# Revision 1.2  1999/08/26 17:21:39  saima
# Added more error checks.
#
# Revision 1.1  1999/05/27 17:03:15  tam
# Initial release.
#

# Create simple interface to data products information.
# Based upon the HDBdpl Perl server.
#
# Developed by Tom McGlynn (May 1999)

# see the end of this file for pod.

use strict;

package Browse::DPQuery;
use Browse::Config;

sub new {
    my ($self,$table) = @_;
    my $obj = {};
    
    foreach my $match (keys %::dpq) {
       if ($table =~ /$match/) {
           my $type = "Browse::".$::dpq{$match};
           eval "package Browse::_firesafe; require $type";
           bless $obj, $type;
           last;
       }
    }

    unless (ref($obj) =~ /Browse/) {
      Browse::Utils::Error("DPQ type " . ref($obj) . 
	" table=$table not found! See Config.pm");
    }

    $obj->setTable($table);
    $$obj{'isVizier'} = 1 if $table =~ /\//;
    return $obj;
}

sub clear {}

sub getSets {
    my ($self) = @_;
    $self->findSets() unless defined $$self{'DPSets'};
    return $$self{'DPSets'};
}
sub setSets {
    my ($self,$sets) = @_;
    $$self{'DPSets'} = $sets;
}
sub getFormats {
    my ($self) = @_;
    $self->findSets() unless defined $$self{'DPSetFormats'};
    return $$self{'DPSetFormats'};
}
sub setFormats {
    my ($self,$formats) = @_;
    $$self{'DPSetFormats'} = $formats;
}
sub getDataProducts {
    my ($self,$pkeys,$sets) = @_;
    $self->findDataProducts($pkeys,$sets) unless defined $$self{'DataProducts'};
    return $$self{'DataProducts'};
}
sub setDataProducts {
    my ($self,$products) = @_;
    $$self{'DataProducts'} = $products;
}
#sub getMultiDataProducts {
#    my ($self,$tablehash) = @_;
#    unless defined $$self{'DataProducts'} {
#      foreach my $table (keys %$tablehash) {
#         my $inthash = $$tablehash{$table};
#         my $dpobj = new DPQuery($table);
#         $dpobj->findDataProducts($$inthash{keys},$$inthash{sets}) unless 
#                 defined $$dpobj{'DataProducts'}{$table};
#         $$self{'DataProducts'}{$table} = $$dpobj{'DataProducts'}{$table};
#      }
#    }
#    return $$self{'DataProducts'};
#}


sub setTable {
    my ($self,$table) = @_;
    $$self{'table'} = $table;
}
sub getTable {
    my ($self) = @_;
    return $$self{'table'};
}

sub setKeyString {
    my ($self,$key) = @_;
    $$self{'keystring'} = $key;
}
sub getKeyString {
    my ($self) = @_;
    $self->findKeyString() unless defined $$self{'KeyString'};
    return $$self{'KeyString'};
}

sub getProductColumns {
    my ($self) = @_;
    $self->findProductColumns() unless defined $$self{'ProductColumns'};
    return $$self{'ProductColumns'};
}
sub setProductColumns {
    my ($self,$productColumns) = @_;
    $$self{'ProductColumns'} = $productColumns;
}

sub getShortcuts {
    my ($self) = @_;
    $self->findShortcuts() unless defined $$self{'Shortcuts'};
    return $$self{'Shortcuts'};
}
sub setShortcuts {
    my ($self,$Shortcuts) = @_;
    $$self{'Shortcuts'} = $Shortcuts;
}

sub getUniqueKeys {
    my ($self) = @_;
    $self->findUniqueKeys() unless defined $$self{'UniqueKeys'};
    return $$self{'UniqueKeys'};
}
sub setUniqueKeys {
    my ($self,$keys) = @_;
    $$self{'UniqueKeys'} = $keys;
}

sub getParamType {
    my ($self,$parameter) = @_;
    $self->findParamType($parameter) unless defined 
       $$self{'ParamTypes'}{$parameter};
    return $$self{'ParamTypes'}{$parameter};
}
sub setParamType {
    my ($self,$parameter,$type) = @_;
    $$self{'ParamTypes'}{$parameter} = $type;
}


#---------------------------------------------------------------------------
# these are the installation-specific routines that must be overridden
#---------------------------------------------------------------------------

# Find the dataproduct sets associated with a given table.
sub findSets {
    #$$self{'DPSets'} = \%dpsets;
    #$$self{'DPSetFormats'} = \%dpsetformats;
}

#---------------------------------------------------------------------------

# Given a table and one or more keys, find all of the
# data products associated with the table.  Return
# this information with additional fields allowing the
# user to determine the specific key and data product set
# that a given data product is associated with.

sub findDataProducts {
    #$$self{'DataProducts'} = \%products;
}

#---------------------------------------------------------------------------
sub findKeyString { #unused at the moment
#
#
# return a comma-separated list of the unique keys and values
}

#---------------------------------------------------------------------------
# Find the columns used in creating data product tags.
# Get the product columns for a given table, i.e.,
# those columns in the table that are needed to define
# the tag strings for the associated products.
sub findProductColumns {
    #$self->setProductColumns(join('|', keys %seen));
}

#---------------------------------------------------------------------------
# Get the type of a given parameter.
sub findParamType {
    #return $$self{'ParamTypes'}{$parameter};
}

#---------------------------------------------------------------------------
# Find and resolve all shortcuts used in data product URL definition.
# Note that shortcuts are not defined on a per-table basis.
sub findShortcuts {
    #$self->setShortcuts($shortcuts);
}


#---------------------------------------------------------------------------
# Find the unique key for a table.
sub findUniqueKeys {
    #$self->setUniqueKeys($key);
}


#########  End of utility functions.  ###########################


# Get key and data product information.
# For each table this routine returns the data product
# sets associated with the table, and a description of
# the unique key giving the name and basic type of
# each column in the key.

sub getKeysAndDPSets {
    #return ($dpsets, $key);
}

1;


__END__

=head1 NAME

DPQuery - parent class for data product query objects

=head1 SYNOPSIS

 use Browse::DPQuery;

 my $dpq = new Browse::DPQuery($table);

 my $prodref = $dpq->getDataProducts($keys,$sets);

 my ($dpsets, $keyinfo) = $dpq->getKeysAndDPSets();

=head1 DESCRIPTION

The DPQuery class creates the data product query object, one example of which
is the HDPQuery object. See Config.pm to add your own dp query object.
The DPQuery object is called for each table desired.

=head1 METHODS

The getDataProducts() and getKeysAndDPSets() are the methods currently used
by other Browse modules.

=head2 GET DATA PRODUCTS

 my $prodref = $dpq->getDataProducts($keys,$sets);

$keys is a ref to an array where each element is a unique key string 
identifying a row of the query results, i.e.: "seq_id=12602000".
$sets is a ref to an array where each element is a short dp set name, i.e.: "images".  The $sets ref may be empty, in which case all sets are returned.
The return value is a ref to a hash  with this structure:

 $prodref = { uniqueid => $setsref ,
              "set_descriptions" => $setdescsref }
 $setsref = { setname => $setprodref }
 $setprodref = [ @$prods ]
 $prods = { prodkey = prodval }


=head2 GET UNIQUE KEY AND DATA PRODUCT SETS

 my ($dpsets, $keyinfo) = $dpq->getKeysAndDPSets();

Returns two values. The first is a ref to a hash where the keys are the 
short dp set names, and their values are longer names of the data sets. 
The second is a string of the format: 
       uniqueidfield:format[,uniqueidfield:format]*
 i.e.: _unique_id:int

=head1 AUTHOR INFORMATION

High Energy Astrophysics Science Archive Research Center (HEASARC),
NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.

You are free to use this code as you wish, but unless you change
it significantly, please leave the above name & address intact.

=head1 BUGS

=head1 SEE ALSO

L<Browse::HDPQuery>

=cut




