# $Id: Display.pm,v 1.85 2008/12/15 15:59:24 lmm Exp $
# $Log: Display.pm,v $
# Revision 1.85  2008/12/15 15:59:24  lmm
# Changes to accomodate multiple pairs of coordinates per heasarc table.
#   Added two subroutines otherRaDecs() and renameRaDecs to parse
#   coordinates and to rename results table column headers for galactic
#   and eliptic coordinates.
#
# Revision 1.84  2008/11/17 14:46:10  lmm
# Fixed bug in the display of table link dependencies by adding a check for
#   leading and trailing spaces.
#
# Revision 1.83  2008/04/07 16:50:50  lmm
# Added class method that returns the prefix used for dynamically
#   generated column names used to determine table links.
# Made a slight change to code that writes column data
#
# Revision 1.82  2008/03/06 18:24:58  lmm
# Adjusted colspan parameter for long strings
#
# Revision 1.81  2008/03/06 17:56:29  lmm
# Added extra empty column values to tables if the header is there but there
#    is no data.
# Fixed FOV link so it accepts all coordinate formats
#
# Revision 1.80  2008/01/30 16:26:49  lmm
# Added back newlines after magnifying glass image because it messes up
#   Text Display.
#
# Revision 1.79  2008/01/30 15:20:50  lmm
# Added call to new Javascript function that adds row search offset to
#   be submitted with query form.
#
# Revision 1.78  2007/11/20 15:05:28  lmm
# Added encoding for Vizier parameter display link.
#
# Revision 1.77  2007/10/09 14:14:38  lmm
# Removed Notification code for the time being.  Added new empty methods
#   beforeNewTable() and afterTable().
#
# Revision 1.76  2007/01/08 16:23:25  lmm
# Improved URL for related links. Tied with change in Parameter.pm.
#
# Revision 1.75  2006/11/06 16:05:46  lmm
# Added FOV to Services legend.
#
# Revision 1.74  2006/10/24 17:17:24  lmm
# Added FOV link (F) to Services column.
# Requery and ReDisplay buttons sre now disabled until page loads.
# Table name is slightly different in summary table to emphasize
#   short name.
# Pulled code out of processData() and created class method writeColumnData()
#   (for slight difference in Tabbed Display)
#
# Revision 1.73  2006/06/23 14:34:07  lmm
# Added GCN option to Services column
# Added hidden inputs to pass parameters for requery
# Added column sort image to table legend
# Changed Hera symbol from X to H and added variable to save offset column
#   target name to pass to Hera Manager
# Added method to define Results class
# Made changes to accommodate separation of Data Products and Further Action
#   option routines in Results class
# Added code for future Notification service (not ready yet)
# Added changes to propogate precision information to TableFormat.pm
#   (code provided by T. McGlynn)
#
# Revision 1.72  2006/01/12 17:21:09  lmm
# Added list of tables as a new argument to resultsSummary().  The summary
#   table will use the sort of this list when displaying data.
#
# Revision 1.71  2005/12/20 21:22:01  lmm
# Corrected meta variable hash key that was preventing tool tip from
#   displaying results table name
#
# Revision 1.70  2005/12/13 19:36:22  lmm
# Changed results summary table to sort by table priority.
# Made changes for new Keyword search and for inclusion of meta data tables.
# Added table list to YourSearchWas() for requery.
# Added calls to new SearchBasedOn package routines and removed code that
#   the package replaces.
# Changed the display of parameter qualifiers to only non-meta data queries.
# Added requerydisplay value as hidden input to maintain current display
#   format.
# Moved redisplay format options to new displayFormatOptions().
#
# Revision 1.69  2005/09/27 15:02:27  lmm
# special treatment to find results summary table id for vizier tables
#
# Revision 1.68  2005/09/07 13:29:01  lmm
# Results Summary now displays even if only one table displayed and the
#   table style has changed slightly.
# Blank spaces have been added between table link names.
#
# Revision 1.67  2005/08/11 14:50:43  lmm
# Removed submit() from onClick directive for redisplay options to prevent
#   extra window with "no action" error message
#
# Revision 1.66  2005/07/18 20:45:47  lmm
# Improvements to allow additional types of table link dependencies
#   (changes contributed by E. Sabol)
#
# Revision 1.65  2005/07/06 17:26:56  lmm
# Add link to Hera page in the Services legend
#
# Revision 1.64  2005/07/05 20:14:52  lmm
# Added Anonymous Hera (X) to the legend
#
# Revision 1.63  2005/06/23 19:33:31  lmm
# All submit buttons now have an onclick directive to define target window
#
# Revision 1.62  2005/03/23 19:23:19  lmm
# Long parameter query strings are no longer displayed in
#   "Search was based on" box
#
# Revision 1.61  2005/03/07 18:48:38  lmm
# Corrected table link query in get_row_links().  Added/deleted some
#   debug statements
#
# Revision 1.60  2005/02/15 21:38:11  lmm
# Object/Coordinates entry in "Search was based on" area is now
#   HTML-encoded before being displayed
#
# Revision 1.59  2005/02/15 15:51:34  lmm
# Added new service (X) that transfers data products for current row to
#   Hera Manager and automatically starts up fv.
# Made changes to display table and bibliography links when associated
#   parameter is not in default display set.
# Removed traces of Hera/wgacat table test.
# Some cosmetic code cleanup
#
# Revision 1.58  2005/01/19 15:19:15  lmm
# Changed method name resultsSummary()
#
# Revision 1.57  2004/12/29 15:03:37  lmm
# Made Summary table on Results page one size no matter how many rows
#
# Revision 1.56  2004/12/17 20:12:04  lmm
# Moved Bibcode services link to end of services list for aesthetic reasons
#
# Revision 1.55  2004/12/17 20:04:20  lmm
# Added check for JavaScript cababilty before creating Table Name/Hits
#   Summary
#
# Revision 1.54  2004/12/16 14:15:48  lmm
# Added summary table of Browse table names and row count
# Reorganized code by creating subroutines createNewHeader(),
#   tableStart(), createSelectDPBox(), getBiblink(0
# Made changes to improve table column sort for galactic coordinates
# Added a new service capability to analyze a row of data using Hera
#   (disabled for now)
# Removed wgacat specific Hera links (being replaced by new general Hera
#    Analysis service)
# Added link dependency check to relatedlinks()
# Made to bibcode values links to ADS abstract
# Added Skyview image size to display tag for quicker page load
#
# Revision 1.53  2004/01/16 15:12:15  lmm
# Added alt tag to image
#
# Revision 1.52  2004/01/15 20:42:54  lmm
# New method setNullTables and other code added to save information
#   on tables not valid for current query but may be valid for a requery.
# Added call to javascript function noNewWindowSubmit() to prevent new
#   windows for query and object file downloads
# Added capability to display information on dataset IDs stored in
#   zzext table
# Save Query button now only appears on result page from Main Search form.
#
# Revision 1.51  2003/12/05 18:16:20  lmm
# Fixed bug where positions were counted for searches not based on position.
#
# Revision 1.50  2003/12/02 21:35:23  lmm
# Added method to set the pass along the serialized Query string.
# Fixed bug that displayed wrong radius units in Search Was based On box.
# Added limit to number of targets displayed in Search Was based On box.
# Added Save Query to File and Save All Targets to File buttons.
#
# Revision 1.49  2003/09/12 20:45:20  lmm
# Corrected the documentation link for 2nd table header row
#
# Revision 1.48  2003/09/12 17:30:39  lmm
# Added additional <td> and <th> color styles to tables with
#    2 display table rows for each query results row  (eg. tables with
#    long string parameters like HETE2GRB)
#
# Revision 1.47  2003/09/09 17:51:55  lmm
# Information about skyview images now only appears if skyview
#    images are displayed
#
# Revision 1.46  2003/09/08 19:23:15  lmm
# Using new css style to make table borders thinner
# Made table scroll area taller
# Corrected image size listed under optical SkyView image
#
# Revision 1.45  2003/09/03 21:19:15  lmm
# Table legend now appears once on page.
# Tables have a new streamlined look (thinner borders).
# Thumbnail images are now links to larger images at SkyView site
# Link to Skyview optical image added to Services column (O)
#
# Revision 1.44  2003/08/29 17:32:39  lmm
# Added methods for defining page header text and style and removed
#    HEASARC footer
# Defined more meaningful text for linked table and other page headers
# Added small SkyView DSS and RASS images of query target
# Radius can now be changed for requery
# Added small image next to random tip to draw attention to it
#
# Revision 1.43  2003/07/31 18:50:43  cwh
# Changed YourSearchWas to print entry box for coords even if user entered
# coordinates (was doing it only for unresolved object names).
# Separated out the legend from the tableHeader, since not all display modes
#   will want to print the legend.
#
# Revision 1.42  2003/07/31 17:57:48  cwh
# Fixed YourSearchWas form to allow editing of coordinates as well as obj names.
#
# Revision 1.41  2003/07/31 15:26:00  lmm
# Included data products in scrolled area under results
# Added Printer-Friendly Version button
# Added Random Tip and Tip archive link
# Moved Requery Button to Query Display section
#
# Revision 1.40  2003/07/31 14:03:27  cwh
# Allow stylesheet class for each data row to be set by an object property.
# Added support for linked tables.
# Added form elements for requery for Entry and maxrows.
#
# Revision 1.39  2003/05/20 19:10:28  cwh
# Added VOTable display option to the redisplay menu.
#
# Revision 1.38  2003/05/20 17:11:00  cwh
# Changed chopTable to an object method.
# Added explanation of 'B' link to Services legend.
# Fixed check so that 'D' is only printed if datasets exist.
# Passing 'class' (stylesheet color code) to servicelinks sub (not using it yet,
#   but may need it later, if another table cell is begun within this sub.
# Printing AND vs OR in search summary box between quals.
# Added Pure text display option.
#
# Revision 1.37  2002/12/16 03:41:49  cwh
# Added quotes where needed.
# Only doing single encoding of unique key string in url, not double.
#
# Revision 1.36  2002/12/10 21:39:13  cwh
# Added id to form element.
# Added quotes where needed.
# Added cunique to service links call.
#
# Revision 1.35  2002/10/17 19:24:47  cwh
# removed plotsql var.
# HTML improvements:
#  - added tbody (thead starts in Results.pm)
#  - added alternating datacell1 and datacell2 (can set rows to alt colors)
#  - added titles to single-character links (accessibility)
#
# Revision 1.34  2002/09/13 20:01:02  cwh
# Changed setMaxRows to MaxRows(), now can either set or get.
# Changed setInput to Input(), now can either set or get.
# Isolated formname in var, passing it to javascript functions
# Calling Decor::Header instead of Results::Header.
# Now passing input along to the w3hdprods form
# Added Bib link stuff
# Changed dp handling
# Removed some inputs to Your Search Was, now getting them through obj props
# Printing 2 decimal places on radii
# Printing resolver message if any
# Smarter redisplay format option printing.
#
# Revision 1.33  2002/05/16 16:41:26  cwh
# Fixed bug where lines were broken from cut-and-paste.
# Added mjd conversion for end_time metaparams in YourSearchWas().
#
# Revision 1.32  2002/05/14 18:04:57  cwh
# Now dynamically loading display modules as needed.
# Added YourSeachWas method here, moved from Results.pm.
# Fixed bug in html link.
#
# Revision 1.31  2002/04/11 18:36:09  cwh
# Only print out service links when ra and dec are defined.
# Data cells are now marked with their own stylesheet class, so they can be
#   shown without line breaks but other table cells do have line breaks.
# w3hdprods.pl data link has cgi url forced rather than allowing the db-perl url.
#
# Revision 1.30  2002/03/25 18:20:20  cwh
# Added Coordinates and Equinox to the things passed in the url for the 'D'
# and magnifier links, so the coordinates will be displayed as the user wants.
#
# Moved the Service links into a subroutine, so they can be called from other
# display objects without repeating the code. (This should actually probably
# go in Services.pm.)
#
# Revision 1.29  2002/03/22 22:52:12  cwh
# Commented out the links to results tables.
# Added legend for service links.
# Fixed bug in savedec (+/-).
# Moved services column to the left side of the results table.
# Added SkyView: RASS-Cnt Broad to service links.
# Put back border around data sets / action box.
# Fixed table cell html when no data sets.
#
# Revision 1.28  2002/03/18 02:05:25  cwh
# Only check whether to print contents if $tables is an array ref.
# Don't display Services for cross-correlations, since they don't work yet.
#
# Revision 1.27  2002/03/15 23:51:47  cwh
# Got rid of 'Please select' directions, replaced with list of 'contents' links
# to results tables.
# Added pink magnifying glass icon to legend of Vizier result tables.
# Removed /div at 25-line table break, so the scrolbox doesn't break there.
# Added on-the-line service links.
# Turned off the table border around the data sets and actions.
# Use short table names for errors and warnings.
#
# Revision 1.26  2002/02/15 16:31:52  cwh
# Fixed bug introduced when I added quotes around href string in navtrail.
#
# Revision 1.25  2002/02/14 18:14:27  cwh
# HTML cleanup: changed <font> elements to <span> elements. Added quotes
#   around links. Fixed nonexistent </br> tag.
# Navtrail: if no referrer url, create one to the main interface.
# Added the </div> that should have been there, which was causing the pink in
#   Vizier tables to leak.
# Changed the regex for unique keys to allow non-digits.
# Moved the part where empty cells have &nbsp; put in them to just before the
#    cell is printed.
#
# Revision 1.24  2001/12/18 15:47:55  cwh
# Moved the DisplayLinks call to as soon as possible - printing it last was
# causing a problem when the user clicked on a sort arrow before the table
# finished displaying.
#
# Revision 1.23  2001/12/05 23:48:14  cwh
# Removed restriction on printing sort legend for cross-correlations.
# Added "$keyval=" to unique key string encoded in results.
# Passing $$self{"offset_var"} to Results::PrintTableParams().
#
# Revision 1.22  2001/11/20 15:47:43  cwh
# Made looking for "w3_" be case-insensitive.
#
# Revision 1.21  2001/11/08 20:17:48  cwh
# Added Browse:: to sub calls and to blessed $mode.
#
# Revision 1.20  2001/10/25 22:20:00  cwh
# Instead of calling DisplayLegend, now printing 1 or two lines at the top of
# each table with magnifying glass and sort arrows explained.
#
# Revision 1.19  2001/10/25 20:17:17  cwh
# Separated out StreamDisplay, BatchDisplay, and TextDisplay as separate
#   modules.
# Added a 'setInput' sub to set the input url as a object property, to pass
#   to Results::Heraform.
# Added call to Results::Heraform.
# Changed the right arrow buttons to links, which popup in a new window.
#
# Revision 1.18  2001/09/15 14:54:41  cwh
# Took table layout out of Results subroutines, added it here.
# Now passing tablename into Results::DisplayOptions() instead of a dp flag.
#
# Revision 1.17  2001/08/28 19:29:52  cwh
# Position no longer has raf and decf on front.
# Add link to help on ra,dec error flags.
# Now using magnifying glass icons instead of right arrows.
# Replaced hardcoded url to icons with $::images.
# Storing ra, dec format as accuracy to be used in line transformation.
#
# Revision 1.16  2001/07/31 15:17:47  cwh
# Fixed typo.
#
# Revision 1.15  2001/07/31 14:49:17  cwh
# Added use FitsDisplay;
# Added alt text to vizier logo and right arrow
# Added empty Directions() sub to textdisplay and streamdisplay.
#
# Revision 1.14  2001/07/23 16:39:59  cwh
# Added a getCoordSys routine.
# Passing Coordinates and Equinox as hidden variables.
# Added a showText routine, which just prints anything you send it, and is
#   currently being used to print the search criteria summary box.  Only defined
#   for non-TextDisplay, non-Batchdisplay, non-StreamDisplay objects.
# Pulled out the directions line into a sub, so I can call it after showing the
#   search criteria summary box.
#
# Revision 1.13  2001/07/02 20:52:43  cwh
# Now only display the 1st column with the right arrow and checkbox if there
# are key fields.
#
# Revision 1.12  2001/07/01 01:38:19  cwh
# Changed 'vizier' in 's/asu-xml/vizier/' to 'VizieR'.  Some VizieR servers are
# case sensitive and require it this way.
#
# Revision 1.11  2001/06/20 21:56:06  cwh
# Removed hidden var stuff that was needed for old method of sorting.
# Moved ValidateInput::checkResults to within the pageHeader (was in tableHeader).
#
# Revision 1.10  2001/06/15 03:26:00  tam
# Make data sets array null if empty.
#
# Revision 1.9  2001/06/13 20:57:53  cwh
# commented out debugging statements
#
# Revision 1.8  2001/06/12 16:11:17  tam
# For double key fields make sure key is
# written to full precision.
# Don't use VizieR right arrows for cross correlations.
#
# Revision 1.7  2001/05/30 21:20:52  cwh
# Added targets to windows.
#
# Revision 1.6  2001/05/24 17:05:55  cwh
# Now passing along max rows to self so that when user does a resort, only max
# rows are returned.
#
# Revision 1.5  2001/05/22 17:52:03  cwh
# Added Id and Log to files without them.
#

use strict;

use Browse::Utils;
use Browse::Results;
use Browse::ValidateInput;
use Browse::HDB;
use Browse::SearchBasedOn;

# Standard display object.
# This class, the default Display object,
# returns a page with results formatted in HTML tables.

package Browse::Display;

# Create all Display objects.  The only argument is
# the class of the object to be created.

sub new {

    my ($self,$mode) = @_;
    
    my $obj = {};
    if ($mode) {
        $mode =~ /([\-\w]*)/; $mode=$1;   # untaint $mode
        #dynamically load display module
        $mode = "Browse::$mode";
        eval "package Browse::_firesafe; require $mode";
        bless $obj, $mode;
    } else {
        bless $obj;
    }
    $obj->init();
    return $obj;
}

sub init {
   my ($self) = @_;
   $self->results("Results");
   $self->depLinkPrefix("(as )*_link_");
}

sub setMissionList{}

sub depLinkPrefix {
   my ($self, $prefix) = @_;
   $$self{"dlprefix"} = $prefix if defined($prefix);
   return $self->{"dlprefix"};
}

# Static function to chop $::local_origin off the beginning
# of a table name.

sub chopTable {

    my ($self,$table) = @_;
    #'g' because xcorr table names have multiple table names in them.
    $table =~ s/$::local_origin//gi; 
    return $table;
}

sub results {
    my ($self, $results) = @_;
    $self->{"results"} = Browse::Results->new($results) if defined($results);
    return $self->{"results"};
}

# Set the coordinate system for output.

sub setCoordSys {

    my ($self, $coordSys, $equinox) = @_;
    $$self{"coordsys"} = $coordSys;
    if (defined($equinox)) {
        $$self{"equinox"} = $equinox;
    }
}

# Set the color for the page header
sub setHeaderColor {
    my ($self, $stylecolor) = @_;
    if (defined($stylecolor)){
       $$self{"stylecolor"}=$stylecolor;
    }
}

# Set text for page header
sub setPopupRef {
    my ($self, $popupFrom) = @_;
    if (defined($popupFrom)){
       $$self{"popupFrom"}=$popupFrom;
    }
}

# Get the coordinate system for output.
sub getCoordSys {
    my ($self) = @_;
    return ($$self{"coordsys"},$$self{"equinox"});
}

sub MaxRows {

    my ($self, $maxrows) = @_;
    $$self{"maxrows"} = $maxrows if defined($maxrows);
    return $$self{"maxrows"};
}


# Set the output coordinate format.

sub setCoordFormat {

    my ($self, $coordFormat) = @_;
    
    $$self{"coordformat"} = $coordFormat;
}

# Set the output date format.

sub setDateFormat {

    my ($self, $dateFormat) = @_;
    $$self{"dateformat"} = $dateFormat;
}
       
sub setSavedQuery {

    my ($self, $savedQuery) = @_;
    $$self{"savedquery"} = $savedQuery;
}

sub setNullTables {
    my ($self, $nullTablesHtml) = @_;
    $$self{"nullTablesHtml"} = $nullTablesHtml;
}

# Print out the very top of the page.  From
# the HTTP headers until we begin displaying
# information from a particular table.

sub pageHeader {

    my ($self, $navtrail) = @_;
    my $formname="w3hdprods";
    

    if ($$self{"text"}) {
        print Browse::CGI::Header();
    } else {
        Browse::Decor::Header( $$self{"popupFrom"} ,0,0,0,
		$$self{"stylecolor"}); #dont close table, cols, no banner, color

        print "<script src=\"$::w3bdocs/w3bSubmit.js\"></script>";

        Browse::ValidateInput::checkResults($formname);
        unless ($navtrail=~ /\w/) { 
            my $referrer = ($ENV{HTTP_REFERER} or "/".$::cgiurl."/w3browse.pl");
            $navtrail = "<a class='navpast' href='$referrer'> Choose Tables</a>";
        }
        print "$navtrail &gt; <span class='navcurrent'>Search Results</span>" .
	   " &gt; <span class='navfuture'>Choose Data Products</span><br>";
        if ($ENV{REQUEST_METHOD} eq "GET") {
            $navtrail .= " &gt; &lt;a class='navpast' href='".
		$ENV{SCRIPT_NAME}.'?'.$ENV{QUERY_STRING}.
		"'&gt;Search Results&lt;/a&gt;";
        } else {
            $navtrail .= " &gt; Search Results";
        }
        $navtrail =~ s/\</&lt;/g;
        $navtrail =~ s/\>/&gt;/g;
    }
    #-- Start form 
    print "\n<form method='POST' target='_blank' name='$formname' " .
	" action='w3hdprods.pl'>\n";

    #--- original input should exist so grab some parameters and pass them on
    #print "Display input=" . Browse::CGI::Decode($$self{input}) .
	#"<br>" if $::debug;

    my $decodedInput = Browse::CGI::Decode($$self{input});
    if ($decodedInput) {
       my ($dtime, $dnr);
       if ($decodedInput =~ /&Time=([^&]*)&/) { $dtime=$1;}
       if ($decodedInput =~ /&NR=([^&]*)&/) { $dnr=$1;}
       #---Save for other methods
       $self->{"NR"}=$dnr;
       $self->{"Time"}=$dtime;
       print "<input type='hidden' name='NR' value='".
  	   $dnr ."'>\n" if $dnr;
       print "<input type='hidden' name='Time' value='".
	   $dtime . "'>\n" if $dtime;
       print "Display input=" . $dtime ." -- " .$dnr .
	"<br>" if $::debug;
    }

    print "\n<input type='hidden' name='navtrail' value='$navtrail'>\n";
    print "<input type='hidden' name='Coordinates' value='".
	$$self{"coordsys"}."'>\n";
    print "<input type='hidden' name='Equinox' value='".
	$$self{"equinox"}."'>\n";
    print "<input type='hidden' name='input' value='".$$self{"input"}."'>\n";
    print "<input type='hidden' name='savedQuery' value='".
	$$self{"savedquery"}."'>\n";

    # Pass along header to be used on new page
    my ($popup)=$$self{"popupFrom"};   #lmm

}

# this is currently being used to echo the search criteria.
sub showText {
   shift;
   print @_;
}

# print directions
sub Directions {

    my ($self) = @_;
    #my ($self,$tables,$table_info) = @_;
    #$self->rowCountTable($table_info);

}

#--- Displays names of tables and the number of rows in that table
sub resultsSummary {
    my ($self,$tinfo, $tablesSorted) = @_;

    #--- get number of browse tables to determine size of summary 
    #--- display area and to keep track of rows
    my @tinfokeys =keys (%$tinfo);
    #print "Disp: resultsSum: <br>" if $::debug;
    #Browse::Utils::DumpRef($tinfo) if $::debug;
    my $tableCount=$#tinfokeys;

    #--- Control size of area with style sheet entry
    my $tClass = "rowcounttablemedium";   
    my $bClass = "oneplustablerowcount";
    my $hClass = "rowcounttablemediumhead";   
    my $sTablename="Table Name/Row Count Summary";
    if ($tableCount == 0) {
       #--- slightly different display for one table result
       $tClass = "rowcounttableonerow";   
       $hClass = "rowcounttableonerowhead";   
       $sTablename="Table Name and Row Count";
    }
print <<EOT;
    <script type=\"text/javascript\" language=\"JavaScript\">
EOT
print <<EOT;
    if (document.getElementById) {
    document.writeln('<div class="$hClass">');
    document.writeln('$sTablename');
    document.writeln('</div>');
EOT
    #document.writeln('<center>$sTablename</center>');
    if ($tableCount > 0) {
print <<EOT;
    document.writeln('<center><font size="-1">Click on table name to view search results</font></center>');
EOT
}
print <<EOT;
    }
EOT
print <<EOT;
    if (document.getElementById) {
    document.writeln('<div class="$tClass">');
EOT

    #--- display the table with names and # of hits  
print <<EOT;
    //document.writeln('<table border="1" align="center" id="rowcount" width="80%"><tbody id="counttableTB">');
    document.writeln('<table class="$bClass" align="center" id="rowcount" width="80%"><tbody id="counttableTB">');
    document.writeln('<tr></tr>');
EOT

    #--- page load status line
print <<EOT;
    document.writeln('<tr>');
    document.writeln('<td colspan ="2" align="center" id="rowcountupdate"> </td>');
    document.writeln('</tr>');
    document.writeln('<tr>');
EOT


    #--- Gather table names from sorted list and add description to 
    #--- summary table
    my $currTable=0;
    my $rowcount=0;
    for my $sTable (@$tablesSorted) {
       my $table = $tinfo->{$sTable};
       my $tableId;
       
       #--- special processing for vizier
       if ($table->{"table_name"} =~/\//) {
          $tableId=Browse::Vizier::getFinalTableName($table->{"table_name"});
          #print "alert('table=' +'" . $table->{table_name} ."' + 'id='+ '$tableId');\n";

          if ($tableId !~ /\w/ ) {$tableId=$table->{"table_name"}.'/out';}
       } else {
          $tableId=$self->chopTable($table->{"table_name"});
       }


       my $description=$table->{"table_description"};
       $description =~ s/\n/ /g;

       #--- debugging
       my $mission=$table->{"observatory_name"};
       my $pri=$table->{"table_priority"};
       #--- end debugging

       my $cellCountId=$tableId . "CC";
       my $cellNameId=$tableId . "NC";
       if ($description) {
print <<EOT;

          document.writeln('<td >');
EOT
print <<EOT;
          document.writeln('  <table  width="100%" bgcolor="#ffffff" cellspacing="0" id="counttable$currTable"><tr>');
          document.writeln('  <td align="left" id="$cellNameId"> <b>$tableId</b>:$description </td>');
          document.writeln('  <td align="right" id="$cellCountId"> </td>');
          document.writeln('  </tr></table>');
EOT
#document.writeln('  <td align="left" id="$cellNameId"> $description ($tableId) </td>');
          my $newcell=$currTable % 2;
          ++$currTable;
print <<EOT;
          document.writeln('</td>');
EOT

          #--- start a new row 
          if ($newcell && $rowcount < $tableCount) {
print <<EOT;
             document.writeln('</tr><tr>');
EOT
          }
          ++$rowcount;
       }
    }
print <<EOT;
    document.writeln('</tr></tbody></table>');
    document.writeln('</div>');
    } //-- end check for javascript method capability
EOT
    print "</script>\n";

}


sub Input {
    my ($self,$input) = @_;
    $$self{"input"} = $input if $input;
    return $$self{"input"};
}

# Print out everything at the bottom of the table
# after the last table information.

sub pageFooter {
    my ($self) = @_;
    
   
    $self->closeTable();

    Browse::Results::Footer();
    Browse::Decor::EmailFooter();
    Browse::Results::Heraform($$self{"input"});
    print "</body></html>\n";
}


# Print the information at the beginning of a tables results.

sub tableHeader {
    my ($self) = @_;

    my ($meta) = $$self{"meta"};
    my $results = $self->results();
    if ($results) {
       $results->DisplayLinks($meta);
    } 

    #Browse::Results::DisplayLinks($meta);
    
    my $table_info = $$meta{"tableinfo"};
    my $table      = $$meta{"table"};
    
    $$self{"meta"} = $meta;
    $$self{"rowcount"} = 0;
    
    
    my ($sql) = $$meta{"sql"};
    
    if (defined($$table_info{"default_search_radius"})) {

	my $raf  = $$table_info{"right_ascension"};
	my $decf = $$table_info{"declination"};

	if (substr($raf, 0, 1) eq "@") {
	    $raf = substr($raf, 1);
	}
	if (substr($decf, 0, 1) eq "@") {
	    $decf = substr($decf, 1);
	}
	    
    }
	
	
    my $radius;
    if ($$meta{"positions"}) {
	my $pos = $$meta{"positions"};
	my $pos1 = $$pos[0];
	my ($ra, $dec, $rad) = split('\|', $pos1);
	$radius = $rad;
    }
    
    if ($$meta{"resource_name"} && $$meta{"resource_name"} ne $$self{"last_resource"}) {
        my $url = $$meta{"query_url"};
	$url =~ s/asu-xml/VizieR/;
        print "<hr><img alt=\"VizieR logo\" src='$::vizgif' " .
	   "height=20 width=20>  <a target=helpwin href='$url'" .
	   ">$$meta{resource_title} ($$meta{resource_name})</a>\n";
	$$self{"last_resource"} = $$meta{"resource_name"};
    }

    Browse::Results::PrintTableName($table, $table_info, $radius);

}

sub legend {
    my ($self) = @_;
    my $table      = $$self{"meta"}{"table"};
    #print legend
    print "<hr><span class=\"small\"><strong>Table Legend:</strong>" .
	"<br><img src = \"$::images/mag5.gif\" height='15' width='15'" .
	" border='0' alt=\"Display all params\"> Display all parameters" .
	" for a row";

    print "&nbsp;&nbsp;&nbsp;&nbsp;<img src = \"$::images/magviz.gif\" " .
	"height='15' width='15' border='0' " .
	"alt=\"Display all params direct from VizieR\"> Display all " .
	"parameters directly from VizieR" if $table =~ /\//;

    print "<br><img src = \"$::images/down.gif\" height='15' width='15' " .
	"alt=\"Sort\"> Sort by a column in order: " .
	"1,2,3&nbsp;&nbsp;&nbsp;&nbsp;<img src = \"$::images/up.gif\" " .
	"height='15' width='15' alt=\"Reverse Sort\"> Sort by column in " .
	"reverse order: 3,2,1&nbsp;&nbsp;&nbsp;" .
	"<img src = '$::images/down_sort.gif'  height='15' width='15' " .
	"alt='Current Sort 1,2,3'>/" .
	"<img src = '$::images/up_sort.gif'  height='15' width='15' " .
	"alt='Current Sort 3,2,1'> " .
	"Current table sort" .
	"<br>";

    #print "Services links: R: ROSAT All-Sky Survey image, N: NED objects near coordinates, S: SIMBAD objects near coordinates, D: get list of data products, B: ADS bibliography holdings<br>\n";  
    my $gcn;
    if ($$self{"gcnlink"}) {
        $gcn = ", G: GRB Coordinate Network (GCN) notices";
    }

    print "<table><tr><td nowrap valign='top'>Services links:</td>" .
	"<td valign='top'> O: Digitized Sky Survey image, " .
	"R: ROSAT All-Sky Survey image, N: NED objects near coordinates, " .
	"S: SIMBAD objects near coordinates, D: get list of data products, " .
	"H: analyze data products using <a href='/hera'>Hera</a>, " .
	"B: ADS bibliography holdings" .
	$gcn . ", " .
	"F: FOV plot for observation " .
	"</td></tr></table>\n";  

    print "Scroll down below tables to select Data Products and " .
	"Further Actions.<br>\n";
    print "</span>\n";
}

# Extract a specified set of fields from a table
# Finish off any information for a table.

sub tableFooter {
    my ($self,$msg) = @_;
  
    $self->closeTable($msg);
    
}

# Process a metadata input.  This implies that
# information from a new table will be forthcoming.


sub processMeta {
    my ($self, $meta) = @_;
    
    $self->closeTable();
    
    # Just save the metadata for later use.
    $$self{"meta"} = $meta;
    $$self{"rowcount"} = 0;
    #print "process meta<br>" if $::debug;
    #Browse::Utils::DumpRef($meta) if $::debug;
    
    # Make sure we don't hold onto an invalid transform.
    $$self{"transform"} = undef;
   
    $$self{gcnlink} = $$meta{tableinfo}{grb_name};

}

# Process a row of data.

sub processData {
    my ($self, $data) = @_;
    my $row = $$self{"rowcount"};
    if (!$$self{"transform"}) {
        $$self{"transform"} = $self->setTransform($data);
    }
    
    #--- find search offset column for Hera Analysis
    my $colNames =  $$data{"names"};
    my $offsetCol;
    my $statusCol;
    
    for (my $i=0; $i <= $#$colNames; $i += 1) {
         #print "Display::$$colNames[$i]<br>" if $::debug;
         if ($$colNames[$i] =~ /status/) {
	    $statusCol=$i; 
         }
         if ($$colNames[$i] =~ /$$self{"offset_var"}/) {
	    $offsetCol=$i; 
         #   last;
	 }
    }

    my $meta      = $$self{meta};
    my $tablename  = $$meta{tableinfo}{table_name};
    my $shortname = $self->chopTable($tablename);

    #--- Add new header row after 25 rows
    if ($row % 25 == 0)  {
        $self->createNewHeader($meta);
    }
    
    #--- Format rows
    my $class = $$self{"class"} || 
        ($row % 2 ? "thinbordercell1" : "thinbordercell2");
    my $skip = $$self{"skipfield"};
    
    my $line = $$data{"data"};
    my $trans = $$self{"transform"};
    #Browse::Utils::DumpRef($trans) if $::debug;
    
  
    my ($racol, $deccol) = ($$trans{"racols"},$$trans{"deccols"});
    my $savera = Browse::CGI::Encode($$line[$$racol[0]]) if $$racol[0];
    my $savedec = Browse::CGI::Encode($$line[$$deccol[0]]) if $$deccol[0];
    $savedec = "%2B".$savedec unless (!$savedec or $savedec =~ /^-|^%2b|^%2d/i);

    print "Display::processData::transforming line<br>" if $::debug;
    Browse::TableFormat::transformLine($line, $trans);

    #--- get offset value to parse for target
    (my $offsetTarget = $$line[$offsetCol]) =~ /\d*\((.*)\)/ if  $offsetCol =~ /\d/;
    my $fovStatus = $$line[$statusCol];
    $offsetTarget = $1;
    #print "Display::offset= $offsetTarget<br>" if $::debug;
    #print "Display::status= $fovStatus<br>" if $::debug;

    # We defer the transformat of spaces into NBSPs to here
    # because this is the point that we know that
    # the data is going to be presented in HTML.
    my $col;
    foreach $col(@$racol,@$deccol) {
        $$line[$col] =~ s/ /\&nbsp\;/g;
        #$$line[$col] =~ s/!!/<a href="$::w3bhelp#coorderror">!!<\/a>/g;
        $$line[$col] =~ s/!!/ (coordinate value error)/g;
    }
    
    
    my $unique = $$self{"meta"}{"table"} . "|||";
    #print "Display::1 unique= $unique<br>" if $::debug;

    my $types = $$data{"types"};

    my $longs = $$trans{"longs"};
    my ($names) = $$self{"names"};
    
    # Check if our skip fields are for multi-table result.
    # We want to put double pipes at the end of the keys
    # for each table.
    my $n0 = substr($$names[0], 0, 5);
    if ($n0 !~ /w3\_[a-z]\_/i) {
        $n0 = "w3_";
    }
   
    # Get the biblink info needed for the services link
    my $biblink= $self->getBiblink($line,$names);
    my $fovlink= $self->getFovlink($tablename,$line,$names,$offsetTarget,
	$fovStatus,$savera,$savedec) if $offsetTarget =~ /\w/;
 
    my $dlink="";
    my $xlink="";
    my $oldsep;
    my $i;
    for ($i=0; $i <= $#$line; $i += 1) {
       #print "Display::looking at $$line[$i] $i $skip $$names[$i]" .
	#"  $$types[$i]<br>" if $::debug;

        if ($i < $skip) {
	
            #print "Display::not skipped $$names[$i] $$types[$i]<br>" if $::debug;
	    my $str='';
 	    my $name='';
	
	    if ($$types[$i] == 6) {
	        # Ensure adequate precision for double values.
		# By default the string representation of doubles
		# isn't adequate.
		if (length($$line[$i]) > 0) {
		    $str .= sprintf("%30.20g", $$line[$i]);
		    $str =~ s/^ *//;
		    $str =~ s/ *$//;
		}
	    } else {
	        $str = $$line[$i];
	    }
            $name = $$names[$i];
            $name =~ s/^w3_//i;
            $name =~ s/^\w_// if $$self{"meta"}{"table"} =~ / x /;


	    # Handle nulls.
	    if (length($str) > 0) {
                $unique .= $name ."=". $str . '|';
	    } else {
	        $unique .= $name."=".' |';
	    }
            #print "Display::2 unique= $unique<br>" if $::debug;
	    
	    if ($$names[$i+1] !~ /^$n0/i) {
	        $unique .= "|";
		$n0 = substr($$names[$i+1], 0, length($n0));
	    }
            #print "Display::3 unique= $unique<br>" if $::debug;

	    
	} else {
	    if ($i == $skip ) {


                my $cunique  = Browse::CGI::Encode($unique);

                # Link for Data Products Preview
                my($popupRef) = $$self{"popupFrom"};   #lmm
                $dlink = "/$::cgi/w3hdprods.pl?files=Preview" .
		   "&Coordinates=$$self{coordsys}&Equinox=$$self{equinox}" .
		   "&CheckSize=1&showgifs=1&Target=$cunique" .
		   "&popupFrom=$popupRef" 
		   if scalar keys %{$$self{"meta"}{"dpsets"}};

                #--- Prepare coords and radius for Hera analysis link
	        my ($xlinkra, $xlinkdec, $xlinkrad);
                if ($$meta{"positions"}) {
	            my $pos = $$meta{"positions"};
	            my $pos1 = $$pos[0];
	            ($xlinkra, $xlinkdec, $xlinkrad) = split('\|', $pos1);
	            if ($xlinkrad =~ /Default/) { 
                       $xlinkrad=
			  $self->{meta}{tableinfo}{default_search_radius};
	            } else {
                       $xlinkrad *= 60; #convert to arc minutes
                    }
                }
                 
                #--- Hera analysis link
                $xlink = "/$::cgi/w3hdprods.pl?files=Analyze" .
		   "&Coordinates=$$self{coordsys}&Equinox=$$self{equinox}" .
		   "&Entry=$xlinkra,$xlinkdec&Radius=$xlinkrad" .
		   "&CheckSize=1&showgifs=1&Target=$cunique" .
		   "&dptype=anonhera" .
		   "&popupFrom=$popupRef&searchOffset=$$line[$offsetCol]"
		   if scalar keys %{$$self{"meta"}{"dpsets"}};

                #--- assign which multiple target applies
                $xlink .= "&offsetTarget=" . 
		   Browse::CGI::Encode($offsetTarget) .
		   "_unique_" . "$unique"  
		   if scalar keys %{$$self{"meta"}{"dpsets"}} && $offsetTarget;
		my $glink;
		if ($$self{gcnlink}) {
		    if ($$self{gcnlink} ne "setup") {
		        my $ginfo = {};
			my $namefield  = $$self{meta}{tableinfo}{grb_name};
			my $flagfield  = $$self{meta}{tableinfo}{grb_flag};
			
			$namefield =~ s/^\@//;
			$flagfield =~ s/^\@//;
			
			my $matchvalue = $$self{meta}{tableinfo}{grb_flag_value};
			
			my $nameindex = -1;
			my $flagindex = -1;
			
			for (my $i=0; $i<@$names; $i += 1) {
			    if ($$names[$i] =~ /^(\_link\_)?$namefield$/i) {
			        $nameindex = $i;
			    }
			    if ($$names[$i] =~ /^(\_link\_)?$flagfield$/i) {
			        $flagindex = $i;
			    }
			}
			if ($nameindex < 0) {
			    $$self{gcnlink} = undef;
			    
			} else {
			    $$ginfo{name} = $nameindex;
			    $$ginfo{flag} = $flagindex;
			    $$ginfo{val}  = $matchvalue;
			    $$self{gcnlink} = "setup";
			    $$self{gcninfo} = $ginfo;
			}
		    }
		    if ($$self{gcnlink}) {
		        my $ginfo = $$self{gcninfo};
			my $namind = $$ginfo{name};
			my $flgind = $$ginfo{flag};
			
			if (defined($$line[$namind])) {
			    if ( $flgind < 0  || 
			        ($flgind >= 0 && ($$line[$flgind] eq $$ginfo{val}))) {
				
				$$line[$namind] =~ /^(GRB)?\s*(\d{6}[a-z]?$)/i;
				if (defined($2)) {
				    
				    my $str = "$2 or GRB $2 or GRB$2";
				    $str =~ s/ /\%20/g;
				    $glink = "http://gcn.gsfc.nasa.gov/cgi-bin/docs/gcn/swish-cgi.pl?results=0&query=$str";
				}
			    }
			}
		    }
		}

    
                my $begin=
		   $self->createSelectDPBox($unique,$cunique, $shortname, 
			$offsetTarget);
                print "<tr><td class='$class'>$begin ";
                #print "<input type='hidden' name='offsetTarget' " .
		   #"value='$offsetTarget" . "_unique_" . "$unique'>" 
		   #if $offsetTarget =~ /\w/;
                print "</td>\n"; 

                $self->relatedLinks($names, $line,$unique,$class);

                if ($unique !~ / x /i) {
                  print "<td class='$class'>\n"; #Services cell
                  #print $self->servicelinks($savera,$savedec,$dlink,$xlink,
			#$biblink,$cunique,$class,$$self{"coordsys"},
			#$$self{"equinox"});
                  print $self->servicelinks($savera,$savedec,$dlink,$xlink,
			$biblink,$glink, $fovlink,
			$cunique,$class,$$self{"coordsys"},
			$$self{"equinox"});

                  print "</td>";
                }

	    }

	    # Skip parameters that have not been requested
	    if ($$names[$i] =~ /^_link_/) {
                print "skipping $$names[$i]<br>" if $::debug; 
	        next;
	    }

	    # Skip long strings.
	    if ($longs  && grep($_ == $i, @$longs)) {
	        next;
	    }

            my $format = 
              $$self{"meta"}{"paraminfo"}{$$names[$i]}{"parameter_format"};
            #print "format = $format<br>" if $::debug;
            if ($format =~ /bibcode/) {
              $$line[$i] = 
                 "<a href=\"http://adsabs.harvard.edu/cgi-bin/bib_query?" .
	         "$$line[$i]\">$$line[$i]</a>"
            } elsif ($format !~ /html/) {

            #print "format =  $$line[$i]<br>" if $::debug;
              $$line[$i] =~ s/\s+/\&nbsp\;/g; # don't want to break the line
              $$line[$i] =~ s/</\&lt\;/g;   # encode so not interpreted as tags
              $$line[$i] =~ s/>/\&gt\;/g; 
            #print "format =  $$line[$i]<br>" if $::debug;
            }

            $self->writeColumnData($class,$$line[$i]);

            # make empty cells look right
            print "&nbsp;" unless $$line[$i] =~ /\S/;
            print "</td>";
	}
    }
    #---  format empty last columns (seems to affect Vizier)
    my $emptycolumns = scalar(@$names) - scalar(@$line);
    for (my $ec=0; $ec <$emptycolumns;$ec +=1) { 
            $self->writeColumnData($class,"&nbsp;");
            print "</td>";
    }
    print "</tr>\n";
    
    # Deal with long strings
    for ($i=0; $i <= $#$longs; $i += 1) {
        #my $span = $#$line - $skip - 1;   #seems to be 1 column off (HTE2GRB)
        #my $span = $#$line - $skip ;   
        my $span = $#$names ;
	if ($span <= 0) {
	    $span = 1;
	}
	
        #print "<tr ><td colspan='2'><td colspan='$span'>";
        print "<tr><td class='$class' colspan='2'></td>" .
		"<td class='$class' colspan='$span'>";
	
	my $text = $$line[$$longs[$i]];
	$text =~ s/\0/<br>\n/g;
        print "$text</td></tr>\n";
    }
    $$self{"rowcount"} += 1;
    
}

sub servicelinks {
    my ($self, $savera, $savedec, $dlink, $xlink, $biblink,$glink, $fovlink, $cunique,$class,$coordsys,$equinox) = @_;   


    my $html="";

    #--- Create links for SkyView images
    my($skvDssUrl)=Browse::SkyView::prepareLink($savera,
	    $savedec,'Digitized+Sky+Survey',$coordsys,$equinox);
    my($skvRassUrl)=Browse::SkyView::prepareLink($savera,
	    $savedec,'RASS-Cnt+Broad',$coordsys,$equinox);

    #--- Optical Image link
    $html .= "\n<a title='View SkyView Digitized Sky Survey image " .
	"for these coordinates' target='services' " .
	"href='http://skyview.gsfc.nasa.gov/cgi-bin/nnskcall.pl?" .
	"VCOORD=$savera,$savedec&SURVEY=Digitized+Sky+Survey'>" .
	"O</a>&nbsp;" if $savera =~ /\d/;
    #--- Xray Image link
    $html .= "\n<a title='View SkyView ROSAT All-sky survey image " .
	"for these coordinates' target='services' " .
	"href='http://skyview.gsfc.nasa.gov/cgi-bin/nnskcall.pl?" .
	"VCOORD=$savera,$savedec&SURVEY=RASS-Cnt+Broad'>R</a>&nbsp;" 
	if $savera =~ /\d/;
    #--- NED link
    $html .= "\n<a title='Query NED for these coordinates' " .
	"target='services' " .
	"href='http://nedwww.ipac.caltech.edu/cgi-bin/nph-objsearch?" .
	"search_type=Near+Position+Search&" .
	"in_csys=Equatorial&in_equinox=J2000.0&lon=".
	$savera."d&lat=$savedec&radius=2.0&out_csys=Equatorial" .
	"&out_equinox=J2000.0&obj_sort=Distance+to+search+center" .
	"&zv_breaker=30000.0&list_limit=5&img_stamp=YES" .
	"&z_constraint=Unconstrained&z_value1=&z_value2=" .
	"&z_unit=z&ot_include=ANY&nmp_op=ANY'>N</a>&nbsp;" if $savera =~ /\d/;
    #--- SIMBAD link
    $html .= "\n<a title='Query SIMBAD for these coordinates' " .
	"target='services' href='http://simbad.harvard.edu/sim-id.pl?" .
	"Radius=2&Ident=$savera+$savedec'>S</a>&nbsp;\n" if $savera =~ /\d/;
    #--- preview data products link
    $html .= "\n<a title='Preview data products for this row' " .
	"target='services' href='$dlink'>D</a>&nbsp;"
	if (scalar keys %{$$self{"meta"}{"dpsets"}} and $dlink);
    #--- Analyze data products with Hera link
    $html .= "\n<a title='Analyze data products for this row using Hera' " .
	"href='$xlink'>H</a>&nbsp;" 
	if (scalar keys %{$$self{"meta"}{"dpsets"}} and $xlink);
    #--- Bibliography link
    $html .= "\n<a title='See bibliographic references for this row' " .
	"target='services' " .
	"href='http://adsabs.harvard.edu/cgi-bin/bib_query?" .
	"$biblink'>B</a>&nbsp;" if $biblink;
    #--- Fovlink
    $html .= "\n<a title='See FOV plot for this observation' " .
	"target='services' " .
	"href='/$::cgi/chkin_hea.pl?" .
	"$fovlink'>F</a>&nbsp;" if $fovlink;
    # GCN link
    if (defined($glink)) {
        $html .= "\n<a title='GCN Notices for this burst'" .
	 " href='$glink'>G</a>&nbsp;";
    }

    return $html;
} # end of servicelinks


# Process empty tables or error messages.

sub showEmpties {

    my ($self, $empties, $errs) = @_;
    $self->closeTable();

    #-- Add HTML for nulltables for possible user requery
    if ($$self{"nullTablesHtml"})  {
        print $$self{"nullTablesHtml"}; 
    }

    if ($#$empties >= 0) {
	
	print "<hr><p> <strong>No matches for</strong><br><dl>\n";
	my $niltab;
	foreach $niltab(@$empties) {
            my $stable = $$niltab[0];
            $stable =~ s/$::local_origin//g;
	    print "<dd> <a name=\"$stable\">$$niltab[1] ($stable)</a>\n";
	}
	print "</dl>\n";
    }
    my $lastErr;
    if ($#$errs >= 0) {
        print "<hr><p> <strong>Search errors</strong><br><dl>\n";
	my $errtab;
	foreach $errtab(@$errs) {
            my $stable = $$errtab[1];
            $stable =~ s/$::local_origin//g;
	    if ($$errtab[0] ne $lastErr) {
	        print "<dt> $$errtab[0] ";
	    }
	    print "<dd> <a name=\"$stable\">$$errtab[2] ($stable)</a>\n";
	}
	print "</dl>\n";
    }
}
        
# This is private method that sets up the
# table transformation to be used to format
# the table for output.

sub setTransform {

    my ($self, $data) = @_;

    my $names =  $$data{"names"};
    #print "Display::setTransform:names=@$names<br>\n" if $::debug;
    my $types =  $$data{"types"};
    my $precisions = $$data{"precisions"};   #long strings tam
    #print "types=@$types<br>\n" if $::debug;
    
    my $coordsys  = $$self{"coordsys"};
    
    my $meta      = $$self{"meta"};
    my $tableinfo = $$meta{"tableinfo"};
    
    $$self{"unitFlds"} = undef;
    
    my @radecs;
    #print "Display::setTransform<br>" if $::debug;
    #Browse::Utils::DumpRef($tableinfo) if $::debug;
    
    my $rac  = $$tableinfo{"right_ascension"};
    my $decc = $$tableinfo{"declination"};
    
    my (@raf)  = split('\|', $rac);
    my (@decf) = split('\|', $decc);
    
    if (defined($rac) && defined($decc)) {
        my $i;
        for ($i=0; $i <= $#raf; $i += 1) {
            $rac = $raf[$i];
            $decc = $decf[$i];

            if (substr($rac, 0, 1) eq "@") {
    	        $rac = substr($rac, 1);
    	    }
            if (substr($decc, 0, 1) eq "@") {
	        $decc = substr($decc, 1);
	    }

            push(@radecs,"$rac,$decc");
        }
    }

    #---- other ras and decs 
    #--- example of format: chandra_ra,@chandra_dec||esis_ra,@esis_dec
    my $oradecs  = $$tableinfo{"other_ra_dec"};
    if ($oradecs) {
       my (@radeclist)  = split('\|\|', $oradecs);
       my ($ora,$odec);
       foreach (@radeclist)  {
          ($ora,$odec) = split("\s*,\s*"); 
          if (defined($ora) && defined($odec)) {
            if (substr($ora, 0, 1) eq "@") {
    	       $ora = substr($ora, 1);
    	    }
            if (substr($odec, 0, 1) eq "@") {
	       $odec = substr($odec, 1);
	    }
            push(@radecs,"$ora,$odec");
          }
       } 
    }
    
    
    my $paraminfo = $$meta{"paraminfo"};
    #my ($accuracy) = $$paraminfo{$rac}{"parameter_format"} =~ /:\.(\d+)f/; 
    my ($accuracy) = $$paraminfo{$rac}{"parameter_format"} =~ /:\d*\.(\d+)f/; 
    #print "Display:  acc=" if $::debug;
    #Browse::Utils::DumpRef($accuracy) if $::debug;
    
    my @mjds;
    my $name;
    
    my @unitFlds = ();
    my $unitCount = 0;

    my @truncs;
    foreach $name (@$names) {


        my ($unit) = $$paraminfo{$name}{"parameter_unit"};
	$unit =~ s/^ *//;
	if (length($unit) > 0) {
            if ($unit eq "mjd") {
	        push(@mjds, $name);
		push(@unitFlds, "");
		
	    } elsif (substr($unit,0,3) ne "deg") {
	        push(@unitFlds, "[$unit]");
		$unitCount += 1;
	    } else {
	        push(@unitFlds, "");
	    }
	#} elsif ($name  =~ /\_.*offset/i) {
	#    # Won't display units if this is the only one, 
             # do don't increment unitCount
	#        push(@unitFlds, "[']");
	} else {
            
	    push(@unitFlds, "");
	}
	
	
	#if ($name =~ /\_.*(offset|delta)/i) {
	#    push(@truncs, "$name,\%0.3f");
	#}
	
	
	my ($form) = $$paraminfo{$name}{"parameter_format"};
        print "Display::setTransform: $form $name<br>" if $::debug;
    
	my ($bas, $out) = split('\:', $form);
	if (defined($out) and $out =~ /\d/) {
	    push(@truncs, "$name,\%".$out);
	}
	    
    }
    
    if ($unitCount > 0) {
        $$self{"unitFlds"} = \@unitFlds;
    }
    
    # Just collect up the constants that we'll use in the
    # transformation of each line into a single array.
    #my ($trans) = Browse::TableFormat::setTransform($names, $types, 
	#\@radecs, \@mjds, \@truncs, $$self{"coordsys"});
    # long strings tam
    my ($trans) = Browse::TableFormat::setTransform($names, $types, 
	$precisions, \@radecs, \@mjds, \@truncs, $$self{"coordsys"});  
		     
    $$trans{"equinox"}     = $$self{"equinox"};
    $$trans{"dateformat"}  = $$self{"dateformat"};
    $$trans{"coordformat"} = $$self{"coordformat"};
    $$self{"names"}        = $$trans{"names"};
    $$trans{"accuracy"} = $accuracy if $accuracy =~ /^\d+$/;
    
    return $trans;
}

# This prints out the columns at the beginning of each HTML table block.

sub printColumnHeader {

    my ($self, $names) = @_;
    
    my $meta  = $$self{"meta"};
    my $trans = $$self{"transform"};
    my $longs = $$trans{"longs"};
    #my ($links,$html);
    my ($links,$html, $data_id, $dataid_prefix);   #data_id
    #print "Display ** $$self{sortcolumn} <br>" if $::debug;
    print "<div class='lscrolltable'>" unless $$self{"intable"}; #lmm
    
    my $results = $self->results();
    if ($results) {
       ($$self{"skipfield"},$links,$html, $data_id, $dataid_prefix) = 
	  $results->PrintTableParams($$meta{"table"}, 
	  $$meta{"tableinfo"}, $$meta{"sortable"}, $names, $$self{"unitFlds"},
	  $longs, $$self{"offset_var"},$$self{"sortcolumn"});
    }

    #($$self{"skipfield"},$links,$html, $data_id, $dataid_prefix) =   #data_id
      #Browse::Results::PrintTableParams($$meta{"table"}, $$meta{"tableinfo"}, 
         #$$meta{"sortable"}, $names, $$self{"unitFlds"}, $longs,
         #$$self{"offset_var"},$$self{"sortcolumn"});

    my $bg;
    if ($$meta{"table"} =~ /\//) {   # if  '/' in the name, must be VizieR
        $bg = "class='Vizier'";
    }

    $self->tableStart($meta);

    #--- header start
    print "<thead>\n<tr $bg>$html</tr>";

    my $fld;
    #print "links " . ref($links) . " " . scalar @$links ." <br>" if $::debug;
    $$self{"related_links"} = $links;
    $$self{"dataset_id"} = $data_id;
    $$self{"dataid_prefix"} = $dataid_prefix;

    # Handle long strings
    foreach $fld(@$longs) {
        #my $span = $#$names - $$self{"skipfield"} - 1;   #seems to be 1 col off
        #my $span = $#$names - $$self{"skipfield"} ;
        my $span = $#$names; 
	
        if ($span <= 0) {
            $span = 1;
        }

        print "<tr><th class='thinborderhead' colspan='2'></th>" .
	   "<th class='thinborderhead' scope='col' colspan='$span' " .
	   "align='left'>";   #checking for blank column in HTE table lmm
	my $fn = $$names[$fld];
	my $doc = $$meta{"tableinfo"}{"table_document_url"};
	print "<a target='helpwin' href='$doc#$fn'> $fn </a>";
	print "</th></tr>\n";
    }
        print "</thead>\n";	
  
}

# Finish up processing of a table.  Close any open HTML blocks....

sub closeTable {

    my ($self, $msg) = @_;
    
    if ($$self{"intable"}) {
        print "</tbody>\n";
        print "</table>\n";
	$$self{"intable"} = 0;
        my $s = "";
        if ($$self{"rowcount"} > 1) {
            $s = "s";
        }
        my $table = $$self{"meta"}{"table"};
        my $xtable = $self->chopTable($table);

        print "$$self{rowcount} row$s retrieved from $xtable<p>\n";

        if ($$self{"dataset_id"})  { #data_id
           my $datasetLink=$table . $$self{"dataset_id"};
           print "<a name='" . $datasetLink . "'></a><font size='-1'> Note: " .
	        "The standard " .
		"<b>Dataset ID</b> used in referring to this data in" .
		" journals and other publications is the content of this" .
		" column prefixed with the string '" .
		$$self{"dataid_prefix"} . "'. " .
		# More information can be" .
		#" found at the <a href='http://vo.ads.harvard.edu/dv/'>" .
		#"Astrophysics Data System.</a> " .
		"<p></font>";
        }
	
        print "$msg<br>\n" if $msg;

	my $meta = $$self{"meta"};
	my $dp   = $$meta{"dpsets"};
	if (ref($dp)  && scalar keys %$dp == 0) {
	    $dp = undef;
	}
	

	my $tableinfo = $$meta{"tableinfo"};
        print "<table class='othertable'><tr class='othertable'>\n";
	if ($dp) {
            print "<td class='othertable' valign='top'>\n";
            $$tableinfo{table_name} = $table;
            my $results = $self->results();
            if ($results) {
	       $results->DisplaySets($dp, $tableinfo);
            }
	    Browse::Results::displayFilter($table);
	    Browse::Results::displayDPOptions();
            print "</td>\n";
	}
        print "<td class='othertable' valign=top>\n";
        my $coords = ($$tableinfo{right_ascension}? 1:0);

        my $notifyLink="";
        #$notifyLink=$self->notificationForm();

	$self->{results}->DisplayTableOptions($table);
	$self->{results}->DisplayServiceOptions($table,$coords,$dp, $notifyLink);
        print "</td></tr></table></div>\n";

    }
}

sub YourSearchWas {
    my ($self,$positions,$qualifiers,$sorts,$dirs,$resmsg, $tables) = @_;

    #--- Prepare to display SkyView images and Seach Based On box
    my $searchDisplayer = Browse::SearchBasedOn->new();
    $searchDisplayer->CoordSys($self->{coordsys}, $self->{equinox});
    $searchDisplayer->Positions($positions);

    #--- zzgen query uses non-user parameter seeting.  Do not display.
    my @nonzz = grep !/^zz/, @$tables;
    if (scalar (@nonzz) > 0) {
       $searchDisplayer->Qualifiers($qualifiers);
    }

    $searchDisplayer->MaxRows($self->MaxRows());

    # Display DSS and RASS images (if 1 object queried) and search criteria
    my $string = "";
    $string .= " <br><center>\n";
    $string .= "<!--images and search based on start--> <table><tr>\n";

    # If only one object queried display its SkyView image
    my $numpos=$searchDisplayer->Positions(); 
    my $num = 0;
    $num =  scalar(@$numpos) if $numpos;
    if ($num == 1) {
       $string .= "<td> <!--- start images cell --->\n";
       $string .= $searchDisplayer->displayImages();
       $string .="</td><!-- end images cell -->\n";
    }

    #--- Add resolver message
    if ($resmsg =~ /\w/) {
       $searchDisplayer->Resmsg($resmsg);
    }

    #--- Add saved query string
    if ($self->{savedquery} =~ /\w/) {
       $searchDisplayer->SavedQuery($self->{savedquery});
    }

    my $searchHTML = "<td><!-- start search based on cell -->";
    $searchHTML .= $searchDisplayer->displaySearch();

    #--- Add the requery button 
    #--- get the display mode to make sure it does not change upon re-query
    my $displayformat= ref($self);
    $displayformat=~ s/Browse\:\://;
    $searchHTML .= "<input type=hidden name='requerydisplay' value='" .
	$displayformat ."'>\n";
    $searchHTML .= "</td><!--end search based on cell -->\n";
    $string .= $searchHTML;

    #--- Display other format options
    $string .= "<td><!-- start other format options cell -->\n";
    $string .= "<table>";
    $string .= $self->displayFormatOptions();

    #--- Add button for saving targets to file used for target file upload
    $string .="<tr><td align='left' colspan='2'><input type='submit' " .
	"name='redisplay' value='Save All Objects To File' " . 
	"onClick=\"noNewWindowSubmit(this.form)\"></td></tr>\n"; 
    

    #--- Add Reset button 
    $string .="<tr><td align='left' colspan='2'><input type='reset'>" .
	"</td></tr>\n";

    $string .= "</table></td><!-- end other formats cell --></tr>\n";
    $string .= "</table></center>\n";

print $string;

#--- Add tip
print <<EOT;
<p><script language="JavaScript">
<!-- start hide
chooser = Math.round(Math.random()*(numberofitems-1));
document.write("<img src='$::images/dblbball.gif' align='bottom' alt='little blue ball'> <font size='-1'><b>Browse Tip</b>:  " +questions[chooser]+"</font>");
// stop hide -->
</script>
EOT


}

#--- Display format options for re-display
sub displayFormatOptions {
  my ($self) = @_;   

  my $ofstring .="<tr><td align='right'>"; 
  $ofstring .=
	"<input type='submit' name='redisplay' value='Redisplay'" .
	" onclick=\"this.form.target='redisplay'\" disabled></td>" .
	"<td>as <select name='displaymode'>\n";
  $ofstring .= "<option value='Display'>HTML Table\n" 
	unless ref($self) =~ /::Display/;
  $ofstring .= "<option value='TabbedDisplay'>Tabbed Display\n" 
	unless ref($self) =~ /::Tabbed/;
  $ofstring .= "<option value='TextDisplay'>Text Table\n" 
	unless ref($self) =~ /::Text/;
  $ofstring .= "<option value='FitsDisplay'>FITS Table\n" 
	unless ref($self) =~ /::Fits/;
  $ofstring .= "<option value='XMLDisplay'>astrores XML\n" 
	unless ref($self) =~ /::XML/;
  $ofstring .= "<option value='VODisplay'>VOTable XML\n" 
	unless ref($self) =~ /::VO/;
  $ofstring .= "<option value='ExcelDisplay'>Excel-compatible\n" 
	unless ref($self) =~ /::ExcelDisplay$/;
  $ofstring .= "<option value='PureTextDisplay'>Pure Text\n" 
	unless ref($self) =~ /::PureTextDisplay$/;

  $ofstring .= "</select></td></tr>\n";

  $ofstring .="<tr><td align='left' colspan='2'><input type='submit' " .
	"name='redisplay' value='Printer-Friendly Version' " .
	"onclick=\"this.form.target='redisplay'\" disabled></td></tr>\n" 
	unless ref($self) =~ /::PureTextDisplay$/;

  return $ofstring;
}

sub get_row_links {

  # links		related links for this row
  # names		column names for this row 
  # line		column values 
  # unique 		unique id
  # tablename           full table name
  # paraminfo           parameter format for each column

  #my ( $links,$names,$line,$unique,$tablename,$paramInfo) = @_;   #lmm
  my ($self, $names,$line,$unique) = @_;   #lmm

  #--- now that $self is passed
  my $links=$$self{"related_links"};
  my $tablename = $$self{meta}{tableinfo}{"table_description"};
  my $paramInfo =$$self{"meta"}{"paraminfo"};

  return undef unless $$links[0][0] =~ /\w/; 

  my ($mastertable,$keyval) = split('\|\|\|', $unique, 2);

  #print "get_links " . $$links[0][0] . "$keyval<br>" if $::debug;
  $keyval =~ s/\|\|$//; # strip off trailing ||
  $keyval =~ s/\|/ and /; # | separates each name/value pair

  # values for each row
  my %values;
  #my %params;    # holds format of each field
  foreach my $i (0 .. $#$names) {
     $$line[$i] =~ s/&nbsp;/ /g;
     $values{$$names[$i]} = $$line[$i];
     #print "line=$$line[$i] name=$$names[$i]<br>" if $::debug;
     #-- get format of all fields
     #$params{$$names[$i]} = $paramInfo->{$$names[$i]}{"parameter_format"};
  }

  my $returnlink = "&nbsp;";
  foreach my $link (@$links) {
     # link is an array of table_name,
     #                     link_table_name,
     #                     link_criterion,
     #                     link_priority,
     #                     link_symbol,
     #                     link_description,
     #                     link_dependency ordered by priority

     # First check if there is a link dependency
     #--- changes suggested by e.sabol 7/05 
     
     if (defined($$link[6])
	 && $$link[6] =~ /^(.*)([\=\>\<]|\s+in\s+|\s+not\s+in\s+)(.*)/) {

         # Parse the dependency:  field <=> value
	 #$$link[6] =~ /(.*)([\=\>\<])(.*)/;
	 #my ($field, $op, $tvalue) = ($1, $2, $3);
	 my($field, $op, $tvalue) = ($1, lc($2), $3);
	 
	 $field =~ s/^ *//;
	 $field =~ s/ *$//;
	 
	 $tvalue =~ s/^ *//;
	 $tvalue =~ s/ *$//;
         ##-----------------------------------------------------
	 
	 #print "** Testing: $field, $op, $tvalue<br>\n" if $::debug;
	 if ($op =~ /^\s+(not\s+)?in\s+$/ && $tvalue =~ /^\(\s*select/) {

	     $tvalue =~ s/^\(\s*//;
	     $tvalue =~ s/\s*\)$//;

	     $op =~ s/^\s*//;
	     $op =~ s/\s*$//;
	     #print "** Testing select: $field, $op, $tvalue<br>\n" if $::debug;

	     unless ($self->{link_dep_subqueries}{$tvalue}) {
		 my($data,$pnames) = Browse::DBQ::query($tvalue);
		 my @tvalues = ();
		 foreach my $row (@$data) {
                     
		     push(@tvalues,$row->[0]);
		 }
		 # Cache results of link_dependency subquery for future calls.
		 $self->{link_dep_subqueries}{$tvalue} = \@tvalues;
	     }

	     my $xvalue = exists($values{$field}) ? $values{$field} :
	       exists($values{checkNonDefaults($field)})
	       ? $values{checkNonDefaults($field)} : undef;
	     next unless (defined($xvalue));
             $xvalue =~ s/^\s+//;
             $xvalue =~ s/\s+$//;
	     #print "*** Testing: \"$xvalue\" $op ($tvalue)<br>\@tvalues = (",join(',',@{ $self->{link_dep_subqueries}{$tvalue} }),")<br>\n" if $::debug;

	     my $match = grep(($_ eq $xvalue),
			      @{ $self->{link_dep_subqueries}{$tvalue} });
	     if ($op eq 'in') {
		 next unless ($match);
	     } else { # not in
		 next if ($match);
	     }

	 } else {

	     my $alpha = 0;
	     # Check for quotes and if so treat as string.
	     if ($tvalue =~ /(^[\'\"])(.*)[\'\"]$/) {
	         $alpha = 1;
	         $tvalue = $2;
	     }
	  
	     # Up to here could be done outside the loop for rows...
	     my $xvalue = exists($values{$field}) ? $values{$field} :
	        exists($values{checkNonDefaults($field)})
	        ? $values{checkNonDefaults($field)} : undef;
	     next unless (defined($xvalue));
 
	     #print "*** Testing: $xvalue, $tvalue, $alpha<br>" if $::debug;
	  
	     # Sneaky way of handling all six cases with just two
	     # ifs.  Note that characters '<=>' are in lexical order.
	     if ($alpha) {
	         if (($xvalue cmp $tvalue) != ($op cmp '=')) {
	             next;
	         }
	     } else {
	         if (($xvalue <=> $tvalue) != ($op cmp '=')) {
	             next;
	         }
	     }
	 } 
     }

     my %tInfo = Browse::HDB::TableInfo($$link[1]);
     my $tname =$tInfo{"table_description"}; 
     my ($popupRef) = "Linked Result to $tname " .
	"<br><font size=\"-1\">from $tablename</font>";

     #$returnlink .= "&nbsp;<a title='$$link[5]' target='linkwin' " .
     $returnlink .= "<a title='$$link[5]' target='linkwin' " .
	"href='w3query.pl?tablehead=name%3D$$link[1]&" .
	"displaymode=SimpleDisplay&popupFrom=" .
        Browse::CGI::Encode($popupRef); 


     #print "This link: ($$link[4]) __ $$link[2]<br>\n" if $::debug;


     foreach my $criterion (split('&',$$link[2])) { 
     #print "Display: crit=$criterion mtable=$mastertable<br>\n" if $::debug;

       foreach my $field ($criterion =~ /$mastertable\.([\w]+)/ig) {
          #print "Display:: select $field from $mastertable where $keyval<br>" 
		#if $::debug;

          if (defined $values{$field}) {
             $criterion =~ s/$mastertable\.$field/$values{$field}/;
          } else {
             $criterion =~ 
		s/$mastertable\.$field/(select $field from $mastertable where $keyval)/;
             
          }
       }
       
       if ($criterion =~ /^-cone/) {
          $criterion =~ s/-cone:\s*//ig;
          my($ra,$dec,$radius) = split(',',$criterion);
          $returnlink .= "&Entry=".Browse::CGI::Encode($ra);
          $returnlink .= ",".  
		Browse::CGI::Encode($dec) if defined $dec and $dec != ',';
          $radius ||= "Default";
          $returnlink .= "&Radius=$radius";
       } else {
          #print "Display: criterion=$criterion<br>" if $::debug;
          my ($newfield,$crit) = $criterion =~ /([\w]+)(.*)/;

          #print "Display: new field='$newfield' crit=$crit <br>" if $::debug;
          #--- For links, replace the = operator with == for more effecient
          #--- SQL processing later on (Parameter::MakeParamQuery2())
          $crit =~ s/={1}/==/;
          #print "Display: new  crit=$crit <br>" if $::debug;
          $crit=Browse::CGI::Encode($crit);

          # took out extra unnecessary = sign before $crit --lmm 2/2004
          $returnlink .= "&bparam_$newfield $crit";

       }
     } # end foreach criterion
     $returnlink .= "'>$$link[4]</a>&nbsp;";
  }
 
  return $returnlink;
}

#--- Set up table class, id, etc
sub tableStart {
    my($self,$meta)=@_;
    #--- Give the table an id to reference from table/row number display box 
    my $tableID=$self->chopTable($$meta{"table"}) ."T";
    #--- preheader start
    print "\n<table id='$tableID' border='0' cellspacing='1' rules='cols' " .
	"class='thinbordertable' " .
         #"bgcolor='#000000' " .
         "title='Query results for ".
	$$meta{"tableinfo"}{"table_description"}.
	"'>";

}

#--- create data products selection box and icons
sub createSelectDPBox {
   my ( $self,$unique,$cunique, $shortname, $offsetTarget) =@_;

   my $begin = "<a target=\"moreinfo\" href=\"w3hdprods.pl?".
      "files=P&Target=$cunique&Coordinates=$$self{coordsys}" .
      "&Equinox=$$self{equinox}\">" .
      "<img src=\"$::images/mag5.gif\" ".
      "height='16' width='16' border='0' " .
      "alt=\"Display all params\" onLoad=\"incrementCount('$shortname')\">".
      "</a>";

   if ($unique =~ /\// && $unique !~ / x /i) {
      my ($table,$keyval) = split('\|\|\|', $unique, 2);
      ($keyval) = $keyval =~ /=\s*(.+)\s*\|\|$/;  
      my $key = $$self{meta}{tableinfo}{"query_key"};
      $begin .= "<a target=\"moreinfo\" href=".
      "\"$::vizier/viz-bin/VizieR-5?-source=$table&".
      "$key=" .Browse::CGI::Encode($keyval) . 
      "\"><img src=\"$::images/magviz.gif\" ".
      "height='16' width='16' border='0' " .
      "alt=\"Display all params (from VizieR)\">".
      "</a>";
   }
   my $offValue= $offsetTarget . "_unique_" . "$unique" 
		   if $offsetTarget =~ /\w/;
    
   $begin .= "<input type='checkbox' name='Target'" .
       " id='Target' value='$unique'";
   if ($offsetTarget =~ /\w/) { 
	$begin .= " onClick=\"addHiddenOffset(this,'$offValue');\"";
   } 
   $begin .= ">";

   return $begin;
}

#--- Create new header for split table
sub createNewHeader {

   my($self,$meta) = @_;

   #--- close previous  html table if over 25 rows
   if ($$self{"intable"}) {
      print "</tbody>\n";
      print "</table>\n";
   }

   my $names = $$self{names};
   my $coordsys  = $$self{coordsys};
   my $tablename  = $$meta{tableinfo}{table_name};

   #--- Format column header for non-equatorial coordinates (with 
   #--- sort if appropriate)
   if ($coordsys !~ /equatorial/i) {
      my $tableinfo = $$meta{tableinfo};

      my $ra = $$tableinfo{"right_ascension"};
      $ra =~ s/^\@//;
      my $dec = $$tableinfo{"declination"};
      $dec =~ s/^\@//;
      my $target_name = $$tableinfo{"target_name"};
      $target_name =~ s/^\@//;

      #--- find names of other ra/dec columns
      #my ($oras, $odecs) = $self->otherRaDecs($tableinfo);
      my ($oras, $odecs) = otherRaDecs($tableinfo);

      my $names = $$self{'names'};
      #my ($sortCheck, $res, $pnames);

      foreach (@$names) {
         #$_ = $self->renameRaDecs($_, $ra, $dec, $oras, $odecs, $coordsys, 
         $_ = renameRaDecs($_, $ra, $dec, $oras, $odecs, $coordsys, 
		$tablename);
         #if (scalar(@$oras) && scalar(@$odecs)) {
            #my @check_other_ras = grep /^$_$/, @$oras;
            #my @check_other_decs = grep /^$_$/, @$odecs;
            #if ($coordsys =~ /galactic/i ) {
               #if (scalar(@check_other_ras)) {
                  #$_ =~ s/ra$/lii/;
               #}
               #if (scalar(@check_other_decs)) {
                  #$_ =~ s/dec$/bii/;
               #}
            #} elsif ($coordsys =~ /ecliptic/i) {
               #if (scalar(@check_other_ras)) {
                  #$_ =~ s/ra$/elon/;
               #}
               #if (scalar(@check_other_decs)) {
                  #$_ =~ s/dec$/elat/;
               #}
            #}
         #}
#
         #if ($_ eq $ra) {
            #if ($coordsys =~ /galactic/i) {
               ##$_ = "_Lii";
               ##-- Check if table has galactic coordinate parameters
	       ##-- If it does do not attach the leading underscore
               ##-- This is for table display and column sort purposes
               ##--  only (lii & bii values are transformed ra & dec)
               #$sortCheck="select distinct parameter_format " .
		  #" from zzpar " .  "where table_name= '$tablename'" .
                  #" and parameter_name='lii'";
               #($res,$pnames)=Browse::DBQ::query($sortCheck);
    	       #if ($#$res < 0) {
                  #$_ = "_Lii";
               #} else {
                  #$_ = "lii";
               #}
            #} elsif ($coordsys =~ /ecliptic/i) {
               #$_ = "_Elon";
            #}
         #} elsif ($_ eq $dec) {
            #if ($coordsys =~ /galactic/i) {
               ##$_ = "_Bii";
               ##-- Check if table has galactic coordinate parameters
	       ##-- If it does do not attach the leading underscore
               ##-- This is for table display and column sort purposes
               ##--  only (lii & bii values are transformed ra & dec)
               #$sortCheck="select distinct parameter_format " .
	          #" from zzpar " .  "where table_name= '$tablename'" .
                  #" and parameter_name='bii'";
               #($res,$pnames)=Browse::DBQ::query($sortCheck);
    	       #if ($#$res < 0) {
                  #$_ = "_Bii";
               #} else {
                  #$_ = "bii";
               #}
            #} elsif ($coordsys =~ /ecliptic/i) {
               #$_ = "_Elat";
            #}
         #}
      }
   }
	
   $self->printColumnHeader($$self{"names"});

   #--- add id to table for table/row number display box
   my $tID = $self->chopTable($tablename) . "TB";
   print "<tbody id=$tID>\n";
   #print "<tbody>\n";
	
   #--- set flag to indicate new html table (25 rows) has been started
   $$self{"intable"} = 1;
}

#--- create link to related tables
sub relatedLinks {
   my ($self, $names, $line,$unique,$class)= @_;
   
   # Get table name of current table ("from link")
   my $tdesc = $$self{meta}{tableinfo}{"table_description"};
   my $links = $self->get_row_links($names, $line,$unique);

   if ($links =~ /\w/) {
      print "<td class='$class'>\n"; #Related links
      print $links;
      print "</td>";
   } else { 
      #--- if there are no links for this row add empty cell 
      #--- so other cells line up under the correct header

      #--- check to see if there are any links at all for this table
      my $testlinksize= scalar @{$self->{related_links}};
      if ($testlinksize == 1) {
         my @test=@{$self->{related_links}}[0];
         $testlinksize = scalar @{$test[0]}; 
      }
      if ($testlinksize) {
         print "<td class='$class'>\n"; #Related links
         print "&nbsp;";
         print "</td>";
      }
   }
}

#--- create link to bibliography for Services section
sub getFovlink {
   my ($self,$table,$line,$names,$target,$status,$savera,$savedec,)=@_;
   my $fovlink;
   my ($ra,$dec,$obsid);
   if ($self->{coordsys} =~ /\w/ && scalar(grep /$table/, @::fovtables)) {
   #if ($self->{coordsys} =~ /equatorial/i && 
                   #scalar(grep /$table/, @::fovtables)) {
       if ($status =~ /\w/ && scalar(grep /$status/, @::fovstatus)) {
          for (my $i=0; $i <= $#$names; $i += 1) {
             $$line[$i] =~ s/&nbsp;/ /g;
             #print "Display::line=$$line[$i] name=$$names[$i]<br>" if $::debug;
             if ($$names[$i] eq 'ra') {$ra=$$line[$i];}
             if ($$names[$i] eq 'dec') {$dec=$$line[$i];}
             if ($$names[$i] eq 'obsid') {$obsid=$$line[$i]; $obsid =~ s/^\s+//}
          }
          #$fovlink = "position=" .Browse::CGI::Encode($target).
          $fovlink = "position=" ."$savera,$savedec".
	      "&obsposition=$savera,$savedec&showplot=on&fov=$obsid" . 
              "&displayimage=on&displayresults=on";
       }
   }
           #print "D getfov $table $status $fovlink<br>" if $::debug;
   return $fovlink;
}

sub getBiblink {
    my($self,$line,$names)= @_;
    #--- Get field used to find bib links
    my $bkeys = $$self{"meta"}{"tableinfo"}{"bibkeys"};

    my $bkeystring;
    my $i;
    #--- look for matching table parameter and build link for current row
    for ($i=0; $i <= $#$line; $i += 1) {
       my $sep = '|' if length($bkeystring);
          #$bkeystring .= $sep."$$names[$i]=$$line[$i]" 
		#if $bkeys =~ /\b$$names[$i]\b/; 
       if ($bkeys =~ /\b$$names[$i]\b/) { 
          $bkeystring .= $sep."$$names[$i]=$$line[$i]" ;
       } else {
          #--- Check if bibkey is a parameter that is not in the display set.
          if (checkNonDefaults($bkeys) =~ /\b$$names[$i]\b/) { 
             $bkeystring .= $sep."$bkeys=$$line[$i]" ;
          }
       }

    }
    # remove spaces in case format from metadata is incorrect
    $bkeystring=~s/\s//g;
    
    my @biblink = 
	split(/ /,$$self{"meta"}{"tableinfo"}{"bibcodes"}{$bkeystring});
    @biblink=grep(/\w/,@biblink); # get rid of any nulls
    my $biblink = "bibcode=".join("&bibcode=",@biblink) if scalar(@biblink);
    
    return $biblink; 
}

#--- convert a parameter to the designated name used if a parameter is not 
#--- a ddefault or requested parameter but is included in the search query
sub checkNonDefaults {
   my ($field) =@_;
   return "_link_".$field;
} 

sub writeColumnData {
   my ($self,$class,$data) = @_;
   #print "<td class='$class'>$data"; 
   print "<td ";
   if ($class) {print "class='$class'";}
   print ">$data"; 
}

#--- overridden in sub classes
sub beforeNewTable {}
sub afterTable {}

sub otherRaDecs {
   #my ($self,$tinfo) = @_;
   my ($tinfo) = @_;
   #--- find names of other ra/dec columns
   my $other_radecs = $$tinfo{"other_ra_dec"};
   my @oras;
   my @odecs;
   if ($other_radecs) {
      my @other_list = split(/\|\|/, $other_radecs); 
      foreach (@other_list) {
         my ($ora, $odec) = split(/\s*,\s*/); 
         if (defined($ora) && defined($odec)) {
            $odec =~ s/^\@//;
            $ora =~ s/^\@//;
            push(@oras, $ora);
            push(@odecs, $odec);
         }
      }
   }
   return \@oras, \@odecs;
}


sub renameRaDecs {
   #my ($self, $name, $ra, $dec, $oras, $odecs, $coordsys, $tablename) = @_;
   my ($name, $ra, $dec, $oras, $odecs, $coordsys, $tablename) = @_;
   my ($sortCheck, $res, $pnames);

   #--- other table ras and decs 
   if (scalar(@$oras) && scalar(@$odecs)) {
      my @check_other_ras = grep /^$name$/, @$oras;
      my @check_other_decs = grep /^$name$/, @$odecs;
      if ($coordsys =~ /galactic/i ) {
         if (scalar(@check_other_ras)) {
            $name =~ s/_ra$/_lii/;
            $name =~ s/^ra_/lii_/;
         }
         if (scalar(@check_other_decs)) {
            $name =~ s/_dec$/_bii/;
            $name =~ s/^dec_/bii_/;
         }
      } elsif ($coordsys =~ /ecliptic/i) {
         if (scalar(@check_other_ras)) {
            $name =~ s/_ra$/_elon/;
            $name =~ s/^ra_/elon_/;
         }
         if (scalar(@check_other_decs)) {
            $name =~ s/_dec$/_elat/;
            $name =~ s/^dec_/elat_/;
         }
      }
   }

   if ($name eq $ra) {
      if ($coordsys =~ /galactic/i) {
        if ($tablename) {
           #-- Check if table has galactic coordinate parameters
	   #-- If it does do not attach the leading underscore
           #-- This is for table display and column sort purposes
           #--  only (lii & bii values are transformed ra & dec)
           $sortCheck="select distinct parameter_format " .
		  " from zzpar " .  "where table_name= '$tablename'" .
                  " and parameter_name='lii'";
           ($res,$pnames)=Browse::DBQ::query($sortCheck);
    	   if ($#$res < 0) {
              $name = "_Lii";
           } else {
                  $name = "lii";
           }
        } else {
              $name = "_Lii";
        }
      } elsif ($coordsys =~ /ecliptic/i) {
               $name = "_Elon";
      }
  } elsif ($name eq $dec) {
     if ($coordsys =~ /galactic/i) {
        if ($tablename) {
           #-- Check if table has galactic coordinate parameters
	   #-- If it does do not attach the leading underscore
           #-- This is for table display and column sort purposes
           #--  only (lii & bii values are transformed ra & dec)
           $sortCheck="select distinct parameter_format " .
		  " from zzpar " .  "where table_name= '$tablename'" .
                  " and parameter_name='lii'";
                   
           my ($res,$pnames)=Browse::DBQ::query($sortCheck);
    	   if ($#$res < 0) {
              $name = "_Bii";
           } else {
              $name = "bii";
           }
        } else {
           $name = "_Bii";
        }
     } elsif ($coordsys =~ /ecliptic/i) {
        $name = "_Elat";
     }
   }
   return $name;
}

## Successful load
1;
