#$Id: DisplayTableHits.pm,v 1.11 2009/04/17 19:46:23 lmm Exp $
#-----------------------------------------------------------
#--- Displays a summary of all tables and the number of associated
#--- rows that match the user criteria
#-----------------------------------------------------------

use strict;

package Browse::DisplayTableHits;

use Browse::Config;
use Browse::ValidateInput;
use Browse::SearchBasedOn;
use Browse::Parameter;
use HEASARCMaster qw(query_master_table); 
use HTM;
use Benchmark;


sub new {
    my ($self,$type) = @_;
    my $obj = {};
    my $packageName="Browse::$type";
    eval "package Browse::_firesafe; require $packageName";
    return bless $obj, $packageName;
}
   
sub hello {
   my  ($self,$in) = @_;
   print "Hello! You have reached Browse::DisplayTableHits<br>";
}

sub setNumberOfColumns {
   my  ($self, $columns) = @_;
   $columns=4 unless $columns;
   $self->{number_of_columns}=$columns;    # set number of columns in table
   $self->{columnWidth}=100/$self->{number_of_columns};
   $self->{missionWidth}=$self->{columnWidth} * 2/3;
   $self->{columnWidth} += 1/3/($self->{columnWidth}-1);

}

#-------------------------------------------------------------------------
#--- Display results: Heasarc Table names and number of hits
#--- with links to table results
sub display {
   my ($self,$in) = @_;
   my %dummy;
   $self->{allpositions} = \%dummy;

   #--- set up for local debugging ------------------
   $self->{debug}=0;   # debug for this package only
   $self->{benchmark}=0;   # calculate timing for variuos stages of query
   print "Content-Type: text/html\n\n" 
	if ($self->{debug} || $::debug || $self->{benchmark}) ;
   Browse::Utils::DumpRef($in) if $self->{debug};
   #--------------------------------------------------


   Browse::Config::AlwaysRun();
   if ($$in{Entry} =~ /\w/) {
       $$in{Entry} =~ s/\0/;/g; # multiple targets should be sepd by ';'
   }

   ##--- save orginal radius to send with table query
   $self->{radius}        = $in->{Radius}; 
   $self->{radius_unit}   = $in->{Radius_unit};   ##--- save org. units

   #--- resolve object name if necessary, changes radius value
   my $parameter = Browse::Parameter->new("Parameter");
   my $start_bench;
   if ($self->{benchmark}) {
      $start_bench = new Benchmark;
      print "start Benchmark:: Resolving sources <br>" ;
   }
   my ($positions,$resmsg)=$parameter->MakeConeQuery($in);
   if ($self->{benchmark}) {
      my $end_bench = new Benchmark;   # benchmark
      print "end=$end_bench  <br>" if $self->{debug};
      my $difference = timediff($end_bench, $start_bench); #benchmark
      print "<br><---Cone Search-->", timestr($difference), "<-------><br>" ;
   }


   print "time ="   . $in->{Time} ."<br>" if ($self->{debug} || $::debug) ;
   print "radius =" . $in->{Radius} ."<br>" if ($self->{debug} || $::debug) ;
   print "entry ="  . $in->{Entry} ."<br>" if ($self->{debug} || $::debug) ;

   my $logmessage ="[". DateTime::asctim()."] $::REMOTE_HOST ::" .
	" [MasterTable Search], " ; 
   $logmessage .= "Entry: " . $in->{Entry} if $in->{Entry}; 
   $logmessage .= " Radius: " . $in->{Radius} if $in->{Radius}; 
   $logmessage .= " Time: " . $in->{Time} if $in->{Time}; 
   Browse::Log::WriteLog("$logmessage", $::queryLog);

   # Specify search radius  
   my $search_radius = $in->{Radius};
   $search_radius= undef if $in->{Radius} =~ /Default/;

   #--- following added to deal with coordinates of the form 
   #--- [xx xx xx +yy yy yy].  Adds a comma between coords
   #--- Update 01/08 - MakeConeQuery resolved a lot of this issue.
   my @entries=split(/;/,$in->{Entry});
   my @newentries; 
   foreach  (@entries) {
      #my ($r, $d ) = /(.*)[^,]\s*(\+.*)/;

      #Clean up entered source
      my $e=$_;
      $e=~s/\s+/ /;
      $e=~s/^\s+//;
      $e=~s/\s+$//;

      push (@newentries, $e);
   }
   $self->{entry}         = join(';',@newentries);
   print "entry e=$$self{entry}<br>"  if ($self->{debug} || $::debug) ;
   print "entry encoded e=" . Browse::CGI::Encode($$self{entry}) ."<br>"  
	if ($self->{debug} || $::debug) ;

   $self->{resolver}      = $in->{NR};
   $self->{equinox}       = $in->{Equinox};
   $self->{coordinates}   = $in->{Coordinates};
   $self->{timerange}     = $in->{Time};   # may be more than 1 time range
   $self->{search_radius} = $search_radius;

   $self->{positions}     = $positions;
   print "pos $$positions[0]<br>"  if ($self->{debug} || $::debug) ;
   
   #--- table color scheme
   $self->{altmissioncolor1}= "#DDBBBB";
   $self->{altmissioncolor2}= "#BBBBDD";
   $self->{altcolor1}= "#FFDDDD";
   $self->{altcolor2}= "#DDDDFF";
   $self->{missioncolor} = "#AADDDD";
   $self->{headercolor} = "#F9ECA0";

   #--- used to convert time to qualifiers if time range has been entered
   $self->{qualifiers}=();


   #--- Simple page header
   Browse::Decor::Header(
	"Browse Search Results: Results Summary for all Missions",
	0,0,0,"displayhits");

   #--- Search the master table
   my $tableHits;
   $self->queryMasterTable();

   

   #--- include Javascript to have all mission tables selected when
   #--- Mission box is clicked
   print "<script type=\"text/javascript\" language=\"JavaScript\">\n";
   print "<!--\n";
   Browse::ValidateInput::tickTable();
   print "</script>\n";

   #--- Display criteria the results are based on
   $self->displayCriteria($positions);

   #--- form start
   my $formname="w3table";
   print "\n<form method='POST' name=\"$formname\" action=\"w3table.pl\">\n";

   #--- Display the results
   $self->displayTable();

}   #--- end display
#-------------------------------------------------------------------------


#-------------------------------------------------------------------------
#--- Create HTML table that displays Mission name and rows of Heasarc 
#--- table names with the number of hits
sub displayTable {
   my ($self)=@_;

   my($num_tables, $num_hits);
   my($table_name, $hit_list);
   my($entry, $resolver );
   my($total_hits);
   
   #--- Need at least one hit.
   scalar(keys %{$self->{allpositions}}) > 0 or 
	Browse::Utils::Error('No Tables satisfied the entered criteria !'); 

   $num_tables = scalar keys %{$self->{allpositions}};
   $total_hits = 0;
   $self->{bgcolor}=$self->{altcolor1};

   #--- Query database to get HEASARC table names
   #--- Get all mission & table names  
   my $sql="select a.table_name,a.parameter_value, b.table_description" .
	" from zzext a, zzgen b where a.parameter_name='observatory_name' " .
	"and a.table_name=b.table_name order by a.parameter_value";

   #--- alt query Get regime for future table sorting
   #my $sql="select a.table_name,  a.parameter_value, c.table_description, " .
	#"b.parameter_value from zzext a, zzext b, zzgen c where" .
	#" (a.parameter_name='observatory_name' and " .
	#"b.parameter_name='frequency_regime') and " .
	#"a.table_name=b.table_name and a.table_name=c.table_name " .
	#"order by a.parameter_value";


   #--- Send query
   my ($results) = Browse::DBQ::query($sql);

   #--- Create arrays of table names for each mission type 
   #--- See Config file until database table is available to query
   #--- this information
   my @active   = ();
   my @archived = ();
   my @catalog  = ();

   foreach my $result (@$results) {            

      #--- second element is mission name
      if ($::missiontypes{$$result[1]} =~/archived/i) {
         push (@archived,$result);
      } elsif ($::missiontypes{$$result[1]} =~ /active/i) {
         push (@active,$result);
      } elsif ($::missiontypes{$$result[1]} =~ /catalog/i) {
         push (@catalog,$result);
      }
   }

   my %allMissions=(
      active=>\@active,
      archived=>\@archived,
      catalog=>\@catalog,
   );

   my ($shortname, $longname, $mission, $columns, $regime);
   
   my $currentMission="";
   my $missionCount=0;
   my $newcolor;

   #--- Set number of columns in displayed table for formatting
   $self->setNumberOfColumns();

   $self->{newcell}=0;   #--- used to keep track of unpopulated cells
   my $first_table=1;

   #--- may be used later
   #$self->createMissionAnchors($hit_table, $results);

   #--- Display Submit buttons
   print "<input type='submit' name='Action' value='View Selected Tables' " .
	"onClick=\"return(pageCheck('on'))\">&nbsp;" .
	"<input type='reset' value='Reset'> &nbsp; &nbsp; &nbsp; &nbsp;\n";

   #--- Add parameters from Requery form to main form
   if ($self->{timerange}) {
         print "<input type='hidden' name='Time' value='" . 
		$self->{timerange} . ">'\n"; 
   }
   if ($self->{coordinates}) {
         print "<input type='hidden' name='Coordinates' value='" . 
		$self->{coordinates} . "'>\n"; 
   }
   if ($self->{equinox}) {
         print "<input type='hidden' name='Equinox' value='" . 
		$self->{equinox} . "'>\n"; 
   }
   if ($self->{entry}) {
         print "<input type='hidden' name='Entry' value='" . 
		$self->{entry} . "'>\n"; 
   }
   if ($self->{radius_unit}) {
         print "<input type='hidden' name='Radius_unit' value='" . 
		$self->{radius_unit} . "'>\n"; 
   }
   #if ($self->{search_radius}) {
   if ($self->{radius}) {
         print "<input type='hidden' name='Radius' value='" . 
		$self->{radius} . "'>\n"; 
   }

   print "<input type='hidden' name='ResultMax' value='0'>\n"; 


   #--- Start table formatting
   my $tableHTML = "\n<table align='left' cellspacing='1' border='0' " .
	"width='95%' cellpadding='1'>\n";    #--- main table


   #--- for each mission category display mission tables and # of hits
   foreach my $type (sort keys %allMissions) {

      my $typeCount=0;  #--- looking for missions categories with no results

      #--- Finish displaying blank cells (color only) before starting new 
      #--- mission category
      if ($self->{newcell} %  $self->{number_of_columns}-1 > 0) {
         while ($self->{newcell}  % $self->{number_of_columns}-1 > 0) {
            $tableHTML .= "<!--completing tables-->\n <td bgcolor='" .
		$self->{bgcolor} . "'>" .
		"<table width='100%'>" .
		"<tr>\n <td>&nbsp;</td>\n <td>&nbsp;</td>\n " .
		"<td>&nbsp;</td>\n" .
		" </tr></table></td>\n";
            ++$self->{newcell};
         }
      }

      #--- Create header row for mission type
      if ($type =~ /active/) {
           $tableHTML .= "<tr><td bgcolor='" .
	   $self->{headercolor} .
	   "' colspan='" . 
	   $self->{number_of_columns} . "' align='center'>" .
	   "<b>Active HEASARC Missions</b></td></tr>\n";
      } elsif ($type =~ /archived/) {
           $tableHTML .= "<tr><td bgcolor='" .
	   $self->{headercolor} .
	   "' colspan='" . 
	   $self->{number_of_columns} . "' align='center'>" .
	   "<b>Other Missions</b></td></tr>\n";
      } elsif ($type =~ /catalog/) {
           $tableHTML .= "<tr><td bgcolor='" .
	   $self->{headercolor} .
	   "' colspan='" . 
	   $self->{number_of_columns} . "' align='center'>" .
	   "<b>Object Catalogs</b></td></tr>\n";
      }


      #--- For each HEASARC table check master table results for a match
      #--- Display if match

      foreach $columns (@{$allMissions{$type}}) {            
         $shortname=shift(@$columns);	# short table name
         $mission=shift(@$columns);   # mission
         $longname=shift(@$columns);  # long table name
  
         #--- Get number of hits for current HEASARC table
         $hit_list=0;    #-- reset counter for each table
         
         if ($self->{allpositions}{$shortname}) {
            $hit_list = $self->{allpositions}{$shortname};
          } 
          if ($hit_list) {  # include table only if table has associated hits

            ++$typeCount;

            if ($mission ne $currentMission) {   #-- start new mission section

               #-- color fill empty cells of previous mission section 
               #-- before creating new mission section
               if ($self->{newcell} %  $self->{number_of_columns}-1 > 0) {
                  while ($self->{newcell}  % $self->{number_of_columns}-1 > 0) {
                     $tableHTML .= "<!--completing tables-->\n <td bgcolor='" .
			   $self->{bgcolor} . "'>" .
			   "<table width='100%'>" .
		   	   "<tr>\n <td>&nbsp;</td>\n <td>&nbsp;</td>\n " .
			   "<td>&nbsp;</td>\n" .
		   	   " </tr></table></td>\n";
                     ++$self->{newcell};
                  }
               }

               #--- Close previous mission row unless this is the very first
               if (!$first_table) {
                   #--- close previous table
                   $tableHTML .= "</tr>\n" .
			   "<!--- end mission table-->\n";
               } else {
                   $first_table=0;
               }
         
               #-- alternate colors for each mission section
               $newcolor=$missionCount % 2;
               if ($newcolor==1) {
	          $self->{bgcolor}=$self->{altcolor1}; 
	          $self->{missioncolor}=$self->{altmissioncolor1}; 
               }
                 else {$self->{bgcolor}=$self->{altcolor2};
                 $self->{missioncolor}=$self->{altmissioncolor2};
               }
               

               #--- new mission section   
               $tableHTML .= "<!--***********************************-->\n\n";
            
               #--- replace blanks in mission name with encoded spaces
               $_=$mission;
               #tr/ /&nbsp;/;
               tr/ /_/;
               my $mission_help=$_;

               $tableHTML .= "\n<!---start mission table-->\n" .
		"<tr bgcolor='" . $self->{bgcolor} . "'>" .
                #--- mission name cell 
		"<td width='" .$self->{missionWidth} . "%' bgcolor='" .
		$self->{missioncolor} . "'>" .
                "<!--- mission name cell -->\n" .
		"<table width='100%' border='0'>\n" .
		"<td align='left' width='5%' bgcolor='" .
                $self->{missioncolor} . 
		"'>" .
		"<input type='checkbox' name='table' " .
		"id='$mission' " .
		"onClick=" .
		"\"tickTable(this.form.tablehead,this.checked,'".
		#Browse::CGI::Encode($mission)."')\"" .
		$mission."')\"" .
		">" .   "</td><td align='left' bgcolor='" .
                $self->{missioncolor} . 
		"'>" .
                "<font size='-1'>" .
		"<label for='$mission'>" .
		"<a target='helpwin' href='w3table.pl?MissionHelp=" . 
	        lc($mission_help) . "'>" .
		"$mission" .
	    	"</a></label>\n" .
                "</font>" .
		"</td>" .
	        "</tr></table></td>\n\n";

                $self->{newcell}=1;   #--- reset for each new mission 

                ++$missionCount;   # increment mission count for color change
            }   #-- end new mission section

            $num_hits = $hit_list or 
		Browse::Utils::Error("$table_name is empty!");

            #--- display table info & number of hits
            $tableHTML .=  $self->displayCell($shortname, $longname, $num_hits,$mission);

            #--- count used to determine when new row is needed and to
            #--- format empty cells at end of mission section
            ++$self->{newcell};

            #-- end table/hits cell
            $tableHTML .= " </td><!--- end checkbox-table-hits cell -->\n";   

            #--- start a new row 
            if ($self->{newcell}==$self->{number_of_columns}) {
                $tableHTML .= "</tr><!--- end row of checkbox-table-hits -->\n";
            }

            $currentMission=$mission;
         }   #--- if there are hits for this table

       
      }   #--- end table

      #--- Display message if there were no missions in the current category
      if ($typeCount == 0) {
           $tableHTML .= "<tr><td bgcolor='" .
	   $self->{bgcolor} .
	   "' colspan='" . 
	   $self->{number_of_columns} . "' align='center'>" .
	   "<font size='-1'>No data for this category</font></td></tr>\n";
       }
   }   #--- for each mission category 

   #--- last set of empty table cells
   if ($self->{newcell} %  $self->{number_of_columns}-1 > 0) {
      while ($self->{newcell}  % $self->{number_of_columns}-1 > 0) {
         $tableHTML .= "<!--completing tables-->\n <td bgcolor='" .
		$self->{bgcolor} . "'>" .
		"<table width='100%'>" .
		"<tr>\n <td>&nbsp;</td>\n <td>&nbsp;</td>\n " .
		"<td>&nbsp;</td>\n" .
		" </tr></table></td>\n";
         ++$self->{newcell};
      }
   }

   $tableHTML .= "</tr></table>\n";
   $tableHTML .= "</td></tr></table>\n";
    
   print $tableHTML;

   #--- Display another set of Submit buttons
   print "<p><input type='submit' name='Action' " .
	"value='View Selected Tables' " .
	"onClick=\"return(pageCheck('on'))\">&nbsp;" .
	"<input type='reset' value='Reset'>\n";
   print "</form>\n";
   print "<script language='JavaScript' type='text/javascript' " .
	"src='$::w3bdocs/wz_tooltip.js'></script>";
    print "</body></html>\n";

}  #--- end displayTable 
#-------------------------------------------------------------------------

#-------------------------------------------------------------------------
#--- format a checkbox/table/hits cell for Browse query
sub displayCell {
   my ($self,$shortname,$longname, $num_hits, $mission)=@_;

   #--- Get table info for checkbox values
   my $cellHTML="";
         
   my $tableinfo = {Browse::HDB::TableInfo($shortname)};
   $tableinfo->{'radius'} =  $tableinfo->{'default_search_radius'} 
                     if exists $tableinfo->{'default_search_radius'};   
   $tableinfo->{'radius'} =  "***" if($tableinfo->{'radius'} < 0);   
   $tableinfo->{'url'} ||= $tableinfo->{'table_document_url'};
   $tableinfo->{'mission'} ||= $tableinfo->{'observatory_name'};
   $tableinfo->{'tabletype'} ||= $tableinfo->{'table_type'};
   $tableinfo->{'description'} ||= $tableinfo->{'table_description'};
   $tableinfo->{'archive'}="Y"; # if ($gotsets{$table});
   $tableinfo->{'archive'} ||= 'N';
   $tableinfo->{'priority'} = '5' unless defined $tableinfo->{'priority'};
   # priority can be '0'

   #--- Choose one mission id table associated with multiple missions 
   my @multimissions = split(/\|\|/,$tableinfo->{'mission'});

   #print "missions=" .  $tableinfo->{'mission'} . " mm num=" .
	#scalar(@multimissions) ,"<br> " if ($self->{debug} || $::debug) ;

   $tableinfo->{'mission'} = $multimissions[0];
   if (scalar(@multimissions) > 1) {
      foreach my $mm (@multimissions) {
       if ($mm =~ /\b$mission\b/i) {
          $tableinfo->{'mission'} = $mm;
          last;
       }
      }
   }
   #print "missions after=" .  $tableinfo->{'mission'} . "<br> " 
	#if ($self->{debug} || $::debug) ;

   my($info)  =  'name=' . $shortname .'&'.
      'description=' . $tableinfo->{'description'}.'&'.
      'url=' .         $tableinfo->{'url'}. '&' .
      'archive=' .     $tableinfo->{'archive'}.'&'.
      'radius=' .      $tableinfo->{'radius'} . '&' .
      'mission=' .     $tableinfo->{'mission'} . '&' .
      'priority=' .    $tableinfo->{'priority'} . '&' .
      'tabletype=' .   $tableinfo->{'tabletype'}; 

   if ($self->{timerange}) {$info .= "&Time=" .  $self->{timerange}; }

      if ($self->{newcell} % $self->{number_of_columns} == 0) {
            #--- new row of tables-hits for current mission
            $cellHTML .= "<tr bgcolor='" . $self->{bgcolor} . 
		"'><td></td><!-- table-hits row  -->\n\n";   
            ++$self->{newcell};
      }
      $cellHTML .= " <td width='" .$self->{columnWidth} . 
	 "%' align='left'>" .
	 "<!--- checkbox-tablename-hits cell -->\n";
      $cellHTML .= " <table   width='100%'> <tr>\n";
         #--- 
      $cellHTML .= " <td align='left' width='5%'>\n" .
            " <input type='checkbox' name='tablehead' id='$shortname' " .#label?
	    "value='$info'>";
      $cellHTML .="</td>\n";
      $cellHTML .= " <td align='left' >" ."<font size='-1'>" .
	    "<label for='$shortname'>" .
	    "<a target=\"helpwin\" href=\"" .$tableinfo->{'url'}. "\">";
      $cellHTML .= "$longname "  . 
	 "</a>" .  "</label>" .
	"</font></td>\n\n";
      $cellHTML .= " <td align='right' ><font size='-1'> <a href=\"";
      my $url =
	   "Radius=" . $self->{radius} . 
	   "&Radius_unit=" . $self->{radius_unit} .
	   "&Coordinates=" . $self->{coordinates} .
	   "&Equinox=" . $self->{equinox} .
	   "&ResultMax=" . "0" .
	   "&NR=" .Browse::CGI::Encode($self->{resolver}) .
	   "&Entry=" .Browse::CGI::Encode($self->{entry}); 
      $url .= "&tablehead=$info\"";
	   
      my $completeurl = "w3query.pl?" .$url;

      $cellHTML .= "$completeurl>" .  "$num_hits</a></font></td>\n";
      $cellHTML .= " </tr></table>\n";
      return $cellHTML;
}  #--- end displayCell
#-------------------------------------------------------------------------

#-------------------------------------------------------------------------
#--- Display parameters entered by user for search, SkyView images and
#--- image that describes how to use summary table
sub displayCriteria {
   my ($self, $positions) =@_; 

   #--- Display search criteria and SkyView Images

   #--- separate form for "Search based on" Submit button
   my $formname="w3table_sb";
   print "\n<form method='POST' name=\"$formname\" action=\"w3table.pl\">\n";
   #--- Add time range and coordinates to pass on
   if ($self->{timerange}) {
   print "<input type='hidden' name='Time' value='" . 
		$self->{timerange} . "'>\n"; 
   }
   if ($self->{coordinates}) {
         print "<input type='hidden' name='Coordinates' value='" . 
		$self->{coordinates} . "'>\n"; 
   }
   if ($self->{equinox}) {
         print "<input type='hidden' name='Equinox' value='" . 
		$self->{equinox} . "'>\n"; 
   }
   print "<input type='hidden' name='Resultmax' value='0'>\n"; 

   my $displayer = Browse::SearchBasedOn->new();
   $displayer->CoordSys($self->{coordinates}, $self->{equinox});
   $displayer->Positions($positions);
   $displayer->Qualifiers($self->{qualifiers});
   
   my $imageHTML="<table width='95%'><tr>";

   #-- Search Based On
   $imageHTML .= "<td align='center' valign='middle'>\n";
   $imageHTML .= $displayer->displaySearch();
   $imageHTML .= "</form >\n";
   $imageHTML .= "</td>\n";

   #-- SkyView Images
   if ($positions && scalar(@$positions) ==1) {
      $imageHTML .= "<td align='center'>\n";
      $imageHTML .= $displayer->displayImages();
      $imageHTML .= "</td>\n";
   }

   #-- display "how to read table" image
   $imageHTML .= "<td align='center' valign='middle'>\n";
   $imageHTML .= "<table><tr><td valign='bottom' align='center'>\n";
   $imageHTML .= "<font size='-1'>" .
	"How to use the information on this page<br></font>";
   $imageHTML .= "<img src='$::images/browse-key.gif'>";
   $imageHTML .= "</td></tr></table>\n";
   $imageHTML .= "</td>\n";

   $imageHTML .= "</tr></table>\n";
   print "$imageHTML<br>";

   #-- Disable max rows field, may want to allow changes later
   print "<script language='JavaScript' type='text/javascript'>\n"; 
   print "<!--\n";
   print "if (document.getElementById) {\n";
   print "var mr=document.getElementById('maxrows');\n";
   print "if (mr){mr.disabled=true;} else {alert('not possible');}\n";
   print "}\n";
   print "//-->\n";
   print "</script>\n";
   


}  #--- end displaycriteria
#------------------------------------------------------------------------

#-------------------------------------------------------------------------
#--- not used right now
#sub createMissionAnchors {
   #my ($self, $hit_table,$results) = @_;
   #my $r=$results;  
   #my ($list, $shortname, $mission, $longname);
#
   #my $html = "<b>Index: </b>";
   #my $currentMission="";
   ##foreach my $columns (@$results) {            
   #foreach my $columns (@$r) {            
#
      #$shortname=shift(@$columns);	# short table name
      #$mission=shift(@$columns);   # mission
      #$longname=shift(@$columns);   # mission
      #$list = $hit_table->{$shortname};
      #if ($list && $mission ne $currentMission) {     
         #$html .= "<a href='#$mission'>$mission</a>, \n";   
         #$currentMission=$mission;
      #}
   #}
   #print "$html<p>";
#
#}  #--- end createMission Anchors
##-------------------------------------------------------------------------

#--- tell the user what to do after page is displayed  - not used
sub instructionsText {
    my ($self) = @_;
    my $text="<font size='-1'>The table below displays the number of rows " .
	"that satisfy your selection criteria in each Browse " .
	"table (tables ordered by mission).\n";
    $text .= "Select desired tables and then hit the Submit Search button OR " .
        "click on the number of rows to go directly to results for " .
	"the corresponding table. " .
	"Clicking the box next to the mission name will select all " .
	"associated tables. ";
    $text .= "</font>";
    return $text;
}

#-------------------------------------------------------------------------
#--- Query the MasterTable  
#--- Note: Time only queries are treated slightly differently

sub queryMasterTable {
   my($self)= @_;

   my @timeranges;
   #--- separate time ranges
   @timeranges=$self->getTimeRanges();

   my(%args) = ();

   my $tableHits;
   my ($start_time, $end_time);
   my $positions = $self->{positions};

   print scalar(@$positions) . " positions::<br>" 
	if ($positions && $self->{debug});

   if ($positions && scalar(@$positions)) {

      my $allpositionhits;
      print "****New allposition<br>" if $self->{debug};

      #--- define search radius for query
      $args{search_radius} = $self->{search_radius} 
	if defined $self->{search_radius};

      foreach my $target (@$positions) {
      


         my $positionhits;

         #--- new position
         my ($ra_target,$dec_target,$search_radius,$entry,$resolver)= 
	   split('\|',$target);


         print "----Looking at time::" . $self->{timerange} .
	   " target::$ra_target, $dec_target radius=$$self{search_radius}<br>" 
		if $self->{debug};
 
         #--- These args apply to time range and non-time range searches
         $args{ra_target}     = $ra_target;  #if defined $self->{ra_target};
         $args{dec_target}    = $dec_target; #if defined $self->{dec_target};
         $args{counts}    = 1;                #return counts not ids
         Browse::Utils::DumpRef(\%args) if $self->{debug};
         #-------------------------------------------------------------
         #--- data products time tests
         my $start_bench;
         if ($self->{benchmark}) {
            $start_bench = new Benchmark;
            print "start Benchmark:: $target <br>" ;
         }

         
         if (!$self->{timerange} ) {
            #--- No time range specified ----

            #Browse::Utils::DumpRef(\%args) if $self->{debug};
            $tableHits= query_master_table(%args);

            #Combine with previous queries
            print "Query Master Table::position: merging...<br>" 
		if $self->{debug};

            $self->mergeHits($tableHits);

         } else {

            #--- Time range has been specified ----

            ##--- do master table query for each time range, filter 
            ##--- for duplicates
            foreach my $time (@timeranges) {
      
               ($start_time, $end_time) = split(/\|/,$time);
               print " Time and position time=$start_time $end_time <br>" 
		   if $self->{debug};
      
               # Assemble the argument hash.
               $args{start_time} = $start_time if defined($start_time);
               $args{end_time} = $end_time if defined($end_time);

               print "sending args: <br>" if $self->{debug};
               Browse::Utils::DumpRef(\%args) if $self->{debug};

               $tableHits= query_master_table(%args);

               print "Query Master Table::time range: merging...<br>" 
			if $self->{debug};
               my $timecheck=1;
		  $self->mergeHits($tableHits,$timecheck);
           }
        }
         
        print "Query Master Table::all hits: merging...<br>" 
	   if $self->{debug};
        
        #--- debugging
        if ($self->{debug}) {
           print "All positions so far:<br>";
           foreach my $tablename (keys(%$allpositionhits)) {
              print "$tablename  " . 
		scalar(@{$allpositionhits->{$tablename}}) . "<br>" ;
           }
           print "---<br>"; 
        }
        #-------------------------------------------------------------
        #--- data products time tests
        if ($self->{benchmark}) {
           my $end_bench = new Benchmark;   # benchmark
           my $difference = timediff($end_bench, $start_bench); #benchmark
           print "<-------------->", timestr($difference), "<-------><br>" ;
       }
      }

      print "saving final hits...<br>" if $self->{debug};
      $self->{finalHits}=$allpositionhits;

   } else {
      #--- Time range only (no objects or coordinates entered)

      my $hits;
      #--- separate time ranges
      
      foreach my $time (@timeranges) {
         print "time only time=$time <br>" if $self->{debug};
         #-------------------------------------------------------------
         #--- data products time tests
         my $start_bench;
         if ($self->{benchmark}) {
            my $start_bench = new Benchmark;
            print "start Benchmark:TIME RANGE ONLY:: $time <br>" ;
         }

         ($start_time, $end_time) = split(/\|/,$time);

         # Assemble the argument hash.
         $args{start_time} = $start_time if defined($start_time);
         $args{end_time} = $end_time if defined($end_time);
         $args{counts} = 1;

         print "sending args: <br>" if $self->{debug};
         Browse::Utils::DumpRef(\%args) if $self->{debug};
         $tableHits= query_master_table(%args);
         print "merging time only <br>" if $self->{debug};

         my $timecheck=1;
         $self->mergeHits($tableHits,$timecheck);
     
         #-------------------------------------------------------------
         #--- data products time tests
         if ($self->{benchmark}) {
           my $end_bench = new Benchmark;   # benchmark
           print "end=$end_bench  <br>" if $self->{debug};
           my $difference = timediff($end_bench, $start_bench); #benchmark
           print "<br> <------------->", timestr($difference), "<-------><br>" ;
         }
       }
   }  #-- end time range only query
         
   #--- Convert unique id list to counts list before returning

}  #--- end queryMasterTable

##-------------------------------------------------------------------------
sub getTimeRanges {
    my ($self) =@_;
    my $timeEntry=$self->{timerange};
    print " getTimeRanges: time=$timeEntry <br>" if $self->{debug};
    my $tfield  = "_metaparam_start_time";
    my $tefield = "_metaparam_end_time";
    
    my @timeranges;
    if ($timeEntry =~ /\d/) {
       my (@timesets) = split(/\s*;\s*/,$timeEntry);
       my $timeset;
       foreach $timeset (@timesets) {
          $timeset =~ s/^\s*//g;
          my ($tmin,$tmax) = split(/\s*\.\.\s*/,$timeset);
          $tmin = DateTime::iso2mjd($tmin) if 
	      (defined $tmin and $tmin !~ /\d{5}\.?\d?/);
          $tmax = DateTime::iso2mjd($tmax) if
	      (defined $tmax and $tmax !~ /\d{5}\.?\d?/);
          if (($tmin =~ /error/i) || ($tmax =~ /error/i)) {
              Browse::Utils::Error("Error: Date values must be entered in " .
		     "either the ISO " .
                     "standard format (YYYY-MM-DD hh:mm:ss) or in Modified " .
                     "Julian Date format (integer or real number). Please go " .
                     "back and enter an appropriate date value for Time.");
          }
          if (!defined $tmax) { # make +/- 1 day
             $tmax = $tmin + 1;
             $tmin -= 1;
          }
          push @{$self->{qualifiers}}, "$tfield|<=|$tmax";
          push @{$self->{qualifiers}}, "$tefield|>=|$tmin";


          push @timeranges, $tmin . '|' . $tmax;
        }
    }
    return @timeranges;
} # ----- end getTimeRanges --

#-------------------------------------------------------------------------
#--- look for duplicates from previous time or object name searches
#-------------------------------------------------------------------------
sub mergeHits {
   my($self, $currentHits, $timecheck) =@_;
   
   my $dupcount=0;
   my $finalHits;
   foreach my $tablename (keys(%$currentHits)) {

      if (!$self->{allpositions}{$tablename}) {

            #--- table does not exist yet so add it 

            $self->{allpositions}{$tablename} = $currentHits->{$tablename};
      } else {
            ##--- hits for this table have already been found so add 
            ##--- current hit counts
            print "Table exists adding  to $tablename..." . 
		$self->{allpositions}{$tablename} ."<br>"  if $self->{debug};

            #my $tableEntries=$currentHits->{$tablename};
            $self->{allpositions}{$tablename} += $currentHits->{$tablename};
            print "Totals for  $tablename " .  
		$self->{allpositions}{$tablename} ."<br>"  if $self->{debug};
               
      }
   }
}

#-------------------------------------------------------------------------
#-- No longer used
sub finalCounts {
   my ($self)= @_;
   my %allHits;
   my $hits=$self->{finalHits};

   foreach my $tablename (keys %$hits) {
      print "Wrapping it up" .$tablename . " " . 
	scalar(@{$hits->{$tablename}}) . "<br>" if $self->{debug};

      $allHits{$tablename} = scalar(@{$hits->{$tablename}});
   }
   return \%allHits;

}

#-------------------------------------------------------------------------
###########
###  EOF  #
#########
1;

#$Log: DisplayTableHits.pm,v $
#Revision 1.11  2009/04/17 19:46:23  lmm
#Removed check for duplicate rows to improve query speed.
#
#Revision 1.10  2008/01/30 14:54:58  lmm
#Made maximum table row limit uneditable
#Fixed mission checkbox by removing HTML encoding
#Fixed errors caused by tables associated with multiple missions
#
#Revision 1.9  2008/01/23 18:13:16  lmm
#Removed a lot of the position/source cleanup now that MakeConeQuery
#  is used.
#
#Revision 1.8  2007/10/09 14:19:06  lmm
#Now calling MakeConeQuery() as a Parameter class method.
#
#Revision 1.7  2006/12/13 19:11:49  lmm
#Added message to Browse query log file
#
#Revision 1.6  2006/11/29 16:06:13  lmm
#Fixed two bugs involving format of user coordinates and a change to
#  radius value before Browse query is processed.
#
#Revision 1.5  2005/07/06 17:48:34  lmm
#Missing '>' at end of input tags were causing big problems for Requery
#
#Revision 1.4  2005/03/28 14:20:25  lmm
#Replaced spaces with underscores for mission help link
#
#Revision 1.3  2005/03/22 14:16:32  lmm
#Corrected link for mission help page
#
#Revision 1.2  2005/02/23 19:11:59  lmm
#Added coordinates and equinox to link URL associated with hits number
#
#Revision 1.1  2005/02/22 19:05:42  lmm
#Displays summary of tables and rows that satisfy user's criteria
#
