use strict;

package Browse::FitsDisplay;

use vars qw( @ISA );
@ISA = qw( Browse::TextDisplay );
use Browse::TextDisplay;

# Print just the HTTP header and ensure
# that positions and times are output as simple numbers.

sub pageHeader {
    my ($self) = @_;

    $self->setCoordFormat("decimal");
    $self->setDateFormat("mjd");

    #print "Content-Type: text/html\n\n" if $::debug;  #--debug
    print "Content-Type: image/x-fits\n" .
	"Content-Disposition: attachment; filename=browse_results.fits\n\n";
}

# No table header.
sub tableHeader {
}

sub legend { }

# No footer.
sub pageFooter {
}

# Handle closing the table.  This
# is when we actually do our output.

sub closeTable {

    my ($self) = @_;
    
    if ($$self{"intable"}) {
	$$self{"intable"} = 0;
	
	my $array = $$self{"dataarray"};
	my $meta  = $$self{"meta"};
	my $names = $$self{"names"};
	my $types = $$self{"types"};
        my $paraminfo = $$meta{"paraminfo"};
#print "types<br>" if $::debug;
#Browse::Utils::DumpRef($types) if $::debug;
	
	my $nskip = 0;
	while ($nskip <= $#$names) {
	    if (substr($$names[$nskip], 0, 3) !~ /w3_/i) {
	        last;
	    }
	    $nskip += 1;
	}
	
	# Print out an initial dummy FITS HDU if needed.
	if (!$$self{firstTable}) {
	    $self->primaryArray();
	}
        
        my $coordsys  = $$self{coordsys};
        my $tableinfo= $$meta{tableinfo}; 
        if ($coordsys !~ /equatorial/i) {
            my $ra = $$tableinfo{"right_ascension"};
            $ra =~ s/^\@//;
            my $dec = $$tableinfo{"declination"};
            $dec =~ s/^\@//;
            #my ($oras, $odecs) = $self->otherRaDecs($tableinfo);
            my ($oras, $odecs) = Browse::Display::otherRaDecs($tableinfo);

            my $target_name = $$tableinfo{"target_name"};
            $target_name =~ s/^\@//;
            my $names = $$self{'names'};

            foreach (@$names) {
               #--- prepare column header and format for other ra and decs
               #--- Can't use Display::renameRaDecs because of paraminfo 
               #--- assignment
               if (scalar(@$oras) && scalar(@$odecs)) {
                  my @check_other_ras = grep /^$_$/, @$oras;
                  my @check_other_decs = grep /^$_$/, @$odecs;
                  if ($coordsys =~ /galactic/i ) {
                     if (scalar(@check_other_ras)) {
                        $_ =~ s/_ra$/_lii/;
                        $_ =~ s/^ra_/lii_/;
                        $$paraminfo{$_}=$$paraminfo{$check_other_ras[0]};
                     }
                     if (scalar(@check_other_decs)) {
                        $_ =~ s/_dec$/_bii/;
                        $_ =~ s/^dec_/bii_/;
                        $$paraminfo{$_}=$$paraminfo{$check_other_decs[0]};
                     }
                  } elsif ($coordsys =~ /ecliptic/i) {
                     if (scalar(@check_other_ras)) {
                        $_ =~ s/_ra$/_elon/;
                        $_ =~ s/^ra_/elon_/;
                        $$paraminfo{$_}=$$paraminfo{$check_other_ras[0]};
                     }
                     if (scalar(@check_other_decs)) {
                        $_ =~ s/_dec$/_elat/;
                        $_ =~ s/^dec_/elat_/;
                        $$paraminfo{$_}=$$paraminfo{$check_other_decs[0]};
                     }
                  }

               }

                if ($_ eq $ra) {
                    if ($coordsys =~ /galactic/i) {
                        $_ = "_Lii";
                    } elsif ($coordsys =~ /ecliptic/i) {
                        $_ = "_Elon";
                    }
                    $$paraminfo{$_}=$$paraminfo{$ra};
                } elsif ($_ eq $dec) {
                    if ($coordsys =~ /galactic/i) {
                        $_ = "_Bii";
                    } elsif ($coordsys =~ /ecliptic/i) {
                        $_ = "_Elat";
                    }
                    $$paraminfo{$_}=$$paraminfo{$dec};
                }
            }
        }

	
	# Make all of the columns line up nicely.
	Browse::TableFormat::neatenTable($array, $names, $types, undef);
	
	# Print the header for this table.
	$self->printCard("XTENSION", "TABLE   ", "ASCII table extension");
	$self->printCard("BITPIX", 8, "Mandatory value");
	$self->printCard("NAXIS", 2, "Mandatory value");
	
	# Get rid of the unique key columns.
	splice(@$names, 0, $nskip);
	splice(@$types, 0, $nskip);
	
	my $size;
	my @r0 = @{$$array[0]};
	splice(@r0, 0, $nskip);
	
	foreach (@r0) {
	    $size += length($_) + 1;
	}
	
	$self->printCard("NAXIS1", $size, "Length of row");
	$self->printCard("NAXIS2", ($#$array +1), "Number of rows");
	
	$self->printCard("PCOUNT", 0, "Mandatory value");
	$self->printCard("GCOUNT", 1, "Mandatory value");
	
	$self->printCard("TFIELDS", ($#r0 + 1), "Number of fields");
	$self->printCard();
	
	my $tab  = $$meta{"tableinfo"}; 
	
	$self->printCard("EXTNAME", $self->chopTable($$tab{table_name}), "Extension name");
	$self->printCard("TABLE", $$tab{"table_name"}, "Table designator");
	$self->printCard("TAB_DESC", $$tab{"table_description"});
	$self->printCard("TAB_URL", $$tab{"table_document_url"});
	
	$self->printCard();
	
        my $i;
	
	my $ccount = 9;
	my $offset = 0;

	
	# Loop over each column
	
        my @revised;  #-- place to hold names after they are revised for header

	for ($i=0; $i <= $#$names; $i += 1) {
	
	    my $name = $$names[$i];
	    my $type = $$types[$i];
	    
	    my $len = length ($r0[$i]);
	    my $form;
            #-- remove initail and trailing spaces
            $name =~ s/^\s+|\s+$//g;
            my $format =$$paraminfo{$name}{"parameter_format"};
            my $unit=$$paraminfo{$name}{"parameter_unit"};
            print "<br>---FD:: ro=$r0[$i] type=$type name=$name " .
		"format=$format  unit=$unit len=$len<p>" if $::debug;

            #--- Move any leading underscores to end of string
            $name =~ s/^(_+)(.*)/$2$1/;
	    #$self->printCard("after", $name);

            #--- Upper case for header display 
            my $ucname=uc($name);

            my $same=0;    #-- count for duplicate TTYPE keyword search
            my $safetycount=0;  #-- used to prevent infinite loop
            if (scalar(@revised)) { 
               #--- Look for revised keyword in list of those already revised
               #--- If found add a _ to the end and search again until there
               #--- are no duplicates 
               while ($same=grep /^$ucname$/, @revised) { 
                   $ucname .= "_";
                   #--- just a precaution in case an infinite loop occurs
                   ++$safetycount;
                   if ($safetycount > scalar(@revised)) {
                      last;
                   }
               }
            }
            #--- Save revised keyword for next check
            push (@revised,$ucname);

	    $self->printCard("TTYPE".($i+1), $ucname);
	    
            #--- type == 6 && format ='' is for _search_offset 
            #--- which is displayed as character string
	    if ($type <= 1  || $name eq "class" 
                            || ($type == 6 && $format eq '')) {
	        $form= "A$len";
	    } elsif ($type eq 6 or $type eq 7) {
   	             $form = "F$len.0";
	    } else {
	        $form = "I$len";
	    }

            #--- expand format out to eight characters
            if (length($form) < 8) {
	       $form .= " " x (8-length($form)) ;
	    } 

	    $self->printCard("TFORM".($i+1),$form);
	    $self->printCard("TUNIT".($i+1),$unit) if $unit =~ /\w/;
            print "TFORM format = $format form=$form <br>" if $::debug;
            if ((my $acc) = $format =~ /float\d*:\d*\.(\d*)f/) { 
                     my $disp = "F$len.$acc"; 
                     
	             $self->printCard("TDISP".($i+1),$disp);
            }
	    $self->printCard("TBCOL".($i+1),($offset+1));
	    $ccount += 3;
	    
	    $offset += $len + 1;
	}
	$self->printCard("END");
	
	$self->flushBlock(" ");
	
	my $row;
	
	# Loop over each row in the ouput.
	foreach $row (@$array) {
	    # Remember to get rid of the unique key data.
	    splice(@$row, 0, $nskip);
	    $self->output(join(" ", @$row)."\n");
	}
	$self->flushBlock(" ");
    }
}

sub primaryArray {
    my ($self) = @_;
    
    $self->printCard("SIMPLE", "T", "Browse (FitsDisplay V0.1)");
    $self->printCard("BITPIX", "8");
    $self->printCard("NAXIS", 0, "Dummy HDU");
    $self->printCard("EXTEND", "T", "Extensions allowed");
    $self->printCard("END");
    $self->flushBlock(" ");
    $$self{firstTable} = 1;
}

# Print a FITS header card.
sub printCard {

    my ($self, $key, $value, $comment) = @_;
    
    my $line = " " x 80;
    
    # Just print out a blank if there is no key.
    
    if (defined($key)) {
    
        # Make sure the key takes up exactly 8 spaces.
        if (length($key) < 8) {
	    substr($line, 0, length($key)) = $key;
	    $key .= " " x (8-length($key)) ;
	} elsif (length($key) > 8) {
	    substr($line, 0, 8) = substr($key, 0, 8);
	} else {
            substr($line, 0, 8) = $key;
	}

        # Is there a value?
	# Note that we don't support COMMENT style cards in this routine.
	
        if (defined($value)) {
	
	    substr($line, 8, 1) = '=';
	    my $len;
	
	    # We assume that these two strings are meant to be booleans.
            if ($value eq 'T' || $value eq 'F') {
                substr($line, 29, 1) = $value;
		$len = 20;
	
	    # Probably a better way of checking for a numeric value than adding 0
	    # but I don't know it offhand.
            } elsif ( ($value + 0) eq $value) { 
                $len = length($value);
	        substr($line, 30-$len, $len) = $value;
		$len = 20;
	
	    # Anything else is a string value.
            } else {
                $len = length($value);
		 
		# Can't run off the card.
		if ($len > 68) {
		    $value = substr($value, 0, 68);
		    $len = 68;
		}
                substr($line, 10, $len+2) = "'$value'";
		$len += 2;
            }
            
	    # Comment begins no earlier than column 32 and later
	    # if a long string is present.
            if (defined($comment)) {
	        $len += 10;
		if ($len < 30) {
		    $len = 30;
		}
		
		if ($len + 3 < 80) {
                    substr($line, $len+1, 1) = '/';
	            substr($line, $len+3, length($comment)) = $comment;
		}
	    }
	}
    }
    
    # Make sure we didn't grow the line.
    $line = substr($line, 0, 80);
    $self->output($line);
}


sub printComment {
    my ($self, $text) = @_;
    
    my $line = "COMMENT    $text";
    my $len = length($line);
    if ($len > 80) {
        $line = substr($line, 0, 80);
    } elsif ($len < 80) {
        $line .= " " x (80-$len);
    }
    
    $self->output($line);
}
   

# Output data and keep track of how long the data is.
sub output {

    my ($self, $data) = @_;
    
    print uc($data);
    $$self{"outputSize"} += length($data);
}

# Finish up the current 2880 byte block using
# the specified character for fill (space for
# headers and ASCII tables, \000 for everything
# else.
sub flushBlock {

    my ($self, $char) = @_;
    
    my $size = $$self{"outputSize"};
    if ($size % 2880) {
        $self->output( substr($char, 0, 1) x (2880 - $size%2880));
    }
}

sub showEmpties {

    my ($self, $empties, $errors) = @_;
    if ($#$empties >= 0 || $#$errors >= 0) {
    
        if (!$$self{firstTable}) {
            $self->primaryArray();
	}
	
	$self->printCard("XTENSION", "IMAGE   ", "Tables not returning data");
	$self->printCard("BITPIX", 8);
	$self->printCard("NAXIS", 0);
	$self->printCard("PCOUNT", 0);
	$self->printCard("GCOUNT", 1);
	$self->printCard();
	
	my $line;
	foreach $line(@$empties) {
	    $self->printComment("$$line[0] : No matching rows");
	}
	foreach $line(@$errors) {
	    $self->printComment("$$line[1] : Error in query (cone search not supported?)");
	}
	$self->printCard("END");
	$self->flushBlock(" ");
    }
}

sub resultsSummary {}
sub YourSearchWas {}
sub beforeNewTable {}
sub afterTable {}
## Successful load
1;
