# $Id: HDB.pm,v 1.48 2008/04/07 16:44:57 lmm Exp $
# $Log: HDB.pm,v $
# Revision 1.48  2008/04/07 16:44:57  lmm
# ZZPAR query now includes parameter_ucd (change by tam).
#
# Revision 1.47  2007/10/16 19:04:50  lmm
# Another tweak of the SQL statement so it includes only selected table
#   types.
#
# Revision 1.46  2007/10/11 13:07:51  lmm
# Corrected sql statement that was omitting tables that do not have a
#   default_search_radius and possibly other parameters. fix provided
#   by ed sabol.
#
# Revision 1.45  2007/08/17 18:40:02  lmm
# Reduced the size of a huge SQL query using CASE statements (done in
#   conjuction with dbms upgrade)
#
# Revision 1.44  2006/08/24 14:38:05  lmm
# Changes to the order of parameters on the Search By Parameter page.
#   Changes supplied by Tom McGlynn.
#
# Revision 1.43  2006/06/23 14:49:52  lmm
# Saves all missions that a table is associated with (now necessary
#   for Tabbed Display)
#
# Revision 1.42  2006/01/25 19:10:37  lmm
# Moved class select table sort to w3query.pl (new sort is by name)
#
# Revision 1.41  2004/07/01 14:25:58  lmm
# Added persistent table information hash to avoid repeated equivalent
#   queries
#
# Revision 1.40  2003/12/02 19:47:43  lmm
# cosmetic change only
#
# Revision 1.39  2003/07/23 20:41:44  cwh
# Changed olegacy to heasarcdev.
#
# Revision 1.38  2002/12/17 18:47:09  cwh
# Fixed problem where tables were flagged as being available to cross-correlate
#   by radius even though they had no default_search_radius.
#
# Revision 1.37  2002/12/10 21:35:33  cwh
# Added seen array to ensure that xcorr tables are only seen once.
#
# Revision 1.36  2002/09/13 21:03:10  cwh
# Get ZZBIB info and save it in table info
# Get mission name for xcorr.
#
# Revision 1.35  2002/04/10 22:00:06  cwh
# Fixed bug where priority was set to 5 if priority =0.
#
# Revision 1.34  2001/12/05 23:16:25  cwh
# Added Display_tables_list(), which will print a list of all tables available
# to be queried.  This is for the batch users who enter a nonexistent table
# name, or who want to get a list of tables to query.
#
# Revision 1.33  2001/11/20 15:49:57  cwh
# Lowercasing metadata column names. (Oracle returns them in uppercase.)
#
# Revision 1.32  2001/11/07 23:34:18  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.31  2001/06/20 22:57:32  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.30  2001/06/18 19:07:56  cwh
# Added call to Vizier::tableInfo in get_table_meta, called from w3table.pl for
#    single tables.
#
# Revision 1.29  2001/06/04 15:06:54  cwh
# When getting the list of tables in a mission, now only gets those with
# w3browse_display != 'N'.
#
# Revision 1.28  2001/05/17 02:31:29  tam
# Deleted old comments and unused functions.
#
# Revision 1.27  2001/04/25 17:42:33  cwh
# changes Tom made: removed SQL(), made changes for VizieR tables.
#
# Revision 1.26  2001/04/09 20:02:03  cwh
# Ed's fix - calls getKey to get unique key.
#
# Revision 1.25  2001/04/03 16:17:28  cwh
# - added skip of bogus table '1254'.
# - added subroutine get_table_meta to add mission-type metadata to a table
#
# Revision 1.24  2001/03/12 16:31:21  cwh
# No longer dies when no tables found for missions.  There may be tables in
# addition to missions.
#
# Added priority to metadata that is saved for tables.  This can now be printed
# by w3table.pl in debug mode.
#
# Revision 1.23  2001/01/18 22:00:15  cwh
# Changes Tom made: First uses query_key as the unique key, if it exists.  If
#   query_key doesn't exist, then uses unique_key.
#
# Revision 1.22  2001/01/02 23:18:17  cwh
# Now gets tabletypes.
#
# Added table types as criteria for searches.
#
# Previous comments deleted 2001-05-16
#
# Initial revision
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=======================================================================#
# Package Name:   HDB.pm
# Package Authors:Edward J. Sabol, Hughes STX; Saima Zobair (RITSS)
# Date:           03/96
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
#   This module contains functions for invoking HDBmc and parsing the
#   results into useful data structures. Also included are functions
#   for invoking HDBrect and HDBfilter.
#=======================================================================#

use strict;
use Browse::TableFormat;
use Browse::Vizier;

package Browse::HDB;
#------------------------------------------------------------------------------
#
# This package references the following external variables and subroutines:
#
#   Variables:
#     $::debug:	indicates whether to print debugging information or not (1 or 0)
#     $::hbin:	directory where the HEASARC clients and utilities software are
#		  located
#     $::hdbmc:	name of the HEASARC database client executable to use for queries
#     $::addr:	string containing comma-separated list of e-mail address to
#		  notify if there is a problem with the database or the HEASARC
#		  database client/server
#
#   Subroutines:
#     &::Error:	Subroutine that takes an error message as an argument and
#		  displays it to the user or does whatever you want when an
#		  error occurs.
#     &::MailMessage: Subroutine that e-mails a message to a comma-separated
#		  list of e-mail addresses
#     &::DumpAssoc: Subroutine that prints the key/value pairs of a hash.
#
#------------------------------------------------------------------------------
sub Observatories {

    my ($res) = Browse::DBQ::query("select distinct parameter_value from zzext where parameter_name = 'observatory_name' order by parameter_value");

    if ($#$res < 0) {
        &HDBerror("Error locating observatories");
    }

    my @obs;
    my $line;
    foreach $line(@$res) {
        push(@obs, $$line[0]);
    }
    return(@obs);
} #eosub--HDB::Observatories
#------------------------------------------------------------------------------
sub TableInfo {

    # Function  input: name of table for which you want the metabase info
    # Function output: name/value associative array of table info
    my($tablename) = @_;
    my($table);
    my($alltable_info) = {};
    my(@tables) = split(',', $tablename);
    my($multi) = (scalar(@tables) > 1) ? 1 : 0;

    my($tablestring);
    
    my $comma = "";

    if ($#tables > 0) {
        foreach $table (@tables){
            $$alltable_info{$table} = singleTableInfo($table);
        }
        return $alltable_info;
    } else {
        my $tinfo = singleTableInfo($tablename);
        return %$tinfo;
    }
}

sub singleTableInfo {

    my ($table) = @_;

    if ($HDB::tableSave{$table}) {
        return $HDB::tableSave{$table};
    }

    if ($table =~ /\//) {
        return Browse::Vizier::tableInfo($table);
    }
    #print "HDB:: table=$table <br>" if $::debug;

    # Get ZZGEN information (we may be table_name twice but this make
    # we have cleaner code).
    
    #print "HDB:: select table_name,table_location,".
                           #"table_description,table_document_url,create_date,".
                           #"modify_date,table_rows from zzgen where ".
                           #"table_name ='$table'<br>" if $::debug;

    my ($results, $names) = Browse::DBQ::query("select table_name," .
			   "table_location,".
                           "table_description,table_document_url,create_date,".
                           "modify_date,table_rows from zzgen where ".
                           "table_name ='$table'");
    if ($#$names <= 0) {
            &HDBerror("No table info found for $table");
	    exit;
    }
   
    my $line = $$results[0];
    
    # Store ZZGEN info in structure.

    my $table_info = {};

    my $i;
    for ($i=0; $i <= $#$names; $i += 1) {
        $$names[$i] = lc($$names[$i]); # for ISDC
        if ($::server =~ /heasarcdev/i  and $$names[$i] eq "table_document_url") {
            $$line[$i] =~ s/heasarc/heasarcdev/;
        }
        $$table_info{$$names[$i]} = $$line[$i];
    }

    # Now add in anything from ZZEXT...  Suppose we could have
    # problems if there's a table in ZZEXT not in ZZGEN.

    ($results) = Browse::DBQ::query("select table_name,parameter_name," .
		"parameter_value from zzext where table_name = '$table'");

    foreach $line(@$results) {
        $$line[2] =~ s/^\@//;   # Get rid of column indicators.
        #T#$$table_info{$$line[1]} = $$line[2];
        #--- If table belongs to more than 1 mission, store all mission 
        #--- names.  added for TABBED interface  lmm:05/06
        if ($$table_info{$$line[1]}) {
           $$table_info{$$line[1]} .= '||' . $$line[2];
        } else {
           $$table_info{$$line[1]} = $$line[2];
        }
    }

    if ($$table_info{"query_key"}) {
        $$table_info{"unique_key"} = $$table_info{"query_key"};
    }

    # Now see if table is in ZZBIB
    my ($bibflag) = Browse::DBQ::query("select bibcode, id_fields, id ".
                    "from zzbib where table_name = '$table'");
    if ($$bibflag[0]) {
       my %bibcodes;
       foreach my $bib(@$bibflag) {
          next unless ($$bib[0] and $$bib[1] and $$bib[2]);
          $$table_info{"bibkeys"} = $$bib[1];
          # handle cases of multiple id_fields
          my @ukeys = split(',',$$bib[1]);
          my @ukeyvals = split(',',$$bib[2]);
          my $ukeystring='';
          foreach my $ukey (@ukeys) {
              my $sep='|' if length($ukeystring);
              $ukeystring .= $sep . $ukey."=".shift(@ukeyvals);
          }
          $bibcodes{$ukeystring} .= Browse::CGI::Encode($$bib[0])." ";
       }
    $$table_info{"bibcodes"} = \%bibcodes;
    }

    $HDB::tableSave{$table} = $table_info;

    return $table_info;    

} #eosub--HDB::TableInfo
#------------------------------------------------------------------------------
sub GetUnique {
    # Function input :  table info
    # Function output:  string containing unique keys 
 
    my($table_info, $key_datatype, $prefix, $alias) = @_;
    if (!defined($alias)) {
        $alias = "w3_";
    }
    
    #my($unique) = (defined($table_info->{'unique_key'}))
    #              ? $table_info->{'unique_key'}
    #              : "";
    my $unique = Browse::DPL::getKey($table_info->{'table_name'});

    if (! $unique){
      my($msg) = "Unique key(s) not defined for " .
		  "$table_info->{'table_name'}.\n\n" .
                  "** Please investigate immediately! **\n\n";
      Browse::MailMessage::MailMessage($msg,$::addr);
      Browse::Utils::Error("Database temporarily unavailable. Please " .
		  "try again later.");
    }

    my(@key_datatype) = split (',',$key_datatype);
    if (! scalar(@key_datatype) > 0){
       push(@key_datatype,$key_datatype);
    } 

    $unique = "";
    foreach (@key_datatype) {
     my($name, $type) = split('\:',$_);
     if ($type =~ /float/){
       $unique .= "convert(varchar,$prefix$name) as $alias$name,"; 
     } else {
       $unique .= "$prefix$name as $alias$name,"; 
     } 
    }

    print "<br>HDB::GetUnique Unique keys SQL line: $unique \n" if ($::debug);
    return($unique); 
} #eosub--HDB::GetUnique
#------------------------------------------------------------------------------
sub ParamInfo {
    # Function  input: name of table for which you want the metabase info
    #                    Note: Table name can actually be a list of tables
    #                    separated by commas.
    # Function output: reference to name/value hash of parameter info

    my($table_name) = shift(@_);

    my($parameters) = {};
    my(@output) = ();
    my($all_params) = {};
    my($default_params) = {};
    my($not_default_params) = {};
    my($mjd_params) = {};

    my(@tables) = split(",", $table_name);
    
    my %parameterHash;
    my $table;
    foreach $table(@tables) {
        if ($table =~ /\//) {
            $parameterHash{$table} = Browse::Vizier::paramInfo($table);
        } else {
            $parameterHash{$table} = singleParamInfo($table);
        }
    }
    return \%parameterHash;
}

sub singleParamInfo {

    my ($table) = @_;

    my($results, $names) = Browse::DBQ::query("select table_name," .
			      "parameter_name,parameter_description," .
			      "parameter_format,parameter_unit,".
                              "parameter_is_index,parameter_minval," .
			      "parameter_maxval,parameter_comment," .
			      "parameter_default,parameter_ucd " .
			      "from zzpar where ".
                              "table_name = '$table' order by ".
                              "parameter_default + 1/(parameter_default+.0001)");

    if ($#$names < 0) {
        &HDBerror("No parameter info found for $table");
    }
    my $line;
    my $parameters = {};

    $$parameters{"#all#"} = [];
    $$parameters{"#default#"} = [];
    $$parameters{"#not_default#"} = [];
    $$parameters{"#mjd#"} = [];

    foreach $line(@$results) {

        my $parameter_name = $$line[1];
        #print "HDB name=$parameter_name<br>" if $::debug;

        my $i;
        for($i=0; $i <= $#$names; $i += 1) {
            my $field = lc($$names[$i]); # for ISDC
            $$parameters{$parameter_name}{$field} = $$line[$i];

            if ($field eq 'parameter_default') {

                #--- new code supplied by tam
                if ($$line[$i] == 0) {
                    push(@{$$parameters{"#not_default#"}},
                            $parameter_name);
                } else {
                    push(@{$$parameters{"#default#"}},
                            $parameter_name);
                }
                #push(@{$$parameters{"#all#"}},$parameter_name);
                #if ($$line[$i] eq '0') {
                    #push(@{$$parameters{"#not_default#"}},
                            #$parameter_name);
                #} else {
                    #push(@{$$parameters{"#default#"}},
                           ##$parameter_name);
                #}

            } elsif(($field eq "parameter_unit")&&($$line[$i] eq "mjd")) {
               push(@{$$parameters{"#mjd#"}},$parameter_name);
            }
        }
    }
    # When listing all parameters in the table do one of two things.
    #
    # If the natural order of the parameters is such that the default parameters
    # come out in the order specified in the parameter_default field, then
    # use the natural order (which may have non-default parameters interleaved
    # with default parameters.
    # Otherwise have all of the default parameters followed by the
    # non-default parameters where the later are in the natural order.
    #
    # Get the natural order (An amusing where clause!)
    #--- new code supplied by tam
    ($results, $names) = Browse::DBQ::query("select * from $table where 1 = 0");
    
    my %ord;
    my $OK = 1;
    
    # Create a hash parameter_name -> position in natural order
    for (my $i=0; $i<@$names; $i += 1) {
        $ord{$$names[$i]} = $i;
    }
    
    my $dft = $$parameters{"#default#"};
    # For each default parameter check that if it comes before the next
    # default parameter in the natural order.
    for (my $i=0; $i<@$dft - 1; $i += 1) {
        my $a = $$dft[$i];
        my $b = $$dft[$i+1];
	if ($ord{$a} > $ord{$b}) {
	    $OK = 0;
	    last;
	}
    }
    
    if ($OK) {
        $$parameters{"#all#"} = $names;
    } else {
    
        # Some default parameter is out of order.
	# So sort the non-defaults according to the natural ordering.
        my $ndft = $$parameters{"#not_default#"};
	my @all;
	push(@all, @$dft);
	my @srt = sort { $ord{$a} <=> $ord{$b} } @$ndft;
	push(@all, @srt);
	
        $$parameters{"#all#"} = \@all;
    }
    return($parameters);
} #eosub--HDB::ParamInfo
#------------------------------------------------------------------------------
sub quoteList {
    # Put quotes into a comma separated list
    my ($list) = @_;
    $list =~ s/\s*,\s*/','/g;
    $list = "'$list'";
    return $list;
} #eosubs--HDB::quoteList
#------------------------------------------------------------------------------
sub GetParamList {
    # Function  input: name of table for which you want the list of parameters,
    #                    string determining whether you want the list of "all"
    #                    parameters or only "default" parameters
    # Function output: two strings containing comma-delimited lists of:
    #   1) the default (standard) set of parameters (in default order)
    #      with the other remaining parameters appended if "all" parameters are
    #      requested
    #   2) the list of parameters with MJD units

    my($table,$set) = @_;

    my($parameters) = Browse::HDB::ParamInfo($table);
    my($heasarc_table) = "\L$table\E";
    
    my $hash = $$parameters{$table};

    my(@params) = @{ $$hash{'#default#'} };
    
    @params = (@params,@{ $$hash{'#not_default#'} })
      if ($set =~ /(all)/i);
      
    my($param_list) = join(',',@params);
    my($plotparam)  = join(',',@{$$hash{'#default#'}}) . ',' . 
                      join(',',@{$$hash{'#not_default#'}});
    my($mjd_list) = join(',',@{ $$hash{'#mjd#'} });

    #get numeric parameters
    my($t,$numeric);
    foreach $t(split(',',$plotparam)) {
        my($p) = $parameters->{$table}{$t};
        if (($p->{'parameter_format'} =~ /float/) ||
            ($p->{'parameter_format'} =~ /int/)){
            $numeric .= $t . ",";
        }
    }
    print "HDB::GetParamList ($table, $set): returning " . scalar(@params) .
          " parameters: $param_list, Numeric: $numeric<br>\n" if $::debug;
    return($param_list,$mjd_list,$plotparam,$numeric);
} #eosub--HDB::GetParamList
#------------------------------------------------------------------------------
sub GetParamLists {
    # Function  input: array of tables for which you want the list of parameters,
    #                    string determining whether you want the list of "all"
    #                    parameters or only "default" parameters
    # Function output: hash with key/value ==> table name/string containing
    #                    comma-separated list of parameters

    my($set) = pop(@_);
    my(@tables) = @_;

    my($parameters) = Browse::HDB::ParamInfo(join(',',@tables));
    my(%params) = ();
    my($table);

    foreach $table (@tables) {
	if ($set =~ /(all|override)/i) {
	    $params{$table} = join(',',
		(@{ $parameters->{"\L$table\E"}{'#default#'} },
		@{ $parameters->{"\L$table\E"}{'#not_default#'} }));
	} else {
	    $params{$table} = join(',',
		@{ $parameters->{"\L$table\E"}{'#default#'} });
	}
	$params{"$table#mjd"} = join(',',
		@{ $parameters->{"\L$table\E"}{'#mjd#'} });
    }

    Browse::Utils::DumpAssoc(%params) if $::debug;
    return(%params);
} #eosub--HDB::GetParamLists
#------------------------------------------------------------------------------
sub TableObs {
    # Function  input: name of mission(s) (comma-separated) for which you want
    #                    the list of tables which belong to those mission(s)
    # Function output: array of table names
    my(@tables) = ();
    my(@output) = ();
    my($name,$value);
    my($missions) = uc(shift(@_));

    if ($missions =~ /^\s*([\-A-Z0-9_, ]+)/) {
	$missions = $1;
    } else {
	die "Invalid mission argument (\"$missions\") passed " .
		"to Browse::HDB::TableObs!\n";
    }

    my($results);
    if ($missions ne "") {
    
        my $xmission = quoteList($missions);
	#($results) =Browse::DBQ::query("select distinct table_name from zzext ".
	#                       " where parameter_name='observatory_name' and ".
	#	               " parameter_value in($xmission)");
	($results) =Browse::DBQ::query(
			       "select distinct a.table_name from zzext a".
	                       " where ".
                               " a.parameter_name='observatory_name' and ".
		               " a.parameter_value in($xmission)".
                               " and a.table_name not in (select ".
                               " table_name from zzext where ".
                               " parameter_name='w3browse_display' and".
                               " parameter_value='N')");
	if ($#$results < 0) {
	    &HDBerror("Unable to find tables for $missions");
	}  
    }

   my($line,@results); 
   foreach $line (@$results){
     push(@results,$$line[0]); 
   }
   return @results;
} #eosub--HDB::TableObs
#------------------------------------------------------------------------------
sub GetQueryTables {

    my($missions,$tabletypes) = @_;
    my($sql,@results,@missions,@tabletypes);
   
    if (ref($missions) eq 'ARRAY') {
	@missions = @{ $missions };
    } 
    if (ref($tabletypes) eq 'ARRAY') {
	@tabletypes = @{ $tabletypes };
        $tabletypes=0;
    }
    # *** Change '4' in next line if another type added!  This undefs 
    #     tabletypes if all tabletypes were selected
    #undef($tabletypes,@tabletypes) if scalar(@tabletypes) == 4;  
    $tabletypes = "'" . $tabletypes . "'" if $tabletypes;
    $tabletypes = "'" . join("','",@tabletypes) . "'" if @tabletypes;

    #print "HDB::GetQueryTables: tabletypes: $tabletypes\n<br>" if($::debug);
    #print "HDB::GetQueryTables: missions: @missions \n<br>" if($::debug);

    my(@my_missions) = ();
    foreach (@missions) {
       /^\s*([-a-zA-Z0-9_, ]+)/ && push(@my_missions,$1);
    }
    if (!scalar(@my_missions)) {
       warn ("Invalid mission(s) (\"" . join(',',@missions) .
   	     "\") passed to Browse::HDB::GetQueryTables!\n");
	    Browse::Utils::Error("Error: Invalid mission name(s).");
    }

    my($mission_list) = "'" . join("','",@my_missions) . "'";
    $mission_list =~ tr/a-z_/A-Z /;

    # Case syntax used for tables that may or may not have
    # data products, default_search_radius, and priority
    $sql = "select " .
       "a.table_description, " .
       "a.table_name, " .
       "a.table_document_url, " .
       "(case when a.table_name in  " .
       "(select distinct table_name from zzdpsets) then 'Y' " .
             "else 'N' end) as archive, " .
       "(case when a.table_name not in " .
       "(select distinct table_name from zzext where " .
             "parameter_name='default_search_radius') then '-1' " .
             "else c.parameter_value end) as search_rad, " .
       "b.parameter_value as mission, " .
       "e.parameter_value as tabletype, " .
       "(case when a.table_name not in " .
       "(select distinct table_name from zzext where  " .
             "parameter_name='table_priority') then '5' " .
             "else d.parameter_value end) as priority " .
       "from zzgen a, zzext b, zzext c, zzext d, zzext e " .
       "where a.table_name=b.table_name " .
       "and a.table_name*=c.table_name " .
       "and a.table_name*=d.table_name " .
       "and a.table_name=e.table_name " .
       "and b.parameter_name='observatory_name' " .
       "and b.parameter_value in ($mission_list) " .
       "and c.parameter_name='default_search_radius' " .
       "and d.parameter_name='table_priority' " .
       "and e.parameter_name='table_type' " .
       "and e.parameter_value in ($tabletypes) " .
       "and a.table_name not in (select distinct table_name from zzext where " .
              "parameter_name='w3browse_display' and parameter_value='N') ";

    #print "$sql<br>" if $::debug;

    my ($results) = Browse::DBQ::query($sql);

    if ($#$results < 0) {
        #print "HDB:results < 0 " . scalar(@tabletypes) . "<br>" if $::debug;
        if (scalar(@tabletypes) != 4) {
            #&HDBerror("No ". join(" or ",@tabletypes) .
            #           " tables found for missions: $mission_list");
         } else {
            #&HDBerror("Unable to locate queryable tables for missions: $mission_list");
         }
    
    }

    my($table_desc,$table_short,$table_archive,$table_radius,
       $table_mission,@columns);
       
    my($tablelist) = [];
    
    my $columns;
    foreach $columns (@$results) {
        #print "HDB:: columns dump<br>" if $::debug;
        #Browse::Utils::DumpRef($columns) if $::debug;

        my($tables) = {};
	$tables->{'description'} = shift(@$columns);
	$tables->{'name'}        = shift(@$columns);
	$tables->{'url'}         = shift(@$columns);
	$tables->{'archive'}     = shift(@$columns);
	$tables->{'radius'}      = shift(@$columns);
	$tables->{'mission'}     = shift(@$columns);
	$tables->{'tabletype'}   = shift(@$columns);
	$tables->{'priority'}    = shift(@$columns);
        $tables->{'url'} =~ s/heasarc/heasarcdev/ if($::server =~ /heasarcdev/);
	push(@ { $tablelist },$tables);
   } 
   
   return ($tablelist);
} #eosub--HDB::GetQueryTables
#------------------------------------------------------------------------------
sub GetCrossTables {

    my(@results,$sql);

    # table has a default_search_radius and no start time
    $sql = "select distinct a.table_name,a.table_description, b.parameter_value as default_search,' ' as start_time, c.parameter_value as mission ".
           "from  zzgen a, zzext b, zzext c ".
           "where (a.table_name = b.table_name) ".
           "and   (a.table_name = c.table_name) ".
           "and   (b.parameter_name = 'default_search_radius') ".
           "and   (a.table_name in (".
                  "select table_name from zzext where parameter_name='right_ascension')) ".
           "and   (a.table_name in (".
                  "select table_name from zzext where parameter_name='declination')) ".
           "and   (a.table_name not in (".
                  "select a.table_name from zzext a,zzpar b ".
                  "where a.table_name = b.table_name ".
                  "and a.parameter_name='start_time' ".
                  "and a.parameter_value = '\@' + b.parameter_name ".
                  "and b.parameter_unit = 'mjd')) ".
           "and   (c.parameter_name = 'observatory_name') ".
           "and   (a.table_name not in (".
                  "select table_name from zzext where parameter_name='w3browse_display' and parameter_value='N')) ";

    $sql .=" union ";

    # table has start_time and no search radius
    $sql .= "select distinct a.table_name,a.table_description, ' ' as default_search, b.parameter_value as start_time, d.parameter_value as mission ".
           "from  zzgen a, zzext b, zzpar c, zzext d ".
           "where (a.table_name = b.table_name) ".
           "and   (a.table_name = c.table_name) ".
           "and   (a.table_name = d.table_name) ".
           "and   (b.parameter_name = 'start_time') ".
           "and   (c.parameter_unit = 'mjd') ".
           "and   (b.parameter_value = '\@' + c.parameter_name) ".
           "and   (d.parameter_name = 'observatory_name') ".
           "and   (a.table_name not in (".
                  "select table_name from zzext where parameter_name='right_ascension')) ".
           "and   (a.table_name not in (".
                  "select table_name from zzext where parameter_name='declination')) ".
           "and   (a.table_name not in (".
                  "select table_name from zzext where parameter_name='w3browse_display' and parameter_value='N')) ";

    $sql .=" union ";

    # table has a default_search_radius and start_time
    $sql .="select distinct a.table_name, a.table_description, b.parameter_value as default_search, c.parameter_value as start_time, e.parameter_value as mission ".
           "from  zzgen a, zzext b, zzext c, zzpar d, zzext e ".
           "where (a.table_name = b.table_name) ".
           "and   (a.table_name = c.table_name) ".
           "and   (a.table_name = d.table_name) ".
           "and   (a.table_name = e.table_name) ".
           "and   (c.parameter_name = 'start_time') ".
           "and   (c.parameter_value = '\@' + d.parameter_name) ".
           "and   (d.parameter_unit = 'mjd') ".
           "and   (b.parameter_name = 'default_search_radius') ".
           "and   (e.parameter_name = 'observatory_name') ".
           "and   (a.table_name in (".
                  "select table_name from zzext where parameter_name='right_ascension')) ".
           "and   (a.table_name in (".
                  "select table_name from zzext where parameter_name='declination')) ".
           "and   (a.table_name not in (".
                  "select table_name from zzext where parameter_name='w3browse_display' and parameter_value='N')) "
           ;
#print "sql=$sql<br>\n";
    my ($results, $names, $types) = Browse::DBQ::query($sql);

    my($tables) = {};

    $tables->{'list'} = [];
    $tables->{'descriptions'} = {};

    #my($table_name,$table_description,$table_radius,$start,$mission,@columns);
    my(@columns);
    my %seen = ();
    my $line;
    foreach $line(@$results) {
      my ($table_name ,$table_description,$table_radius,$start,$mission) = @$line;
        push(@{ $tables->{'list'} },$table_name) unless $seen{$table_name}++;
        $tables->{'descriptions'}{$table_name} = $table_description;
        $tables->{'radii'}{$table_name} = $table_radius
          unless (($table_radius eq 'n/a') or ($table_radius =~/^\s+$/));
        $tables->{'start_time'}{$table_name} = $start;
        $tables->{'mission'}{$table_name} = $mission;
    }
    #(print "HDB::GetCrossTables: tables = (",
           #join(',',sort(@{ $tables->{'list'} })),")<br>\n") if $::debug;
#Browse::Utils::DumpRef($tables) if $::debug;
    return($tables);
} #eosub--HDB::GetCrossTables
#------------------------------------------------------------------------------
sub GetMJDParams {
    # Function  input: name of table
    # Function output: array of parameters that have MJD units

    my($tablename) = shift(@_);
    my(@mjdlist) = ();

    $tablename =~ tr/A-Z/a-z/;
    if ($tablename =~ /^\s*([a-z0-9_]+)/) {
	$tablename = $1;
    } else {
	die "Invalid tablename argument (\"$tablename\") passed to Browse::HDB::GetMJDParams!\n";
    }

    if ($tablename ne "") {

	# Get list of MJD fields for this table.
	my ($results) = Browse::DBQ::query("select distinct parameter_name from zzpar where table_name = '$tablename' and parameter_unit = 'mjd'");
        foreach (@$results) {
            push(@mjdlist, $_[0]);
        }
	#print "MJD parameters: ",join(',',@mjdlist),"<br>\n" if $::debug;
    }
    @mjdlist;
} #eosub--HDB::GetMJDParams
#------------------------------------------------------------------------------
sub GetClassinfo {
    # Function input : list of tables separated by ','
    # Function output: array with class_id and class_name separated by '|'
    my($tables,$classparam) = @_;

    my(@tables) = split ('\,',$tables);

    my($sql,@resarr);
    foreach (@tables) {
      $sql = "select distinct $classparam from $_ ";
      my($results) = Browse::DBQ::query($sql);
      foreach(@$results) {
        push(@resarr, $$_[0]) unless (Browse::Utils::memq($$_[0], @resarr));
      }
    }
    if (scalar(@resarr) < 0) {
        &HDBerror("Unable to query CLASS table");
    }
    #@resarr = sort @resarr;
    return(\@resarr); 

} #eosub--HDB::GetClassInfo
#------------------------------------------------------------------------------
sub HDBerror {
    my($msg) = @_;
    my $mailMsg = "Database anomaly detected: \n$msg\nDBI status: $DBI::errstr\n".
	      "    ** Please investigate immediately! ** process: $$\n\n";
    #Browse::MailMessage::MailMessage($mailMsg,$::addr);
    $msg = "Error: $msg<p>".
           "This may be an error in your specification or our database may be unavailable.";
    if (length($DBI::errstr) > 0) {
        $msg .= "<p>The database reports an error condition:";
	if ($DBI::errstr =~ /Server message number=(\d+) .* text=(.*)/i) {
	    $msg .= " Msg number $1: $2";
	} else {
	    $msg .= "<br>$DBI::errstr";
	}
    } else {
        $msg .= "<p>No error status reported from the database.";
    }
    Browse::Utils::Error($msg);
    exit;
} #eosub--HDB::HDBerror
#------------------------------------------------------------------------------

sub get_table_meta {
# input: a single table name
# output: ref to hash with mission-type meta (description, name, etc)

    my ($table) = @_;
    
    if ($table =~ /\//) {
        #print "HDB::get_table_meta: passing $table to Vizier<br>" if $::debug;
        return Browse::Vizier::tableInfo($table);
    }

    my $tableinfo = {Browse::HDB::TableInfo($table)};
    $tableinfo->{'name'} =  $tableinfo->{'table_name'};
    $tableinfo->{'radius'} =  $tableinfo->{'default_search_radius'}
                   if exists $tableinfo->{'default_search_radius'};
    $tableinfo->{'radius'} =  "***" if($tableinfo->{'radius'} < 0);
    $tableinfo->{'url'} ||= $tableinfo->{'table_document_url'};
    $tableinfo->{'mission'} ||= $tableinfo->{'observatory_name'};
    $tableinfo->{'tabletype'} ||= $tableinfo->{'table_type'};
    $tableinfo->{'description'} ||= $tableinfo->{'table_description'};
    $tableinfo->{'archive'} ||= 'N';
    $tableinfo->{'priority'} = $tableinfo->{'table_priority'};
    $tableinfo->{'priority'} = '5' unless defined $tableinfo->{'priority'};

    return $tableinfo;

}

#------------------------------------------------------------------------------
sub Display_tables_list {
  # Print a list of all tables that can be queried
  # input: format - the display mode
  # output: prints results in the selected display mode

   my ($format) = @_;
   my($results) = Browse::DBQ::query(
           "select a.parameter_value, a.table_name from zzext a where ".
           " a.parameter_name='observatory_name' and a.table_name not in".
           " (select table_name from zzext where ".
           " parameter_name='w3browse_display' and".
           " parameter_value='N') order by a.parameter_value,a.table_name");

  my ($results2) = Browse::DBQ::query("select table_name,table_description from zzgen");
   my ($line2,%descs);
   foreach $line2 (@$results2){
     $descs{$$line2[0]} = $$line2[1];
   }

   my($line);
   foreach $line (@$results){
     my $desc = $descs{$$line[1]};
     $$line[2] = $desc;
     $$line[1]=~s/$::local_origin//g;
   }

   my $displayer = Browse::Display->new($format);
   $displayer->{intable}=1;
   $displayer->{dataarray} = $results;
   $displayer->{names} = ["Mission", "Table", "Table Description"];
   $displayer->{types} = ["char","char","char"];
       # Content type and BatchStart already sent
   $displayer->pageHeader() unless $format =~ /Batch/;  
   $displayer->closeTable();

}



1;
#########
#  EOF  #
#########
