# $Id: HDPQuery.pm,v 1.10 2007/04/10 19:06:33 lmm Exp $

use strict;

package Browse::HDPQuery;
use Browse::DPQuery;
# Inheritance.
use vars qw( @ISA );
@ISA = qw( Browse::DPQuery );



#### The findXXX routines extract needed information from
#### the database and store it in the object variables

# Find the dataproduct sets associated with a given table.
sub findSets {
    my ($self) = @_;
    my $table = $self->getTable();
    my $query = "select set_name, tag_format, set_description from zzdpsets".
                "   where table_name= '$table' order by set_name";
		
    my ($results) = Browse::DBQ::query($query);
    my %dpsets;
    my %dpsetformats;
    
    foreach my $line(@$results) {
        print "$$line[0], $$line[1], $$line[2]<br>" if $::debug;
        $dpsetformats{$$line[0]} = $$line[1];
        $dpsets{$$line[0]} = $$line[2];
    }

    $self->setSets(\%dpsets);
    $self->setFormats(\%dpsetformats);
}

#---------------------------------------------------------------------------

# Given a table and one or more keys, find all of the
# data products associated with the table.  Return
# this information with additional fields allowing the
# user to determine the specific key and data product set
# that a given data product is associated with.

sub findDataProducts {
    my ($self,$pkeys,$setNames) = @_;
    $setNames = [] unless $setNames;
    my $table = $self->getTable();

    my $dpsets = $self->getSets();


    my $fullsets=scalar (keys %$dpsets); 

    if (scalar(@$setNames)) {  # only keep requested sets
       my $mydpsets;

       #my $fullsets=scalar (keys %$dpsets); 
       #my $selectedSets = scalar(@$setNames);

       #if (scalar (keys %$dpsets) != scalar(@$setNames)) {
          #Browse::Log::DPLogger($table, $fullsets,$selectedSets,
	  #join(',',@$setNames)); 
       #}

       foreach my $set (@$setNames) {
          $$mydpsets{$set} = $$dpsets{$set};
       }
       $dpsets = $mydpsets;
    } else {
       @$setNames= keys %$dpsets;
    }
    
    #--- Log number of available data product sets and number selected
    #--- (for statistics)
    my $selectedSets = scalar(@$setNames);
    Browse::Log::DPLogger($table, $fullsets,$selectedSets,join(',',@$setNames)); 

    my $keyFieldString = join(',',@$pkeys) if ref($pkeys);
    $self->setKeyString($keyFieldString);
    my $productColumns = $self->getProductColumns();

    # If no products column exist abandon this search.  This may happen
    # with cross correlation
    #if (!defined($keyFieldString) || !defined($productColumns)) { #lmm 7/04
    if (!$keyFieldString || !$productColumns) {
       return undef;
    }

    my $columns;
    #Find the required columns and create a hash of pointers to them.
    @$columns = split('\|', $productColumns);
    my $i;
    my $hashIndex;
    for ($i=0; $i <= $#$columns; $i++) {
          $$hashIndex{$$columns[$i]} = $i;
    }
    # Now look  at the keys.
    my @keyNames = ();
    my $pkey;
    foreach $pkey (@$pkeys) {
        my $key;
        foreach $key (split('\|',$pkey)) {
           $key =~ s/=[^\|]*//;
           push(@keyNames,$key);
        }
    }

    # Save the types of the key fields so we know how to do comparisons later
    my @types;
    foreach my $keyName (@keyNames) {
        push (@types, $self->getParamType($keyName));
    }

    # Select out the product columns.
    
    my $q1 = "select ".join(",", @$columns);
    my $query;
    my $results = [];
    my $union;
    my $keynumber = 0;
    
    # Since we assume relatively good connections with
    # the database we submit a separate query for each key.
    
    my $key;
    
    foreach $key (@$pkeys) {
    
        # Add in the keynumber so we know which key
	# the results come from.
	
	$query = "$q1,$keynumber from $table where ";
	
        my (@fields,@values);
        if ($key =~ /=/) {
           foreach (split('\|', $key)) {
              my ($field,$val) = split (/\s*=\s*/,$_);
              print "got field=$field,val=$val<br>\n" if $::debug;
              push (@fields,$field);
              push (@values,$val);
           }
        } else {
           @fields = @keyNames;
	   @values = split('\|', $key);
        }
	$keynumber += 1;
	    
	# Now put in the where clause the match to the current key.
	my $col;
	my $and=" ";
	for ($col = 0; $col <= $#values; $col += 1) {
	   if ($types[$col] =~ /char/i) {
	       $query .= " $and $keyNames[$col] = '$values[$col]' ";
	   } elsif ($types[$col] =~ /float/i) {
	       $query .= " $and convert(varchar,$keyNames[$col]) = '$values[$col]' ";
	   } else {
	       $query .= " $and $keyNames[$col] = $values[$col] ";
	   }
	   $and = " and ";
	}
        print "DPL::getDataProducts: query=$query<br>\n" if $::debug;
	
	# Get the results and add them to the stack
        my ($newres) = Browse::DBQ::query($query);
        @$results = (@$results, @$newres) if $newres;
    }
    
    if ($#$results < 0) {
        print "DPL::getDataProducts::No Results!<br>\n" if $::debug;
        return undef;
    }
    
    
    # Make sure the shortcuts are set up.
    $self->findShortcuts();
		
    # Get the data products.
    my $prods = $self->getProducts(
		$self->substituteTags($results, $hashIndex, $setNames)
		);
		
    my %products = ();
    my $setarr=();
    foreach my $line(@$prods) {
	my $set = $$line{'set'};
	my $key = $$line{'key'};
	$$line{'dp_url'} = $self->resolveShortcuts($$line{'dp_url'});
	push(@{$products{$$pkeys[$key]}{$$setNames[$set]}}, $line);
    }

    $products{"set_descriptions"} = $dpsets;
    $self->setDataProducts(\%products);
}

#---------------------------------------------------------------------------
sub findKeyString {
#
#
# return a comma-separated list of the unique keys and values
}

#---------------------------------------------------------------------------
# Find the columns used in creating data product tags.
# Get the product columns for a given table, i.e.,
# those columns in the table that are needed to define
# the tag strings for the associated products.
sub findProductColumns {
    my ($self) = @_;
    my $dpsets = $self->getSets();
    my @fields;
    my $dpformats = $self->getFormats();
    my %seen;
    foreach my $pform (values %$dpformats) {
	 my @matches = $pform =~ /\{[^\}]+\}/g;
	 foreach my $column(@matches) {
             #print "--HDPQ::findProductCols:col=".$column."<br>" if $::debug;
             #print "--HDPQ::findProductCols:subs=".
		#substr($column, 1, length($column)-2) ."<br>" if $::debug;
	     $seen{lc(substr($column, 1, length($column)-2))}++;
         }
    }
    #print "HDPQ::findProductCols:fields=".join('|',keys %seen)."<br>\n" 
	#if $::debug;
    $self->setProductColumns(join('|', keys %seen));
}

#---------------------------------------------------------------------------
# Get the type of a given parameter.
sub findParamType {

    my ($self, $parameter) = @_;
   
    # Special column for keys.
    if ($parameter eq "_unique_id") {
        $self->setParamType($parameter,"int");
    }

    my $table = $self->getTable();
    my $query = "select parameter_name, parameter_format" .
                "  from zzpar where table_name = '$table'";
    
    my ($results) = Browse::DBQ::query($query);
    if ($#$results < 0) {
        Browse::HDB::HDBerror("DPL error: unable to determine ".
                              "parameter types for $table");
    }
    
    foreach my $line (@$results) {
        $self->setParamType($$line[0],$$line[1]);
    }
}

#---------------------------------------------------------------------------
# Find and resolve all shortcuts used in data product URL definition.
# Note that shortcuts are not defined on a per-table basis.
sub findShortcuts {
    my ($self) = @_;

    my $query = "select parameter_name, parameter_value from zzext ".
                "where table_name='zzdp'";
    my ($results, $names) = Browse::DBQ::query($query);
    
    my ($line, %shortcuts);
    foreach $line (@$results) {
	$shortcuts{$$line[0]} = $$line[1];
    }
    
    # Now resolve the shortcuts.
    
    foreach my $name (keys %shortcuts) {
    
        my $count = 0;
        while ($shortcuts{$name} =~ /\$\{([^\}]+)\}/  && $count < 20) {
	    $shortcuts{$name} =~ s/\$\{([^\}]+)\}/$shortcuts{$1}/;
	    $count += 1;
	}
	if ($count >= 20) {
	    print STDERR "Unresolved shortcut for $name\n";
	}
    }
    $self->setShortcuts(\%shortcuts);
}

#---------------------------------------------------------------------------
# Replace strings of the form ${...} with the appropriate shortcut.
sub resolveShortcuts {

    my ($self,$str) = @_;
    my $shortcuts = $self->getShortcuts();
    $str =~ s/\$\{([^}]*)\}/$$shortcuts{$1}/g;
    #$str =~ s/\/tartarus.gsfc/\/hera4.gsfc/; ### hack
    return $str;
}



#---------------------------------------------------------------------------
# Find the unique key for a table.
sub findUniqueKeys {

    my ($self) = @_;

# may be moved to VizierProds
    if ($$self{'isVizier'}) {
       $self->setUniqueKeys("recno");
       return;
    }
    
    my $table = $self->getTable();

    my $query = "select _unique_id from $table where 0=1";
    my ($results) = Browse::DBQ::query($query);
    if (!defined $results) {
        $query = "select table_name, parameter_value from zzext ".
                 "  where parameter_name='query_key' and table_name='$table'";
        ($results) = Browse::DBQ::query($query);
        if ($#$results < 0) {
           $query = "select table_name, parameter_value from zzext ".
                "  where parameter_name='unique_key' and table_name='$table'";
           ($results) = Browse::DBQ::query($query);
        } 
        if ($#$results < 0  && $table =~ /^zz/) {
	    $results = [["table_name"]];
	}

        if ($#$results < 0) {
            Browse::HDB::HDBerror("DPL error: Unable to find unique key ".
                                  "information for $table");
        }
        $self->setUniqueKeys($$results[0][1]);
    } else {
        $self->setUniqueKeys("_unique_id");
    }
#    print "unique keys=".$self->getUniqueKeys()."<br>\n" if $::debug;
}


#########  End of utility functions.  ###########################


# Get key and data product information.
# For each table this routine returns the data product
# sets associated with the table, and a description of
# the unique key giving the name and basic type of
# each column in the key.

sub getKeysAndDPSets {

    my ($self) = @_;
  
    my $table = $self->getTable(); 
    my $key = "";
    my $dpsets = {};
    
      if ($$self{'isVizier'}) {
          ($key,$dpsets) = ("_unique_id:int",[]);
      } else {
          ($key, $dpsets) = $self->keyInfoTable();
      }
	
    return ($dpsets, $key);
}

#---------------------------------------------------------------------------
# This routine processes a single table for getKeysAndDPSets.
sub keyInfoTable {

    my ($self) = @_;
    my $keyString = $self->getUniqueKeys();
    
    my $xstring;
    
    if (length($keyString) > 0) {
	my $tcomma = "";
	foreach my $key(split(',',$keyString)) {
	    my $form = $self->getParamType($key);
	    if ($form =~ /text/ || $form =~ /char/) {
	        $form = "char";
	    } elsif ($form =~ /int/) {
	        $form = "int";
	    } elsif ($form =~ /real/ || $form =~ /float/) {
	        $form = "float";
	    } else {
	        $form = "?";
	    }
	    $xstring .= "$tcomma$key\:$form";
	    $tcomma = ",";
	}
	
	
    }
    my $dpsets = $self->getSets();
# now returning a hash as dpsets instead of an array ref
    return ($xstring, $dpsets);
}



#---------------------------------------------------------------------------
# Get the products corresponding to the retrieved tags.
sub getProducts {

    my ($self,$tagArray) = @_;
    
    my $qs = "select dp_tag, dp_type, dp_format, dp_level, dp_url";
    
    my @productsArray;
    
    # For each input line.
    # We'll send a separate query for each original key.
    
    while (my $tagline = shift(@$tagArray) ) {
        #print "tagline=$tagline<br>\n" if $::debug;
    
        my @linesets = split('\|', $tagline);
	my $lineno = shift(@linesets);  # Get line number.
	my $setno = 0;

	my $query = "";
        # For each data product set
	foreach  my $settags(@linesets) {
            #print "settags=$settags<br>\n" if $::debug;
	        
            # For each tag definition within the set.
	    foreach my $tag(split(",", $settags)) {
              #print "tag=$tag<br>\n" if $::debug;
              #print "setno=$setno<br>\n" if $::debug;
		
                $query .= " union all " if length($query) > 0;
		$query .= "$qs,$lineno,$setno from zzdp where ";

		if (index($tag,'*') >= 0) {
		    $tag =~ tr/\*/\%/;
		    $query .= " dp_tag like '$tag%' ";
		} else {
		    $query .= " dp_tag='$tag'";
		}
			
	    }
	    $setno += 1; 
	}
        print "DPL::getProducts query=$query<br>\n" if $::debug;
	my ($res) = Browse::DBQ::query($query);
        #print "no prods!<br>\n" if !$res && $::debug;
        #print "done with prod query<br>\n" if $::debug;
        foreach my $line (@$res) {
           my %hashres=(dp_tag => $$line[0],
                        dp_type => $$line[1],
                        dp_format => $$line[2],
                        dp_level => $$line[3],
                        dp_url => $$line[4],
                        key => $$line[5],
                        set => $$line[6]);
	   push(@productsArray, \%hashres);
        }
    }    
    
    #return (cleanResults(\@productsArray));
    return (\@productsArray);
    
}

# Substitute the product column values into the tag definitions in zzdpsets
sub substituteTags {
    
    my ($self, $results, $hashIndex, $reqsets) = @_;
    return [] if (scalar @$reqsets == 0);
   
    my $table = $self->getTable(); 
    my $dpformats = $self->getFormats();
    $results = cleanResults($results);
    my $tagArray = [];
    
    foreach my $line (@$results) {
    
        
	my $keyno = pop(@$line);
	my $match = 0;
	my $tags = "$keyno|";

        foreach my $set (@$reqsets) {	
	
	    my $buffer = $$dpformats{$set};
	    print "HDPQ-:start $buffer<br>" if $::debug;
	    
	    # The next code looks for escape sequences of the form 
            # @{...} and replaces them with the current value in the 
            # appropriate row in the table.  Remember that $hashIndex is
	    # a pointer to a hash table giving the index of
	    # a given column within the table ($line).
            # if there is no value, remove @{...} and any formatting 
            # that before and after it (between the periods)
	  
            #--- replacing this code
            #$buffer =~ s/\@\{([^\}]+)\}/$$line[$$hashIndex{$1}]/g;
	    
            #-- More than one substitution may be required so save them to list
            #-- before testing
            my @sublist = ($buffer =~ /\@\{([^\}]+)\}/g);  
            
            my $testForValue;

            foreach my $subval (@sublist) {
               $testForValue=$$line[$$hashIndex{$subval}];
               print "subval=$subval hash=$$hashIndex{$subval} line=$testForValue <br>" if $::debug;
               if (!$testForValue) { 
                  $buffer =~ s/\.[^\.]*\@\{([^\}]+)\}*.\./\.\./g;
               } else {
                  $buffer =~ s/\@\{([^\}]+)\}/$testForValue/;
               }
            }
	    print "HDPQ-:final $buffer<br>" if $::debug;
	    print "-----------------------<br>" if $::debug;
	    $tags .= lc("$buffer|");
	    $match += 1;
	}
	
	push (@$tagArray, $tags) if ($match > 0);
    }
    return $tagArray;
}

# Get rid of white space in the assumed 2-D array.
sub cleanResults {

    my ($results) = @_;

    my $col;
    my $val;
    foreach $col (@$results) {
        foreach $val (@$col) {
	    $val =~ s/^\s*|\s*$//;
	}
    }
    return $results;
}    
sub clear {
}

# $Log: HDPQuery.pm,v $
# Revision 1.10  2007/04/10 19:06:33  lmm
# Altered changes made in last revision to make all substitutions in tag
#   definition (not just the first).
#
# Revision 1.9  2007/01/08 16:33:04  lmm
# Changed substituteTags() to deal with tag definitions that are null.
#
# Revision 1.8  2005/11/21 18:40:33  lmm
# Added query processing for meta data
#
# Revision 1.7  2005/06/17 14:51:45  lmm
# Removed hard coding of system to access heasarc_tartarus data products
#
# Revision 1.6  2005/01/07 16:22:49  lmm
# Added log entries for all data product retrieval and display processes
#   to have better statistics
#
# Revision 1.5  2004/12/17 20:18:10  lmm
# Added logging of user selection of data products types from the Results
#   page to find out how frequently this feature is used
#
# Revision 1.4  2004/07/13 18:09:49  lmm
# Corrected check for data products by checking for variable values
#   rather than just seeing if they are defined in findDataProducts()
#
# Revision 1.3  2003/07/31 14:04:21  cwh
# Added hack since tartarus url wasn't working.
#
# Revision 1.2  2003/03/03 22:21:30  cwh
# Added bit so that an error isn't generated when one target has data products but another doesn't.
#
# Revision 1.1  2002/09/05 18:23:38  cwh
# Data prod query methods specific to HEASARC-style metadata tables.
#
1;
