# $Id: HProdGen.pm,v 1.19 2007/10/03 15:16:39 lmm Exp $
# See the pod for ProdMain.pm for documentation of inherited methods.
# See the end of this file for cvs logs and pod.

use strict;

use Browse::ProdMain;
use Browse::Log;
use LWP::Simple;
require LWP::UserAgent;
use Sys::Hostname;
use Browse::Utils;

package Browse::HProdGen;
# Inheritance.
use vars qw( @ISA );
@ISA = qw( Browse::ProdMain );

eval { require Filesys::DiskSpace; };
unless ($@) {
  $Browse::Products::dfmod=1;
  use lib $::dflib;        # for Filesys::DiskSpace-req'd libs
}

#----------------------------------------------------------------------------

sub doProd {
   my ($self,$val) = @_;
   my $url = $$val{'dp_url'};

   if ($url =~  /ftp\:\/\/[^\/]*(\/.*)/) {
      #--- file is local
      my $file = "/FTP$1";
      $file = $url unless -s $file;
         
      if ($self->fileFilter() !~ /\w/) {   
         #-- if no filter entered save file or directory
         $self->Request($file);
      } else {
         #--- if file filter has been entered do not save directories.  
         #--- Descend directories and apply filter to files in directories
         my $filelist=();

         if ($$val{'dp_format'} =~/DIRECTORY/) {
            #--- descend directories if current product is a directory
            $filelist = $self->descendtree($url, $filelist);
            foreach my $fileFromDir (@$filelist) {
               if ($self->applyFileFilter($fileFromDir)) {
                    $self->Request($fileFromDir);
               }
            }
         } else {
            #--- data product is a file so apply filter directly
            if ($self->applyFileFilter($file)) {
                  $self->Request($file);
            }
         }
      }
   } else {
         #--- data product is not local file so save URL
         if ($self->applyFileFilter($url)) {
            $self->Request($url);
         }
   }

}

sub finish {
    my ($self,$in) = @_;
    foreach (keys %$in) {
       if (/\bFile/) {
          foreach my $file (split("\0",$$in{$_})) {
            $self->addUrltoReq($file);
          }
       }
    }
    $self->startRequest($in);
}

sub pageHeader {
    my ($self,$in) = @_;
    #Browse::Decor::Header("TAR File Creation");
    Browse::Decor::Header($$in{"popupFrom"},0,0,0,"dprodtable" );
    print "pid: $$, pcount: $::pcount<br>\n" if $::debug;

    my $navtrail = $in->{'navtrail'};
    unless ($navtrail) {
      my $referrer = ($ENV{"HTTP_REFERER"} or "/".$::cgiurl."/w3browse.pl");
      $navtrail = "<a class=navpast href=\"$referrer\"> Choose Tables</a>";
    }
    print "$navtrail &gt; <span class=navcurrent>Retrieve Data Products</span><br><p>\n";

}

sub pageFooter {
    my ($self) = @_;
    Browse::Decor::Acknowledge();
    #Browse::Decor::Footer();
    Browse::Decor::EmailFooter();
    Browse::Decor::EndTags();

}

sub dotarscript {
    my ($file,$moretarfiles,$allfiles) = @_;
    my $tarout = "$::tarftpbase/$::tarurlbase/w3browse-$$".
                 $::pcount.".".scalar(@$moretarfiles).".tar";

    while (-s $tarout) {
        $tarout =~ s/\.tar/1.tar/; # ensure unique
    }

    push(@$moretarfiles,$tarout);

    #--- Parse URL to build parameter list for POST
    my ($url,$args) = split(/\?/,$file);
    $file=$url;

    my (@pairs) = split(/&/,$args);
    my %params;
    foreach my $pair (@pairs) {
       my ($k, $v) = split(/=/,$pair);
       $params{$k}=$v;
       print "$k --> $v<br>" if $::debug;
    }
    #-- Add tar file name and data products file list for 
    #-- duplicate file elimination
    my $filelist=join(',',@$allfiles);
    $params{flist}=$filelist;
    $params{tar}=$tarout;

    #--- need to stay on same host so I know where to look for tarfile
    my $host = Sys::Hostname::hostname();
    $file =~ s/heasarc|heasarcdev/$host/;

    if ($::debug ) {$file =~ s/cgi-bin\/W3Browse/cgi-bin\/w3bdev\/W3Browse/;}

    print "Please wait - getting subset tarfile...<br>\n";

    #--- Use UserAgent and POST because URL can be long with the file list.
    use HTTP::Request::Common qw(POST);
    my $ua = LWP::UserAgent->new();
    my $request = POST $file,\%params;
    my $response = $ua->request($request); 
   

    unless (-s $tarout) {
       my $msg = "Error: couldn't create subset tarfile: $!\n";
       Browse::MailMessage::MailMessage($msg,$::addr) && 
          print "<br>$msg<br>\n";
    }
    return $moretarfiles;
}

sub doremotefile {
    my ($file) = @_;
            # if too long, don't print full URL to external file.
            my $linktext='';
            if (length($file) > 70) {
               $linktext = "Link";
            } else {
               $linktext = $file;
            }
	    print "Link to remote data product (not tarred): ".
                  "<a target='_blank' href='$file'>$linktext</a><br>\n";

}

sub startRequest {

    my ($self,$in) = @_;
    #my $msg = "No data products found.<br><br>Please go back and make sure you have at least one table row and at least one data product type selected.  If so, the reason may be that we do not have any data products of the type selected for the table rows selected - for example, an observation you chose may be scheduled, or performed, but not yet in the archive.";
    unless ($self->Request()) {
         #Browse::Utils::Error("$msg");
         $self->noProductsMessage();
    }
    my @allfiles = keys %{$self->Request()};
   
    unless (scalar(@allfiles)) {
       #Browse::Utils::Error("$msg");
       $self->noProductsMessage();
    }

    my $total;
    my @files;   

    my %tarhash = {};
    for (@allfiles) {
       $_ =~ s/\/$//;
       $tarhash{$_} = 1;
    }

    #--- Check the file list for files that are already included in 
    #--- a directory and estimate tar file size
    foreach my $file(@allfiles) {
            if (!Browse::Utils::isDuplicate($file,\%tarhash)) {
               push(@files,$file);
               $total += $self->fileSize($file) if $file =~ /FTP/;
               #print "$file $total<br> " if $::debug;
            }
    }
    #my $estsize = $total;  #save this for later comparison

    $self->checktarsize($total);
    #$self->checkdiskspace($total);

    print "<strong>Estimated size of TAR file: ".$self->optsize($total).
          "</strong><br>" if $total > 0;

    #--- First time through check tar file size and if over limit
    if (defined($::tarsizeasklimit) && $total >= $::tarsizeasklimit 
				    && !$in->{createTarFile}) {
        $self->printform($in);
    } else {$self->createTarFile($in,@files);}
}

#--- Display form that gives user choice to continue tar download
#--- or create download script.
sub printform {
   my ($self, $in)=@_;
   print "<p>This tar file will take a long time to create and download. " .
      "To avoid possible time-outs we strongly suggest that you use " .
      "the Create Download Script option.  The  script provides " .
      "<a href='http://www.gnu.org/software/wget/wget.html'>wget </a>" .
      "commands that use time stamps to allow resumption of interrupted " .
      "downloads and allow files to be downloaded individually.\n";
   
   print "<form name='files' method=POST " .
    "action='/$::cgiurl/w3hdprods.pl'>\n".
    "<p><b>Choose a Data Products Download Option.</b></p>\n".
    "<p><table><tr><td>" .
    "<input type='Submit' name='files'  value='" .
    "Continue Download'></td><td></td>\n".
    "<td> " .
    "<input type='submit' name='files'  value='" .
    "Create Download Script '>" .
    " </td></tr></table> \n";
   foreach my $key (keys %$in) {

      if ($key =~ /^RetrieveSet|^Target/) {
          foreach my $set (split("\0",$$in{$key})) {
            print "$key  ==>  $set<br>" if $::debug;
             print "<input type='hidden' name='$key' value='" . $set . "'>\n";
          }
       } else {
            
            print "..$key  ==>  $$in{$key}<br>" if $::debug;
           print "<input type='hidden' name=\"$key\" value='".
        	$in->{$key} ."'>\n" unless $key =~ /controlid|password/;
       }
   }

   print "</form>\n";
}  #--- end printform


#--- Create tar file
sub createTarFile {

    my ($self,$in,@files) = @_;
    my $tarfile = "$::tarurlbase/w3browse-$$".$::pcount.".tar";
    my $tarurl  = "ftp://$::ftphost/".$tarfile;
    my $httpurl  = "http://$::host/FTP/".$tarfile;
    $tarfile = "$::tarftpbase/$tarfile";
    while (-s $tarfile) {
      $tarfile =~ s/\.tar/1.tar/; # ensure unique
    }


    print <<EOT;
    Your TAR file is being created now.  When finished you may retrieve it via the following link
    <dl>
    <dt><dd><a href='$httpurl'>$httpurl</a>.
    </dl>
    <b>Note</b>: We have phased out retrieval of data product tar files via FTP.  
    <p>Please wait until the "TAR complete" message appears below before 
    retrieving.<p>
    <strong>Data products included in the TAR file:</strong> (filenames ending 
    in '.gz' or '.Z' have been compressed for faster downloading.)<br>
    <br>
EOT
    #print <<EOT;
    #Your TAR file is being created now.  When finished you may retrieve it via
    #<dl>
    #<dt>FTP at <dd><a href='$tarurl'>$tarurl</a> or
    #<dt>HTTP at <dd><a href='$httpurl'>$httpurl</a>.
    #</dl>
    #Please wait until the "TAR complete" message appears below before 
    #retrieving.<p>
    #<strong>Data products included in the TAR file:</strong> (filenames ending 
    #in '.gz' or '.Z' have been compressed for faster downloading.)
    #<br>
#EOT

    Browse::Log::CallingTar("$$"."$::pcount","-C ".join(' -C ',@files));

    my $tflag = "c";
    my @moretarfiles=(); # holds names of extra tar files
    my @remotefiles = ();

    my %tarhash = {};
    for (@files) {
       $_ =~ s/\/$//;
       $tarhash{$_} = 1;
    }
    foreach my $file(@files) {
     
        if ($file =~ /[\!\$\;]/) {
            print "Invalid file: $file<br>\n";
	} elsif ($file =~ /xteTar\.pl/) {
            dotarscript($file,\@moretarfiles,\@files);
	} elsif ($file =~ /\:|%3A/i) {
            # added encoding to remove dangerous characters from 
            # file name (per ed sabol's suggestion)
	    push(@remotefiles,Browse::CGI::Encode($file));   
            #print " HProdGen: encoded file=" . Browse::CGI::Encode($file) .
		#"<br> " if $::debug;
        } else {
            #--- Add file to tar file but first check 
            #--- if file is in a directory that is in the list
            if (!Browse::Utils::isDuplicate($file,\%tarhash)) {
	       $tflag=dolocalfile($file,$tflag,$tarfile);
            }
        }
     }
   
    
    if (scalar @remotefiles) {
        my $host = Sys::Hostname::hostname();
        my $script = "http://$host/$::cgi/remotetar.pl";
        my $tempfile = "$::tarftpbase/$::tarurlbase/remote$$.tar";
        print "gonna run $script?tar=$tempfile&File=".
		join("&File=",@remotefiles)."<br>\n" if $::debug;

        print "Please wait - getting remote products...$tempfile<br>\n";
        LWP::Simple::get("$script?tar=$tempfile&File=".
                          join("&File=",@remotefiles) );
        push(@moretarfiles,$tempfile);
    }


    appendtarfiles(\@moretarfiles,$tarfile);
    my $size = (-s $tarfile);
   
    if ($size) { 
       #if ($estsize and ($size/$estsize < 0.67 or $size/$estsize > 1.3)) {
       #   my $msg = "Warning: Tarfile $tarfile size ($size) is ".
       #              "more than 33% off from estimate ($estsize)!\n)";
       #   Browse::MailMessage::MailMessage($msg,$::addr);
       #}
       print "<p><Strong>TAR complete</strong>: Actual size: ".
             $self->optsize($size) .".<br>";
       my $savetables = join(',',@{$self->Tables()}) || $$in{'savetables'} || 
		"unknown";
       Browse::Log::CreatedTarfile($savetables,$size/1024); 
    } elsif ($tflag =~ /c/) {
       print "<p><strong>Nothing to tar:</strong> All products are remote - " .
		"click on the links above to access them.";
    } else {
       my $msg="TAR file not created: files=@files, tarfile=$tarfile";
       Browse::MailMessage::MailMessage($msg,$::addr) && 
       print "<p><strong>TAR not complete:</strong> Something may be wrong " .
		"with our server.  Email has already been sent to the " .
		"maintainers to alert us to the problem.";
    }
}  #--- end createTarFile

sub appendtarfiles {
   my ($moretarfiles,$tarfile) = @_;
   # Now append the xteTar tarfiles
   foreach my $tar (@$moretarfiles) {
     print "Appending subset tarfile $tar.<br>" if $::debug;
     my $cmdout = `$::tar -Af $tarfile $tar`;
     if ($cmdout) {
         print "Error tarring $tar: $cmdout<br>\n" &&
              Browse::MailMessage::MailMessage($cmdout,$::addr);
     } else {
         print "Appended subset tarfile.<br>\n";
     }
     unlink $tar if -e $tar;
   }
}

sub dolocalfile {
     my ($file,$tflag,$tarfile) = @_;
            $file =~ /^([^;\s]*)$/;
	    $file = $1;

       my ($dir,$xf) = cutfile($file);
            my $cmd = "$::tar -".$tflag."hf $tarfile -C $dir $xf 2>&1";
            my $cmdout = `$cmd`;
            if ($cmdout) { 
              if ($cmdout =~ /file changed as we read it/i) {
                 #--- subtle system problem that is not interfere with tar
                 print "Tarred $file<br>\n";
                 Browse::Log::WriteLog("[". DateTime::asctim().
		      "] $::REMOTE_HOST :: Error:: " . $cmdout, $::tarLog);
                 Browse::MailMessage::MailMessage("$cmd :: $cmdout",
			'lmm@milkyway.gsfc.nasa.gov');
              } else {
                 print "Error tarring $file: $cmdout<br>\n" &&
                 #Browse::MailMessage::MailMessage($cmdout,
			#'lmm@milkyway.gsfc.nasa.gov');
                 Browse::MailMessage::MailMessage($cmdout,$::addr);
              }

            } else { print "Tarred: $file<br>\n"; }
	    $tflag = "r";
     return $tflag;
}

sub cutfile {
     my ($file) = @_;
	    my $cut = index($file, "//");
	    
	    # If a TAR point is specified use it, otherwise
	    # go two directory levels above bottom.
	    
	    my ($dir, $xf);
	    if ($cut > 0) {
		$dir = substr($file, 0, $cut);
		$xf  = substr($file, $cut+2);
		
	    } else {
		$cut = rindex($file, "/");
		$cut = rindex($file, "/", $cut-1);
		if ($cut > 0) {
		    $dir = substr($file, 0, $cut);
		    $xf = substr($file, $cut+1);
		} else {
		    $dir = "/";
		    $xf = $file;
		}
	    }
     return $dir,$xf;
}

#sub optsize {
# get the file size in the optimal units for printing
    #my ($size) = @_;   # assuming it is in bytes
    #my $unit="";
#
    #if ($size < 1024) {
            #$unit = "B";
        #} elsif ($size < 1022976) { #1022976 = 999 kB
            #$size = int(($size+512)/1024);
            #$unit = "kB";
        #} elsif ($size < 1047527424) { #1047527424 = 999 MB
            #$size = int( ($size+512*1024)/(1024*1024));
            #$unit = "MB";
        #} else {
            #$size = int( ($size+512*1024*1024)/(1024*1024*1024));
            #$unit = "GB";
        #}
    #return "$size $unit";
#}

sub checktarsize {

    # check to see if the tar file is estimated to be larger than the limit
    #my ($total) = @_;
    my ($self, $total) = @_;
    if ($total >= $::tarsizelimit) {  
        Browse::Utils::Error("Tar file size is expected to be ".
              $self->optsize($total).".  We have set a limit of ".
              $self->optsize($::tarsizelimit)." per tar file.  Please go back and ".
              "reduce the number of observations or files selected, " .
	      "or select Create Download Script to create" .
              " a editable script of download commands."); 
    }
    #print "checking tar file size: estimated tar file size: ".
         #$self->optsize($total).", limit set to: ".
          #$self->optsize($::tarsizelimit)."<br>\n" if $::debug;
} #eosub--checktarsize

sub checkdiskspace {

    # check to see if there is disk space to handle the estimated tar file size
    #my ($total) = @_;
    my ($self, $total) = @_;
    my $dir = "$::tarftpbase/$::tarurlbase";
    my $avail="";
    if ($Browse::Products::dfmod) {
       $avail = (Filesys::DiskSpace::df($dir))[3];
    } else {
       $avail = (split(/\s+/, `$::df $dir`))[10]; # may need changing
    }
    $avail *= 1024;

    if ($total >= $avail) {
       Browse::Utils::Error("<dl><dt>Insufficient disk space to create tar ".
            "file:<dd>available disk space : ".$self->optsize($avail).
	    "</dd><dd>expected size of tar file : ".
	    $self->optsize($total)."</dd></dt></dl>");
    }
    print "checking disk space: estimated tar file size: ".
          $self->optsize($total).", available disk space: ".
          $self->optsize($avail)."<br>\n" if $::debug;
} #eosub--checkdiskspace
	    
1;

__END__

# $Log: HProdGen.pm,v $
# Revision 1.19  2007/10/03 15:16:39  lmm
# Changes to call more efficient Utils::isDuplicate().
#
# Revision 1.18  2007/05/04 13:41:22  lmm
# Making sure global variable for a tar size limit is defined
#
# Revision 1.17  2006/10/27 15:14:14  lmm
# Removed FTP download link to simplify disk mounting/access  configuration.
#
# Revision 1.16  2006/10/24 17:23:42  lmm
# New class method noProductsMessage() now called to display
#    generic "No data products" message.
#
# Revision 1.15  2005/11/21 18:58:22  lmm
# Added user specified file filter check for remote files.
#
# Revision 1.14  2005/10/31 17:13:06  lmm
# Corrected spelling mistake
#
# Revision 1.13  2005/09/19 17:02:26  lmm
# Added call to descendtree() to filter fles when user has entered a filter.
# Relocated tar file creation code to new subroutine createTarFile() to
#   give users the choice to create download script if tar file is large.
#
# Revision 1.12  2005/08/05 14:32:28  lmm
# Removed duplicate files from tar file size estimate
#
# Revision 1.11  2005/07/11 19:50:58  lmm
# Changed GET to POST in dotarscript() to accommodate long URL.
#
# Revision 1.10  2005/05/16 15:14:44  lmm
# Local files are not included in tar file if they are
#   located in a directory included in the data products file list
#
# Revision 1.9  2005/04/14 15:22:38  lmm
# Fixed call to checktarsize()
#
# Revision 1.8  2004/10/04 20:22:33  lmm
# Moved subroutine optsize() into parent class ProdMain so it can be used
#   by other derived classes
#
# Revision 1.7  2004/05/24 14:16:31  lmm
# Encoded file names for data products that are retrieved from remote servers
#
# Revision 1.6  2003/08/28 21:33:20  lmm
# Define text and style for page header and remove HEASARC footer
#
# Revision 1.5  2003/02/25 21:32:05  cwh
# Now reading the savetables param if $self->Tables isn't set, so that we can
# correctly log the tables the files came from.
#
# Revision 1.4  2002/12/17 20:56:16  cwh
# Added catch for requests with no products found.
# Added longer error explanation for when no products are found.
#
# Revision 1.3  2002/10/28 17:32:22  cwh
# Instead of running the doremotefile() subroutine, now just pushing remote
# files onto a list, then running the remotetar.pl script with all the remote
# files as input. Once the tarfile of remote files is generated, the tarfile
# is treated the same as the tarfiles created by xteTar.pl, which are added
# to the main tarfile.
#
# Revision 1.2  2002/10/23 16:49:50  cwh
# Added for handling xtetar requests - need to stay on the same host so I know
# whether to look for the resulting tarfile on retrieve1 or retrieve2.
#
# Revision 1.1  2002/09/05 18:26:56  cwh
# ProdMain is a class which creates objects for displaying data products and
# for generating data product packages for retrieval.  HProdDisplay and
# HProdGen are provided as examples of display and generation classes which
# inherit from ProdMain.  The expectation is that other sites using this code
# will modify the HProd* modules for their needs.
#

=head1 NAME

HProdGen - Methods for generating a delivery of data products to the user. Inherits from ProdMain.  See L<Browse::ProdMain> for description of inherited methods
.

=head1 SYNOPSIS

 use Browse::ProdMain;

 my($in) = Browse::CGI::ReadParse();
 my $dp = new Browse::ProdMain($in);
 $dp->doProducts($in);

=head1 DESCRIPTION

The HProdGen class is created by and inherits from the ProdMain class.

The doProducts() method is a complete product processor, but if you want
more control you can directly call the methods used in doProducts().

=head1 OPTIONAL: USE METHODS DIRECTLY

See L<Browse::ProdMain> for the description of inherited methods.
These are the additional methods in HProdGen that you can use if you
choose not to use doProducts():


=head2 START THE DELIVERY REQUEST

 $dp->startRequest();

Called from within finish().  This initiates the product delivery.
There are separate subroutines for handling the individual products,
depending on whether they are local filenames, CGI scripts, or urls.
This implementation assumes that the final product offered to the user is 
a tar file.

=head1 AUTHOR INFORMATION

High Energy Astrophysics Science Archive Research Center (HEASARC),
NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.

You are free to use this code as you wish, but unless you change
it significantly, please leave the above name & address intact.

=head1 BUGS

=head1 SEE ALSO

L<Browse::ProdMain>, L<Browse::HProdDisplay>

=cut

