# $Id: Log.pm,v 1.23 2005/09/19 17:25:46 lmm Exp $
# $Log: Log.pm,v $
# Revision 1.23  2005/09/19 17:25:46  lmm
# Added routine to log data products download script information
#
# Revision 1.22  2005/01/07 16:31:52  lmm
# Made a small change to DPLogger() output format
#
# Revision 1.21  2004/12/17 20:15:24  lmm
# Added subroutine DPLogger() to log data products selections from Results
#   page
#
# Revision 1.20  2002/09/05 18:17:23  cwh
# Don't log if nolog flag is set.
#
# Revision 1.19  2002/02/26 18:05:02  cwh
# Added '($$ % 20) or' so that the error messages are only mailed if the pid
# is evenly divisible by 20.
#
# Revision 1.18  2001/11/07 23:43:55  cwh
# Moved the logfile definitions from here to Config.pm.
# Changed 'warn's to MailMessage() since we don't want users to see the warnings.
#
# Revision 1.17  2001/10/25 19:28:47  cwh
# Added logging of tarfile (actually processid which is unique part of file
#   name).
#
# Revision 1.16  2001/08/08 20:51:50  cwh
# Added logging of the Time entry from the first page.
#
# Revision 1.15  2001/07/31 15:47:04  cwh
# Cleaned up log::Query() - easier to read and more consistent.
#
# Revision 1.14  2001/07/17 21:06:21  cwh
# Created getLogs() so that we can get the log variables into each subroutine.
#
# Revision 1.13  2001/07/17 20:40:57  cwh
# Changed logDir to ::logDir, which is set in w3config.pl.
# Changed Cone to Resort.
# Entries is only printed if $entries.
# ImgSort is followed by whatever is sent as $query.
#
# Revision 1.12  2001/07/17 18:39:01  cwh
# Added VizieRLogger subroutine.
# Changed the flag for determining if Cross Query is of results or full tables.
#
# Revision 1.11  2001/04/25 17:03:39  cwh
# removed unnecessary 'my'
#
# Revision 1.10  2000/07/26 18:17:34  saima
# Removed reference to $in->{'JavaPlot'}.
#
# Revision 1.9  2000/03/08 20:10:39  saima
# W3Browse to Browse name change.
#
# Revision 1.8  1999/12/20 17:51:27  saima
# Fixed a remote host problem where the environment variable always
#   contained 'legacy' as the remote host. The correct name now comes
#   from a CGI variable added by the webserver. This is stored in
#   $::REMOTE_HOST.
#
# Revision 1.7  1999/10/07 20:12:07  saima
# Modified layout of log displays.
#
# Revision 1.6  1999/09/30 18:32:16  saima
# Modified format for SQL logging.
#
# Revision 1.5  1999/09/03 20:37:55  saima
# Added query logging. Also, made some code structure changes.
#
# Revision 1.4  1999/03/19 20:52:44  saima
# Corrected tar log file path/name.
#
# Revision 1.3  1999/03/18 21:02:38  saima
# Ooops commented out the actually print statements to LOG file!
#
# Revision 1.2  1999/03/18 20:48:37  saima
# Print log messages only if $::debug is on.
#
# Revision 1.1  1999/03/18 20:44:31  saima
# Initial Revision
#
#
#-----------------------------------------------------------------------#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
#  it significantly, please leave the above name & address intact.
#-----------------------------------------------------------------------#
# Module Name:    Log.pm
# Module Author:  Saima Zobair, Raytheon ITSS
# Date:           3/99
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
#
#
#
#=======================================================================#
# Outline:
#
#
#
#=======================================================================#

use strict;
use DateTime;
use Browse::MailMessage;

package Browse::Log;

# Locations of log files.

#------------------------------------------------------------------------
sub TableQuery {

  my($scriptname,$in,$querytype,$missions) = @_;

  my($logstuff) = "[". DateTime::asctim()."] $::REMOTE_HOST :: ";
  $missions = join(',',@ {$missions}) if ($missions);

  if ($querytype =~ /Display/) {
     $logstuff   .= "[$querytype], Missions: $missions"; 
     $logstuff   .= ", SortBy: $in->{'sortby'}" if ($in->{'sortby'}); 
  } 
  elsif ($querytype =~ /^Help/) {
     $logstuff   .= "[$querytype], Mission: $in->{'MissionHelp'}"; 
  }
  WriteLog($logstuff, $::queryLog);

} #eosub--Log::TableQuery
#------------------------------------------------------------------------
sub Query {

  my($scriptname,$in,$querytype,$tables,$query) = @_;

  my $table = join(',',@{$tables});
  $table =~ s/\s*//g;

  my $entries;
  if($in->{'Entry'}) {
       $entries = join(';',split(/\s*\0\s*/,$in->{'Entry'}));
  }

  if ($querytype =~ /Build Parameter/) {
    $querytype = scalar(@ {$tables}) > 1 ? 'Build Multiple Parameter Search' : 
                                        'Build Parameter Search';
  } elsif ($querytype =~ /Parameter Search/) {
     $querytype = "$querytype + Cone Search";
  } elsif ($querytype =~ /Cross/) {
     if (defined $query){
        $querytype  = "Cross on Results"; 
     } else {
        $querytype  = "Cross on Full Tables"; 
     }
  }  
  
  my($logstuff) = "[".DateTime::asctim()."] $::REMOTE_HOST :: ";
  $logstuff .= "[$querytype]" if $querytype;
  $logstuff .= ", Tables: $table" if $table;
  $logstuff .= ", Entries: $entries" if $entries; 
  $logstuff .= ", Where: $query" if $query;
  $logstuff .= ", Time: $in->{'Time'}" if $in->{'Time'};
  $logstuff .= ", XTime: $in->{'XTime'}" if $in->{'CrossTime'};
  $logstuff .= ", XRadius: $in->{'XRadius'}" if $in->{'CrossRadius'};
  $logstuff .= ", ImgSort: $query" if (grep(/imgSort/,keys(%{$in}))); 
  WriteLog($logstuff, $::queryLog);

} #eosub--Log::Query
#------------------------------------------------------------------------
sub VizieRLogger {
 
  my ($url,$tables,$quals,$positions) = @_;
  my($logstuff) = "[".DateTime::asctim()."] $::REMOTE_HOST :: ";
  $logstuff    .= "[VizieRQuery]";
  $logstuff    .= ", Tables: $tables" if $tables;
  $logstuff    .= ", Entries: $positions " if $positions;
  $logstuff    .= ", Where: $quals " if $quals;
  $logstuff    .= ", URL: $url ";

  WriteLog($logstuff, $::VizLog);

} #eosub--Log::VizieRLogger
#------------------------------------------------------------------------
sub CallingTar{
  my($tarfile,$files) = @_;

  my($logstuff) = "[".DateTime::asctim()."] $::REMOTE_HOST :: Tarfile: $tarfile, ";
  $logstuff    .= "Tarring $files\n";

  WriteLog($logstuff, $::tarLog);

} #eosub--Log::CallingTar
#------------------------------------------------------------------------
sub CommandsDisplayed {
  my($tables,$size) = @_;

  my($logstuff) = "[".DateTime::asctim()."] $::REMOTE_HOST :: ";
  $logstuff    .= "Created commands for $tables, ($size kilobytes)\n";

  WriteLog($logstuff, $::tarLog);

} #eosub--Log::CommandsDisplayed
#------------------------------------------------------------------------
sub CreatedTarfile{
  my($tables,$size) = @_;

  my($logstuff) = "[".DateTime::asctim()."] $::REMOTE_HOST :: ";
  $logstuff    .= "Created tarfile for $tables, ($size kilobytes)\n";

  WriteLog($logstuff, $::tarLog);

} #eosub--Log::CreatedTarfile
#------------------------------------------------------------------------
sub SqlLogger {
  my ($sql, $db) = @_;

  my($logstuff) = "[". DateTime::asctim()."] $::REMOTE_HOST :: ";
  $logstuff    .= "db=$db ::  sql=$sql";
  WriteLog($logstuff, $::sqlLog);

} #eosub--Log::SqlLogger
#------------------------------------------------------------------------
sub DPLogger {
  my ($table,$total,$selectedTotal,$setlist) = @_;
  my($logstuff) = "[". DateTime::asctim()."] $::REMOTE_HOST :: ";
  $logstuff    .= "table=$table |$total sets| :: " .
	"selected sets=$setlist |$selectedTotal sets|";
  WriteLog($logstuff, $::tarLog);
}
#------------------------------------------------------------------------
sub WriteLog { 
  my($logstuff, $logfile) = @_;
  return if $::nolog;
  
  (open(LOG,">>$logfile") or open(LOG,">$logfile") ) || ( ($$ % 20) or Browse::MailMessage::MailMessage(DateTime::asctim()." Warning: Could not open Browse log file $logfile :$!",$::addr));
  print LOG "$logstuff\n";
  close LOG || ( ($$ % 20) or Browse::MailMessage::MailMessage(DateTime::asctim()." Warning: Could not close Browse log file $logfile: $!",$::addr));

} #eosub--Log::WriteLog
#------------------------------------------------------------------------

1;
#########
#  EOF  #
#########
