# $Id: MailMessage.pm,v 1.9 2001/11/07 23:41:43 cwh Exp $
# $Log: MailMessage.pm,v $
# Revision 1.9  2001/11/07 23:41:43  cwh
# Changed this to a module.
#
# Revision 1.8  2001/07/02 17:35:46  cwh
# Just changed the 'from' address to be the same as the 'to' address instead
# of hardcoding 'w3browse@athena...'.
#
# Revision 1.7  1999/12/20 17:51:39  saima
# Fixed a remote host problem where the environment variable always
#   contained 'legacy' as the remote host. The correct name now comes
#   from a CGI variable added by the webserver. This is stored in
#   $::REMOTE_HOST.
#
# Revision 1.6  1999/05/27 17:08:01  tam
# Added 'package main' to work with Apache mod_perl.
#
# Revision 1.5  1998/12/23 16:31:45  saima
# Changed the perl path.
#
# Revision 1.4  1997/11/25 23:17:12  saima
# Added -T switch to perl version.
#
# Revision 1.3  1997/03/21 16:01:30  sabol
# Removed unnecessary e-mail address argument to sendmail invocation.
#
# Revision 1.2  1996/12/19 23:18:18  sabol
# Now uses SCRIPT_NAME environment variable when available instead of $0.
#   Constructs the full URL of the script sending the message and reports
#   that. It also reports REQUEST_METHOD and HTTP_REFERER.
#
# Revision 1.1  1996/11/22 19:35:33  sabol
# Added to W3Browse project. Based on John Kressel's version, but updated to be
#   Perl 5-savvy and use 'strict' compiler directive. Simplified code for
#   readability and efficiency.
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA / Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=======================================================================#
# Module Name:    MailMessage.pl (based on John Kressel's mailer.pl)
# Module Author:  Edward J. Sabol, Hughes STX
# Date:           11/96
# Project:        HEASARC Web Library
#=======================================================================#
# Module Description:
#  This module contains a subroutine for sending mail to humans when
#  critical errors are detected by a CGI program.
#=======================================================================#

use strict;

#------------------------------------------------------------------------
package Browse::MailMessage;

sub MailMessage {

    my($msg,$address,$subject) = @_;

    # Discover the name of the executing script.
    my($progname) = ($0 ? $0 : $ENV{'SCRIPT_NAME'});
    my($from_address) = "\"$progname CGI Program\" "
      . '<' . $address . '>';

    unless (scalar(@_) >= 2) {
	print <<EOT;
<pre>Usage requires the following input arguments whose:
 1st argument is the message string
 2nd argument is the address to mail to (delimit multiple addresses with commas)
 3rd argument (optional) is the Subject: line. If no subject is provided,
     this program will assign a subject of 'Human assistance required.'

 Example: &MailMessage("Help me\nPlease!",'user\@node.domain','Send Help.');
</pre>
EOT
       return;
    }

    $subject = "Human assistance required." if (! $subject);
    $address = 'www@athena.gsfc.nasa.gov' unless ($address =~ /\@/);

    # Construct the full URL of the CGI script sending the message.
    my($fullURL) = "http://" . $ENV{'SERVER_NAME'};
    ($fullURL .= ":" . $ENV{'SERVER_PORT'}) if ($ENV{'SERVER_PORT'} != 80);
    $fullURL .= $ENV{'SCRIPT_NAME'} . $ENV{'PATH_INFO'};
    ($fullURL .= "?" . $ENV{'QUERY_STRING'}) if (length($ENV{'QUERY_STRING'}));

    open(MAIL,"|/usr/lib/sendmail -oi -t");
    print MAIL <<EOT;
To: $address
From: $from_address
Reply-To: $address
Subject: $subject

** This message was sent automatically from $progname. **

$msg

------------------------------------------------------------------------------

WWW browser used:	$ENV{'HTTP_USER_AGENT'}
Remote host:		$::REMOTE_HOST
Remote IP address:	$ENV{'REMOTE_ADDR'}
Server name and port: 	$ENV{'SERVER_NAME'}:$ENV{'SERVER_PORT'}
Server protocol:	$ENV{'SERVER_PROTOCOL'}
Server software:	$ENV{'SERVER_SOFTWARE'}
Server cgi-bin version:	$ENV{'GATEWAY_INTERFACE'}
CGI URL referrer:	$ENV{'HTTP_REFERER'}
CGI request method:	$ENV{'REQUEST_METHOD'}
CGI URL:		$fullURL

------------------------------------------------------------------------------
EOT

    close(MAIL);

} #eosub--MailMessage
#------------------------------------------------------------------------
1;
