# $Id$
# $Log$

# This class displays the results in a format that has the look of the default
# HTML display, except that the data is displayed in plain text (but with other
# HTML markup).

use strict;

package Browse::NonTableDisplay;

use vars qw( @ISA );
@ISA = qw( Browse::Display );

# Can't print rows until we've got them all since
# we need to 'neaten' the table output and make all rows
# have columns of the same width.  So transform
# the fields but just save the results for later printing.

sub processData {
    
    my ($self, $data) = @_;
    
    my $row = $$self{"rowcount"};
    my $xn = $$data{"names"};
    
   
   if (!$$self{"transform"}) {
   
       $$self{"transform"}  = $self->setTransform($data);
       $$self{"dataarray"} = [];
       $$self{"intable"} = 1;
       $$self{"names"} = $$data{"names"};
       $$self{"types"} = $$data{"types"};
    }
    
    # Copy the line data -- the data array may be reused.
    my $line  = $$data{"data"};
    my @line  = @$line;
    my $trans = $$self{"transform"};
    my $arr   = $$self{"dataarray"};
    
    
    Browse::TableFormat::transformLine(\@line, $trans);
 
    my ($racol, $deccol) = ($$trans{"racols"},$$trans{"deccols"});
    foreach (@$racol,@$deccol) {
        $$line[$_] =~ s/!!/<a href="$::w3bhelp#coorderror">!!<\/a>/g;
    }
   
    
    push(@$arr, \@line);
    $$self{"rowcount"} += 1;
    
}

# For this class, closeTable actually
# prints the results after 'neatening' the table.

sub closeTable {

    my ($self) = @_;
    
    if ($$self{"intable"}) {
	$$self{"intable"} = 0;
	
	my $array = $$self{"dataarray"};
	my $meta  = $$self{"meta"};
	my $names = $$self{"names"};
	my $types = $$self{"types"};
	
	my $nskip = 0;
	while ($nskip <= $#$names) {
	    if (substr($$names[$nskip], 0, 3) !~ /w3_/i) {
	        last;
	    }
	    $nskip += 1;
	}
	
	Browse::TableFormat::neatenTable($array, $names, $types, undef);
	
	my $trans = $$self{"transform"};
	my $longs = $$trans{"longs"};
        my $table = $$self{"meta"}{"table"};
	
	print "<p><pre>\n";
	$self->displayTextLine($names, $nskip, $longs,$types,$names,$table,1);
	
	my $line;
	foreach $line(@$array) {
	    $self->displayTextLine($line, $nskip, $longs, $types, $names,$table);
	}
	
        my $s = "";
        $s = "s" if $$self{"rowcount"} > 1;
	print "</pre>\n";
        print "$$self{rowcount} row$s retrieved from ". $self->chopTable($table) ."<p>\n"; 
	
	$$self{rowcount} = 0;

        my $dp   = $$meta{"dpsets"};
        $dp = undef if (ref($dp)  && $#$dp < 0);

        print "<table border=1><tr><td valign=top>\n";
        if ($dp) {
            my $tableinfo = $$meta{"tableinfo"};
            $$tableinfo{table_name} = $table;
            Browse::Results::DisplaySets($dp, $tableinfo);
        }
        print "</td><td valign=top>\n";
        Browse::Results::DisplayOptions($table);
        print "<p>";
        print Browse::Results::DisplayLegend();
        print "</td></tr></table>\n";

        Browse::Results::DisplayLinks($meta);
	
    }
}


sub displayTextLine {
    my ($line, $skip, $longs,$types,$names,$tablename,$nocheckbox) = @_;
    print "\n";
    my $unique = $tablename . "|||";

 #my $right = "right.gif";
    my $right = "mag5.gif";
    if ($unique =~ /\// && $unique !~ / x /i) {
        #$right = "rightdiv.gif";
        $right = "mag5viz.gif";
    }

    # Check if our skip fields are for multi-table result.
    # We want to put double pipes at the end of the keys
    # for each table.
    my $n0 = substr($$names[0], 0, 5);
    if ($n0 !~ /w3\_[ab]\_/i) {
        $n0 = "w3_";
    }


    my $i;
    for ($i=0; $i <= $#$line; $i += 1) {
        if ($i < $skip) {

            my $str;

            if ($$types[$i] == 6) {
                # Ensure adequate precision for double values.
                # By default the string representation of doubles
                # isn't adequate.
                if (length($$line[$i]) > 0) {
                    $str .= sprintf("%30.20g", $$line[$i]);
                    $str =~ s/^ *//;
                    $str =~ s/ *$//;
                }
            } else {
                $str = $$line[$i];
            }

            # Handle nulls.
            if (length($str) > 0) {
                $unique .= $str . '|';
            } else {
                $unique .= ' |';
            }

            if ($$names[$i+1] !~ /^$n0/) {
                $unique .= "|";
                $n0 = substr($$names[$i+1], 0, length($n0));
            }

        } else {
            if ($i == $skip and $skip > 0) {
                #$unique  = Browse::CGI::Encode($unique);
                my $begin = "  <input type='image' alt=\"Display all params\"" .
                   " src='$::images/$right' height=16 width=16 border=0" .
                   " name='imgDisp|$unique'>" .
                   "<input type='checkbox' name='Target' value='$unique'>";

                $begin = "       " if $nocheckbox;
                print $begin;
            }

            # Skip long strings.
            if ($longs  && grep($_ == $i, @$longs)) {
                next;
            }

            print "$$line[$i]|";
        }
    }
    print "\n       " . "_"x (length("@$line")-20) if $nocheckbox;

}

#sub displayTextLine {
#
#    my ($line, $skip, $longs) = @_;
#    
#    print "|";
#    my $i;
#    for ($i=$skip; $i <= $#$line; $i += 1) {
#        if ($longs && grep($_ == $i, @$longs)) {
#	    next;
#	}
#	
#        print $$line[$i]."|";
#    }
#    print "\n";
#    
#    for ($i=0; $i <= $#$longs; $i += 1) {
#        my @spl = split("\0", $$line[$$longs[$i]]);
#	foreach (@spl) {
#	    print "      |".$_."\n";
#	}
#	print "\n";  # Extra line to separate multiple long fields.
#    }
#}

## Successful load
1;
