# $Id: ProdMain.pm,v 1.12 2007/10/03 19:57:42 lmm Exp $
# See the end of this file for cvs logs and pod.

use strict;
use Browse::Utils;
package Browse::ProdMain;

sub new {
    my ($self,$in) = @_;
    my ($action) = $$in{"files"} =~ /([\w-]*)/;

    unless ($::prods{$action}) {
       Browse::Utils::Error("Prods type for $action unknown! See Config.pm");
    }
    my $obj = {};
    my $type = "Browse::".$::prods{$action};
    eval "package Browse::_firesafe; require $type";
    bless $obj, $type;

    $obj->init($in);
    return $obj;
}

sub init {
    my ($self,$in) = @_;
    $$self{"gifstogo"} = $::MaxGIFs;
    $self->checkSize(1);
    $self->showGIFs(1);
    $self->checkSize($$in{"CheckSize"}) if defined $$in{"CheckSize"};
    $self->showGIFs($$in{"showgifs"}) if defined $$in{"showgifs"};
    my $coordsys  = $$in{"Coordinates"} || "Equatorial";
    my $equinox   = $$in{"Equinox"} || 2000;
    $self->coosys($coordsys,$equinox);
}

sub coosys {
    my ($self,$coords,$equinox) = @_;
    $$self{'coosys'} = [$coords, $equinox] if defined $coords; 
    return $$self{'coosys'};
}


sub showGIFs {
    my ($self,$flag) = @_;
    $$self{'showGIFs'} = $flag if defined $flag; 
    return $$self{'showGIFs'};
}
sub checkSize {
    my ($self,$flag) = @_;
    $$self{'checksize'} = $flag if defined $flag; 
    return $$self{'checksize'};
}
sub Tables {
    my ($self,$tables) = @_;
    $$self{'tables'} = $tables if defined $tables;
    return $$self{'tables'};
}
sub Products {
    my ($self,$products) = @_;
    $$self{'products'} = $products if defined $products;
    return $$self{'products'};
}
sub getallkeysfortable {
    my ($self,$table) = @_;
    return $$self{'products'}{$table}{"keys"};
}
sub getallsetdescsfortable {
    my ($self,$table) = @_;
    return $$self{'products'}{$table}{"prods"}{"set_descriptions"};
}
sub getallprodsforset {
    my ($self,$table,$key,$set) = @_;
    return $$self{'products'}{$table}{"prods"}{$key}{$set};
}
sub fileFilter {
    my ($self,$f) = @_;
    $$self{"filefilter"} = $f if defined $f;
    print "file filter $f added<br>" if defined $f && $::debug;
    return $$self{'filefilter'};
}
sub havelocaldata {
    my ($self,$flag) = @_;
    $$self{"localdata"} = $flag if defined $flag;
    return $$self{'localdata'};
}
sub numprods {
    my ($self,$flag) = @_;
    $$self{"numprods"}++ if defined $flag;
    return $$self{'numprods'};
}
sub rowid {
    my ($self,$flag) = @_;
    $$self{"rowid"}++ if defined $flag;
    return $$self{'rowid'};
}
sub gifstogo {
    my ($self,$flag) = @_;
    $$self{"gifstogo"}-- if $flag and $$self{"gifstogo"} !=0;
    return $$self{"gifstogo"};
}
sub transInfo {
    my ($self,$hashref) = @_;
    $$self{"transInfo"} = $hashref if defined $hashref;
    return $$self{'transInfo'};
}

sub Request {  #hash, so no duplicate requests
    my ($self,$req) = @_;
    $$self{"request"}{$req}++ if defined $req;
    #print "ProdMain:: adding request $req<br>" if $::debug;
    #Browse::Utils::DumpRef($self->{request}) if $::debug;

    return $$self{'request'};
}
sub addUrltoReq {
    my ($self,$url) = @_;
    #print "ProdMain:: addURL $url<br>" if $::debug;
    $self->Request($url);
}

#----------------------------------------------------------------------------
#  doProducts is a complete product displayer, but you can use it as an
#   example to roll your own.
#----------------------------------------------------------------------------

sub doProducts {

  my ($self,$in) = @_;
  #-------------------------------------------------------------
  #--- data products time tests
  #use Benchmark;
  #my $start_time = new Benchmark;
  
  my (@targets) = split("\0", $$in{"Target"});
  my %sets=();

  foreach my $key (keys %$in) {
     print "ProdMain:: key $key=>" . $$in{$key} . "<br>" if $::debug;
     if (my ($tab) = $key =~ /RetrieveSet_(.*)/) {
        push(@{$sets{$tab}},split("\0",$$in{$key}));
     }
  }

  print "ProdMain: getting RetrieveSets...." . scalar (keys(%sets)) . 
	"<br>\n" if $::debug;


  if ($::debug) {
     foreach (keys %sets) {
      print "$_ => ".join(',',@{$sets{$_}})."<br>\n"; 
     }
  }

  $self->Products($self->findProducts(\@targets,\%sets));
  $self->pageHeader($in);
  $self->instructions();
  $self->Tables( [sort keys %{$self->Products()}] );

  print "<br>ProdMain= tables: <br>\n" if $::debug;
  foreach my $table(@{ $self->Tables() }) {
    my $tableInfo = {Browse::HDB::TableInfo($table)};

    print "<br>ProdMain= looking at$table <br>\n" if $::debug;
    
    

    my $filter = $$in{"dpfilefilter_" . $tableInfo->{table_name}}; 

    #--- if no table table is associated with filter all tables
    #--- are meant to be filtered
    if (!$filter) {
       $filter = $$in{"dpfilefilter_"}; 
    }

    # remove everything other than numbers,letters, underscores, ; and *
    $filter =~ s/[^a-zA-Z0-9_\*\/\.\;]//g;  
    print "<br>filter=$filter for $table <br>\n" if $::debug;
  
    $filter =~ s/([^a-zA-Z0-9*;])/\\$1/g;  # put a slash before / and . 
    print "filter=$filter for $table <br>\n" if $::debug;
  
    $filter =~ s/\*/\.\*/g;  # convert * to .*
    print "filter=$filter for $table <br>\n" if $::debug;

    $filter =~ s/\;/|/g;  # convert ; to | (or)
    print "filter=$filter for $table <br>\n" if $::debug;

    #--- save filter to use in subclasses
    $self->fileFilter($filter);

    
    $self->doTable($table, $tableInfo);

    my $keys = $self->getallkeysfortable($table);
    foreach my $key (@$keys) {
        $self->doKey($table,$tableInfo,$key);

        my $set_descriptions = $self->getallsetdescsfortable($table);
        foreach my $set(keys %$set_descriptions) {

            my $vals = $self->getallprodsforset($table,$key,$set);
            
            #--- Sort by product name for display and retrieve page
            my @sortedvals = sort sortByType @$vals if defined(@$vals);

            #$self->doSet($$set_descriptions{$set}, ref($vals));
            $self->doSet($$set_descriptions{$set}, 
		defined($vals) && scalar(@$vals));

            foreach my $val (@sortedvals) {
	      $self->doProd($val);
            }
        }
        $self->closeRow();
    }
  }

  $self->finish($in);
  #-------------------------------------------------------------
  #--- data products time tests
  #my $end_time = new Benchmark;   # benchmark
  #my $difference = timediff($end_time, $start_time); 	#benchmark
  #print "<p>$::host";
  #print "<br> ", timestr($difference), "<br>" ;
  #-------------------------------------------------------------

  $self->pageFooter();


} #eosub doProducts

#----------------------------------------------------------------------------
#  Override these!
#----------------------------------------------------------------------------

sub pageHeader {Browse::Decor::Header("Data Products")}
#sub pageFooter {Browse::Decor::Footer()}
sub pageFooter {
  Browse::Decor::EmailFooter();
  Browse::Decor::EndTags();
}
sub instructions {}
sub closeRow {}
sub doTable {}
sub doKey {}
sub doSet {}
sub doProd {}
sub finish {}

#----------------------------------------------------------------------------
#  Shared functions used in both viewing and writing products
#----------------------------------------------------------------------------
sub findProducts {

    my ($self,$targets,$sets) = @_;
    my %allTables = ();

    # handle sets in xcorr
    foreach $_ (keys %$sets) {
       if ($_ =~ / x /) {
          foreach (@{$$sets{$_}}) {
             my @setarr = split(/\s/,$_);
             while (@setarr) {
                my $set = shift @setarr;
                my $table = shift @setarr;
                my $prefix = shift @setarr;
                push(@{$$sets{$table}},$set);
             }
          }
       }
    }

    my %seen={};
    foreach my $t(@$targets) {

        #$t = Browse::CGI::Decode($t);
        my ($tab, $key) = split('\|\|\|', $t, 2);

        my @tables = split(" x ", $tab);
        foreach (@tables) {
            $_ =~ s/ [^ ]$//; # Get rid of table prefix
        }

        my @xkeys   = split('\|\|', $key);

        # Get rid of null keys at end.
        while ($#xkeys >= 0 && length($xkeys[$#xkeys]) == 0) {
            pop(@xkeys);
        }

        # Add in keys
        foreach (@xkeys) {

            addKey(shift(@tables), $_, \%allTables, \%seen);
        }
    }

    foreach my $table(keys %allTables) {

        print "ProdMain table=$table<br>" if $::debug;
        my $dpquery = new Browse::DPQuery($table);
        $allTables{$table}{"prods"} =
           $dpquery->getDataProducts($allTables{$table}{"keys"},$$sets{$table});
    }
    return \%allTables;
}

sub addKey {

    my ($tab, $key, $allTables, $seen) = @_;

    if ($tab =~ /\// || $$seen{"$tab.$key"}) {
        return;
    }
    print "ProdMain addKey=$tab $key<br>" if $::debug;
    Browse::Utils::DumpRef($allTables) if $::debug;

    if (!defined($$allTables{$tab})) {
        $$allTables{$tab} = {};
        $$allTables{$tab}{"keys"} = [];
    }

    $$seen{"$tab.$key"}++;

    my $arr = $$allTables{$tab}{"keys"};
    push(@$arr, $key);
}

sub addSetstoProds {
    # may not be useful - add specific sets to the products hash structure
    # this is analogous to addSetstoReq
    #$self->addSetstoProds("heasarc_ascapublic","seq_id=12601000",["images"]);
    my ($self,$table,$key,$sets) = @_;
    $sets = [] unless $sets;
    my $currprods = $self->Products();
    my %sets = ($table => $sets);
    my $newprods = $self->findProducts(["$table|||$key"],\%sets);
    foreach my $set (keys %{$$newprods{$table}{prods}{$key}}) {
        $$currprods{$table}{prods}{$key}{$set} =
         $$newprods{$table}{prods}{$key}{$set};
        $$currprods{$table}{prods}{set_descriptions}{$set} =
         $$newprods{$table}{prods}{set_descriptions}{$set};
        push(@{$$currprods{$table}{keys}},@{$$newprods{$table}{keys}});
    }
    $self->Products($currprods);
}

sub addSetstoReq {
    # add prods for a single key in a single table (multiple sets)
    # example:
    #$self->addSetstoReq("heasarc_ascapublic","seq_id=12601000",["images"]);
    my ($self,$table,$key,$sets) = @_;
    $sets = [] unless $sets;
    my %sets = ($table => $sets);
    my $newprods = $self->findProducts(["$table|||$key"],\%sets);
    foreach my $set (keys %{$$newprods{$table}{prods}{$key}}) {
        foreach my $val (@{$$newprods{$table}{prods}{$key}{$set}}) {
           $self->doProd($val);
        }
    }
}


sub fileSize {
     
    my ($self,$file) = @_;

    my $size;
    if (!-e $file) {
        return 0;
    }

    if (-l $file) {
        $file = replaceLink($file);
    }
    if (!-d $file) {
        $size = (-s $file);
    } else {
        $size = dirSize($file);
    }

    return $size;
}


sub dirSize {

    my ($dir) = @_;

    if (!opendir(DIR, $dir)) {
        return 0;
    }

    my @files = readdir(DIR);
    closedir(DIR);

    my $file;
    my $total;

    foreach $file (@files) {
        if (substr($file, 0, 1) eq ".") {
            next;
        }
        $file = "$dir/$file";
        if (-l "$dir/$file") {
            $file = replaceLink($file);
        }
        if (-d $file) {
            $total += dirSize($file);
        } else {
            $total += ( -s $file);
        }
    }
    return $total;
}

sub replaceLink {

    my ($file) = @_;

    my $maxLink = 5;
    while (-l $file && $maxLink > 0) {
        $maxLink -= 1;
        my $nf = readlink($file);
        if (substr($file, 0, 1) ne "/") {
            my $lp = rindex($file, "/");
            $file = substr($file, $lp).$nf;
        } elsif (substr($nf, 0, 1) ne "/") {
            #--not sure this change will work for all linked files -lmm 10/07
            my $lp = rindex($file, "/");
            $file = substr($file, 0, $lp)."/$nf";
            #$file = substr($file, -$lp)."/$nf";
        } else {
            $file = $nf;
        }
    }
   return $file;
}

#--- applies user-specified file name filter to URL name in Data Products 
sub applyFileFilter {
   my ($self,$url) = @_;
   my $filter= $self->{filefilter};

   #print "ProdMain applyFileFilter filter=$filter<br>\n" if $::debug;
   
   if ($filter !~ /\w+/) {return 1;}  #-- no filter so return
   print "applying filter $filter to $url<br>" if $::debug;


   if ($url !~ /$filter/) {
      return 0;
   }
   return 1;
}



#---------------------------------------------------------------------------
#--- create a list of files in a directory
sub descendtree {

    my ($self,$url,$filelist) = @_;
    print "descending $url<br>\n" if $::debug;
    my $dir;
    #--- replace system name with local directory path
    $url=~ /ftp\:\/\/(heasarc|heasarcdev|legacy|olegacy).gsfc.nasa.gov\/(.*)/;
    $dir = "$2";
    $dir = $url if $dir !~ /\w/;
    $dir .= "/" if $dir !~ /\/+$/; 
    $dir = "/FTP/". $dir unless $dir =~/^\/FTP\//;

    if (opendir(DIR, "$dir")) {
        my @files = readdir(DIR);
        closedir(DIR);
	my $file;
	foreach $file (@files) {

	    next if $file =~ /^\./; # ignore hidden files

	    if (-d "$dir$file") {
                print "$dir$file is dir so descend again<br>\n" if $::debug;
	        $filelist = $self->descendtree("$dir$file/",$filelist);
	    } else { 
		push(@$filelist, "$dir$file");
                #if ($dir =~ /tdrss/i) { print "pushing $dir$file<br>\n";}
                print "pushing $dir$file<br>\n" if $::debug;
	    }
	}
    } else {
         Browse::Utils::Error("Error opening data products directory " . $dir);
    }
    return $filelist;
}
    

sub optsize {
# get the file size in the optimal units for printing
    my ($self, $size) = @_;   # assuming it is in bytes
    my $unit="";
    if ($size < 1024) {
            $unit = "B";
        } elsif ($size < 1022976) { #1022976 = 999 kB
            $size = int(($size+512)/1024);
            $unit = "kB";
        } elsif ($size < 1047527424) { #1047527424 = 999 MB
            $size = int( ($size+512*1024)/(1024*1024));
            $unit = "MB";
        } else {
            $size = int( ($size+512*1024*1024)/(1024*1024*1024));
            $unit = "GB";
        }
    return "$size $unit";
}

sub sortByType {
   $$a{dp_type} cmp $$b{dp_type};
}

sub noProductsMessage {
  my ($self) = @_;
    my $msg = "Data products are not available for one of the " .
	"following reasons:<p>" .
        "<ul><li>Data product selection may not be correct." .
	" Please go back and select at least one table row and at least " .
	"one data product type. </li> " .
        "<li> The data associated with selected row(s) or product type(s)" .
	" are not available because:<ul>" .
   	"<li> the observation has not taken place</li>" .
   	"<li> data have not been processed</li>" .
   	"<li> data are still proprietary </li></ul></ul><p>" .
        "Please consult the table documentation for information on " .
	"data availability.";
        Browse::Utils::Error("$msg");

}

1;

__END__

# $Log: ProdMain.pm,v $
# Revision 1.12  2007/10/03 19:57:42  lmm
# Tried to correct the replaceLink() to follow xte file links.
#
# Revision 1.11  2007/03/26 19:23:31  lmm
# Slight change to "Data Products not found" message
#
# Revision 1.10  2006/10/24 19:05:57  lmm
# Added Sort by product name (for Display & Retrieve).
# Made change to prevent set decriptions from appearing on Display
#   & Retrieve page when there are no data products for a given row.
# Added method noProductsMessage() that can be overwritten by subclasses.
#
# Revision 1.9  2005/11/21 20:12:15  lmm
# Added processing for a file filter that is applied to all results tables
#   (added for revised Text Table format).
#
# Revision 1.8  2005/09/19 15:27:52  lmm
# Changes made to defer file filter check to subclasses of ProdMain:
#   added new class method fileFilter(), relocated descendtree() to
#   here and made it class method, made applyFileFilter() a class method
#
# Revision 1.7  2005/06/23 19:22:27  lmm
# Some changes to the filter conversion to Perl regular expression
#
# Revision 1.6  2005/06/15 16:03:16  lmm
# Added new subroutine applyFileFilter() for new data products file name
#   filter option
#
# Revision 1.5  2004/12/15 20:44:04  lmm
# Added parameter to doSet() method to indicate number of products in set
#
# Revision 1.4  2004/10/04 21:03:50  lmm
# Moved optsize() from HProdGen.pm to make it available to derived classes
#
# Revision 1.3  2003/08/29 16:48:45  lmm
# Removed HEASARC footer
#
# Revision 1.2  2002/12/16 18:20:57  cwh
# Fixed bug where data sets were being ignored in cross-correlations.
# No longer decoding targets.
#
# Revision 1.1  2002/09/05 18:26:56  cwh
# ProdMain is a class which creates objects for displaying data products and
# for generating data product packages for retrieval.  HProdDisplay and
# HProdGen are provided as examples of display and generation classes which
# inherit from ProdMain.  The expectation is that other sites using this code
# will modify the HProd* modules for their needs.
#

=head1 NAME

ProdMain - parent class for product-display and product-generation objects.

=head1 SYNOPSIS

 use Browse::ProdMain;

 my($in) = Browse::CGI::ReadParse();
 my $dp = new Browse::ProdMain($in);
 $dp->doProducts($in);

=head1 DESCRIPTION

The ProdMain class creates the ProdDisplay and ProdGen objects. Which object
is created depends on the value of the $$in{'action'} parameter.
See Config.pm to view or change the mapping.

The doProducts() method is a complete product processor, but if you want 
more control you can directly call the methods used in doProducts().

=head1 OPTIONAL: USE METHODS DIRECTLY

These are the methods you can use if you choose not to use doProducts():

=head2 GET/SET THE PRODUCTS:

 $dp->Products($products); 
 $products = $dp->Products(); 

If given hashref $products (result of findProducts), sets the products.
If given no argument, returns the products hashref.

=head2 FIND THE PRODUCTS:

 $products = $dp->findProducts($targets,$sets); 

Where $targets is a ref to an array, where each element is:
     "table_name|||unique_key_string", i.e.:
     "heasarc_ascapublic|||seq_id=12601000"
And $sets is a ref to an array of short dp set names, i.e. "images".

Returns a ref to a hash with this structure:
 
 $products = { table_name => $tablehashref };
 $tablehashref = { "prods" => $prodref,
                   "keys"  => $keysref }
 $keysref = [ @unique_id_strings ]
 $prodref = { uniqueid => $setsref ,
              "set_descriptions" => $setdescsref }
 $setsref = { setname => $setprodref }
 $setprodref = [ @$prods ]
 $prods = { prodkey = prodval }

=head2 GET/SET THE PRODUCT GENERATION REQUEST:

 $dp->Request($singlerequest); 
 @requests = keys %{ $dp->Request() };

If given a single local filename or url, adds it to the request hash. 
A hash is used only to ensure that duplicate files/urls aren't requested.
If given no argument, the request hash is returned.  Take the keys to get
the array of requested files/urls.

=head2 ADD A URL TO THE PRODUCT GENERATION REQUEST:

 $dp->addUrltoReq($url);

Add a single url to the request hash.

=head2 ADD ALL PRODUCTS IN A SET TO THE PRODUCT HASH:

 $dp->addSetstoProds($table,$key,$sets);

Add all products in a dataset for a given unique row to the products hash.
$sets is an array reference where each element is a short dataset name.

=head2 ADD ALL PRODUCTS IN A SET TO THE PRODUCT GENERATION REQUEST:

 $dp->addSetstoReq($table,$key,$sets);

Add all products in a dataset for a given unique row to the request hash.
$sets is an array reference where each element is a short dataset name.

=head2 PRINT A PAGE HEADER:

 $dp->pageHeader();
 
Print the page header.  Overridden in subclasses.

=head2 PRINT INSTRUCTIONS:

 $dp->instructions();

Print instructions. Overridden in subclasses.

=head2 GET/SET THE TABLES:

 $dp->Tables($tables);
 $tables = $dp->Tables();

If given arrayref $tables, sets the tables.
If given no argument, returns the tables arrayref.
 
=head2 DO SOMETHING FOR EACH TABLE:

 $dp->doTable($table,$tableInfo);

Do something for the given table. Overridden in subclasses.
$table is the table name, $tableInfo is the result of Browse::HDB::TableInfo.

=head2 DO SOMETHING FOR EACH ROW OF THE QUERY RESULTS:

 $dp->doKey($table,$tableInfo,$key);

Do something for the given table. Overridden in subclasses.
$table is the table name, $tableInfo is the result of Browse::HDB::TableInfo.
$key is the unique key identifying this row.

=head2 DO SOMETHING FOR EACH DATA PRODUCTS SET:

 $dp->doSet($set);

Do something for the given data products set name. Overridden in subclasses.
$set is the short dp set name, i.e.: "images".

=head2 DO SOMETHING FOR EACH DATA PRODUCT:

 $dp->doProd($prod);

Do something for the given data product. Overridden in subclasses.
$prod is a ref to a hash containing information about the product.

=head2 CLOSE ROW PROCESSING:

 $dp->closeRow();

Do whatever needs to be done after each row of the query results. 
Overridden in subclasses.

=head2 FINISH:

 $dp->finish($in);

Do whatever needs to be done after processing the products structure.
Overridden in subclasses.
$in is the result of Browse::CGI::ReadParse().

=head2 PRINT A PAGE FOOTER:

 $dp->pageFooter();

Print a page footer.  Overridden in subclasses.
 
=head1 AUTHOR INFORMATION

High Energy Astrophysics Science Archive Research Center (HEASARC),
NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.

You are free to use this code as you wish, but unless you change
it significantly, please leave the above name & address intact.

=head1 BUGS

=head1 SEE ALSO

L<Browse::HProdDisplay>, L<Browse::HProdGen>

=cut


