# $Id: Results.pm,v 1.63 2002/12/16 18:33:44 cwh Exp $
# $Log: Results.pm,v $
# Revision 1.63  2002/12/16 18:33:44  cwh
# Added labels, ids, and scopes for section 508 compliance.
#
# Revision 1.62  2002/10/17 19:22:28  cwh
# Simplified column header code.
# Removed plotsql var.
# HTML improvements:
#  - added datahead class to col header cells
#  - added scope=col to col header cells (accessibility)
#  - added rules=cols so lines aren't ruled bt rows.
#  - added class=datatable to data table
#  - added title to data table (accessibility)
#  - added thead (accessibility) (tbody is in Display.pm)
#
# Revision 1.61  2002/09/13 20:48:19  cwh
# Encoding table name in tickTable call.
# No longer getting set description from splitting off the set string.
#
# Revision 1.60  2002/07/10 18:37:17  cwh
# Removed goHera function.  This is now done with the saveFile.pl script.
# Removed list of services.  Now using the Services::get_services_list func.
# Changed the calls to tickTable (see ValidateInput cvs log).
#
# Revision 1.59  2002/05/14 18:10:53  cwh
# Moved YourSearchWas to Display.pm.
# Uncommented Save to Hera.
#
# Revision 1.58  2002/04/11 18:12:55  cwh
# Fixed minor html problems.
#
# Revision 1.57  2002/03/22 23:01:00  cwh
# Moved Services column to the left.
# Minor html adjustments.
# Added SkyView: ROSAT All-Sky image to services menu.
# Removed kludge I had added to pass data set description.
# Added &nbsp;'s to the resolved coordinates in the YourSearchWas box.
#
# Revision 1.56  2002/03/18 02:06:30  cwh
# Don't display Services for cross-correlations, since they don't work yet.
#
# Revision 1.55  2002/03/17 18:52:47  cwh
# Deleted Directions(), MakeQueryHeaders(), DisplayLegend(), and DisplayLegend2().
# Added internal links on table names.
# Changed use of <dd><dt> to <div class="indent">
# Changed hard-coded style: background-color: pink to class="Vizier".
# Added Services column.
# Added &nbsp; between up and down arrows, and added line break before arrows.
# Added help links in action box.
#
# Revision 1.54  2002/02/14 18:26:29  cwh
# HTML cleanup: changed all <font> elements to <span> elements, added type=
#   "text/javascript" to <script> elements.
# Added table-level service menus.
# Added a new Save to Hera button which calls perl module instead of javascript.
# Print short table name instead of full name on xcorr results, but retain
#   unaltered long table name for later.
# Comment out all the javascript alerts.
# Set target for submit to new window.
#
# Revision 1.53  2001/12/18 15:50:07  cwh
# Added links to help for xoffset and tdelta.
# Added XML as a display mode.
#
# Revision 1.52  2001/12/11 19:50:58  cwh
# Now showing plot button for cross-correlation results.
#
# Revision 1.51  2001/12/06 16:58:02  cwh
# Encode entire table name in sort identifier - don't strip $::local_origin.
# No longer hardcoding the offset varname - using passed value.
# Don't strip prefix from sort field in cross-correlations results.
# Encode table name in req_table.
#
# Revision 1.50  2001/11/30 20:07:57  cwh
# Got rid of PrintHiddenInfo and PrintXText, since they aren't used anymore.
# Removed the poll, since we're forbidden by NASA to have surveys on websites.
# Added a couple of case-insensitive flags to pattern matches for Oracle.
#
# Revision 1.49  2001/11/08 20:28:41  cwh
# Added Browse:: to sub calls.
# Added a poll to ask users if which interface thye liked better.
#
# Revision 1.48  2001/10/25 22:28:18  cwh
# Changed labels on sort arrows.
# Changed label for View button to 'Preview and Retrieve'.
# Removed 'into a TAR file' text from beside 'Retrieve' button since made it
#   sound like it would results in a different thing than the Preview button.
# Changed the reporting of sort direction in search summary box.
#
# Revision 1.47  2001/10/25 19:49:50  cwh
# Changed url to w3hdprods so that the tar is always done without mod_perl.
#
# Revision 1.46  2001/10/25 19:15:36  cwh
# Changed _Offset to _Search_Offset for added clarity.
# Added sort arrows to _Search_Offset column.
# Added link to help from _Search_Offset column heading.
# Added hook for nontable display option.
# Added Heraform section to isolate form for use by Hera form manager.
#
# Revision 1.45  2001/10/04 22:15:52  cwh
# Added revised Hera functions.
#
# Revision 1.44  2001/09/19 16:55:28  cwh
# In YourSearchWas, output metaparam_start_times in iso format instead of MJD.
# In YSW, added line break after each param in a multiple param query.
#
# Revision 1.43  2001/09/15 14:53:07  cwh
# Rearranged buttons to make more sense.
# Got rid of 'show file sizes' and 'show GIF previews' options - now hidden vars
#   that are on by default.
# Added Cornerstone-style questions.
# YourSearchWas now gets displaymode passed in - offers users the ability to
#   redisplay the results in other formats (doesn't offer the current format).
#
# Revision 1.42  2001/08/28 19:26:18  cwh
# Moved up/down images to the sub where they are used.
# Changed hardcoded url in images to use $::images instead.
# Now using magnifying glass instead of right arrow.
# Added $table to ticktable name for data sets (but causes problems in xcorr)
# Removed dependence on CoordConv, now uses Astro lib.
# Not passing time in to YourSearchWas separately, and not handling it separately.
# In YourSearchWas, position comes without raf and decf.
# Simplified coord conversion in YourSearchWas.
# Reporting metaparam fields in italics in YourSearchWas.
#
# Revision 1.41  2001/08/08 21:36:29  cwh
# Added tablename to checkall boxes for data sets to make them unique.
#
# Revision 1.40  2001/08/08 20:59:12  cwh
# Passing in time.
# Added more complicated parsing of min/max/op, to reflect actual handling.
# Added time reporting.
#
# Revision 1.39  2001/07/31 14:53:53  cwh
# Added alt text to up/down arrows.
# Cleaned up table tags in search summary table.
#
# Revision 1.38  2001/07/23 20:06:33  cwh
# The 'All' checkboxes are now written by javascript so they are not printed
# when javascript is off or unavailable.
#
# Revision 1.37  2001/07/23 16:44:05  cwh
# Cleaned up the formatting of the 'Search radius used' string.
# Added the YourSearchWas() subroutine, which creates the search criteria summary
#   box.
#
# Revision 1.36  2001/07/09 15:31:37  cwh
# Modified to display help links properly for data products for cross correlations.
#
# Revision 1.35  2001/07/01 01:52:06  cwh
# When displaying units, change underscores to spaces and 'micro' to '&micro;'.
#
# Revision 1.34  2001/06/20 22:00:27  cwh
# Now set a javascript variable to 1/0 to indicate whether to check in
#   pageCheck to see if at least one dataset is selected.
#
# Revision 1.33  2001/06/19 16:26:36  tam
# Fixed bug that prevented VizieR units from showing up
# in tabular display.
#
# Revision 1.32  2001/06/13 20:02:04  cwh
# increased font size on param units, and got rid of a hack dealing with Viz.
#
# Revision 1.31  2001/05/31 20:48:38  cwh
# Now passing a xcorr flag to ColumnHTML.  This solves a problem where columns
# which started with a single letter and an underscore (ie e_Count) were being
# interpreted as cross-correlation col headings and the initial letter was being
# stripped off.  The column was then being linked to a nonexistent $doc, and
# it affected all the column headings after it.
#
# Revision 1.30  2001/05/17 20:03:12  cwh
# Changed the name of the checkall checkbox for the data sets list - now calls
# tickTable with the string 'RetrieveSet', with no table specified.
#
# Revision 1.29  2001/05/17 19:25:01  cwh
# Plot and Cross corr buttons are not offered for results of cross-correlations
# for now.
#
# Revision 1.28  2001/05/17 18:20:41  cwh
# Prints Correlation radius and/or delta time used.
# Adds 'recno' as the Vizier unique id.
#
# Revision 1.27  2001/05/17 02:48:54  tam
# Got rid of unused cross-correlation routines.  Added support for
# new plotting and correlation links.
#
# Revision 1.26  2001/05/03 19:03:18  cwh
# Changed heasarc_ to $::local_origin.
# Added Tom's sub DisplayLinks.
#
# Revision 1.25  2001/04/25 18:41:03  cwh
# Tom pretty much completely rewrote this.
#
# Revision 1.24  2001/04/09 19:35:53  cwh
# Ed's fix: added call to getKey.  Master tables in text display weren't getting
# their unique id right.
#
# Revision 1.23  2001/04/04 17:04:53  cwh
# Long parameter names are split when printing them in column headers.  After
# every 8 characters, a break is inserted before the next underscore.
#
# Revision 1.22  2001/04/03 16:19:54  cwh
# - added space after alt tag on up/down arrows.
# - made Show file sizes and Show GIF previews checked by default.
#
# Revision 1.21  2001/03/12 16:39:41  cwh
# Got rid of extra <center>...</center> that was acting up when no matches
# found.
#
# The checkall box with the data sets is now checked by default, since all the
# data sets are checked.  Also each checkall box has its own name, since it has
# to operate on only that one set.
#
# Revision 1.20  2001/01/22 20:41:15  cwh
# Pulled the xcorr and plot buttons out of PrintTableName and made a new sub
# called PrintTableButtons, which prints them and also the Display button.
#
# Changed 'select all' to 'select/clear all' on top of category listing.
#
# Revision 1.19  2001/01/02 23:26:15  cwh
# Added Hera section, commented out.
#
# Added target=helpwin for all help links.
#
# Added check for $hits before printing plot and cross-corr buttons.
#
# Cleaned up the actions boxes.
#
# Added showgifs checkbox.
#
# Got rid of 'list all products for selected obs'.
#
# Cross-corr is no longer a checkbox option - it is a submit button.
# Tablename is passed as part of the submit button name.
#
# Categories of data products are now a checkbox list instead of scroll menu.
#
# Revision 1.18  2000/08/04 14:52:05  saima
# Added $textdisplay to list of parameters passed to
#   Results::PrintTableName() so that plot/xcorr icons dont get
#   displayed in textmode.
#
# Revision 1.17  2000/04/25 22:15:18  saima
# The 'All' checkbox is added only if more than one result exists.
#
# Revision 1.16  2000/04/25 19:28:06  saima
# Added an 'All' checkbox on Results Page so that if a user selects it,
#   all 'Target' rows are selected for that table. This feature works
#   for JavaScript 1.2 enabled browsers.
#
# Revision 1.15  2000/03/10 16:55:23  saima
# Made name change to Browse. Added a new icon for plotting and added
#   an icon for cross-correlation next to catalog
#   name. PrintHiddenQueryInfo() no longer prints metabase information
#   such as default, notdefault.
#
# Revision 1.14  2000/01/27 18:34:18  saima
# Modified so that if the flag UKeyDisplay is on, then unique key
#   columns are displayed.
#
# Revision 1.13  2000/01/11 18:19:56  saima
# Made minor modifications to PrintText().
#
# Revision 1.12  1999/12/20 17:59:50  saima
# Modified the PrintText() display for batch requests.
#
# Revision 1.11  1999/09/24 18:49:40  saima
# Sends $hits to Results::PrintTableName(). Also, code checks to make
#   sure table has results before displaying the plot icon.
#
# Revision 1.10  1999/09/08 17:37:26  saima
# Modified link to plot icon.
#
# Revision 1.9  1999/09/07 19:08:09  saima
# Added the plotting icon after the table name. Also, fixed
#   a bug in PrintXCorrelation() where the filter information was
#   not being saved in the page. Fixed another bug in PrintXText()
#   where the text results were not being displayed correctly.
#
# Revision 1.8  1999/08/26 19:35:35  saima
# Added an icon in PrintTableName() which brings up the W3Browse
#   Plotting window.
#
# Revision 1.7  1999/05/27 17:18:09  tam
# Major changes to handle new features in cross-correlations.
#   Added check for foreign table before displaying sort arrows.
#
# Revision 1.5  1999/03/19 17:22:40  saima
# Code now truncates to $maxlimit for textdisplay option as well.
#
# Revision 1.4  1999/03/18 20:46:00  saima
# Modified cross-correlations results display so that 2000 is not the
#   display limit anymore.
#
# Revision 1.3  1999/01/07 20:49:43  saima
# Removed reference to $sorturl. Sorts from the results page are no
#   longer server redirects.
#
# Revision 1.2  1998/12/28 19:04:31  saima
# Fixed PrintHiddenQueryInfo() so that the hidden parameters dont have
#   to be CGI Encoded.
#
# Revision 1.1  1998/12/23 17:06:44  saima
# User can now set the max number of results displayed. Added
#   PrintHiddenParamInfo() so that parameter query information can be
#   saved. Also, modified PrintHiddenParams() and PrintHiddenQueryInfo()
#   for new interface.
#
# Revision 1.41  1998/08/03 21:59:58  saima
# Replaced the form name 'target' with 'Target'. The name 'target' gives
#   problems with JavaScript on Netscape 2.
#
# Revision 1.40  1998/07/24 20:11:41  saima
# Minor fix to the batch interface output.
#
# Revision 1.39  1998/07/15 19:06:24  saima
# The routine PrintBatch() was replaced by PrintText(). PrintText()
#   displays query results in text only format. This routine is used by
#   W3Browse as well as the batch interface. The PrintXText() routine
#   displays cross correlation results in text format.
#
# Revision 1.38  1998/06/12 19:46:09  saima
# Added JavaScript code for validating user input. Also added alt image
#   tags for arrow images.
#
# Revision 1.37  1998/05/12 16:29:41  saima
# Minor change in printing cross-correlation results.
#
# Revision 1.36  1998/05/11 20:23:32  saima
# Minor tweaks in cross-correlation display.
#
# Revision 1.35  1998/05/07 20:15:57  saima
# Rewrote PrintXCorrelation() completely so that results are displayed
#   in the HTML table layout instead of using <PRE>. Results are aligned
#   much neater and right arrows are displayed to allow expansion of
#   each table row. Also, made changes necessary so that the sort arrows
#   are not displayed if only 1 result row is found.
#
# Revision 1.34  1998/04/21 18:02:24  saima
# Fixed a bug where double spaces were being replaced to single spaces
#   in unique key data.
#
# Revision 1.33  1998/04/08 17:35:24  saima
# Fixed minor bug so that the '/' is recognized by the unique key stuff.
#
# Revision 1.32  1998/03/19 22:47:59  saima
# Modified necessary changes to display 25 rows at a time.
#
# Revision 1.31  1998/03/17 20:27:50  saima
# Fixed another minor cross correlation display problem.
#
# Revision 1.30  1998/03/17 20:03:37  saima
# Minor tweaks to cross correlation display.
#
# Revision 1.29  1998/03/11 20:39:43  saima
# Rewrote most of the code for the new look of Results page. All
#   results are now displayed in HTML table layout. Added arrow buttons
#   to allow sorting by parameter name. Reduced the amount of hidden
#   information that goes in each result row so that the page
#   size is reduced by a large fraction. Added new routines to display
#   set information, and display user options. Added a routine that
#   is called to print batch query results.
#
# Revision 1.28  1997/11/25 23:05:14  saima
# Added -T switch to perl version.
#
# Revision 1.27  1997/08/26 19:55:11  saima
# Fixed a bug related to decoding data product tag names with special
#   characters.
#
# Revision 1.26  1997/08/19 22:04:11  saima
# Removed new icon, the 'heasarc_' prefix is not added to the table name any more.
#
# Revision 1.25  1997/07/23 15:19:15  tam
#  Minor change to PrintTableName changing \n in <pre> environment
#   to <br>.  This fixes problems with Batch interface.
#
# Revision 1.24  1997/07/01 19:39:10  saima
# Fixed a minor bug in displaying "Entries" rather than "ntries" in batch mode.
#
# Revision 1.23  1997/05/01 18:21:19  sabol
# Cleaned up the code slightly. Fixed log entries. Fixed minor HTML error.
#
# Revision 1.22  1997/04/28 14:32:07  tam
# Changed PrintXCorrelation to print header information only once.
#   Also, changed method used to pass cross-correlation results to
#   PrintXCorrelation. A single array contains results for all
#   correlating rows and indices to the appropriate current
#   rows are provided.
#
# Revision 1.21  1997/04/17 23:27:18  sabol
# Move code that HTML-ized the query headers to MakeQueryHeaders(). Print()
#   now accepts an optional $entry argument for specifying the query info
#   more distinctly. Print() now uses CGI::Encode and HTML::Encode
#   thoroughly. Added PrintXCorrelation(), PrintHiddenParams(), and
#   PrintHiddenQueryInfo() (and removed hidden parameter stuff from
#   Footer()).
#
# Revision 1.20  1996/12/20 20:51:56  sabol
# Moved all routines into a separate package namespace (Results). Removed
#   "Results" from all of the function names. Renamed Results routine as
#   Results::Print. Results::Header now uses a relative URL instead of an
#   absolute one for specifying the CGI action URL. The URL for the
#   SEARCH_OFFSET column in the results goes to the W3Browse Help file instead
#   of the table documentation URL. Results::PrintTableName now also prints the
#   table description and the default search radius. Added support for the "List
#   Individual Data Products (Selected Sets)" and "Filter Query Results" options
#   in Results::Footer.
#
# Revision 1.19  1996/10/28 18:04:33  sabol
# Updated to Module Author designation to reflect the near total rewrite
#   accomplished by this programmer.
#
# Revision 1.18  1996/10/25 21:51:52  sabol
# Comprehensive changes to make module compliant with Perl 5's strict compiler
#   directive. No more global variables!!! (Well, except for a couple defined
#   in w3config.pl, but that's it!)
#
# Revision 1.17  1996/09/19 15:11:31  sabol
# Minor cosmetic change.
#
# Revision 1.16  1996/09/17 20:42:22  sabol
# Minor cosmetic change to data products retrieval instructions.
#
# Revision 1.15  1996/09/16 20:45:03  sabol
# If a query returns zero rows on a table that supports the Data Products
#   Layer, W3Browse no longer puts it in the list of tables that do not support
#   the Data Products Layer.
#
# Revision 1.14  1996/09/13 21:19:22  sabol
# Remove underlines from data products set names for display to the user.
#   Print a blank line between data products set select/list boxes.
#
# Revision 1.13  1996/09/06 19:50:13  sabol
# Minor change to encode plus signs as hex values in checkbox values.
#
# Revision 1.12  1996/08/26 22:44:48  sabol
# Extensive changes to support Data Products Layer. Minor cosmetic changes.
#   Improved error messages.
#
# Revision 1.11  1996/06/25 21:00:43  sabol
# Updated all source code to be Perl 5-compatiable.
#
# Revision 1.10  1996/05/07 15:21:19  sabol
# W3Browse now displays modified Julian dates in ISO-standard year-month-day
#   format. Created version history file and added a link to it from each
#   W3Browse page. Centralized all source code that interacts with HDBmc.
#   Made the source code more modular with more reusability. Better security
#   on all HDBmc calls. Reduced the number of HDBmc calls in order to do an
#   object name/coordinate search, resulting in better efficiency and faster
#   response. Users can now select multiple missions on the main page and
#   still be able to do a parameter query. Now uses table_document_url ZZGEN
#   parameter for constructing all help links in the parameter query forms.
#   The Fields and GIF Image Display Limit pop-up menus on the parameter
#   query form now retain state between query builds.
#
# Revision 1.9  1996/03/01  16:56:11  kressel
# More work on submit buttons. Including MailMessage routine.
#   Some small changes to aesthetics.
#
# Revision 1.8  1996/02/26  18:12:14  kressel
# Lots of small fixes to display of pages.
#
# Revision 1.7  1996/02/16  14:39:03  kressel
# Resurrecting w3print to recover from unintended delete from
#   repository.
#
# Revision 1.5  1996/02/16  13:52:23  kressel
# Workaround for Mosaic - removed Location: http:// ....w3nprep in
# w3nquery.
#   Changed tabs to spaces in w3browse display of observatories.
#   Only displaying ResultsFooter when necessary.
#   Fix for HDBfilter calls for tables with no coordinate parameters.
#   Fix for virtual parameters with leading @.
#
# Revision 1.4  1996/02/07  17:45:53  kressel
# Replaced gif buttons in Param Query with HTML buttons to standardize
#   the GUI. Removed w3defpars, replaced by GetParams in earlier
#   revisions.
#
# Revision 1.2  1995/12/27  17:27:29  kressel
# Redo of Initial CVS, to exclude binaries; and
#   GetParams.pl replaces w3defpars.pl
#   Presence of decimal point in 'ra' field implies Decimal degrees.
#   Have high level of confidence with handling of Coordinates now.
#   End of major version for December 95.
#
# Revision 1.1.1.1  1995/12/19 19:38:05  kressel
# W3Browse Initial CVS
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    Results.pm
# Module Authors: Edward J. Sabol, Hughes STX; Saima Zobair (Raytheon ITSS)
# Date:           October, 1996
# Project:        Browse : HEASARC
#-----------------------------------------------------------------------#
# Module Description:
#
# The routines in the module comprise most of the necessary steps for
# displaying the results of a database query to the user in a consistent
# style.
#
#=======================================================================#

use strict;

package Browse::Results;
use Browse::Services;




#------------------------------------------------------------------------
sub Header {

    my($op,$cols) = @_;
    Browse::Decor::Header("results",$op,$cols); #dont close table, cols
    # This subroutine initiates the form tag on Results page
    #print "\n<form method='POST' target='_blank' name='w3hdprods' action='/$::cgi/w3hdprods.pl'>\n";
    print "\n<form method='POST' name='w3hdprods' action='/$::cgi/w3hdprods.pl'>\n";

} #eosub--Results::Header
#------------------------------------------------------------------------
sub PrintTableName {

    # This subroutine simply prints out the name of the table with hyperlink.
    my($table, $table_info, $radius, $textdisplay, $xtra) = @_;
    
    my($desc) = $table_info->{'table_description'};
    my($url)  = $table_info->{'table_document_url'};
    my $radunit = "&deg;";
    if ($radius) {
        if ($radius =~ /Default/) {
            $radius = $table_info->{"default_search_radius"};

            # Radius might refer to a field in the table.
            if (substr($radius, 0, 1) eq '@') {
                $radius = "Field(".substr($radius,1).")";
            } else {
                $radunit = "'";
            }
        } else {
            $radius = 60*$radius;
            $radunit = "'";
            if ($radius<1) {
                $radius = 60*$radius;
                $radunit = "\"";
            }
        }
    }

    my $stable = Browse::Display::chopTable(undef,$table);
    my $tablest;
    if ($stable =~ / x /) { #cross-corr
       my ($url1,$url2) = split('\|',$url);
       my ($desc1,$desc2) = split(' with | against ',$desc);
       $desc1 =~ s/^(.* Correlation of)//i;
       $tablest = "<a name='$stable'> </a><strong>$1 <a target='helpwin' href='$url1'>$desc1</a> with <a target='helpwin' href='$url2'>$desc2</a> ($stable)</strong>"; 
    } else { #not cross-corr
       $tablest = "<a name='$stable'> </a><strong><a target='helpwin' href='$url'>$desc ($stable)</a></strong>\n";
    }

    print "<hr>\n";
    print $tablest;
    print "<div class='indent'>\n";

    print "$xtra<br>" if $xtra;

    print "Search radius used: ".sprintf("%.2f", $radius)." $radunit<br>\n" if ($radius);
    print "Correlation radius used: ".sprintf("%.2f",$table_info->{'xradius'})."'<br>\n" if ($table_info->{'xradius'});

    my $xtime = $table_info->{'xtime'};
    if (defined $xtime) {
        my $unit = "day";
        if  ($xtime >1) {
           $unit = "days";
        } elsif ($xtime <1) {
           $xtime = $xtime * 24;
           $unit = "hours";
        }
        print "Correlation delta time used: $xtime $unit<br>\n";
    }
    print "</div>\n";
} #eosub--Results::PrintTableName
#------------------------------------------------------------------------
sub PrintTableParams {

    # This subroutine prints the result parameters and arrows that allow 
    # sorting by parameter name. 
    my($table,          # Table results are for.
       $table_info,     # Metainfo about table
       $sortable,       # Can we sort (and in which directions)
       $fields,         # Fields to be displayed
       $unit,           # Unit information
       $fskip,          # Field indices to skip
       $offsetvar       # the name of the offset variable
      ) = @_;

    my $no_arrow = $$table_info{"foreign_table"};

    my $fld;
    my $skip = 0;
    my $html;
    my $fldUrl;
    my $doc = $$table_info{"table_document_url"};
    my %prefixes;
    my %doc;

    foreach $fld(@$fields) {
        # get all the prefixes for later
        if ($fld =~ /^(._)/) {  #has a single-letter prefix (hope it's a xcorr)
            $prefixes{$1}=1;
        }

        if ($fld =~ /as (.+)$/) {
            $fld = $1;
        }

        if ($fld =~ /^w3_/i) {
            $skip += 1;
            next;
        } else {
            #last;
        }
    }


    # get docs for each prefix
    my $xcorr=0;  # cross-correlation flag
    if ($doc =~ /\|/) {  # cross-corrs have '|' in the doc
       $xcorr=1;
       my (@docs) = split('\|',$doc);
       foreach (sort keys %prefixes) {
         $doc{$_} = shift @docs;  # assuming docs are appended in order originally
         print "doc{$_} = $doc{$_}<br>\n" if $::debug;
       }
    }
my $wholetable;
    if ($skip > 0) {
        # Leave a space for the check box.
         $wholetable = Browse::CGI::Encode($table);
         $table =~ s/$::local_origin//gi;
        ($table) = split(" ", $table);
         #my $temp = "<script type='text/javascript' language='JavaScript1.2'>\n<!-- \ndocument.write (\"<input type='checkbox' id='allobs_$table' name='allobs_$table' onClick=tickTable(this.form.Target,this.checked,'".Browse::CGI::Encode($table)."')><label for='allobs_$table'>All</label>\"); \n// --> \n</script>\n ";
         my $temp = "<script type='text/javascript' language='JavaScript1.2'>\n<!-- \ndocument.write (\"<input type='checkbox' id='allobs_$table' name='allobs_$table' onClick=tickTable(this.form.Target,this.checked,'$table')><label for='allobs_$table'>All</label>\"); \n// --> \n</script>\n ";
        $html .= "<th class='datahead' valign='bottom'><label for='Target'>Select</label><br>$temp</th>\n";
    }

    my $bg;
    if ($table =~ /\//) {   # if there is a '/' in the name, must be VizieR
        $bg = "class='Vizier'";
    }
    unless ($xcorr) {
       #$html .="<th class='datahead' scope='col' valign='top' $bg><a href='$::w3bhelp#Services'>Related Tables</a></th>\n";
       $html .="<th class='datahead' scope='col' valign='top' $bg><a href='$::w3bhelp#Services'>Services</a></th>\n";
    }

    my $i;
    for ($i=$skip; $i <= $#$fields; $i += 1) {

        # Did we ask to skip this field?
        if ($fskip && grep($_ == $i, @$fskip)) {
            next;
        }

        $fld  = $$fields[$i];
        my $unt = $$unit[$i];

        if (($fld =~ /^(._)/) && $xcorr) {   #cross-corr
            $doc = $doc{$1};
        }

        $html .=ColumnHTML($wholetable, $fld, $unt, $doc, $no_arrow, $sortable, $bg,$xcorr,$offsetvar);

    }

    print "\n<table border='1' rules='cols' class='datatable' title='Query results for ".$$table_info{"table_description"}."'><thead>\n<tr $bg>$html</tr>";

    return $skip;

} #eosub--Results::PrintTableParams
#------------------------------------------------------------------------
# Get the HTML for a single column header.
sub ColumnHTML {
my($upgif)  ="'$::images/up.gif' height='15' width='15' border='0' ";
my($downgif)="'$::images/down.gif' height='15' width='15' border='0' ";

    my ($table, $field, $unit, $doc, $no_arrow, $sortable, $bg,$xcorr,$offsetvar) = @_;

    if ($unit) {
        $unit =~ s/_/ /g;
        $unit =~ s/micro/&micro;/g;
    }
 
    my ($splitfield) = $field; # must be before $field is altered in next block
    #remove prefix from col name for cross-corr
    (my $stripfield = $field) =~ s/^._// if $xcorr;
    $field =~ s/^(.)_/$1./ if $xcorr;
    #$field =~ s/^._// if $xcorr;
    my $stripfield = $field unless $stripfield;

    my $html = "  <th scope='col' class='datahead' valign='top' $bg>";
    $sortable ='' if $no_arrow;
    
    # change underscores to spaces to shorten column widths.
     $splitfield =~ s/_/ /g;
    if ($field !~ /$offsetvar/i and substr($field, 0, 1) eq "_") {
        $html .= $splitfield;
    } else {
        my $fldUrl = "<a target='helpwin' href=";
        if ($field =~ /$offsetvar/i) {
            $fldUrl .= "'$::w3bhelp#distance_from_center'";
        } elsif ($field =~ /xoffset/i) {
            $fldUrl .= "'$::w3bhelp#xoffset'";
        } elsif ($field =~ /tdelta/i) {
            $fldUrl .= "'$::w3bhelp#tdelta'";
        } else {
            $fldUrl .= "'$doc#$stripfield'";
        }
        $fldUrl .= ">$splitfield</a>";
        $html  .= $fldUrl ."<br>";
    }
    if ($sortable =~ /down/i) {
        $html .= "<input type='image' alt='Sort (1,2,3)' title='Sort (1,2,3)' ".
                 "src=$downgif name='imgSort|down|$table|$field'>";
    }
    if ($sortable =~ /up/i) {
        $html .= "<input type='image' alt='Reverse Sort (3,2,1)' ".
                 "title='Reverse Sort (3,2,1)' ".
                 "src=$upgif name='imgSort|up|$table|$field'>";
    }
    $html .= "&nbsp;$unit</th>\n";
    return $html;
}


#------------------------------------------------------------------------
sub DisplayOptions {
    my ($table,$coords,$dp) = @_;
    (my $stable = $table) =~ s/$::local_origin//g;

    print "<span class='large'>Further Actions:</span><p>";
    print "Do you want to <input type='submit' name='Plot $table' value='Plot'> your $stable results? <a target='helpwin' href='$::w3bhelp#Java_Plot'>(help)</a><br><br>\n" unless ($table =~ / x / and $table =~ /\//);
    print "Do you want to <input type='submit' name='Correlate $table' value='Cross-correlate'> your $stable results with another catalog or table? <a target='helpwin' href='$::w3bhelp#cross_query'>(help)</a><br><br>\n" if $table !~ / x /; 
    print qq|Do you want to <input type='submit' name="files" onClick="setcheckDP(0);return(pageCheck())" value="Display"> all the columns for the rows selected above? <br><br>|; 

unless ($table =~ / x /) {
   print "Do you want to query other services for the rows selected? <a target='helpwin' href='$::w3bhelp#Services'>(help)</a><br>";
   print "<dl><dd><label for='Services$table'>Services:</label><br><select multiple size='5' id='Services$table' name='Services2'>\n";

   my $service_list = Browse::Services::get_services_list($coords,$dp);
   foreach (@$service_list) {
     (my $s=$_)=~s/ /_/g;
     print "<option value='$table $s'> $_";
   }

   print "</select>\n";
   print "<input type='submit' name='GoService' value='GO'></dd></dl><br><br>\n";
}

}#eosub--Results::DisplayOptions 
#------------------------------------------------------------------------
sub DisplaySets{

   # This subroutine displays set information for a table.
   # Function inputs: 
   #    $sets      -  hash containing data products sets for a table
   #    $table_info  -  hash containing table information 
   my($sets, $table_info) = @_;

   my($url)   = $table_info->{'table_document_url'}; 
   my($table) = $table_info->{'table_name'}; 
   my $xtable = $table;
   $xtable =~ s/$::local_origin//gi;

   return if !$sets;

   print "DisplaySets : table : $table, $xtable<br>\n" if ($::debug);
   print<<EOT;
       <span class='large'>Are you interested in data products?</span><br>
       <div class='small'><ol>
          <li><strong> Select the checkboxes for the rows of interest above,</strong></li>
          <li><strong> un-check any data products you are not interested in:</strong></li>
       </ol></div>
       <table>
EOT

   if ($url =~ /\|/ and $xtable =~ / x /) {
      my @urls = split('\|',$url);
      my @xtables = split(' x ',$xtable);
      print "  <tr><td colspan='2'><b>Data Products available for";
      my $i;
      for ($i=0,$i<=$#urls) {
            print " <a target='helpwin' href='$urls[$i]#dp_sets'>$xtables[$i]</a>" unless $urls[$i] =~ /$::vizier/;
            print " $xtables[$i]" if $urls[$i] =~ /$::vizier/;
            $i==$#urls ? print "<br>" : print ",";
            $i++;
      }
      if ("@urls" =~ /$::vizier/) {
         print "(No data products are available for Vizier tables.)<br>\n";
      }
   } else {
      print "  <tr><td colspan='2'><b><a target='helpwin' href='$url#dp_sets'>".
            "Data Products</a> available for $xtable </b><br>\n" ;
   }
   print <<EOT;
    <script type="text/javascript" language="JavaScript1.2">
    <!-- 
    document.write ('<input type="checkbox" checked name="checkall$table" id="checkall$table" onClick="tickTable(this.form.RetrieveSet_$table,this.checked,\\\'\\\')"> <b><label for="checkall$table">All</label><\/b><br>');
    // -->
    </script>
EOT

    my $set;
    foreach $set (sort keys %$sets){
     #   # e.g: "gif_images [ROSAT GIF Images]" or
     #   #      "gif_images [ROSAT GIF Images] heasarc_rosmaster a "
     #   ($set, my $desc, my $extra) = $set =~ /(.*)\s*\[(.*)\]\s*(.*)/;
     #   (($table, my $prefix) = $extra =~ /\s*(\w*)\s*(.)/) if $extra;
        my $desc = $$sets{$set};
        my($setprint) = $set;
        $setprint =~ s/_/ /g;
     #   $desc =~ s/[\[\]]//g;
        print "    <input type='checkbox' checked name='RetrieveSet_$table' id='RetrieveSet_${table}_$setprint' value='$set'><label for='RetrieveSet_${table}_$setprint'>$desc ($setprint)</label><br>\n";
    }
   print<<EOT2;
        <div class="small"><ol start='3'><li><strong> then click a button below.</strong></li></ol></div>
       </td></tr>
       <tr><td align='right'><input type='submit' name="files" onClick="setcheckDP(1);return(pageCheck());" value="Preview and Retrieve"></td><td> data products for selected rows
       <input type='hidden' name='CheckSize' value='on'>
       <input type='hidden' name='showgifs' value='on'>
       </td></tr>
       <tr><td align='right'><input type='submit' name="files" onClick="setcheckDP(1);return(pageCheck())" value="Retrieve"></td><td> data products for selected rows</td></tr>
       <tr><td align='right'><input type='submit' name='Save to Hera' value='Save to Hera'><br><a target='helpwin' href='http://heasarc.gsfc.nasa.gov/docs/software/hera.html'>What is Hera?</a></td><td>data products for selected rows</td></tr>
        
EOT2
    print "    </table>\n";
} #eosub--Results::DisplaySets
#------------------------------------------------------------------------
sub Footer {

   # This subroutine displays the footer for the Results Page.
   my($dp_on, $in) = @_; 

   print "</form>\n"; 
   if ($dp_on){
     print "<em>The retrieval time for data is dependent upon the number of \n" .
         "entries selected from the above query. We recommend reducing \n" .
         "the above list to a minimum before proceeding.</em>\n<hr>\n" 
   }else{
     print "<hr>\n";
   }

} #eosub--Results::Footer
#------------------------------------------------------------------------

sub DisplayLinks {

    my ($meta) = @_;

    my $tableinfo = $$meta{"tableinfo"};
    my $paraminfo = $$meta{"paraminfo"};

    my $ra  = $$tableinfo{"right_ascension"};
    my $dec = $$tableinfo{"declination"};
    $ra  =~ s/^\@//;
    $dec =~ s/^\@//;

    my @numeric;
    foreach my $fld (keys %$paraminfo) {
     next if ($fld =~ /^\#/ || $fld =~ /^w3_/i);
     push(@numeric, $fld) if $$paraminfo{$fld}{"parameter_format"} !~ /^char/i;
    }

    my $numstr = join(",", @numeric);

    my $req = Browse::CGI::Encode($$meta{"serialization"});
    my $table = Browse::CGI::Encode($$tableinfo{"table_name"});
    print "<input type='hidden' name='req_$table' value='$req'>\n";
    print "<input type='hidden' name='ra_$table' value='$ra'>\n";
    print "<input type='hidden' name='dec_$table' value='$dec'>\n";
    print "<input type='hidden' name='numeric_$table' value='$numstr'>\n";

} #eosub--Results::DisplayLinks

#------------------------------------------------------------------------


sub Heraform {
  my($input) = @_;
  print <<EOT;
<form name="HEASARC-control">
<input type='hidden' name="url" value="/$::cgi/w3query.pl">
<input type='hidden' name="vars" value="$input">
<input type='hidden' name="method" value="POST">
</form>
EOT
}

#########
#  EOF  #
#########
1;
