# $Id: Services.pm,v 1.11 2006/10/24 17:05:55 lmm Exp $
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    Services.pm
# Module Author:  Christina Heikkila, SSAI
# Date:           Dec 2001
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
# Handles the querying of other services from Browse results.
#
#=======================================================================#

use strict;
package Browse::Services;

#-------------------------------------------------------------------------

sub get_services_list {
 # input: flag for coordinates, flag for data products
 # output: ref to a list of valid services.

 my ($coordsflag,$dpflag) = @_;
 my @services=();

 if ($coordsflag) {
    @services = ("NED",
                 "SIMBAD",
                 "SkyView:ROSAT All-Sky",
                 "SkyView:DSS",
                 "CoCo",
                 "Viewing",
                 "nH",
                 "X-ray Background Tool",
                 "Abstracts",
                );
 }

 # nothing dp-dependent right now
 #if ($dpflag) {
 #   push(@services,"placeholder");
 #}

  return \@services;
} #eosub-- get_services_list

#-------------------------------------------------------------------------

sub get_coords {
 # input: table name, unique key string
 # output: ra,dec in decimal J2000
 #print "getting coords<br>\n" if $::debug;

 my ($table,$keystr)  = split('\|\|\|',$_[0]);

        my @tables = split(" x ", $table);
        # Get right of table prefix
        foreach (@tables) {
            $_ =~ s/ [^ ]$//;
        }

        my @xkeys   = split('\|\|', $keystr);

        # Get rid of null keys at end.
        while ($#xkeys >= 0 && length($xkeys[$#xkeys]) == 0) {
            pop(@xkeys);
        }

        # Add in keys
       # foreach (@xkeys) {
       #     addKey(shift(@tables), $_, \%allTables, \%seen);
       # }
$table=$tables[0];
$keystr=$xkeys[0];

# print "table=$table, keystr=$keystr<br>\n";
 my $fieldStr ='';

my %tinfo=();
       %tinfo = Browse::HDB::TableInfo($table);
 unless ($keystr =~ /=/) {
       $fieldStr = $tinfo{"unique_key"};
       if (!$fieldStr) {
           Browse::Utils::Error("Unable to find unique key for $table");
           exit();
       }
 }

     my (@fields,@vals);
    if ($fieldStr=~ /\w/) {
       @fields = split(",", $fieldStr);
       @vals = split('\|', $keystr);
    } else {
       foreach (split('\|', $keystr)) {
          my ($field,$val) = split (/\s*=\s*/,$_);
          push (@fields,$field);
          push (@vals,$val);
       }
    }

#  my $qualstring='';
#  my $and='';
#  for (my $i=0; $i <= $#fields; $i += 1) {
#     $qualstring .= "$and $fields[$i]=$vals[$i]";
#     $and="and";
#  }
 my @bkeys;
 #$bkeys = ",".$tinfo{"bibkeys"} if $tinfo{"bibkeys"};
 @bkeys = split(',',$tinfo{"bibkeys"}) if $tinfo{"bibkeys"};
 my $request = Browse::QueryRequest->new($table);
 $request->qclose();
 $request->setTable($table);
 $request->setQuals();
 for (my $i=0; $i <= $#fields; $i += 1) {
    $request->addQual("$fields[$i]|==|$vals[$i]");
 }
 #my @fields = ("ra","dec");
 #push(@fields,@bkeys) if scalar(@bkeys);
 #$request->setFields(\@fields);
 my @sfields = ("ra","dec");
 push(@sfields,@bkeys) if scalar(@bkeys);
 $request->setFields(\@sfields);
 $request->initiate();

 my $meta = $request->getResult();
 my $data = $request->getResult();

 #my ($results) = Browse::DBQ::query("select ra,dec$bkeys from $table where $qualstring");
 my $line = $$data{"data"};
 my $names = $$data{"names"};
#print "got line=@$line<br>\n" if $::debug;
#print "got names=@$names<br>\n" if $::debug;
my ($ra,$dec,$bkeystring);
 for (my $i=0; $i <= $#$names; $i += 1) {
     my $key=$$names[$i];
     my $val=$$line[$i];
     if ($key =~ /^ra$|^_RAJ2000$/) { $ra = sprintf "%.7f",$val;next}
     if ($key =~ /^dec$|^_DEJ2000$/) { $dec = sprintf "%.7f",$val;next}
     foreach my $bkey (@bkeys) {
        next unless (($bkey =~ /\w/) and ($key =~ /$bkey/));
        my $sep='|' if length($bkeystring);
        $bkeystring .= $sep . $bkey."=$val";
     }
 }
 my @biblink = split(/ /,$tinfo{"bibcodes"}{$bkeystring});
 @biblink=grep(/\w/,@biblink); # get rid of any nulls
 my $biblink = "bibcode=".join("&bibcode=",@biblink) if scalar(@biblink);

 $ra=Browse::CGI::Encode($ra);
 $dec = "+".$dec unless $dec <0;
 $dec=Browse::CGI::Encode($dec);
 return ($table,\%tinfo,$keystr,$ra,$dec,$biblink);
} #eosub-- get_coords

sub addKey {

    my ($tab, $key, $allTables, $seen) = @_;

    if ($tab =~ /\// || $$seen{"$tab.$key"}) {
        return;
    }

    if (!defined($$allTables{$tab})) {
        $$allTables{$tab} = {};
        $$allTables{$tab}{"keys"} = [];
    }

    $$seen{"$tab.$key"}++;

    my $arr = $$allTables{$tab}{"keys"};
    push(@$arr, $key);
}


#-------------------------------------------------------------------------

sub query_services {
 # input: list of services to be queried, list of targets (tablename+uniquekey)
 # output: display of links to services, or actual result display


#$::debug+=1;
 my ($in) = @_;
 my $biblink='';


 my %servicemap = (
   "NED" => "http://nedwww.ipac.caltech.edu/cgi-bin/nph-objsearch?search_type=Near+Position+Search&in_csys=Equatorial&in_equinox=J2000.0&lon=<ra>d&lat=<dec>&radius=2.0&out_csys=Equatorial&out_equinox=J2000.0&obj_sort=Distance+to+search+center&zv_breaker=30000.0&list_limit=5&img_stamp=YES&z_constraint=Unconstrained&z_value1=&z_value2=&z_unit=z&ot_include=ANY&nmp_op=ANY",
   "SIMBAD" => "http://simbad.harvard.edu/sim-id.pl?Radius=2&Ident=<ra>+<dec>",
   "SkyView:ROSAT_All-Sky" => "http://skyview.gsfc.nasa.gov/cgi-bin/nnskcall.pl?PIXELX=300&PIXELY=300&VCOORD=<ra>,<dec>&SURVEY=RASS-Cnt+Broad",
   "SkyView:DSS" => "http://skyview.gsfc.nasa.gov/cgi-bin/nnskcall.pl?PIXELX=300&PIXELY=300&VCOORD=<ra>,<dec>&SURVEY=Digitized+Sky+Survey",
   "CoCo" => "http://heasarc.gsfc.nasa.gov/cgi-bin/Tools/convcoord/convcoord.pl?CoordVal=<ra>,<dec>",
   "Viewing" => "http://heasarc.gsfc.nasa.gov/cgi-bin/Tools/viewing/viewing.pl?mission=CHANDRA&mission=HST&mission=RXTE&mission=SAX&mission=XMM&Entry=<ra>,<dec>&NR=CheckCaches%2FNED%2FSIMBAD&CoordSys=J2000",
   "nH" => "http://heasarc.gsfc.nasa.gov/cgi-bin/Tools/w3nh/w3nh.pl?Entry=<ra>,<dec>&NR=CheckCaches%2FNED%2FSIMBAD&equinox=2000&radius=1.00",
   "X-ray_Background_Tool" => "http://heasarc.gsfc.nasa.gov/cgi-bin/Tools/xraybg/xraybg.pl?Entry=<ra>,<dec>&NR=CheckCaches%2FNED%2FSIMBAD&CoordSys=J2000&radius=1.00",
   "Abstracts" => "http://adsabs.harvard.edu/cgi-bin/bib_query?<biblink>",
   );

  my (@targets)= split("\0",$$in{"Target"});
  my (@services)= split("\0",$$in{"Services2"});

  my %targets_by_table = ();
  foreach my $target (@targets) {
     #$target  = Browse::CGI::Decode($target);
     my ($table,$keystr)  = split('\|\|\|',$target);
     push (@{$targets_by_table{$table}}, $target);
  }

if (scalar(@targets) ==1 and scalar(@services) ==1) {
      my ($targettable, $tinfo, $keystr ,$ra, $dec,$biblink) = get_coords($targets[0]);
      my ($table,$servicename) = split(/\s+/,$services[0]);
      my $myservice =  $servicemap{$servicename};
      $myservice =~ s/<ra>/$ra/g;
      $myservice =~ s/<dec>/$dec/g;
      $myservice =~ s/<biblink>/$biblink/g;
      print "Location: $myservice\n\n";
      exit;
}

#Browse::Decor::Header("Service Links");
Browse::Decor::Header("Service Links",0,0,0,"services");

  my ($target,$service);
  my $table;
  foreach $table (sort keys %targets_by_table) {
    my $first = 1;
    my $targets = $targets_by_table{$table};
    foreach $target (@$targets) {
      my ($targettable, $tinfo, $keystr ,$ra, $dec,$biblink) = get_coords($target);
      my ($row,$names) = Browse::RowDisplay::getSingleRow($targettable,$keystr,
         undef, $tinfo, undef, $$in{"Coordinates"}, $$in{"Equinox"});
      my $i;
      for ($i=0; $i <= $#$row; $i += 1) {
          if ($$names[$i] !~ /^w3_/) {
              last;
          }
      }
      splice(@$row,   0, $i);
      splice(@$names, 0, $i);
      if ($first) {
          (my $ptable = $table) =~ s/$::local_origin//ig;
          print "<p><strong><a target='helpwin' href='$$tinfo{table_document_url}'>$$tinfo{table_description} ($ptable)</a></strong>&nbsp;&nbsp;\n";

          print "<br><table class='othertable' title='service links for selected rows'><thead><tr class='othertable'>".
                "<th class='services' scope='col'>Service<br>Links</th>".
                "<th class='othertable' scope='col'>". join("</th><th class='othertable' scope='col'>",@$names)."</th></tr></thead><tbody>\n"; 
          $first=0;
      }

      print "<tr class='othertable'><td class='services'>";
      foreach $service (@services) {
         my ($table,$servicename) = split(/\s+/,$service);
         next if ($table !~ /alltable/ && $table !~ /$targettable/);
         my $myservice =  $servicemap{$servicename};
         $myservice =~ s/<ra>/$ra/g;
         $myservice =~ s/<dec>/$dec/g;
         $myservice =~ s/<biblink>/$biblink/g if $biblink;
         #print "<a target=\"".Browse::CGI::Encode($target)."\" href=\"$myservice\">$servicename</a><br>" unless $myservice =~ /<biblink>|<ra>|<dec>/;
         print "<a target=\"$target\" href=\"$myservice\">$servicename</a><br>" unless $myservice =~ /<biblink>|<ra>|<dec>/;
      }
      if (!scalar(@services)) {print "None selected.";}
      print "</td>\n<td class='othertable'>" . join("</td><td class='othertable'>",@$row)."</td></tr>\n";
    }
    print "</tbody></table>\n";
  }
#Browse::Decor::Footer();
Browse::Decor::EmailFooter();
Browse::Decor::EndTags();
exit;

} #eosub-- query_services


1;
# $Log: Services.pm,v $
# Revision 1.11  2006/10/24 17:05:55  lmm
# Removed second declaration of @fields
#
# Revision 1.10  2006/06/23 18:43:10  lmm
# Made small change for Tabbed Display
#
# Revision 1.9  2003/08/28 21:39:12  lmm
# Defined style for page header, removed HEASARC footer
#
# Revision 1.8  2003/04/28 01:46:10  cwh
# Added new table style.
#
# Revision 1.7  2002/12/16 18:37:33  cwh
# No longer decoding target.
# Added quotes where needed.
# Added title to table.
# Added scope to table cols for section 508 compliance.
# No longer encoding window target.
#
# Revision 1.6  2002/09/13 20:59:59  cwh
# Replaced old way of getting coords with the better way - using request obj.
#   (handles Vizier tables too)
# Only print service link if there are no unresolved placeholders.
#
# Revision 1.5  2002/07/10 20:05:10  cwh
# Added list of services to get_services_list.
# Added handling of biblinks.
# Removed setargs for now.
#
# Revision 1.4  2002/06/06 20:56:40  cwh
# Fixed html bug - wasn't writing <tr> at the beginning of each row after the
# first row.
#
# Revision 1.3  2002/05/26 02:26:02  cwh
# Oops!  Changed olegacy urls to heasarc.
#
# Revision 1.2  2002/03/22 23:02:12  cwh
# Removed debug line.
# Added radius=2 arcmin to SIMBAD query.
# Added SkyView RASS-Cnt Broad to available services.
#
# Revision 1.1  2002/03/18 02:35:55  cwh
# Initial commit.
#
