# $Id: TabbedDecor.pm,v 1.5 2008/10/16 15:01:55 lmm Exp $
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=========================================================================#
# Module Name:    TabbedDecor.pm
# Module Author:  Laura McDonald (SSAI)
# Date:           June, 2006
# Project:        Browse : HEASARC
#=========================================================================#
# Module Description:
# This module contains subroutines for Header & Footer decorations
# and CSS definitions for Tabbed Display.
#=========================================================================#

use strict;

package	Browse::TabbedDecor;

$| = 1; # Force print buffer flush

#--------------------------------------------------------------------------
sub Header { # Sets up a HTML page for writing to. Inserts header, title

    my($title,$closetable,$cols,$banner, $style, $missionlist) = @_;

    ($title = &ResultsTitle) if ($title =~ /^results/);
    my($mtitle)  = $title;
    $mtitle      =~ s/\<[^>]*?>//g;  #remove html
    $mtitle      =~ s/\&nbsp\;/ /g; 
    $mtitle = "Main Interface" if $mtitle =~ /HEASARC Browse/;
    my $fclass = "larger";
    $fclass = "largest" if $title =~ /HEASARC Browse/;
    my $menu = &Menubar();
    $style="standard" unless $style;

    print  "Content-type: text/html\n\n";

    # set to 0 when content-type header is printed
    $::writeHeader = 0;  

    print <<EOT;
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
            "http://www.w3.org/TR/html4/loose.dtd">
    <html><head><title>HEASARC Browse: $mtitle</title>
    <link rel="stylesheet" href="$::w3bdocs/w3browsetable.css" title="colortable style" type="text/css" media="screen">
    <link rel="alternate stylesheet" href="$::w3bdocs/w3browse.css" title="scrolltable style" type="text/css" media="screen">
    <link rel="stylesheet" href="$::w3bdocs/w3browseprint.css" title="print style" type="text/css" media="print"> 
    <script type='text/javascript' src="$::w3bdocs/tips/tips.js"></script> 
    <script type='text/javascript' src="$::w3bdocs/w3main.js"></script> 
    <script type='text/javascript' src="$::w3bdocs/w3summaryAdd.js"></script> 
    <script type='text/javascript' src="$::w3bdocs/tp.js"></script> 
EOT
    #<script type='text/javascript' src="$::w3bdocs/w3summaryNew.js"></script> 
print <<EOT;
<style type='text/css'>
EOT

print <<EOT;
/* style sheet for tabs */

body {
    background:#FFFFFF;
}
.tabs {
    float:left;
    width:100%;
    font-size:100%;
    line-height:normal;
    } 

ul.tabs {
    margin:0;
    padding:3px 3px 0;  /* space between tab levels */
    background:#FFFFFF url("/Images/w3b/bg.gif") repeat-x bottom;
    list-style:none;
    }


ul.tabs li {

    float:left;
    background:url("/Images/w3b/norm_right.gif") no-repeat right top;
    margin:0;
    padding:0;
    }

ul.tabs a, ul.tabs a.link {
    display:block;
    background:url("/Images/w3b/norm_left.gif") no-repeat left top;
    padding:5px 15px;
    }

ul.table li a:link {
   font-style:italic;
   font-weight:normal;
}

/*ul.tabs a:hover, ul.tabs li a:hover {
   font-weight:600;
   }*/

ul.tabs li.active {
    background-image:url("/Images/w3b/norm_right_on.gif");
    padding-bottom:5px;
}

ul.table {
  margin-left:1%;
   }

ul.mission {
   margin-left:.5%;
   }
/*ul.tabs li  a.active, ul.tabs a:link.active, ul.tabs a:visited.active {*/
ul.tabs li  a.active {
    background-image:url("/Images/w3b/norm_left_on.gif");
    }

/* panels */

div.tabs {
    padding: .5em;
    position: relative;
    width: 100%;
    margin: 0;
    white-space: nowrap;
}
div.tabs div.tabs {
    padding: 0;
}
div.tabs div.tabs  {
    clear: left;
    padding: .03em;
}
</style>
EOT

#print <<EOT;
    #<script language='JavaScript' type='text/javascript'>
#alert('noenter');
    #//--- Prevent submit when return key is pressed
    #function noenter(evt) {
        #//return !(window.event && window.event.keyCode == 13); 
       #evt = (evt) ? evt : event;
       #var charCode = (evt.charCode) ? evt.charCode :
	   #((evt.which) ? evt.which : evt.keyCode); 
       #if (charCode == 13 || charCode == 3) {
          #return  false;
       #} else {
          #return  true;
       #}
      # 
    #}
#alert('noenter');
    #</script>
#EOT

    if ($banner) {
       print <<EOT;
       $menu
EOT
    } else {
       print <<EOT;
       </head>
        <body bgcolor="#ffffff" text="#090909" 
       ink="#0505ff" vlink="#ff0000" alink="#c0f0c0" onLoad="pageLoaded();changeLink()">  
EOT
    }
    print <<EOT;
    <script type='text/javascript' src="$::w3bdocs/wz_tooltip.js"></script> 
    <script language='JavaScript' type='text/javascript'>
    var ua = navigator.userAgent;
    if (ua.indexOf('MSIE') >= 0) {
       document.write("<div id='iewarning'><table align='center' width='85%' border='1' " +
	  "cellpadding='10' bgcolor='#eeeeee'>");
       document.write("<tr><td>Internet Explorer Users: Navigation tabs will " +
	  "not work properly until all query tables are loaded. " +
	  "Please wait for the <b>Processing Query</b> tab to disappear " +
          "before selecting a tab. ");
       document.write("<a href='/W3Browse/w3browse-help.html#display_options'>More information</a>");
       document.write("</td></tr></table></div>");
    }
    </script>
EOT
    print "<noscript><p>";
    print "<table align='center' width='85%' border='1' cellpadding='10' " .
	"bgcolor='#eeeeee'>";
    print "<tr><td>A <b>Javascript-enabled browser</b> is required for " .
	"the <b>Browse Tabbed Display</b>.";
    print "To see all of your query results please enable JavaScript " .
	"(usually from our browser's Preferences panel) or choose one " .
	"of the other displays formats listed on the " .
	"<a href='/cgi-bin/W3Browse/w3browse.pl'>main search form.</a>";
    print "</td></tr></table>\n";

    print "</noscript><p>\n";
print <<EOT;
<div style='border: solid 1px #000;width:950px' >
<table width="100%" cellspacing="0">
        <tr class='$style'>
              <td width="30%" align="left">&nbsp;&nbsp;&nbsp;&nbsp;<a href="/$::cgi/w3browse.pl"<b>Main Search Form</b></a></td>
              <td align="center" class="$fclass" width="30%">Browse Query Results</td>
              <td align="right">
<a href="$::w3bdocs/w3btipsarchive.html" ><img src="$::images/tiparchive.jpg" border="0" alt="Tips Archive"></a>

<a href="http://heasarc.gsfc.nasa.gov/hera"><img border="0" alt="You are using the Hera-enabled Browse" height="32" width="45" src="$::images/test/hera_white_on_gold-red_oval.gif"></a> <a target="helpwin" href="$::w3bhelp"><img border="0" alt="Help" src="$::images/help.gif" height="30" width="30"></a></td>
        </tr>
      </table>
</div>

EOT


} # eosub--Header
#--------------------------------------------------------------------------
sub PageTitle { #returns a title to display using <H2>.

    my($title) = join(' ',@_);

    print "<h2>$title</h2>";

} #eosub--PageTitle
#--------------------------------------------------------------------------
sub ResultsTitle {

     my($today) = &today;
     my($title) = "Search Results of $today";

     return ($title);

} #eosub--ResultsTitle
#--------------------------------------------------------------------------
sub Menubar {

     return Browse::Utils::includefile($::menubar) if -s $::menubar;
}
#--------------------------------------------------------------------------
sub Footer { # includes the LHEA footer, page author, and closes the HTML
    EmailFooter();
    print Browse::Utils::includefile($::footer) if -s $::footer;
    EndTags();


} #eosub--Footer

sub EmailFooter { # display Browse dev Team email address
   print "<div id='feedback'>\n";
   if ($::host !~ /gsfc.nasa.gov/) {
    print <<EOT;
<p>
<em>Maintainer:</em>
<a href="mailto:$::maintainer_email">$::maintainer_name</a> 
($::maintainer_email)
EOT
   } else {
      #print "<p><p><a href='/cgi-bin/Feedback'><span style='font-size:80%';>Send email to the $::maintainer_name</span></a>\n";
      print "<p><br><p><a href='/cgi-bin/Feedback?selected=w3browse' target='feedback'>Browse Feedback</a>\n";

   }
print "</div> <!-- end feedback div -->\n";
}
#--------------------------------------------------------------------------

sub EndTags { # closes the HTML
    my($endtags) = 0;  # this was set to 1, not sure why
    if(scalar(@_)) {
       $endtags = @_;
    }  
    if (!$endtags) {
      print <<EOT;
</body>
</html>
EOT
    }
}

#--------------------------------------------------------------------------
sub today {
    #### Returns a string containing today's date in MM/DD/YY format.
    # Changed to return Month dd yyyy

    my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    my $Month = ("January","February","March","April","May","June","July","August","September","October","November","December")[$mon];
    #$mon++;
    #return "$mon/$mday/".substr($year,length($year)-2, 2);
    return "$mday $Month ". ($year+1900);

} #eosub--today
#--------------------------------------------------------------------------
sub MissionLinks {
    # Function  input: a hash with the following keys:
    #   'missions'   => ref to array of missions
    #   'separator'  => string to be used as the separator (default: ", ")
    # Function output: HTML string that implements a list hyperlinked
    #                   missions names

    my(%args) = @_;
    $args{'separator'} = ', ' unless (defined($args{'separator'}));

    # Build the HTML.
    my($html) = '';
    my(@links) = ();
    my($mission,$link);
    foreach $mission (@{ $args{'missions'} }) {
	$link = lc($mission);
	$link =~ tr/ /\_/;
#	$mission =~ tr/ /\-/;
	push(@links,"<a target=helpwin href=\"w3table.pl?MissionHelp=\L$link\E\">\U$mission\E</a>");
    }

    $html = join($args{'separator'},@links) if (scalar(@links));

    return($html);

} #eosub--MissionLinks
#--------------------------------------------------------------------------
sub Acknowledge {       # Instructs users to acknowledge HEASARC if
                        # a publication results from surfing HEASARC,
                        # provides the acknowledgement text and an address
                        # for sending preprints our way.

    print <<EOT;

<hr>
The <a target=helpwin href="$::heasarc/">HEASARC Online Service</a> is provided by the
Laboratory for High Energy Astrophysics at NASA/Goddard Space Flight Center.
If using this service made a significant contribution to a research project,
please make the following acknowledgment in any resulting publication:
<p><i>
"This research has made use of data obtained through the High Energy
Astrophysics Science Archive Research Center Online Service, provided
by the NASA/Goddard Space Flight Center."
</i><p>
Please send a preprint or reprint of the paper to:<p>
	The HEASARC, Code 660.2, NASA/Goddard Space Flight Center,
	Greenbelt, Maryland, 20771, USA.
EOT

} #eosub--Acknowledge
#--------------------------------------------------------------------------
1;
#########
#  EOF  #
#########

# $Log: TabbedDecor.pm,v $
# Revision 1.5  2008/10/16 15:01:55  lmm
# Added header with link to Main Search Form.
#
# Revision 1.4  2006/10/24 19:16:37  lmm
# Added new JavaScript file main.js.
# Added call to new JavaScript function pageLoaded() to keep Requery and
#   Redisplay buttons disabled until page loads.
#
# Revision 1.3  2006/06/29 19:23:33  lmm
# Removed call to JavaScript function on page load
#
# Revision 1.2  2006/06/23 20:25:56  lmm
# Changed names of javascript scripts to non-test versions
#
# Revision 1.1  2006/06/23 19:28:20  lmm
# initial commit
#
