# $Id: TabbedDisplay.pm,v 1.12 2008/10/16 15:32:08 lmm Exp $
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    TabbedDisplay.pm
# Module Author:  Laura McDonald, SSAI 
# Date:           June, 2006
# Project:        Browse : HEASARC
#========================================================================
#--- Overrides Display methods for Tabbed Display
#========================================================================


use strict;

use Browse::TabbedDecor;
use Browse::TabbedResults;
use Browse::Display;

package Browse::TabbedDisplay;

use vars qw( @ISA );
@ISA = qw( Browse::Display );

sub init {
   my ($self) = @_;
   $self->results("TabbedResults");
}
#--- return true if any tables from specific mission have been processed
sub seenMission {
   my ($self, $mission) = @_;
   return $self->{missionssofar}{$mission . 'tabs'};
}

# Create list of missions and associated tables
sub setMissionList {
  my ($self, $tablelist,$selectedMissions) = @_;
  my %missionlist;
  my %missionssofar;  #- defined here, not populated
  my @missiontabs;
  foreach my $table (keys %$tablelist) {
    my $tablename=$tablelist->{$table}{table_name};
    my $mission;
    if ($tablename =~ /\//) {
       $mission="vizier";
       push(@missiontabs,$mission);
    } else {
       #--- KLUDGE 
       #--- if table belongs to more than one mission compare it to 
       #--- the missions the user selected.  If more than one mission
       #--- has been selected by the user, use the last one encountered
       my $userMissionFound =0;
       my @missions = split(/\|\|/, $tablelist->{$table}{observatory_name});
       #$mission=$tablelist->{$table}{observatory_name};
       foreach my $m (@missions) {
          #print "setMissionList looking at $m ". 
		#scalar(@missions) ."++<br>" if $::debug;
          if (grep /$m/, @$selectedMissions) {
             #print "setMissionList $table adding mission $m ". 
		#scalar(@missions) ."++<br>" if $::debug;
             $userMissionFound = 1;
             $mission = $m;
             push(@missiontabs,$mission);
          } 
       }
       #--- if user selected mission info is not available, use first mission
       if (!$userMissionFound) {
          push(@missiontabs,$missions[0]);
       }
       
    }
    print "setMissionList $tablename .. $mission <br>" if $::debug;
    if ($missionlist{$mission}) {
       push @{$missionlist{$mission}},$table;
    } else {
       $missionlist{$mission}=[$table];
    }
  }
  $self->{missionlist}=\%missionlist;
  $self->{missiontabs}=\@missiontabs;
  $self->{missionssofar}=\%missionssofar;
}

#--- Store table information (hidden input) for redisplay 
sub tableLinks {
    my ($self, $links) = @_;
    $$self{"tablelinks"} .= $links if defined($links);
    return $$self{"tablelinks"};
}

#--- set or get current mission name
sub currentMission {
    my ($self, $mission) = @_;
    $$self{"currentmission"} = $mission if defined($mission);
    return $$self{"currentmission"};
}

#--- get number of missions processed
sub missionCount {
    my ($self) = @_;
   return $self->{mcount};
}

# Print out the very top of the page.  From
# the HTTP headers until we begin displaying
# information from a particular table.

sub pageHeader {

    my ($self, $navtrail,$tables) = @_;
    my $formname="w3hdprods";
    

        $self->{tcount} =0;  #- for tabbed tpSet/tpAdd
        $self->{mcount} =0;  #- for tabbed tpSet/tpAdd
        
        my $missionlist= $self->{missionlist};
        #--- Create menu list that will be converted to tabs
        my $ulId="tabcontrol";
        #my $tabulHtml= '<p><p><div id="' . $ulId . '" style="position:absolute;width:100%;top:0;left:0;">';
        my $tabulHtml= '<p><p><div id="' . $ulId . '" >';

        my $tablesmission="$ulId:";
        foreach my $mname (keys %$missionlist) {
           my $lc_mname = lc($mname);
           $lc_mname=~ s/\s/_/g;
           my $tabs = $missionlist->{$mname};
           foreach my $t (@$tabs) {
              $t = $self->chopTable($t);
           $tablesmission .= $t.'|'.$lc_mname.',';
           }
        }
        $tabulHtml .= "</div><p>\n";
        Browse::TabbedDecor::Header( $$self{"popupFrom"} ,0,0,0,
		$$self{"stylecolor"}, $tablesmission); 

         #--- form needs to be started before all tabbing divs
        print "<script src=\"$::w3bdocs/w3bSubmit.js\"></script>\n";
        print "\n<form method='POST' target='_blank' name='$formname' " .
	    " id='$formname' action='w3hdprods.pl'>\n";

        print $tabulHtml;   #--- tab menu
        print "<div>\n";
	   


        Browse::ValidateInput::checkResults($formname);
        
    #--- original input should exist so grab some parameters and pass them on
    print "Display input=" . Browse::CGI::Decode($$self{input}) .
	"<br>" if $::debug;

    my $decodedInput = Browse::CGI::Decode($$self{input});
    if ($decodedInput) {
       #--- save for display at end of all tables
       my ($dtime, $dnr);
       my ($dequinox, $dcoords);
       if ($decodedInput =~ /&Time=([^&]*)&/) { $dtime=$1;}
       if ($decodedInput =~ /&NR=([^&]*)&/) { $dnr=$1;}
       if ($decodedInput =~ /&Equinox=([^&]*)&/) { $dequinox=$1;}
       if ($decodedInput =~ /&Coordinates=([^&]*)&/) { $dcoords=$1;}
       
       $$self{time}=$dtime;
       $$self{dnr}=$dnr;


       
    #print "<script type=\"text/javascript\" language=\"JavaScript\">";
    #print "resortValues('$dtime','$dequinox','$dcoords','$dnr')"; 
    #print "</script>\n";
    my $displayformat= ref($self);
    $displayformat=~ s/Browse\:\://;
    print "<input type='hidden' name='requerydisplay' value='".
	$displayformat."'>\n";
    print "<input type='hidden' name='NR' value='".$dnr."'>\n";
    print "<input type='hidden' name='Time' value='".$dtime."'>\n";
    print "<input type='hidden' name='Coordinates' value='".$dcoords."'>\n";
    print "<input type='hidden' name='Equinox' value='".$dequinox."'>\n";
    }
    
    #print "<input type='hidden' name='input' value='".$$self{"input"}."'>\n";
    print "<input type='hidden' name='savedQuery' value='".
	$$self{"savedquery"}."'>\n";

        #--- DIV start for Query Information
        print "<div id='intro' t_title='Query Information' title='Click to view information about your query'>\n";

    # Pass along header to be used on new page
    my ($popup)=$$self{"popupFrom"};   
}


#--- Displays names of tables and the number of rows in that table
sub resultsSummary {
    my ($self,$tinfo, $tablesSorted) = @_;

    #--- get number of browse tables to determine size of summary 
    #--- display area and to keep track of rows
    my @tinfokeys =keys (%$tinfo);
    
    my $tableCount=$#tinfokeys;
    my $displayCount=$tableCount+1;

    #--- Control size of area with style sheet entry
    my $tClass = "rowcounttablemediumtabbed";   
    my $bClass = "oneplustablerowcount";
    my $hClass = "rowcounttablemediumhead";   
    my $sTablename="Table Name/Row Count Summary";
    if ($tableCount == 0) {
       #--- slightly different display for one table result
       $tClass = "rowcounttableonerow";   
       $hClass = "rowcounttableonerowhead";   
       $sTablename="Table Name and Row Count";
    }
print <<EOT;
    <script type=\"text/javascript\" language=\"JavaScript\">
EOT
print <<EOT;
    document.writeln('<div id="resultsSummary">');
    if (document.getElementById) {
    document.writeln('<div style="margin-left:50px;"><span class="$hClass">');
    document.writeln('$sTablename: </span><span style="font-size:85%;color:#603;background-color:#DDF;padding:3px;"><span id="querying">Querying table <span id="tcountdown">1 </span> out of $displayCount.</span> <span id="rcountdown"></span></span><p>');
EOT
    if ($tableCount > 0) {
print <<EOT;
    document.write('<font size="-1">Click on table name to view search results</font>');
EOT
}
print <<EOT;
    }
EOT
print <<EOT;
    if (document.getElementById) {
    document.writeln('<div class="$tClass">');
EOT

    #--- display the table with names and # of hits  
print <<EOT;
    document.writeln('<table class="$bClass"  id="rowcount" width="80%"><tbody id="counttableTB">');
    document.writeln('<tr></tr>');
EOT

    #--- page load status line
print <<EOT;
    document.writeln('<tr>');
    document.writeln('<td colspan ="2" align="center" id="rowcountupdate"> </td>');
    document.writeln('</tr>');
    document.writeln('<tr>');
EOT

    #--- Gather table names from sorted list and add description to 
    #--- summary table
    my $currTable=0;
    my $rowcount=0;
    for my $sTable (@$tablesSorted) {
       my $table = $tinfo->{$sTable};
       my $tableId;
       my $description=$table->{"table_description"};
       
       #--- special processing for vizier
       if ($table->{"table_name"} =~/\//) {
          $tableId=Browse::Vizier::getFinalTableName($table->{"table_name"});
          if ($tableId !~ /\w/ ) {$tableId=$table->{"table_name"}."/out";}
       } else {
          $tableId=$self->chopTable($table->{"table_name"});
       }


       #my $description=$table->{"table_description"};
       $description =~ s/\n/ /g;

       #--- debugging
       my $mission=$table->{"observatory_name"};
       my $pri=$table->{"table_priority"};
       #--- end debugging

       my $cellCountId=$tableId . "CC";
       my $cellNameId=$tableId . "NC";
       if ($description) {
print <<EOT;

          document.writeln('<td >');
EOT
print <<EOT;
          document.writeln('  <table  width="100%" bgcolor="#ffffff" cellspacing="0" id="counttable$currTable"><tr>');
          document.writeln('  <td align="left" id="$cellNameId"> <b>$tableId</b>:$description </td>');
          document.writeln('  <td align="right" id="$cellCountId" > </td>');
          document.writeln('  </tr></table>');
EOT
          my $newcell=$currTable % 2;
          ++$currTable;
print <<EOT;
          document.writeln('</td>');
EOT

          #--- start a new row 
          if ($newcell && $rowcount < $tableCount) {
print <<EOT;
             document.writeln('</tr><tr>');
EOT
          }
          ++$rowcount;
       }
    }
print <<EOT;
    document.writeln('</tr></tbody></table>');
    document.writeln('</div>');
    document.writeln('</div>');
    document.writeln('</div>');
    } //-- end check for javascript method capability
EOT
    print "</script>\n";
    ##--- Create No matches and Errors area
    print "<script type=\"text/javascript\" language=\"JavaScript\">;\n";
    print "if (document.getElementById && document.createElement) { ";
    print "var elem = document.createElement('matches');\n";
    print "elem.setAttribute('id','elem');\n";
    print "var txt = document.createTextNode('');\n";
    print "elem.appendChild(txt);\n";
    print "document.getElementById('resultsSummary').appendChild(elem);\n";
    print "}\n";
    print "</script>\n";
   

print "</div> <!-- end Query Information (intro)-->\n";   
}   #--- end resultsSummary




# Print the information at the beginning of a tables results.

sub tableHeader {
    my ($self) = @_;

    my ($meta) = $$self{"meta"};
    #--- get the table name string and save for later for tabbed output
    my $results = $self->results();
    my $tableLink;
    if ($results) {
       $tableLink= $self->tableLinks($results->DisplayLinks($meta));
    } 

    my $table_info = $$meta{"tableinfo"};
    my $table      = $$meta{"table"};
    
    #-- Display links for resorting this table before table is displayed
    if ($tableLink) {
        (my $ctable= $table)=~ s/_/\%5F/;
        $tableLink =~ /(\<input.*?req_$ctable.*?\>){1}/;
        my @h_inputs = split(/\<input type='hidden'/,$tableLink);
        my @t_links= grep /$ctable/, @h_inputs;
        foreach (@t_links) {
           print "<input type='hidden' " . "$_\n";
        }
        
    }
    $$self{"meta"} = $meta;
    $$self{"rowcount"} = 0;
    
    
    my ($sql) = $$meta{"sql"};
    
    if (defined($$table_info{"default_search_radius"})) {

	my $raf  = $$table_info{"right_ascension"};
	my $decf = $$table_info{"declination"};

	if (substr($raf, 0, 1) eq "@") {
	    $raf = substr($raf, 1);
	}
	if (substr($decf, 0, 1) eq "@") {
	    $decf = substr($decf, 1);
	}
	    
    }
	
	
    my $radius;
    if ($$meta{"positions"}) {
	my $pos = $$meta{"positions"};
	my $pos1 = $$pos[0];
	my ($ra, $dec, $rad) = split('\|', $pos1);
	$radius = $rad;
    }
    
    if ($$meta{"resource_name"} && $$meta{"resource_name"} ne $$self{"last_resource"}) {
        my $url = $$meta{"query_url"};
	$url =~ s/asu-xml/VizieR/;
        print "<hr><img alt=\"VizieR logo\" src='$::vizgif' " .
	   "height=20 width=20>  <a target=helpwin href='$url'" .
	   ">$$meta{resource_title} ($$meta{resource_name})</a>\n";
	$$self{"last_resource"} = $$meta{"resource_name"};
    }

    Browse::Results::PrintTableName($table, $table_info, $radius);

}

sub legend {
    my ($self) = @_;
    my $table      = $$self{"meta"}{"table"};

    print "TabbedDisplay: in legend() for $table<br>\n" if $::debug;
    #--- mission name div  -- check for vizier
    my $missionname;
    my $missiontitle;
    my $title;
    if ($$self{"meta"}{tableinfo}{table_name} =~ /\//) { 
       $missionname = "vizier";
       $title = "vizier";
    } else {
       $missionname = lc($$self{"meta"}{tableinfo}{observatory_name}); 
       $missiontitle = $missionname;
       if ($missionname =~ /\|\|/) {
          foreach my $m (split(/\|\|/, $missionname)) { 
             if (grep /$m/i, @{$self->{missiontabs}})  {
                $missionname = $m;
                $missiontitle = $missionname;
             }
             print "TD:m=$m $missiontitle<br>" if $::debug;
          }
       }
       $missionname =~ s/\s/_/g; 
       $title= $missiontitle;
       
    }

    my $cm=$self->currentMission();
    if ($cm ne $missionname) {   #-- new mission
       
       if ($cm =~ /\w+/) {   #-- if not very first mission

           #--- if current mission is new end its div and add to tabs
           if (!$self->{missionssofar}{$cm.'tabs'}) {
           }
        ++$self->{mcount};
  
        $self->{tcount} =0;  #- rest for tabbed tpSet/tpAdd of new mission

       } 

       #--- start mission div and missiontabs div if not set already
       if (!$self->{missionssofar}{$missionname.'tabs'}) { 
          print "<div id='" . $missionname .
	   "' t_title='" . $title .
	   "' title='Click to see " .$title .
  	   " mission results'>\n";

          #--- Start mission div
          print "<div id='" . $missionname .  "tabs'>\n";
      }

       my @missionarray=($missionname);
       $self->currentMission($missionname);
    }
    #--- table name div. Add "_t" tag to ensure unique div ids. Check for vizier
    my $tabletitle=$self->{"meta"}{tableinfo}{table_name};
    if ($tabletitle =~ /\//) {
       #--- remove "out" suffix for title
       $tabletitle =~ s/\/out//;
    }

    $self->beforeNewTable(
	$self->chopTable($self->{"meta"}{tableinfo}{table_name}),
	$self->chopTable($tabletitle),  
	$self->{"meta"}{tableinfo}{table_description}); 

    #--- start table div, hide table until it is completely rendered
    #print "<div style='visibility:hidden' id='" . 
	#$self->chopTable($self->{"meta"}{tableinfo}{table_name})  . "_t" .
	#"' t_title='" . $self->chopTable($tabletitle) .
	#"' title='" .$self->{"meta"}{tableinfo}{table_description} . 
	#"'>\n"; 
    #----

    #print legend
    print "<p><b>Click mission tabs (middle tab level) to display table tabs. Move cursor over tabs to see more information.</b>";
    print "<hr><span class=\"small\"><strong>Table Legend:</strong>" .
	"<br><img src = \"$::images/mag5.gif\" height='15' width='15'" .
	" border='0' alt=\"Display all params\"> Display all parameters" .
	" for a row";

    print "&nbsp;&nbsp;&nbsp;&nbsp;<img src = \"$::images/magviz.gif\" " .
	"height='15' width='15' border='0' " .
	"alt=\"Display all params direct from VizieR\"> Display all " .
	"parameters directly from VizieR" if $table =~ /\//;

    print "<br><img src = \"$::images/down.gif\" height='15' width='15' " .
	"alt=\"Sort\"> Sort by a column in order: " .
	"1,2,3&nbsp;&nbsp;&nbsp;&nbsp;<img src = \"$::images/up.gif\" " .
	"height='15' width='15' alt=\"Reverse Sort\"> Sort by column in " .
	"reverse order: 3,2,1&nbsp;&nbsp;&nbsp;" .
	"<img src = '$::images/down_sort.gif'  height='15' width='15' " .
	"alt='Current Sort 3,2,1'>/" .
	"<img src = '$::images/up_sort.gif'  height='15' width='15' " .
	"alt='Current Sort 1,2,3'> " .
	"Current table sort" .
	"<br>";


    my $gcn;
    if ($$self{"gcnlink"}) {
        $gcn = ", G: GRB Coordinate Network (GCN) notices";
    }

    print "<table><tr><td nowrap valign='top'>Services links:</td>" .
	"<td valign='top'> O: Digitized Sky Survey image, " .
	"R: ROSAT All-Sky Survey image, N: NED objects near coordinates,<br> " .
	"S: SIMBAD objects near coordinates, D: get list of data products, " .
	"H: analyze data products using <a href='/hera'>Hera</a>,<br> " .
	"B: ADS bibliography holdings" . 
        $gcn .", " .
	"F: FOV plot for observation " .
	"</td></tr></table>\n";  

    
    print "<p><b>Data Products</b>: Click checkbox to add row to Data Product Retrieval List " ;
    print "</span>\n";
}

# Process empty tables or error messages.

sub showEmpties {

    my ($self, $empties, $errs) = @_;
    $self->closeTable();

    #-- Add HTML for nulltables for possible user requery
    if ($$self{"nullTablesHtml"})  {
        print $$self{"nullTablesHtml"}; 
    }

    if ($self->tableLinks()) {
       print $self->tableLinks();
    }

    my $nomatches_list="";
    my $errors_list="";
    if ($#$empties >= 0) {
	
        #--- Fill in No matches and errors area
        $nomatches_list="<hr><p> <strong>No matches for</strong><br><dl>";
	my $niltab;
	foreach $niltab(@$empties) {
            my $stable = $$niltab[0];
            $stable =~ s/$::local_origin//g;
	    $nomatches_list .= "<dd> <a name=\"$stable\">$$niltab[1] ($stable)</a>";
	}
	$nomatches_list .= "</dl>";
    }
    my $lastErr;
    if ($#$errs >= 0) {
	$errors_list .= "<hr><p> <strong>Search warnings</strong>" .
		"<br><dl>";
	my $errtab;
	foreach $errtab(@$errs) {
            my $stable = $$errtab[1];
            $stable =~ s/$::local_origin//g;
	    if ($$errtab[0] ne $lastErr) {
	        $errors_list .= "<dt> $$errtab[0] ";
	        #print "<dt> $$errtab[0] ";
	    }
	    $errors_list .= "<dd> <a name=\"$stable\">$$errtab[2]" .
		" ($stable)</a>";
	}
	$errors_list .= "</dl>";
    }
        #--- Display Errors.  No Matches are not displayed at this time
        print "<script type=\"text/javascript\" language=\"JavaScript\">\n";
        print "if (document.getElementById) {\n";
        print "var intro=tp['intro'];\n";
        print "var tags=intro.getElementsByTagName('matches');\n";
        print "var matchesid=tags[0];\n";
        print "matchesid.innerHTML='$errors_list';\n";
        print "}\n";
	print "</script>\n";

}
        
# This prints out the columns at the beginning of each HTML table block.

sub printColumnHeader {

    my ($self, $names) = @_;
    
    my $meta  = $$self{"meta"};
    my $trans = $$self{"transform"};
    my $longs = $$trans{"longs"};
    my ($links,$html, $data_id, $dataid_prefix);   #data_id

    
    my $results = $self->results();
    if ($results) {
       ($$self{"skipfield"},$links,$html, $data_id, $dataid_prefix) = 
	  $results->PrintTableParams($$meta{"table"}, 
	  $$meta{"tableinfo"}, $$meta{"sortable"}, $names, $$self{"unitFlds"},
	  $longs, $$self{"offset_var"},$$self{"sortcolumn"});
    }

    my $bg;
    if ($$meta{"table"} =~ /\//) {   # if  '/' in the name, must be VizieR
        $bg = "class='Vizier'";
    }

    $self->tableStart($meta);

    #--- header start
    print "<thead>\n<tr $bg>$html</tr>";

    my $fld;
    print "links " . ref($links) . " " . scalar @$links ." <br>" if $::debug;
    $$self{"related_links"} = $links;
    $$self{"dataset_id"} = $data_id;
    $$self{"dataid_prefix"} = $dataid_prefix;

    # Handle long strings
    foreach $fld(@$longs) {
        my $span = $#$names - $$self{"skipfield"} ;
	
        if ($span <= 0) {
            $span = 1;
        }

        print "<tr><th class='thinborderhead' colspan='2'></th>" .
	   "<th class='thinborderhead' scope='col' colspan='$span' " .
	   "align='left'>";   #checking for blank column in HTE table lmm
	my $fn = $$names[$fld];
	my $doc = $$meta{"tableinfo"}{"table_document_url"};
	print "<a target='helpwin' href='$doc#$fn'> $fn </a>";
	print "</th></tr>\n";
    }
        print "</thead>\n";	
  
}

# Finish up processing of a table.  Close any open HTML blocks....

sub closeTable {

    my ($self, $msg) = @_;
    
    if ($$self{"intable"}) {
        print "</tbody>\n";
        print "</table>\n";
	$$self{"intable"} = 0;
        my $s = "";
        if ($$self{"rowcount"} > 1) {
            $s = "s";
        }
        my $table = $$self{"meta"}{"table"};
        my $xtable = $self->chopTable($table);

        print "$$self{rowcount} row$s retrieved from $xtable<p>\n";

        if ($$self{"dataset_id"})  { #data_id
           my $datasetLink=$table . $$self{"dataset_id"};
           print "<a name='" . $datasetLink . "'></a><font size='-1'> Note: " .
	        "The standard " .
		"<b>Dataset ID</b> used in referring to this data in" .
		" journals and other publications is the content of this" .
		" column prefixed with the string \"" .
		$$self{"dataid_prefix"} . "\". " .
		# More information can be" .
		#" found at the <a href='http://vo.ads.harvard.edu/dv/'>" .
		#"Astrophysics Data System.</a> " .
		"<p></font>";
        }
	
        print "$msg<br>\n" if $msg;

	my $meta = $$self{"meta"};
	my $dp   = $$meta{"dpsets"};
	if (ref($dp)  && scalar keys %$dp == 0) {
	    $dp = undef;
	}
	

	my $tableinfo = $$meta{"tableinfo"};
        print "<table class='othertable'><tr class='othertable'>\n";
	if ($dp) {
            print "<td class='othertable' valign=\"top\">\n";
            $$tableinfo{table_name} = $table;

            my $results = $self->results();
            if ($results) {
	       $results->DisplaySets($dp, $tableinfo);
            }
            print "<table><tr><td><div id='dpmessage'></div>\n";
            print "<script type='text/javascript' language='JavaScript'>;\n";
            print "if (document.getElementById && document.createElement) { ";
            print "var dpmess = document.createElement('p');\n";
            print "dpmess.setAttribute('id','dpmess');\n";
            print "var txt = document.createTextNode('');\n";
            print "dpmess.appendChild(txt);\n";
            print "document.getElementById('dpmessage').appendChild(dpmess);\n";
            print "}\n";
            print "</script>\n";
            print "</td></tr>\n";
            print "<tr><td><a href='#' " .
		"onClick=\"tpShow('tabcontrol','dataproducts')\">" .
		" Show current rows selected for Data Products " .
		"Retrieval</a></td></tr>" .
		" </table>\n";

            print "</td>\n";
	}
        print "<td class='othertable' valign=top>\n";
        my $coords = ($$tableinfo{right_ascension}? 1:0);
       
        my $results = $self->results();
        if ($results) {
	   $results->DisplayTableOptions($table);
	   $results->DisplayServiceOptions($table,$coords,$dp);
        }
        print "</td></tr></table>\n";

        #--- end table div
        #print "</div>   <!-- end table  div  -->\n";
        $self->afterTable(); 

        #--- display table so it appears under correct tab
        print "<script type='text/javascript' language='JavaScript'>;\n";
        print "var currentTable=document.getElementById('" . 
		$self->chopTable($table) . "_t');\n";
        print "currentTable.style.visibility='visible';\n";
        print "</script>\n";
     
        my $cm=$self->currentMission(); 
        

        #--- Adjust tabs for current table
        if ($self->{tcount} == 0) { 
           #--- end of first table of current mission
           if (!$self->{missionssofar}{$cm.'tabs'}) {  #-- unseen mission
              #--- this mission has not yet been encountered so set table tab
              #--- and mission tab
              print "</div>   <!-- end $cm tabs  div  -->\n";
              print "<script type='text/javascript' language='JavaScript'>;\n";
              print "tpSet('$cm'+'tabs','tabs table', " .
		"new Array('" . $self->chopTable($table) . "_t'));\n";
              print "</script>\n";

              print "</div> <!-- end $cm div -->\n";
              if ($self->{mcount} == 0) {
                 #--- end of first table of first mission.  Set all other 
                 #--- tabs to make them available as tables load
                 print "</div>   <!-- end all missions tab  div  -->\n";
                 print "<script type='text/javascript' " .
			"language='JavaScript'>;\n";
                 print "tpSet('missiontabs','tabs mission', ".
			"new Array('$cm'));\n";
                 print "tpSet('tabcontrol','tabs', " .
			"new Array('intro', 'missions'));\n";
                 print "</script>\n";
                 print "<script type='text/javascript' " .
			"language='JavaScript'>;\n";
                 print "</script>\n";
                 dpDiv();
                 helpSection();
                 #-- display  (but hide) param display for selected rows option
                 #-- and table services option.  they will be added to 
                 #-- data products retrieval page
                 print "<p><div id='alltable_rowoptions' style='display:none'>";
                 print qq|<span style='font-size:75%;'><input type='submit' name="files" onClick="setcheckDP(0);return(pageCheck())" value="Display" onclick=\"this.form.target='_blank'\"> all the columns for selected rows</span> <br></div>|;
                 $results->DisplayServiceOptions("alltable",1,1);

                 #--- add this mission to encountered list
                 $self->{missionssofar}{$cm.'tabs'}=1;
              }  else {
                 #-- Add new mission to missions tab
                 print "<script type='text/javascript' " .
			"language='JavaScript'>;\n";
                 print "tpAdd('missiontabs', new Array('$cm'));\n";
                 print "</script>\n";
                 $self->{missionssofar}{$cm.'tabs'}=1;
              }
              print "<script type='text/javascript' language='JavaScript'>;\n";
              print "changeSummaryLink('" . 
		   	 $self->chopTable($table) . "', '$cm');\n";
              print "</script>\n";
              
           } else {
              #--- we have seen this mission before so add table to mission tab
              print "<script type='text/javascript' language='JavaScript'>;\n";
              print "tpAdd('$cm'+'tabs',  '" .$self->chopTable($table) .
		 "_t');";
              print "changeSummaryLink('" . $self->chopTable($table) . 
		 "','$cm');\n";
              print "</script>\n";
           }
        } else {
           #--- this is not the first table in this mission so just add
           print "<script type='text/javascript' language='JavaScript'>;\n";
           print "tpAdd('$cm'+'tabs',  '" .$self->chopTable($table) . "_t');";
           print "changeSummaryLink('" . $self->chopTable($table) . 
		"','$cm');\n";
           print "</script>\n";
        }
        ++$self->{tcount};   #--- increment table count for next cycle
    }
}

sub YourSearchWas {
    my ($self,$positions,$qualifiers,$sorts,$dirs,$resmsg, $tables) = @_;

    #use Browse::TableFormat;
    #my $maxObjectListLimit=10;  # limit the number of targets to list

    #my $maxrows=$self->MaxRows;
    #my ($coord_sys,$equinox) = $self->getCoordSys();

    #--- Prepare to display SkyView images and Search Based On box
   # print "<div style='visibility:hidden' id='clickhere'>" .
	#"<p><b>Tabbed Display Format: Click on tabs as they are " .
	#"displayed above to navigate your results</b></div>";
    my $searchDisplayer = Browse::SearchBasedOn->new();
    $searchDisplayer->CoordSys($self->{coordsys}, $self->{equinox});
    $searchDisplayer->Positions($positions);

    #--- zzgen query uses non-user parameter seeting.  Do not display.
    my @nonzz = grep !/^zz/, @$tables;
    if (scalar (@nonzz) > 0) {
       $searchDisplayer->Qualifiers($qualifiers);
    }

    $searchDisplayer->MaxRows($self->MaxRows());

    # Display DSS and RASS images (if 1 object queried) and search criteria
    my $string = "";
    $string .= " <br><center>\n";
    $string .= "<!--images and search based on start--> <p><table><tr>\n";

    # If only one object queried display its SkyView image
    my $numpos=$searchDisplayer->Positions(); 
    my $num = 0;
    $num =  scalar(@$numpos) if $numpos;
    if ($num == 1) {
       $string .= "<td> <!--- start images cell --->\n";
       $string .= $searchDisplayer->displayImages();
       $string .="</td><!-- end images cell -->\n";
    }

    #--- Add resolver message
    if ($resmsg =~ /\w/) {
       $searchDisplayer->Resmsg($resmsg);
    }

    #--- Add saved query string
    if ($self->{savedquery} =~ /\w/) {
       $searchDisplayer->SavedQuery($self->{savedquery});
    }

    my $searchHTML = "<td><!-- start search based on cell -->";
    $searchHTML .= $searchDisplayer->displaySearch();

    #--- Add the requery button 
    #--- get the display mode to make sure it does not change upon re-query
    #my $displayformat= ref($self);
    #$displayformat=~ s/Browse\:\://;
    #$searchHTML .= "<input type=hidden name='requerydisplay' value='" .
	#$displayformat ."'>\n";
    $searchHTML .= "</td><!--end search based on cell -->\n";
    $string .= $searchHTML;

    #--- Display other format options
    $string .= "<td><!-- start other format options cell -->\n";
    $string .= "<table>";
    $string .= $self->displayFormatOptions();

    #--- Add button for saving targets to file used for target file upload
    $string .="<tr><td align='left' colspan='2'><input type='submit' " .
	"name='redisplay' value='Save All Objects To File' " . 
	"onClick=\"noNewWindowSubmit(this.form)\"></td></tr>\n"; 
    

    #--- Add Reset button 
    $string .="<tr><td align='left' colspan='2'><input type='reset'>" .
	"</td></tr>\n";

    $string .= "</table></td><!-- end other formats cell --></tr>\n";
    $string .= "</table></center>\n";

print $string;

#--- Add tip
print <<EOT;
<p><script language="JavaScript">
<!-- start hide
chooser = Math.round(Math.random()*(numberofitems-1));
document.write("<img src='$::images/dblbball.gif' align='bottom' alt='little blue ball'> <font size='-1'><b>Browse Tip</b>:  " +questions[chooser]+"</font>");
// stop hide -->
</script>
EOT


}



#--- create data products selection box and icons
sub createSelectDPBox {
   my ( $self,$unique,$cunique, $shortname, $offsetTarget) =@_;

   my $begin = "<a target=\"moreinfo\" href=\"w3hdprods.pl?".
      "files=P&Target=$cunique&Coordinates=$$self{coordsys}" .
      "&Equinox=$$self{equinox}\">" .
      "<img src=\"$::images/mag5.gif\" ".
      "height='16' width='16' border='0' " .
      "alt=\"Display all params\" onLoad=\"incrementCount(this,'$shortname')\">".
      "</a>";

   if ($unique =~ /\// && $unique !~ / x /i) {
      my ($table,$keyval) = split('\|\|\|', $unique, 2);
      ($keyval) = $keyval =~ /=\s*(.+)\s*\|\|$/;  
      my $key = $$self{meta}{tableinfo}{"query_key"};
      $begin .= "<a target=\"moreinfo\" href=".
      "\"$::vizier/viz-bin/VizieR-5?-source=$table&".
      "$key=". Browse::CGI::Encode($keyval) .
      "\"><img src=\"$::images/magviz.gif\" ".
      "height='16' width='16' border='0' " .
      "alt=\"Display all params (from VizieR)\">".
      "</a>";
   }
   my $offValue= $offsetTarget . "_unique_" . "$unique" 
		   if $offsetTarget =~ /\w/;

   $begin .= "<input type='checkbox' name='Target'" .
       " id='Target' value='$unique' onClick=\"addDP(this);";

   if ($offsetTarget =~ /\w/) { 
	$begin .= "addHiddenOffset(this,'$offValue');";
   } 

   $begin .= "\">";

   return $begin;
}



sub helpSection {
my ($self) =@_;
print "<div  style='visibility:hidden' id='help' t_title='Help' title='Click to view Tabbed Display help'> <!-- help -->\n";
print <<EOT;
<p>General Browse help can be found at the <a href='/W3Browse/w3browse-help.html'>Browse Help</a> page.
<h3>Browse Tabbed Display</h3>
<form>
<textarea cols='80' rows='30' readonly style='border:0px #fff;'> 
The tabbed Browse output organizes mission and table results in a set 
of hierarchical tabbed panels.  Rather than having to navigate a vast 
single page of results, users can jump from table to table.  

The Query Information tab displays the criteria on which the search is 
based, other display format options (i.e., HTML, Text), and the summary 
of results table which lists all table names that match the criteria 
and the number of rows in each table.  The table name is a link to the 
tab displaying the table results.

The Query Results tab lists a tab for each mission (middle tab level) and a 
tab for each table associated with each mission (bottom tab level).   To 
display table results first select the mission tab and then the table tab.
The table tab will display all table results.  From there you can perform 
table-related functions such as plotting and  cross-correlating as well as 
selecting data products.  For data products select the table rows you are 
interested in and the data products types.  These selected rows will now 
appear under the Data Products Retrieval tab.

The Data Products Retrieval tab will display all data products that you have 
selected.  From this page you can chose the download method to retrieve 
the data products or choose Web-based Services options (e.g., SkyView images, 
NED queries).  Deselect any data products you are no longer interested in 
before downloading.

</textarea></form>
EOT

print "</div>\n";

#--- Temporary loading tab
print "<div  id='loading' t_title='Processing Query...' title='This tab will disappear when all tables are displayed'> <h2>Your query results are being processed.</h2><p>Select the <b>Query Results Tab</b> to access completed tables. <br>The <b>Processing Query Tab</b> will disappear when all tables are displayed.  </div>\n";
print <<EOT;
<script type='text/javascript' language='JavaScript'>
var helpsection=document.getElementById('help');
if (helpsection) {helpsection.style.visibility='visible';}
tpAdd('tabcontrol','help');
tpAdd('tabcontrol','loading');
</script>
EOT
print "\n";

}

sub dpDiv {
   print "<div  id='dataproducts' t_title='Data Products Retrieval' " .
	"title='Click to view selected rows and options for data product retrieval'> <!-- dataproducts -->\n";
   
   print "<div style='max-width:500px;' id='prodsform'></div>\n";
   
   print "</div> <!--- end dataproducts div-->\n";
 

   print "<script type=\"text/javascript\" language=\"JavaScript\">;\n";
   print "if (document.getElementById && document.createElement) { ";
   
   print "var prodsform=document.getElementById('prodsform');\n";
   print "displayInstructions(prodsform);\n";
   print "}\n";
   print "tpAdd('tabcontrol','dataproducts')";

   print "</script>\n";

}
sub pageFooter {
   my ($self) = @_;

   print "<p>&nbsp;<p>\n";
   print "<span style='font-size:80%';><p>\n";
   $self->SUPER::pageFooter();
   print "</span>\n";
}
sub writeColumnData {
   my ($self,$class,$data) = @_;
   print "<td class='$class' id='tddata'>$data"; 
}

sub beforeNewTable {
    my ($self,$tablename, $tabletitle,$tabledesc) = @_;
    print "<script type='text/javascript' language='JavaScript'>\n";
    #print "alert('$tablename');\n";
    print "displayCurrentTableCount();\n";
    print "</script>\n";

    #--- start table div, hide table until it is completely rendered
    #
    print "<div style='visibility:hidden' id='$tablename" . "_t" .
	"' t_title='" . $tabletitle . 
	"' title='" .$self->{"meta"}{tableinfo}{table_description} . 
	"'>\n"; 
    #----
   
}
sub afterTable {
   my ($self)=@_;
   print "</div>   <!-- end table  div  -->\n";
}


## Successful load
1;

# $Log: TabbedDisplay.pm,v $
# Revision 1.12  2008/10/16 15:32:08  lmm
# Help section is hidden until fully written and placed in tab.
#
# Revision 1.11  2008/10/16 15:03:06  lmm
# Removed instructions for clicking tabs as they appear.
#
# Revision 1.10  2008/02/25 21:06:36  lmm
# Added hidden inputs for table before table display to fix column
#   resorting bug
#
# Revision 1.9  2008/01/30 15:20:50  lmm
# Added call to new Javascript function that adds row search offset to
#   be submitted with query form.
#
# Revision 1.8  2007/11/20 15:05:01  lmm
# Added encoding for Vizier parameter display link.
#
# Revision 1.7  2007/10/09 14:49:36  lmm
# Added new progress box that indicates how many tables have been processed.
#   Also using new Display class methods beforeNewTable() and afterTable().
#
# Revision 1.6  2006/11/06 16:05:47  lmm
# Added FOV to Services legend.
#
# Revision 1.5  2006/10/24 20:38:31  lmm
# Moved table shortname to beginning of table name in summary table.
# Add target name to offset column even if there is only one target.
# Removed ProcessData(), Instead code that differs from Display is
#   now in overridden Display method writeColumnData().
# PageFooter is overridden to use a different CSS style.
#
# Revision 1.4  2006/06/29 20:20:13  lmm
# Documentation tweaks.
#
# Revision 1.3  2006/06/29 19:28:41  lmm
# Added more instructions for Tabbed Format.
# Made Help textarea readonly.
# Revised call to incrementCount() to solve overincrementing values
#   problem in results summary table (seen in Mozilla, Netscape).
#
# Revision 1.2  2006/06/28 14:31:27  lmm
# When information about which missions the user selected is not available
#   an arbitrary mission associated with a table is picked (temporary fix).
# Minor edits to help text.
# Some initial fixes to problems with Vizier tables and tabbed format.
#
# Revision 1.1  2006/06/23 19:27:36  lmm
# initial commit
#
