# $Id: TabbedParameter.pm,v 1.3 2007/10/09 14:44:53 lmm Exp $
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    TabbedParameter.pm
# Module Author:  Laura McDonald, SSAI 
# Date:           June, 2006
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
# Overides Parameter class methods for Tabbed Display
#
#=======================================================================#

use strict;

package Browse::TabbedParameter;
 
use vars qw( @ISA );
@ISA = qw( Browse::Parameter);

use Browse::TabbedDisplay;

#------------------------------------------------------------------------

sub hiddenMissions {
   my($self,$in,$displayer,$table_info) = @_;
    my @selectedmissions;
    my $hiddenMissions="";
    foreach my $key (keys %$in) {
     
      if ($key=~ /^Observatory_|^mission/i ) {
        push @selectedmissions,   $in->{$key} ;
        $hiddenMissions .= "<input  type='hidden' name='$key' value='" .
	   $in->{$key} ."'>";
      }
      if ($key=~ /^displaymode/i ) {
        $hiddenMissions .= "<input  type='hidden' name='requerydisplay' " .
	   "value='" .  $in->{$key} ."'>";
      }
    }

    $displayer->setMissionList($table_info, \@selectedmissions);  
    return $hiddenMissions;
}

sub beforeAllTables {
   my ($self) = @_;
    print "<div id='missions' t_title='Query Results' " .
	"title='Click to see your query results'>\n";

    print "<div id='missiontabs'></div>\n";
}

sub afterAllTables {
   my ($self,$displayer,$allTablesCount) = @_;
    #--- Finish up divs and tabs after last table
    print "</div> <!--- current missions -->\n";   #TABBED BROWSE
    if ($allTablesCount == 0) {
       print "<script type='text/javascript' language='JavaScript'>;\n";
       print "tpSet('tabcontrol','tabs', new Array( 'intro'));\n";
       print "</script>\n";
    }
 
    my $cm = $displayer->currentMission();
    if (!$displayer->missionCount()== 0) {
       if (!$displayer->seenMission($cm)) {
          print <<EOT;
          <script type='text/javascript' language='JavaScript'>
          tpAdd('missiontabs','$cm');
          </script>
EOT
          print "\n";
       }
    }
          print <<EOT;
          <script type='text/javascript' language='JavaScript'>
          setTab();
          tpRem('tabcontrol','loading');
          </script>
EOT
print "<script type='text/javascript' language='JavaScript'>;\n";
    #print "alert('$tablename');\n";
    print "displayTotalRowCount('$allTablesCount');\n";
    print "</script>\n";

}
sub finishUp {
    my ($self,$hiddenMissions,$displayer) = @_;
    #--- Pass on some parameters that might not be included in
    #--- table re-sort or other results option
    if ($hiddenMissions) {print "$hiddenMissions\n";}
    print "</div></form>\n";
    print "<p><br><p>\n";
    
    Browse::TabbedDecor::EmailFooter();

    print "</body></html>\n";
}

sub displayLegend {
   #---  display legend for every table
   my ($self, $displayer, $first) =@_; 
      $displayer->legend();
}
#------------------------------------------------------------------------
1;
#########
#  EOF  #
#########
 
# $Log: TabbedParameter.pm,v $
# Revision 1.3  2007/10/09 14:44:53  lmm
# Using new Parameter class methods. This should prevent having to
#   change redudant code here and in Parameter.pm.
#
# Revision 1.2  2006/10/24 20:40:19  lmm
# Moved some hidden form inputs to TabbedDisplay.
#
# Revision 1.1  2006/06/23 19:28:47  lmm
# initial commit
#
