# $Id: Utils.pm,v 1.42 2007/10/03 15:21:49 lmm Exp $
# $Log: Utils.pm,v $
# Revision 1.42  2007/10/03 15:21:49  lmm
# isDuplicate() now uses more efficient hash rather than array to hold file list
#
# Revision 1.41  2005/10/20 13:51:15  lmm
# Fixed a variable re-declaration bug in isDuplicate()
#
# Revision 1.40  2005/05/16 16:53:50  lmm
# Added isDuplicate() whichs checks if a file is included in any
#   directory in a file list
#
# Revision 1.39  2004/12/09 19:29:26  lmm
# Combined Content-type and Content-Disposition lines for use with modperl
#   (requested for Akbar)
#
# Revision 1.38  2004/07/06 15:28:32  lmm
# Improved code that checks number of Browse processes currently running
#   (new code provided by Ed Sabol)
#
# Revision 1.37  2003/12/02 21:38:35  lmm
# Added subroutine dataToFile() to save a string to a file
#    on the user's machine
#
# Revision 1.36  2003/08/29 16:47:02  lmm
# Removed HEASARC footer
#
# Revision 1.35  2002/09/13 20:55:55  cwh
# Added checkprocs, which will tell you how many processes are running with
#   that match a string passed in.
#
# Revision 1.34  2002/01/11 20:57:09  cwh
# Had to define an Error sub in the main package, since Astro.pm calls it.
# Also did the same with MailMessage in case that is also called.
#
# Revision 1.33  2002/01/04 23:29:14  cwh
# Fixed a bug introduced recently where the format of the distance string was
# slightly changed, and the HDBmcs.pl script handling USNO A2 queries couldn't
# parse it.
#
# Revision 1.32  2001/12/18 21:24:29  cwh
# Fixed bug in getting real remote host name.
#
# Revision 1.31  2001/11/30 20:04:55  cwh
# Added the distance() subroutine, it was repeated in both the HeasarcQuery and
# CrossRequest modules.
#
# Revision 1.30  2001/11/07 23:41:20  cwh
# Rewrote includefile() to get rid of the call to UNIX 'cat'.
#
# Revision 1.29  2001/07/31 14:55:20  cwh
# Includefile looks for the include file in the docroot area first, and if
# it isn't there, looks for it in the current directory.
#
# Revision 1.28  2001/07/17 19:11:24  cwh
# Added includefile() subroutine, stolen from webspec, which recursively
# includes include files, and strips out any remaining server directives.
#
# Revision 1.27  2001/06/20 22:57:35  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.26  2001/04/25 17:19:08  cwh
# new function DumpRef.
#
# Revision 1.25  2000/03/08 20:13:28  saima
# W3Browse to Browse name change.
#
# Revision 1.24  2000/01/10 19:33:19  saima
# Added a routine to get the $::REMOTE_HOST variable from REAL_REMOTE_HOST.
#
# Revision 1.23  1999/05/27 17:23:58  tam
# Add 'package main;' to be mod_perl friendly.
#
# Revision 1.22  1998/12/23 18:57:33  saima
# Updated perl path. Modified Error() routine so that the html table
#   anchor is closed in case the error is displayed within an html
#   table layout.
#
# Revision 1.21  1997/11/25 23:37:13  saima
# Added -T switch to perl version.
#
# Revision 1.20  1997/05/09 22:50:26  sabol
# Improved &DumpHash() some to use less whitespace.
#
# Revision 1.19  1997/05/06 20:12:58  sabol
# Added DumpHash() function for printing out more complex data structures than
#   DumpAssoc() can handle.
#
# Revision 1.18  1997/04/17 22:09:44  sabol
# Minor programming style change.
#
# Revision 1.17  1997/03/21 16:02:37  sabol
# Made some minor changes to HTML produced by Error().
#
# Revision 1.16  1996/12/19 23:14:47  sabol
# Include function can now accept a list of files to "require."
#   DumpAssoc will now split on "\0" when printing hash values.
#   Changed order of arguments in memq.
#
# Revision 1.15  1996/10/25 21:32:23  sabol
# Now initializes $header_flag to zero in &Error.
#
# Revision 1.14  1996/10/24 18:42:17  sabol
# Now compliant with Perl 5's strict compiler directive.
#
# Revision 1.13  1996/09/13 21:43:33  sabol
# &Error now prints a </pre> tag before calling &Decor::Footer in case the
#   error occurred inside a <pre> section.
#
# Revision 1.12  1996/08/26 22:42:19  sabol
# Made cosmetic changes to Error which now uses Decor for the page header
#   and footer.
#
# Revision 1.11  1996/08/26 18:42:27  sabol
# Added memq and MakeStringList. Also, minor code cleaning.
#
# Revision 1.10  1996/06/27 17:15:49  sabol
# Esthetic improvements to Error subroutine.
#
# Revision 1.9  1996/06/25 21:00:52  sabol
# Updated all source code to be Perl 5-compatiable.
#
# Revision 1.8  1996/03/01  16:56:28  kressel
# More work on submit buttons. Including MailMessage routine.
#   Some small changes to aesthetics.
#
# Revision 1.7  1996/02/26  18:12:40  kressel
# Lots of small fixes to display of pages.
#
# Revision 1.6  1996/02/23  14:14:53  kressel
# Added 'Clean' subroutine (grabbed from CoordConv).
#
# Revision 1.5  1996/02/07  17:46:08  kressel
# Replaced gif buttons in Param Query with HTML buttons to standardize
# the GUI. Removed w3defpars, replaced by GetParams in earlier
# revisions.
#
# Revision 1.2  1995/12/27  17:27:54  kressel
# Redo of Initial CVS, to exclude binaries; and
#   GetParams.pl replaces w3defpars.pl
#   Presence of decimal point in 'ra' field implies Decimal degrees.
#   Have high level of confidence with handling of Coordinates now.
#   End of major version for December 95.
#
# Revision 1.1.1.1  1995/12/19 19:38:05  kressel
# W3Browse Initial CVS
#
# Revision 1.2  1995/11/07  15:15:39  kressel
# Added sub 'DumpAssoc'
#
# Revision 1.1  1995/10/17  17:41:13  kressel
# Initial revision
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    utils.pl
# Module Author:  John Kressel, Hughes STX
# Date:           Sept, 95
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
# This module contains subroutines for common tasks in Browse.
#
#=======================================================================#

use strict;
use lib './lib';


package Browse::Utils;
#------------------------------------------------------------------------
sub Include { # This routine is untested and unused. Maybe someday...

    my($fname);

    foreach $fname (@_) {
	unless (eval (require $fname)) {
	    &Error("'Require' failed for $fname!");
	}
    }

} #eosub--Include
#------------------------------------------------------------------------
sub DumpAssoc { # Dump all keys and their associated values to screen

    my(%ary) = @_;

    # Set up header
    print "<br>\n<pre>";
    print "       Key                               Value\n";
    print "-----------------------    ------------------------------------------\n";

    my($key,@values);
    foreach $key (sort(keys %ary)) {
	@values = split("\0",$ary{$key});
	if (scalar(@values) > 1) {
	    foreach (@values) {
		printf("%-25s  %s\n",$key,$_);
	    }
	} else {
	    printf("%-25s  %s\n",$key,$ary{$key});
	}
    }

    print "</pre>";

} #eosub--DumpAssoc
#------------------------------------------------------------------------
sub DumpHash { # Like DumpAssoc, expect it takes a ref to a hash and
               # can handle complex data structures like hashes of
               # hashes and hashes of hashes of arrays. It doesn't support
               # hashes where multiple values are delimited by "\0" like
               # DumpAssoc() does, and the output style is different.
               # Note: The $indent argument is optional.

    my($hashref, $indent) = @_;

    # Set up header
    print "<br>\n<pre>" unless $indent;

    my($key,$value);
    foreach $key (sort(keys(%{ $hashref }))) {
	(print "    " x $indent) if $indent;
	printf("%s => ",$key);
	if (ref($hashref->{$key}) eq 'ARRAY') {
	    print "[ ",join(', ',@{ $hashref->{$key} })," ]\n";
	} elsif (ref($hashref->{$key}) eq 'HASH') {
	    print "{\n";
	    &DumpHash($hashref->{$key},int($indent) + 1);
	    (print "    " x $indent) if $indent;
	    print "}\n";
	} else {
	    print "$hashref->{$key}\n";
	}
    }

    print "</pre>" unless $indent;

} #eosub--DumpHash
#------------------------------------------------------------------------------
sub DumpRef {

    my($ptr, $indent) = @_;

    # Set up header
    print "<br><pre>\n" unless $indent;

    if (ref($ptr)) {
        if (ref($ptr) eq "ARRAY") {
	    print $indent."[\n";
	    foreach (@$ptr) {
	        DumpRef($_, $indent." ");
	    }
	    print $indent."]\n";
	} else {
	    print $indent."{\n";
	    foreach (keys(%$ptr)) {
	       print "  $indent$_ => ";
	       my $val = $$ptr{$_};
	       if (!ref($val)) {
	           print "$val\n";
	       } else {
	           if (ref($val) eq "ARRAY") {
		       print "[\n";
		       DumpRef($val, $indent."  ");
		   } else {
		       print "{\n";
		       DumpRef($val, $indent."  ");
		   }
	       }
	   }
	   print $indent."}\n";
       }
    } else {
        print "$indent$ptr\n";
    }
}
#------------------------------------------------------------------------------
sub Error { # Print error message.

    my($error) = @_;
 
    Browse::Decor::Header("Error from Browse") if $::writeHeader;
    print "<br>" if $::writeHeader;
    print "<br><h2>$error</h2><br></pre>";
    print "</table></table></table>";
    #Browse::Decor::Footer();
    Browse::Decor::EmailFooter();
    Browse::Decor::EndTags();


    exit;

} #eosub--Error
#------------------------------------------------------------------------
sub Clean {

    local($_) = join(' ',@_);

    # Clean the input string.
    s/\n+/ /;    # change newlines to spaces
    s/^\s*//;	 # trim leading spaces
    s/^\+//;	 # trim leading plus sign
    s/\s*$//;	 # trim trailing spaces
    s/\s{2,}/ /; # squeeze embedded spaces

    return($_);

} #eosub--Clean
#------------------------------------------------------------------------
sub MakeListString {

    my($list) = "";

    if (scalar(@_) > 1) {
	$list = join(', ',@_);
	if (scalar(@_) == 2) {
	    $list =~ s/, / and /;
	} else {
	    $list =~ s/, (\S+)$/, and $1/;
	}
    } elsif (scalar(@_) == 1) {
	$list = shift(@_);
    }

    return($list);

} #eosub--MakeListString
#------------------------------------------------------------------------
sub memq {

    my($test) = shift(@_);
    my(@array) = @_;

    return((scalar(grep(($_ eq $test),@array))) ? 1 : 0);

} #eosub--memq
#------------------------------------------------------------------------
sub getREMOTEHOST {

  # Read in text.
  my $input = $ENV{'QUERY_STRING'};
  if($input =~ /REAL\_REMOTE\_HOST=([^&]*)/) {
    return($1);
  }

} #eosub--getREMOTEHOST
#------------------------------------------------------------------------
sub includefile {
   # read in an include file, recursively including any files it includes
   
     my $infile = ''; 
     if (-s $_[0]) {
         if (open (IN,$_[0])) {
            while (my $line = <IN>) {$infile .= $line}
            close IN;
         } 
     }


     my $include = '';
     while ($infile =~ /\<!--#include virtual="(.*?)" ?--\>/) {
        my $match = $1;
        my $includefile = $::root.$match;
        if (!-s $includefile and -s $match) {$includefile = $match};
        if (-s $includefile) {
           if (open (IN,$includefile)) {
               while (my $line = <IN>) {$include .= $line}
               close(IN);
           }
        }
        $infile =~ s/\<!--#include virtual="$match" ?--\>/$include/;
        $include = '';
     }
     $infile =~ s/\<!--#.*?--\>//g;
     return $infile;

} #eosub--includefile
#------------------------------------------------------------------------

sub distance {
# get the Haversine formula for the distance between two coordinates
#   in a string to be sent to the database
# $raf, $decf, are the field names or values of the ra, dec for the 1st coord
# $ra, $dec, are the field names or values of the ra, dec for the 2nd coord
# ra and dec values are asumed to be in decimal degrees.
# Results are in radians.
# Divide the result by $MTOR to get results in arcminutes.
# Divide the result by $DTOR to get results in degrees.

# NOTE: (HEASARC sites only) If you change the format of this string, you may 
#       cause the USNO server to fail to parse it!  

my($DTOR) = 0.017453292519943295769236907684886127134428718885417;

    my ($raf, $decf, $ra, $dec) = @_;

    foreach my $a ($ra,$dec,$raf,$decf) {
       if ($a !~ /\d/)  {$a = "($a*$DTOR)"}  else {$a *= $DTOR}
    }

    my $dist = "sin(($decf - $dec)/2)*sin(($decf - $dec)/2) + ".
               "cos($dec) * cos($decf) * sin(($ra-$raf)/2) * sin(($ra-$raf)/2)";
    $dist = "2*asin(sqrt($dist+1e-15))";
    return $dist;
}

#------------------------------------------------------------------------
# Save data to file
sub dataToFile {
   my $dataToSave = shift @_;
   my $localFileName = shift @_;

   # if there is no query to save exit with error mesage
   if ($dataToSave !~ /[a-zA-Z0-9]/) {
      Error("There are no download " .
	"capabilities available for the current query");
   }
   my @fileholder;
   # my $files_location = "/tmp/cwh";   #log file, maybe add later

   if ($localFileName eq '') { 

      print "Content-type: text/html\n\n"; 
      print "You must specify a file to download."; 

   } else {

      #-- maybe log file downloads later
      #open (LOG, ">>$files_location/test.log") || 
		#dataToFileError('open', 'file');

      #print LOG "$localFileName\n";
      #close (LOG);

      print "Content-Type:application/x-download\n" .
	"Content-Disposition:attachment;filename=$localFileName\n\n";
      print $dataToSave;
      
   }
exit;
}


#--- Checks if a file is included in any directory in a file list
sub isDuplicate {
   my ($file,$filelist) =@_;  #--- file name string and hash
   my $pos;
   $file =~ s/\/*$//;  #remove all ending slashes from file to be tested
   $pos  =  length $file; # set starting position to end of file name

   #--- move back through file name  path and get each  directory
   print "looking at $file<br>" if $::debug;
   while (($pos=  rindex $file,"/", $pos) >0 ) { 
      my $testdir =substr($file,0,$pos);

      print "<br>**$file, looking for $testdir<br>\n" if $::debug ;
      if ($$filelist{$testdir}) {
         
         print "found $file, while looking at $testdir<br>\n" if $::debug;
         return 1;
      }

      $pos--;
   } 
   print "NOT FOUND<br>\n" if $::debug ;
   return 0;   #-- file is not already in list
}


sub dataToFileError {
    print "Content-type: text/html\n\n";
    print "The server can't $_[0] the $_[1]: $! \n";
    exit;
}
#------------------------------------------------------------------------
# Find out how many browse processes are running
sub checkprocs {
  #my @out = `$::ps | grep $_[0] |grep -v grep`;
  #return scalar(@out);
  return(0) unless ($::ps);
  my @out = `$::ps`;
  my $regex = quotemeta($_[0]);
  return scalar(grep(/$regex/,@out));
}

#------------------------------------------------------------------------
# These are here to redefine any calls to main::Error or main::MailMessage
# in Astro.pm or other "external" modules to the subs here.

package main;
sub Error { Browse::Utils::Error(@_) };
sub MailMessage { Browse::MailMessage::MailMessage(@_) };

1;
#########
#  EOF  #
#########
