# $Id: ValidateInput.pm,v 1.38 2005/10/14 19:25:40 lmm Exp $
# $Log: ValidateInput.pm,v $
# Revision 1.38  2005/10/14 19:25:40  lmm
# Updated the JavaScript error message for when Start Search button is
#    clicked without object/coordinates or time entered.
#
# Revision 1.37  2005/02/23 18:59:57  lmm
# Changes to required user entries to allow master table queries
#
# Revision 1.36  2005/01/06 16:28:41  lmm
# Added back check for mission selection.  Will change again when master
#   table query is in place.
#
# Revision 1.35  2004/09/15 15:08:52  lmm
# Added check for entries separated by semicolon to inputboxTest()
# Added multiple object/coordinates test to checkMissions() to
#   determine if a Mission/Catalog needs to be selected
#
# Revision 1.34  2004/09/14 17:32:33  lmm
# Selection of mission is required for Parameter Search.  Change made for
#   Master Table query search.
#
# Revision 1.33  2004/03/05 18:10:11  lmm
# Removed uneccesary check for upload file entry and fixed some javascript
#    syntax
#
# Revision 1.32  2004/02/06 20:34:09  lmm
# Corrected upload file form name.
#
# Revision 1.31  2003/12/02 21:30:39  lmm
# Added check for Target upload file.
#
# Revision 1.30  2003/06/12 21:54:46  cwh
# Added tickTablestrict for when you need to check for strict equality in your
# string you're matching.
#
# Revision 1.29  2003/05/20 19:25:38  cwh
# Added instructions on how to get all rows of a table.
#
# Revision 1.28  2003/04/28 01:49:10  cwh
# Changed error notice to reflect that remote products are treated just like
# local ones.
#
# Revision 1.27  2002/09/13 21:00:45  cwh
# Changed '>0' to '>-1' in ticktable - 0 is a legal position for the table
#   name to be found in.
#
# Revision 1.26  2002/09/05 18:13:24  cwh
# Now passing in form name.
#
# Revision 1.25  2002/07/10 18:01:21  cwh
# Removed all usage of a specific form number - kept breaking with the
# appearance/disappearance of a form in the page header.
# Cleaned up tickTable.
#
# Revision 1.24  2002/03/17 19:16:26  cwh
# Added inputboxtest for Entry if big vizier tables are selected on param form.
# (This doesn't work yet, because some viz column names have punctuation in
# them (especially '.') that javascript can't handle).
#
# Revision 1.23  2002/02/14 17:44:42  cwh
# HTML cleanup: Added type="text/javascript" to <script> elements.
#
# Revision 1.22  2001/11/07 23:34:21  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.21  2001/10/04 22:12:13  cwh
# Removed hardcoding of variable name in tickVars().
#
# Revision 1.20  2001/09/17 19:26:02  cwh
# checkTablePage now allows either a time or a coord pair (or both). Before
#   a coord pair was required - failed with just a time.
#
# Revision 1.19  2001/09/15 14:07:14  cwh
# corrected typo in comment.
#
# Revision 1.18  2001/08/08 20:57:20  cwh
# Changed front page check so that it complains if position and time are
#   both empty, but either one alone is ok.
# Changed handling in setMax to not set max to **** if range or !range.
# Changed handlig in setMax to only reset max in range/!range if max is set to
#   ******.
# Eliminated check to ensure that at least one dataset is selected.
#
# Revision 1.17  2001/06/20 21:52:18  cwh
# Modified checkResults since the way we do actions is different now (was
#     checkboxes, now individual submit buttons).
# Removed changeOptions() since we don't use it anymore.
# In checkDPList, removed checkbox test for URL (not used anymore) and changed
#     the error message.
#
# Revision 1.16  2001/06/18 18:54:43  cwh
# Reversed prior commit. Not doing any special handling of vizier checkbox now.
#
# Revision 1.15  2001/06/14 22:55:52  cwh
# Now ok not to have a mission checked for Basic queries if vizquery is checked.
#
# Revision 1.14  2001/05/17 20:02:07  cwh
# Now tickTable checks to see if the field name it's supposed to be checking for
# just begins with the string supplied to the function, it doesn't have to match
# the whole string.  This is so for the Cross-correlations, you can just send
# tickTable 'RetrieveSet' and it'll change any checkbox with name 'RetrieveSet*'.
#
# Revision 1.13  2001/04/03 16:27:28  cwh
# - changed checking of 'Observatory' checkboxes to checking of any checkboxes
#   that start with the string 'Observatory'.
# - added checking of tablehead as well as the Observatory* checkboxes.
# - added setadvanced function, so that we can check the adv_query flag to see
#   if we need to check to see if the Entry field is true.
#
# Revision 1.12  2001/01/02 23:33:46  cwh
# Changed all document.forms[1] back to ...[0], since we are once again the top
# form on all the pages.
#
# Revision 1.11  2000/05/05 17:09:53  saima
# Updated message if a user selects to create a tarfile without
#   specifiying files.
#
# Revision 1.10  2000/04/27 17:55:59  saima
# Modified tickTable() so that in cases of multiple ticker buttons, the
#   routine checks if any of them are checked not just the first one.
#
# Revision 1.9  2000/04/25 18:50:54  saima
# Modified tickTable() so that if a user clicks on 'All' checkbox,
#   all 'Target' rows are ticked on. Also, added changeOptions() routine
#   so that if a user selects any set, the radio button for 'Data
#   Products Options' is set to 'List data products in the selected categories'.
#
# Revision 1.8  2000/03/08 20:13:14  saima
# W3Browse to Browse name change.
#
# Revision 1.7  2000/02/28 21:42:29  saima
# Modified JavaScript form number since the menubar has form[0].
#
# Revision 1.6  1999/05/27 17:22:30  tam
# Don't return an error if all of the boxes checked refer
#   to remote URLs. [Probably shouldn't have any affect at the moment].
#
# Revision 1.5  1999/02/22 18:54:02  saima
# The number of mission tables is passed to checkTablePage() to make
#   sure that JavaScript checks are not added if only one table is available.
#
# Revision 1.4  1999/02/03 20:37:36  saima
# Added tickTable() which allows users to select all tables from the
#   tables page.
#
# Revision 1.3  1999/01/28 19:54:18  saima
# Added a JavaScript showRanges() routine which can blank out/reset
#   min/max range fields.
#
# Revision 1.2  1999/01/26 23:23:46  saima
# In checkParameterQuery() JavaScript code to check for 'varon' is not
#   added if multiple tables are being queried.
#
# Revision 1.1  1999/01/26 22:12:12  saima
# Removed all routines that validated deprecated pages. Added
#   checkParameterQuery() to help set the parameters search page.
#   Also added checkTablePage() to validate the table selections page.
#
# Revision 1.6  1998/08/03 22:10:42  saima
# Replaced the form variable name 'target' with 'Target'. The name 'target' gives
#   problems with JavaScript on Netscape 2. Also, made changes to code
#   that checks the checkbox variable validity. JavaScript on IE3 does
#   not know how to create arrays for variables so that the
#   construct 'document.forms[0]$form_var[i]' would not work properly on
#   IE3.
#
# Revision 1.5  1998/07/27 15:30:51  saima
# If the 'build query' option is used, user should be allowed to submit
#   even if he/she has selected a parameter from the select box and not
#   set criteria for it.
#
# Revision 1.4  1998/07/21 19:48:18  saima
# Major revision to the way routines are organized. Different
#   HTML form element tests were split up in separate routines.
#   The new routines: checkboxTest(), inputboxTest(), and
#   selectboxTest() are called to add javascript code for each
#   type of form element.
#
# Revision 1.3  1998/07/15 19:23:55  saima
# Removed an extra routine that should have been deleted before!
#
# Revision 1.2  1998/06/26 21:11:19  saima
# Modified checkParamQuery() so that users can submit after completing the
#   cone search option if they want. Users are forced to complete
#   at least one option (either cone or parameter search option) before submitting.
#
# Revision 1.1  1998/06/12 19:54:21  saima
# Initial Revision.
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    ValidateInput.pl
# Module Author:  Saima Zobair, Raytheon STX
# Date:           June 4, 1998
# Project:        Browse : HEASARC
#-----------------------------------------------------------------------#
# Module Description:
#
# The routines in this module add JavaScript to Browse forms. The routines  
# check user input and make sure that users have filled/selected the
# required form variables.
#
#=======================================================================#

use strict;

package Browse::ValidateInput;
#------------------------------------------------------------------------
sub scriptHeader{

  my($vars) = @_;
  print <<EOT;

<script type="text/javascript" language="JavaScript">
<!--
//function checks if user has entered input where necessary
function pageCheck($vars) {
EOT

} #eosub--ValidateInput::scriptHeader
#------------------------------------------------------------------------
sub addNavigatorCheck{

  print <<EOT;
        //check if the browser is valid
        var nav = navigator.appName;
        var ver = navigator.appVersion;

        if (nav == "") { return(true); }
        if (nav == "Microsoft Internet Explorer") {
          if ((ver.charAt(0)) < 2) { return(true); }
        }
EOT

} #eosub--ValidateInput::addNavigatorCheck
#------------------------------------------------------------------------
sub checkboxTest {

  # Function inputs:
  #    $form_var  - checkbox html form variable to validate
  #    $scriptvar - JavaScript variable name for this checkbox

  my($form,$form_var,$scriptvar) = @_;

  my($script_check)= $scriptvar . "_check";

  print <<EOT;

        //initialize input as invalid and get entered values from form
        //var $script_check = false;
        var $script_check;
        var flds = document.$form.elements;
        //has the checkbox been marked?
        for (var i=0; i<flds.length; i++) {
           var fld = flds[i];
           // does field name start with $form_var and is it checked?
           if ((fld.name.indexOf("$form_var")==0) && fld.checked) {
             $script_check = true;
             break;
           }
        }
EOT
  
} #eosub--ValidateInput::checkboxTest
#------------------------------------------------------------------------
sub inputboxTest{

  # Function inputs:
  #    $form_var  - inputbox html form variable to validate
  #    $scriptvar - JavaScript variable name for this inputbox

  my($form,$form_var,$scriptvar) = @_;

  my($script_val)   = $scriptvar . "_val";
  my($script_check) = $scriptvar . "_check";
  my($script_mult) = $scriptvar . "_mult";
  print <<EOT;
        // initialize input $form_var value as invalid
        var $script_check = false;
        var $script_mult = false;

        //get entered values from form
        var $script_val   = document.$form.$form_var.value;

        //did user enter something for $form_var?
        if (!$script_val){
            $script_check = false;
        } else {
            //check for spaces only
            var sp = true;
            var elen = $script_val.length;
            for (var i=0; i<elen; i++) {
               if ($script_val.charAt(i) != " "){
                $script_check = true;
                 break;
               }
            }
            //-- check for multiple objects
            if ($script_check) {
               var entries=$script_val.split(";");
               if (entries.length > 1) {
                  $script_mult=true;
               }
            }
        }
EOT

} #eosub--ValidateInput::inputboxTest
#------------------------------------------------------------------------
sub selectboxTest{
# NOT CURRENTLY USED

  # Function inputs:
  #    $form_var  - selectbox html form variable to validate
  #    $scriptvar - JavaScript variable name for this selectbox

  my($form,$form_var,$scriptvar) = @_;

  my($script_sel)   = $scriptvar . "_sel";
  my($script_check) = $scriptvar . "_check";
  print <<EOT;
        // initialize input $form_var value as invalid
        var $script_check = false;

        //get entered values from form
        var $script_sel  = document.$form.$form_var.selectedIndex;

        //did user select $form_var?
        if ($script_sel < 0){
           $script_check=false;
        }else {
           $script_check=true;
        }
EOT

} #eosub--ValidateInput::selectboxTest
#------------------------------------------------------------------------
sub checkMissions{
  my($form) = @_;

  &scriptHeader();
  &addNavigatorCheck();
  &checkboxTest($form,"Observatory","obs");
  &checkboxTest($form,"tablehead","obs");
  &inputboxTest($form,"Entry","entry");
  &inputboxTest($form,"Time","time");
  #--- added for user file upload  
  &inputboxTest($form,"upfile","entryfile");

  print <<EOT;

        //if (!obs_check) {
        //--- Mission required for Detailed search and multiple objects
        //if (!obs_check && (adv_query || entry_mult) ) {
        if (!obs_check && (adv_query ) ) {
            alert("Please select a mission or catalog.");
            return(false);
        }
        var flds = document.$form.elements;

        if (!adv_query) {
           if (!entry_check && !time_check && !entryfile_check){
            alert("Please enter a name/coordinate pair if you would " +
                  "like to run a coordinate/name search, or an observation "+
                  "date if you would like to run a time search.\\n\\n" +
                  "If you are interested in searching by other parameters" +
		  " or retrieving all rows of a table:\\n"+
                  "- Click the 'Detailed Mission/Catalog Search' button"+
                  "\\n"+
                  "- Select the table(s) you are interested in.\\n" +
                  "- Click "+
                  "'Specify Additional Parameters' for a parameter search\\n"+
                  "OR\\n" +
                  " Set the 'Limit Results To' value to the one you want " +
		  "(set this to 'No Limit' to return all rows of the " +
		  "selected tables) and click the 'Submit Search' button.");
              return(false);
           }
        }

        // things look good if you are at this point
        return(true);  
}

function setadvanced (val) {
  adv_query=val;
}//-->

</script>
EOT
} #eosub--ValidateInput::checkMissions
#------------------------------------------------------------------------
sub checkParameterQuery{

  my($form,$tables) = @_; 
  my($numtable) = scalar @$tables;
  &scriptHeader();
  &addNavigatorCheck();

  foreach my $table (@$tables) {
    if ($table =~ /B\/2mass|I\/271|I\/252/i) {
        &inputboxTest($form,"Entry","entry");
        last;
    }
  }

  if ($numtable < 2) {
    &checkboxTest($form,"varon","varon");
    print <<EOT;
        // now print error messages where necessary
        if (!varon_check){
            alert("Please check at least one parameter under 'View'.");
            return(false);
        }
EOT
  }
  print <<EOT;
        // things look good if you are at this point
        return(true);
}//end of pageCheck()

function tickVars(ename,on) {
    // on is either true (turn them on) or false (turn them off)
    // ename is the name of the checkboxes to turn on or off.
    var flds=document.$form.elements;
    for(var i=0; i< flds.length; i++){
      if (flds[i].name == ename) {
         if(on) {
           flds[i].checked=1;
         } else {
           flds[i].checked=0;
         }
      }

    }
   return
}//end of tickVars()

function setMax(x,checkvar) {
    //get entered values from form
    var flds  = document.$form.elements;

    for (var i=0; i<flds.length; i++) {
      var fld = flds[i];
      if (fld.name == checkvar) {
         var  script_sel  = fld.selectedIndex;  
         if (script_sel != 0 && (script_sel != 1)){
            flds[i+2].value = '**********'; 
         } else {
            if (flds[i+2].value == '**********') {
              flds[i+2].value = x; 
            }
         }
      }
    }

}//end of setMax()

function showRanges() {
    var flds = document.$form.elements;

    //has the checkbox been marked?
    if (!document.$form.show.checked) {
        for (var i=0; i<flds.length; i++) {
           var fld = flds[i];
           //if w3min or w3max set to ''
           if ((fld.name.charAt(0) == 'w') && (fld.name.charAt(1) == '3') &&
               (fld.name.charAt(2) == 'm')) {
              if ((fld.name.charAt(3) == 'a') && (fld.name.charAt(4) == 'x') ||
                  (fld.name.charAt(3) == 'i') && (fld.name.charAt(4) == 'n')){
                 fld.value = '';
              }
           }
        }
    } else {
        document.$form.reset();
    }
}//end of showRanges()-->
</script>
EOT

}
#------------------------------------------------------------------------
sub checkTablePage{

  my($form,$num_table) = @_;

  &scriptHeader("cone");
  &addNavigatorCheck();
  &inputboxTest($form,"Entry","entry");
  &inputboxTest($form,"Time","time");
  #--- added for user file upload  -- lmm
  #&inputboxTest($form,"EntryFile","entryfile");
  &inputboxTest($form,"upfile","entryfile");

  if ($num_table > 1) { 
    &checkboxTest($form,"tablehead","tablehead");

    print <<EOT;
        // now print error messages where necessary
        if (!tablehead_check){
            alert("Please select a table.");
            return(false);
        }
EOT
  }

  print <<EOT;
        // now print error messages where necessary
        // now print error messages where necessary
        if (cone=="on" && !entry_check && !time_check && entryfilecheck){
            alert("Please enter a name/coordinate pair if you would " +
                  "like to run a coordinate/name search, or an observation "+
                  "date if you would like to run a time search.\\n\\n"+
                  "If you are interested in retrieving all rows of a table:\\n"+
                  "- select the table(s) you are interested in, then click "+
                  "'Specify Additional Parameters'.\\n"+
                  "- change the maximum number of rows to the number you'd "+
                  "like, then click 'Submit'.");
            return(false);
        }

        // things look good if you are at this point
        return(true);
}// pageCheck()

function tickVars(ename) {
    return;
}//tickVars()


EOT
&tickTable();

print "\n</script>";

} #eosub--ValidateInput::checkTablePage
#------------------------------------------------------------------------
sub tickTable{

  print <<EOT;
  function tickTable(flds,ischecked,table) {
    for(var i=0; i< flds.length; i++){
      if ( (table == '') || (flds[i].value.indexOf(table) > -1) ) { 
        flds[i].checked=ischecked;
      }
    }

    return;
}
  function tickTablestrict(flds,ischecked,string) {
    for(var i=0; i< flds.length; i++){
      if ( flds[i].value == string) { 
        flds[i].checked=ischecked;
      }
    }

    return;
}//tickTablestrict()-->
EOT
#//tickTable()-->
} #eosub--ValidateInput::tickTable
#------------------------------------------------------------------------
sub checkResults{
  my ($form,$check) = @_;

  &scriptHeader();
  &addNavigatorCheck();
  print <<EOT;

        // initialize input values as invalid
        var target_check  = false;
        var retrieveset_check  = false;
EOT

  &checkboxTest($form,"Target","target");
  &checkboxTest($form,"RetrieveSet","retrieveset");

  print <<EOT;
        if (!target_check) {
            alert("Please select at least one row in the table(s) of results.");
            return(false);
        }

       //if (checkdp) {
        //if (!retrieveset_check) {
        //    alert("Please select at least one set of data products.");
        //    return(false);
        //}
       //}
       return(true);
  } // pageCheck()

function setcheckDP (val) {
  var checkdp=val;
}

EOT

   &tickTable();
print "\n</script>";

} #eosub--ValidateInput::checkResults
#------------------------------------------------------------------------
sub checkDPList{
  my ($form) = @_;

  &scriptHeader();
  &addNavigatorCheck();
  &checkboxTest($form,"File","file");

  print <<EOT;
        if (!file_check) {
            alert("No data products selected. \\n\\nPlease check "+
                  "the checkboxes next to the products desired and try again.");
            return(false);
        }
        return(true);
}//-->

</script>
EOT

} #eosub--ValidateInput::checkDPList
#------------------------------------------------------------------------
#########
#  EOF  #
#########
1;
