#!/usr/bin/perl -T
#
# This script creates a web page to allow users to create an manage notification
# of updates for sources in current mission BROWSE tables
# 
#
#
use strict;
use POSIX qw(strftime);
$ENV{PATH} = "/bin:/usr/lib:/www/htdocs/cgi-bin/W3Browse/bin/websec";

#--- user directories
my $websecdir = "/www/htdocs/tmp.shared/websec";
my $userdir = "/www/htdocs/tmp.shared/websec/bnotification/users";

#--- prepare statistics file
my $bnotfile= "$userdir/.bnotstats";
if (-e $bnotfile) {
   if ($bnotfile =~ /^([-\@\+:\/\\\w.]+)$/) {
      $bnotfile = $1;
   } else {
      die 'illegal characters in filename. ';
   }
   print "file = $bnotfile\n";
   if ($bnotfile =~ m/\.\./) {
      die "filename contains the sequence '..' ";
   }
   if (defined $bnotfile) {
      print "  removing=$bnotfile\n";
      unlink $bnotfile;
   }
}

#check if runwebsec is aleady running. if so, exit.
$0 =~ /.*\/(.*)/;
my $scriptname = $1;
print "script name =$scriptname!\n";

my $s = `ps -ef|grep $scriptname|grep -v grep|grep -v vi|grep '/usr/bin/perl'`;
print "**$s**\n";
chomp $s;
my @tmp = split /\n/, $s;
my $nprocs = @tmp + 0;
if ($nprocs > 1) {
   print "run websec already running - $nprocs process(es)\n";
   exit;
}

#--- Get user names from command line or user directory
my @users;
if ($#ARGV == 0) {
   # one user allowed on command line
   @users=($ARGV[0]);
} else {
   #--- Read user names from user directory
   opendir USERS, $userdir or die "unable to read $userdir";
   (@users) = grep !/^\.\.?$/, readdir USERS;
   close (USERS);
}

print "There are " +scalar(@users) + " users.\n"; 

if (scalar(@users) == 0) {
   print "There are no users\n";
   exit;
} 

my $out="";
my $cmd="";
my $daysold=30;
my $usercount=0;
#--- Run websec command for each user
foreach my $user (@users) {
   print "******* $user ********** \n";
   #if ($user =~ /yichou/ )  {
      #--- if no url list account is not active so remove archive directory
      if (! -e "$userdir/$user/url.list")  {
         my  $archivedir = "$userdir/$user/archive";
         
         #--- Untaint for removal
         if ($archivedir =~ /^([-\@:\/\\\w.]+)$/) {
            $archivedir=$1;
            #print " archivedir=$archivedir\n";
         } else {
            die 'illegal characters in directory. ';
         }
         if ($archivedir =~ m/\.\./) {
            die "Directory contains the sequence '..' ";
         }
         if (-e "$userdir/$user/url.list.save" && -e $archivedir) {

            #--- notification has been canceled so delete archive directory
            #---- first delete files
            if (opendir(DIR, "$archivedir")) {
              my @files = grep !/^\.\.?$/, readdir DIR; #-not '.' and '..' files
              closedir(DIR);
	      foreach my $file (@files) {
                 #print " before taint check:file = $file\n";
                 $file="$archivedir/$file\n";
                 if ($file =~ /^([-\@\+:\/\\\w.]+)$/) {
                      $file = $1;
                 } else {
                    die 'illegal characters in filename. ';
                 }
                 #print "file = $file\n";
                 if ($file =~ m/\.\./) {
                    die "filename contains the sequence '..' ";
                 }
                 if (defined $file) {
                    #print "  removing=$file\n";
                    unlink $file;
                 }
              }
            }
            #---- now delete directory
            print "removing $archivedir\n";
            rmdir ($archivedir) or print "not able to remove dir $! \n";
         
         } elsif (-d "$userdir/$user" && -M "$userdir/$user" > $daysold) {
            #--- User directory was created but never used so delete it
            my $udir = "$userdir/$user"; 
            if ($udir =~ /^([-\@:\/\\\w.]+)$/) {
               $udir=$1;
               #print "deleting $udir\n";
            } else {
               print "illegal characters in directory $udir.\n ";
               next;
            }
            if ($udir =~ m/\.\./) {
               print "Skipping  $udir. It contains the sequence '..' \n";
               next;
            }
            if (opendir(DIR, "$udir")) {
              my @ufiles = grep !/^\.\.?$/, readdir DIR; #-not '.' '..' files
              closedir(DIR);
              #--- Remove setup files first
	      foreach my $ufile (@ufiles) {
                 $ufile="$udir/$ufile\n";
                 if ($ufile =~ /^([-\@\+:\/\\\w.]+)$/) {
                      $ufile = $1;
                 } else {
                    print "Skipping $ufile. It is tainted \n";
                    next;
                 }
                 #print "file = $ufile\n";
                 if ($ufile =~ m/\.\./) {
                    print "Skipping $ufile. It contains the sequence '..' \n";
                    
                 }
                 if (defined $ufile) {
                    #print "  removing=$ufile\n";
                    unlink $ufile;
                 }
              }
            }
            #--- Remove the directory
            my $rmdirtest;
            $rmdirtest=rmdir($udir);
   
            #--- if there is no error removing the directory rmdir returns 1.
            if ($rmdirtest == 1) { 
                print "$udir deleted. It was greater than $daysold days old\n";
 	    } else {
		print "===>error deleting $udir: $! \n";
            }

         }
         next;
      }

      my @times = localtime(time);
      my $today = "$times[2]:$times[1]:$times[0]";
      #$cmd = "websec.browse --base " .  "$userdir/$user";
      $cmd = "websec.browse --base " .  "$userdir/$user" . " --notfile " .
	"$bnotfile";

      print "$today: running $cmd\n";
      #--- Untaint
      if ($cmd =~ /([-\@\w\/\s\.]+)$/) {
         $cmd=$1;
         print " cmd=$cmd\n";
      } else {
         die 'illegal characters in command. ';
      }
      if (! is_tainted($cmd)) {
         $out=`$cmd`;
         print "out=$out\n";
      }
      ++$usercount;
   #}  #--- end of debugging 
    sleep 5;   #--- comment out if debugging
}
print "processing finished\n";

open( STATS, ">>$bnotfile" ) or print "stat file could not be opened: $!";
print STATS "\n$usercount users are monitoring sources using the " .
	"Notification Service\n";
close (STATS);

sub is_tainted {
   not eval {
     join("",@_), kill 0;
     1;
   }
} 

