#!/usr1/local/bin/perl5 -T
#
# $Id: w3catindex.pl,v 1.31 2003/08/28 20:05:25 lmm Exp $
# $Log: w3catindex.pl,v $
# Revision 1.31  2003/08/28 20:05:25  lmm
# Remove HEASARC footer
#
# Revision 1.30  2003/07/23 20:41:26  cwh
# Changed olegacy to heasarcdev.
#
# Revision 1.29  2003/04/28 02:37:02  cwh
# Added new table style.
# Now repeating column headers for each mission.
#
# Revision 1.28  2002/09/13 18:22:57  cwh
# Pulled out ftp dir and ftp host which were heasarc-specific.  Now using
#    Config vars.
# Corrected some html.
#
# Revision 1.27  2002/02/14 17:34:35  cwh
# Removed '|| 5' in priority = $$_[1] || 5, since 0 is a legal priority.
# Added table type to params output for each table.
# Added code for ordering tables by priority (currently commented out)
# Added 'data prducts available' to the params output for each table.
#
# Revision 1.26  2001/12/03 20:42:41  cwh
# Added query of zzdpsets to set 'archive', now sent in tablehead.
# Added saving of default search radius instead of just sending '1' in tablehead.
# Added sending of priority in tablehead.
# Changed spaces to nbsps in parenthesized table names.
#
# Revision 1.25  2001/11/08 20:07:10  cwh
# Changed 'do w3config.pl' to 'use Config' and AlwaysRun().
# Added Browse:: to all sub calls.
# Changed unnecessary globals to local variables.
# Updated special links, mission name translations.
# Added additional location to look for mission docs in.
#
# Revision 1.24  2001/07/23 20:04:34  cwh
# Added priority back in if host =~ olegacy or debug is on.
#
# Revision 1.23  2001/07/09 17:48:37  cwh
# Just removed the actual number priorities (those are internal).
#
# Revision 1.22  2001/06/14 14:25:56  tam
# Fixed bug in link to TDAT files.
# Deleted obsolete text about links from number of parameters.
#
# Revision 1.21  2001/05/24 16:40:44  cwh
# This mod_perl stuff is killing me.  Got rid of all the BEGIN blocks (those aren't
# called if they've been already done once).  Added the 'use lib "./lib";'s back in.
#
# Revision 1.20  2001/05/03 19:10:02  cwh
# Put call to w3config.pl in a BEGIN statement, pulled all 'use lib's out.
# Now uses global vars defined in w3config.pl wherever possible.
# Changed action of form to 'Parameter+Search'.
#
# Revision 1.19  2001/04/03 15:59:14  cwh
# - Added taint flag
# - added query to get priorities
# - print priority for each table
#
# Revision 1.18  2001/01/22 20:30:59  cwh
# Added navigation trail.
#
# Revision 1.17  2001/01/02 22:59:55  cwh
# Added target=helpwin for all help links.
#
# Revision 1.16  2000/10/12 20:13:39  cwh
# Tom modified this so that catalogs can appear under more than one mission.
#
# Revision 1.15  2000/06/13 14:38:04  saima
# Fixed broken links to mission descriptions.
#
# Revision 1.14  2000/05/24 16:23:09  saima
# Updated to reflect name changes for 'rxte' and 'bepposax'.
#
# Revision 1.13  2000/03/17 16:15:33  saima
# Fixed EUVE and SAX description broken links.
#
# Revision 1.12  2000/03/08 19:56:28  saima
# W3Browse to Browse name change.
#
# Revision 1.11  1999/11/04 18:02:47  saima
# Bug fix.
#
# Revision 1.10  1999/08/26 17:14:18  saima
# Added more error checks.
#
# Revision 1.9  1999/05/27 15:33:10  tam
# Modified to use mod_perl.  Modified to use DBQ.pm instead of SQL.pm.
#
# Revision 1.8  1998/12/21 19:39:29  saima
# Added new url links for parameter search page.
#
# Revision 1.7  1998/12/03 20:40:58  saima
# Added check where if the test olegacy version of W3Browse was in
#   use, the table help files should link to 'olegacy' webserver.
#
#=========================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA/Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=========================================================================#
# Module Name:    w3catindex.pl
# Module Author:  Thomas McGlynn, USRA
# Date:           August, 97
# Project:        Browse : HEASARC
#=========================================================================#
# Module Description:
#  This script generates an index page describing all of the
#  remotely accessible tables in the Browse system.  It queries
#  the metabase to find all information.  This page also
#  links to the missions description pages and the FTP area
#  for information for the mission to which a table pertains.
#=========================================================================#

use strict;
use lib './lib';

use Browse::Config;

Browse::Config::AlwaysRun();

#--------------------------------------------------------------------------

# These two tables address inconsistencies in the
# way we label missions at various points in our system.
# The key is the mission name as specified in the metabase,
# while the value is the mission name used in either
# the FTP area or in the mission documentation.

my %mission_trans_ftp = ( "arielv", "ariel5",
                          "cgro", "compton",
                          "xmm-newton", "xmm",
                          "rxte", "xte",
                          "sas-3", "sas3",
                          "bepposax", "sax",
		          "sas-2", "sas2") ;
my %mission_trans_doc = ( "arielv", "ariel5",
		          "sas-2", "sas2",
		          "sas-3", "sas3",
		          "einstein", "heao2",
		          "smm", "solarmax",
		          "td1", "td-1a",
		          "xmm-newton", "xmm",
		          "venera", "venera1112") ;
			  
# The specialLink's are help for missions that are not
# in our mission standard pages.  This is primarily for non-high-energy
# missions.

my %specialLink = (
   "cgro",	'http://cossc.gsfc.nasa.gov/cossc/cgro/index.html',
   "einstein",  'http://heasarc.gsfc.nasa.gov/docs/einstein/heao2.html',
   "euve",      'http://heasarc.gsfc.nasa.gov/docs/euve/euve.html',
   "hst",	'http://www.stsci.edu/hst/',
   "iras",	'http://www.ipac.caltech.edu/ipac/iras/iras.html',
   "bepposax",	'http://heasarc.gsfc.nasa.gov/docs/sax/instruments/general_l2.html',
   "iso",       'http://www.iso.vilspa.esa.es/',
   "uit",       'http://praxis.pha.jhu.edu/instruments/uit.html',
   "iue",       'http://www.vilspa.esa.es/iue/iue.html',
   "rxte",       'http://legacy.gsfc.nasa.gov/docs/xte/XTE.html');

# Store the number of tables for each mission.
my %missionCount = ();

my ($mission, $table);
my $oldmission;

# Get the information we will be displaying.
my %info = indexInfo(\%missionCount);

# Print the header information.
printHeader();

# Find the files in the FTP area.  We do it this way just
# in case some part of the FTP area is in some mount difficulty.
# We're just reading the main directory rather than checking the
# existence of the sub-files, so even if a particular part
# of the FTP area is not mounted, the mount-point directory will
# still be around.

opendir(FTP, "$::tarftpbase");
my @ftpdirs = readdir(FTP);
closedir(FTP);
my %ftphash;
for (@ftpdirs) {
    $ftphash{$_} = 1;
}


# Print the set of links connecting the file internally.
missionPointers(\%info);

# Print the total number of tables.
my @keydata = keys(%info);
print "<p><b> Total tables: </b>",$#keydata+1,"</b><p>";

# Now loop over all of the tables.
my $key;
my $descrip;

for $key (sort(keys(%info))) {
    ($mission, $descrip, $table) = split('\|', $key);
    
    # If it's a new mission we need to start a new table.
    if ($mission ne $oldmission) {
        missionHeader($oldmission, $mission, \%ftphash, \%missionCount);
	$oldmission = $mission;
    }
    
    # Print the entry for this table.
    printEntry($info{$key});
    
}

print "</table><p><br>\n";
# Print necessary footer information.
#Browse::Decor::Footer();
Browse::Decor::EmailFooter();
Browse::Decor::EndTags();


# end of main
#--------------------------------------------------------------------------
sub indexInfo {
    my ($missionCount) = @_;

    # This subroutine does all the real work of extracting data
    # from the metabase.

    # First find all of the tables that we are interested in, i.e.,
    # all tables in ZZGEN which do not begin with 'zz' and do not
    # have a w3browse_display='n' entry in zzext.
    
    my $sql1 = "select distinct table_name, table_description, table_document_url, table_rows from zzgen where table_name not like 'zz%' ".
               " order by table_name ";
    
    my ($results1) = Browse::DBQ::query($sql1);
    
    if ($#$results1 < 0) {
        Browse::HDB::HDBerror("Index: Unable to access metadata information");
    }
       
    my $sql2 = "select table_name from zzext".
        " where parameter_name='w3browse_display' and ((parameter_value='n') or ".
              " parameter_value='N')";
    
    my ($results2) = Browse::DBQ::query($sql2);
    
    my %hiddenhash;
    for (@$results2) {
         $hiddenhash{$$_[0]} = 1;
    }
    
    # Now find out how many parameters each table has.
    my $sql3 = "select table_name, count(parameter_name) from zzpar group by table_name";
    
    my ($results3) = Browse::DBQ::query($sql3);
    my %numcols;
    
    if ($#$results3 < 0) {
        Browse::HDB::HDBerror("Index: Unable to count parameters in tables");
    }
    
    foreach (@$results3) {
        $numcols{$$_[0]} = $$_[1];
    }

    # Find out which mission each table belongs to.
    my $sql4 = "select table_name, parameter_value from zzext ".
               "   where parameter_name = 'observatory_name'";
    
    my ($results4) = Browse::DBQ::query($sql4);
    
    if ($#$results4 < 0) {
        Browse::HDB::HDBerror("Index: Unable to determine table observatories");
    }
    
    
    my %missions;
    
    foreach (@$results4) {
        if ( $missions{$$_[0]}) {
	    if (ref($missions{$$_[0]})) {
	        push(@{$missions{$$_[0]}}, $$_[1]);
	    } else {
	        $missions{$$_[0]} = [$missions{$$_[0]}, $$_[1]];
	    }
	} else {
            $missions{$$_[0]} = $$_[1];
	}
    }
    
    # Find all tables where we can do a cone search.
    my $sql5 = "select table_name,parameter_value from zzext where parameter_name='default_search_radius'";
    my ($results5) = Browse::DBQ::query($sql5);
    my %radii;
    
    if ($#$results5 < 0) {
        Browse::HDB::HDBerror("Index: Metadata indicates no positional tables available");
    }
    
    for (@$results5) {
        $radii{$$_[0]} = $$_[1];
    }

    # Find priorities for all the tables
   my $sql6 = "select table_name, parameter_value from zzext where ".
              "parameter_name = 'table_priority'";
    my ($results6) =  Browse::DBQ::query($sql6);
    my %priorities;
        if ($#$results6 < 0) {
        Browse::HDB::HDBerror("Index: Unable to determine table priorities");
    }

    for (@$results6) {
        $priorities{$$_[0]} = $$_[1];
    }
    
    # Find table type for all the tables
   my $sql8 = "select table_name, parameter_value from zzext where ".
              "parameter_name = 'table_type'";
    my ($results8) =  Browse::DBQ::query($sql8);
    my %tabletypes;
        if ($#$results8 < 0) {
        Browse::HDB::HDBerror("Index: Unable to determine table types");
    }

    for (@$results8) {
        $tabletypes{$$_[0]} = $$_[1];
    }

#------------------------
   # sort tableinfo by priority
   my @orderedtables=();
   my %maxpriority={};
   my %seen= {};
   my $maxpriority=0;

   my @orderedtables = sort { $priorities{$a} <=> $priorities{$b}} keys %priorities ;
#   foreach (@orderedtables) {
#       my $blah = $$_{"mission"}.$$_{"tabletype"};
#       unless ($seen{$blah}) {
#          $maxpriority{$blah} = $$_{"priority"} unless (($$_{"priority"} > $maxpriority{$blah}) and defined($maxpriority{$blah}));
#          $maxpriority{$blah} = $maxpriority if $maxpriority;
#          $seen{$blah}++;
#       }
#   }
#
#   if ($::debug) {
#       foreach (keys %maxpriority) {
#           print "max priority for $_ : $maxpriority{$_}<br>\n";
#       }
#   }
#
#   foreach (@orderedtables) {
#       my $blah = $$_{"mission"}.$$_{"tabletype"};
#       push @orderedtables, $_ if int($$_{"priority"}) <= $maxpriority{$blah};
#   }
#------------------------

    # Find if have archive data 
   my $sql7 = "select distinct table_name from zzdpsets";
    my ($results7) =  Browse::DBQ::query($sql7);
    my %archive;
        if ($#$results7 < 0) {
        Browse::HDB::HDBerror("Index: Unable to determine dataset existence!");
    }

    for (@$results7) {
        $archive{$$_[0]} = 'Y';
    }

    
    # Loop over all of the tables.  For each table
    # we will create an array of needed information.  Then
    # we'll create a hash with entries point to these arrays.
    
    my %basehash;
    
    my $line;
    foreach $line(@$results1) {
    
        $table = $$line[0];
        my $descrip = $$line[1];
        my $numrow = $$line[3];
        my $url = $$line[2];
        $url =~ s/heasarc/heasarcdev/ if ($::server =~ /heasarcdev/);
        my $mission = $missions{$table};
	my @miss;
	if (ref($mission)) {
	    @miss = @$mission;
	} else {
	    @miss = ($mission);
	}
	foreach $mission(@miss) {
	    my $radius = $radii{$table};
	    my $priority = $priorities{$table};
	    $priority = '5' unless defined $priority;
	    my $tabletype = $tabletypes{$table};
        
            if (!$hiddenhash{$table}) {
	        $$missionCount{$mission} += 1;
                $basehash{"$mission|$descrip|$table"} = 
                  [$mission, $table, $descrip, $url, $numrow, $numcols{$table}, $radius,$priority,$archive{$table},$tabletype];
            }
	}
    }
    return %basehash;

} #eosub--indexInfo
#--------------------------------------------------------------------------
sub printHeader {

# Print the top of the form.
 Browse::Decor::Header('HEASARC Catalog Resources Index');

# navtrail
my $navtrail = "<span class='navcurrent'>Choose Tables</span> <span class='navfuture'>&gt; Parameter Search &gt; Search Results &gt; Choose Data Products</span>";
print "$navtrail<br>\n";
# not a form, so we can't pass the navtrail info along to w3query.


print <<EOT;
<p>
<i>Mission</i> links to FTP area (where available) and mission description<br>
<i>Catalog name</i> links to search of table<br>
<i>Short name</i> links to help for table <br>
<i>Number of entries</i> links to TDAT (ASCII) file containing table data.
<p>
A <a target=helpwin href="http://heasarc.gsfc.nasa.gov/docs/heasarc/missions/alphabet.html"> table of descriptions </a> for
all high-energy missions is available.
<br>
EOT
#<table class='othertable'>
#<tr class='othertable'>
#<th class='othertable'> Example mission (description) </th>
#<th class='othertable' align='right' colspan='5'> Num.Tables</th>
#</tr>
#</table>
#<br>
#EOT

} #eosub--printHeader
#--------------------------------------------------------------------------
sub missionPointers {

    # Print an internal index for the page.

    my ($info) = @_;
    
    my $oldmission;
    my $key;
    
    print '<p><b> Internal Index: </b>'."\n";
    for $key (sort(keys(%$info))) {
    
        my ($mission, $table) = split('\|', $key);
	
	
	if ($mission ne $oldmission) {
	    if ($oldmission) {
	        print ", \n";
	    }
	    print "<a href=\"#$mission\">$mission</a>";
	    $oldmission = $mission;
	}
    }

} #eosub--missionPointers
#--------------------------------------------------------------------------
sub missionHeader {

    # Begin the table for a given mission (and possibly
    # end the previous table.  This includes creating
    # links to FTP and mission description information.
    
    my ($oldMission, $mission, $ftphash,$missionCount) = @_;
    
    # End the old table.
    if ($oldMission) {
        print "</table><p>";
    }

    
    # Create the mission line.
	
    my $miss = $mission;
    $miss =~ tr/A-Z /a-z/d;
    
    my $miss_ftp;
	
    if ($mission_trans_ftp{$miss}) {
	$miss_ftp = $mission_trans_ftp{$miss};
    } else {
	$miss_ftp = $miss;
    }
    
    my $miss_doc;
    if ($mission_trans_doc{$miss}) {
	$miss_doc = $mission_trans_doc{$miss};
    } else {
	$miss_doc = $miss;
    }
    
    if ($$ftphash{$miss_ftp}) {
        print "<a href=\"ftp://$::ftphost/$miss_ftp\">$mission</A> ";
    } else {
        print "$mission ";
    }

    if ($specialLink{$miss}) {
        print "(<a target='helpwin' href=\"$specialLink{$miss}\">description</a>)";
    } elsif (-e "$::root/docs/heasarc/missions/$miss_doc.html") {
	print "(<a target='helpwin' href=\"/docs/heasarc/missions/$miss_doc.html\">description</a>)"
    } elsif (-e "$::root/docs/$miss_doc/$miss_doc.html") {
	print "(<a target='helpwin' href=\"/docs/$miss_doc/$miss_doc.html\">description</a>)"
    } else {
    }
    
    print "<a name=\"" . $mission . "\">";
    print "</a>". "\n";

    my $plural = "";
    if ($$missionCount{$mission} > 1) {
        $plural = "s";
    }
    print ": $$missionCount{$mission} table$plural\n";
    print "<table class='othertable'width='100%'>\n";
print <<EOT;
<tr class='othertable'>
<th scope='col' class='othertable'>Table (short name)</th>
<th scope='col' class='othertable'>Num parameters</th>
<th scope='col' class='othertable'>Num entries</th>
<th scope='col' class='othertable'>Data Products?</th>
<th scope='col' class='othertable'>Priority</th>
<th scope='col' class='othertable'>Table type</th>
</tr>
EOT
}  #eosub--missionHeader
#--------------------------------------------------------------------------
sub printEntry {

    # Print one table entry.
    
    my ($mission, $tab, $descrip, $url, 
        $nrows, $ncols, $radius,$pri,$archive,$tabletype) = @{$_[0]};
	
    my ($tabx);
    my ($missx);

    my($loc_tab) = $tab;
    $loc_tab =~ tr/a-z/A-Z/;
    $tab =~ s/$::local_origin//;

    # The table name is a link to documentation.
    $tabx = $tab;
    $tabx =~ tr/A-Z /a-z/d;

    $mission =~ s/ /\+/g;
    #my($action) = '&Action=Parameter+Search'; 
    my($action) = '&Action=More+Options&Action=Parameter+Search'; 
    my($radoption) = '&ConeAdd=1' if ($radius); 
    my($mis)    = "&mission=$mission"; 
    my($info)   = "name=". lc($loc_tab) ."&".
                  "description=$descrip&".
                  "url=$url&" .
                  "archive=$archive&".
                  "radius=$radius&" .
                  "mission=$mission&". 
                  "priority=$pri";
    $info = '&tablehead='.Browse::CGI::Encode($info);

    print "<tr class='othertable'>\n";
    print "<td align='left' class='othertable'>\n";
    print "<a href=\"w3query.pl?$info$mis$action$radoption\" title='Parameter query of $tab' >$descrip</a>";
    #print "<a href=\"w3query.pl?tablehead=name%3D".lc($tab)."$action\">$descrip</a>";
    print " (&nbsp;<a target='helpwin' href='$url' title='Documentation for $tab'>$tab</a>&nbsp;)\n" ;
    
    print "</td><td align='right' class='othertable'>\n";
    print "$ncols\n";
    print "</td><td align='right' class='othertable'>";
    
    # Number of rows links to the TDAT file.
    if ($loc_tab =~ /HEASARC/ && $loc_tab !~ /VIEW/){ 
         print "<a href=\"$::heasarc/FTP/heasarc/dbase/tdat_files/$::local_origin$tab.tdat.gz\" title='TDAT format dump of $tab'>$nrows</a></td>\n";
    }else{
         print "$nrows</td>\n";

    }
    $archive = "N" unless $archive;
    print "<td class='othertable'> $archive </td>\n";
    my $priority='';
       $priority = "HIGH" if int($pri) <= 3;
       $priority = "LOW" if int($pri) >= 6;
       $priority = "MED" if !$priority;
    print "<td class='othertable'> $priority";
    print "&nbsp;($pri)" if $::host =~ /heasarcdev/ or $::debug;
    print "</td>\n";
    print "<td class='othertable'> $tabletype </td>\n";
    print "</tr>\n";

}  #eosub--printEntry
#--------------------------------------------------------------------------

#########
#  EOF  #
#########
