#!/usr1/local/bin/perl5 -T

#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    plotvalues.pl
# Module Author:  Laura McDonald
# Date:           Sept, 2005
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
#  
#  Display and save (to file) values from Heasarc Java Plot
#
#=======================================================================#
use strict;
use lib './lib';

use Browse::Config;
use Browse::Utils;
Browse::Config::AlwaysRun();
use Browse::CGI;

my($in,$input) = Browse::CGI::ReadParse(1);  # Read and parse the query string

my $timetag=time();
if (defined($in->{downloadValues})) {
  #----------------------------------------------
  #--- Save values to file 

  # -- use process id and time tag for user local file name
  my $localFileName = $::savedPlotValuesFile . ".$$.$timetag.txt";  
  print "Content-Type:application/x-download\n" .
	"Content-Disposition:attachment;filename=" .
	"$localFileName\n\n";

   #print "Content-type:text/plain\n\n";

  if ($in->{plotcolumns} =~ /\w/) {
     print $in->{plotcolumns};
  }
     print "\n";
  if ($in->{plotvalues} =~ /\w/) {
     print $in->{plotvalues};
  } else {
     print "No Plot Values\n";
  }

} else {
   #----------------------------------------------
   #--- Display values on HTML page

   print "Content-type:text/plain\n\n";

   my $saveForDownload="";
   my $saveHForDownload="";

   #--- create html file in FTP area where it will be deleted 
   #--- eventually
   my $htmlFile ="/FTP/$::tarurlbase/javaplot.$$.$timetag.html"; 
   open JAVAPLOT, ">$htmlFile";

   my @pvalues=split(/\|/,$$in{pvalues});
   my @headers=split(/\,/,$$in{columnNames});

   #--- setup HTML page
   print JAVAPLOT "<html><header>\n";
   print JAVAPLOT "<title>Java Plot Information</title>\n";
   print JAVAPLOT "</header>\n";
   print JAVAPLOT "<body>\n";
   print JAVAPLOT "<h2>Plot Values</h2><p>\n";
   print JAVAPLOT "<form method='POST'  name='plotvalues' " .
		"  action='/$::cgi/w3plotvalues.pl'>\n";
   print JAVAPLOT "<input type='Submit' name='downloadValues'" .
	"  value='Download to File'><p>\n";
  
   #--- Plot values table
   print JAVAPLOT "<table cellspacing='6' cellpadding='6'> <tr>";

   #--- Column headers
   foreach my $name (@headers) {
     print JAVAPLOT "<td align='right'>$name</td>\n";
      $saveHForDownload .=" $name |";
   }
   chop($saveHForDownload);
   $saveHForDownload .="\n";

   #--- Data
   print JAVAPLOT "</tr><tr>";

   foreach my $set (@pvalues) {
      my (@fields) = split(/,/,$set);
   
      #--- x axis value
      my $x = shift @fields;
      print JAVAPLOT "<td align='right'>$x</td>";
      $saveForDownload .="$x";

      #--- y axis values
      foreach my $y (@fields) {
        print JAVAPLOT "<td align='right'>$y</td>\n";
        $saveForDownload .=" | $y";
      }
      print JAVAPLOT "</tr><tr>";
        $saveForDownload .="\n";
   }

   #--- close up table
   print JAVAPLOT "</tr></table>";
   print JAVAPLOT "<input type='hidden' name='plotvalues' " .
	"value='$saveForDownload'>";
   print JAVAPLOT "<input type='hidden' name='plotcolumns' " .
	"value='$saveHForDownload'>";
   #--- close up form
   print JAVAPLOT "</form>\n";

   #--- close up HTML file
   print JAVAPLOT "</body></html>\n";
   close JAVADIR;

   print "http://$::host" . "$htmlFile";
}



