/*******************************************************************************

    NED name-to-coordinate resolver interface application.

    AUTHOR: Song Yom (HITC)
    DATE:   05/95

    MODIFICATION HISTORY:
    DATE    PROGRAMMER      DESCRIPTION
    ----    ----------      -----------

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "ned_client.h"

#ifdef sun
#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1
#endif

#if defined(sun) && !defined(__STDC__)
extern int ned_connect();
extern void ned_disconnect();
extern int ned_obj_byname();
extern void ned_free_cp();
extern void ned_free_objp();
extern char *ned_get_errmsg();
#else
extern int ned_connect(void);
extern void ned_disconnect(void);
extern int ned_obj_byname(char *,int *,ObjInfo **,CrossID **);
extern void ned_free_cp(CrossID *);
extern void ned_free_objp(ObjInfo *,int);
extern char *ned_get_errmsg(void);
#endif

#if defined(sun) && !defined(__STDC__)
main(argc,argv)
    int argc;
    char *argv[];
#else
main(int argc,char *argv[])
#endif
{
    int status = EXIT_FAILURE;
    int no_obj;
    CrossID *cp;
    ObjInfo *op;

/*
    Verify the command line, establish connection with the server, query
    the server for the specified object name, print returned right ascension
    and declination values, free internal data structures, and disconnect
    from the server.  Write data or message to standard output.

    PDL:
    if correct argument count
        if successful connection to server
            if object found
                print result
            else
                signal message
            free internal data structures
            disconnect from server
        else
            signal message
    else
        signal usage message
*/

    if (argc == 2)
    {
        if (ned_connect() == 0)
        {
            if (ned_obj_byname(argv[1],&no_obj,&op,&cp) == 0)
            {
                fprintf(stdout,"%f,%f\n",op->ra,op->dec);
                status = EXIT_SUCCESS;
            }
            else
                fprintf(stdout,"error: %s\n",ned_get_errmsg());
            if (cp)
                ned_free_cp(cp);
            if (op)
                ned_free_objp(op,no_obj);
            ned_disconnect();
        }
        else
            fprintf(stdout,"error: %s\n",ned_get_errmsg());
    }
    else
        fprintf(stdout,"usage: %s name\n",argv[0]);

    exit(status);
}
